@@ -23,7 +23,6 @@ import org.apache.hudi.aws.sync.util.GluePartitionFilterGenerator;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieFileFormat;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
-import org.apache.hudi.common.table.TableSchemaResolver;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.util.CollectionUtils;
@@ -81,7 +80,11 @@ import software.amazon.awssdk.services.glue.model.SerDeInfo;
 import software.amazon.awssdk.services.glue.model.StorageDescriptor;
 import software.amazon.awssdk.services.glue.model.Table;
 import software.amazon.awssdk.services.glue.model.TableInput;
+import software.amazon.awssdk.services.glue.model.TagResourceRequest;
 import software.amazon.awssdk.services.glue.model.UpdateTableRequest;
+import software.amazon.awssdk.services.sts.StsClient;
+import software.amazon.awssdk.services.sts.model.GetCallerIdentityRequest;
+import software.amazon.awssdk.services.sts.model.GetCallerIdentityResponse;
 
 import java.net.URI;
 import java.net.URISyntaxException;
@@ -112,12 +115,14 @@ import static org.apache.hudi.config.GlueCatalogSyncClientConfig.META_SYNC_PARTI
 import static org.apache.hudi.config.GlueCatalogSyncClientConfig.PARTITION_CHANGE_PARALLELISM;
 import static org.apache.hudi.config.HoodieAWSConfig.AWS_GLUE_ENDPOINT;
 import static org.apache.hudi.config.HoodieAWSConfig.AWS_GLUE_REGION;
+import static org.apache.hudi.config.GlueCatalogSyncClientConfig.GLUE_SYNC_DATABASE_NAME;
+import static org.apache.hudi.config.GlueCatalogSyncClientConfig.GLUE_SYNC_RESOURCE_TAGS;
+import static org.apache.hudi.config.GlueCatalogSyncClientConfig.GLUE_SYNC_TABLE_NAME;
 import static org.apache.hudi.hive.HiveSyncConfigHolder.HIVE_CREATE_MANAGED_TABLE;
 import static org.apache.hudi.hive.HiveSyncConfigHolder.HIVE_SUPPORT_TIMESTAMP_TYPE;
 import static org.apache.hudi.hive.util.HiveSchemaUtil.getPartitionKeyType;
 import static org.apache.hudi.hive.util.HiveSchemaUtil.parquetSchemaToMapSchema;
 import static org.apache.hudi.sync.common.HoodieSyncConfig.META_SYNC_BASE_FILE_FORMAT;
-import static org.apache.hudi.sync.common.HoodieSyncConfig.META_SYNC_DATABASE_NAME;
 import static org.apache.hudi.sync.common.HoodieSyncConfig.META_SYNC_PARTITION_FIELDS;
 import static org.apache.hudi.sync.common.util.TableUtils.tableId;
 
@@ -141,27 +146,35 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
    * see https://docs.aws.amazon.com/athena/latest/ug/querying-hudi.html
    */
   private static final String ENABLE_MDT_LISTING = "hudi.metadata-listing-enabled";
+  private static final String GLUE_TABLE_ARN_FORMAT = "arn:aws:glue:%s:%s:table/%s/%s";
+  private static final String GLUE_DATABASE_ARN_FORMAT = "arn:aws:glue:%s:%s:database/%s";
   private final String databaseName;
+  private final String tableName;
 
-  private final Boolean skipTableArchive;
+  private final boolean skipTableArchive;
   private final String enableMetadataTable;
   private final int allPartitionsReadParallelism;
   private final int changedPartitionsReadParallelism;
   private final int changeParallelism;
+  private final Map<String, Table> initialTableByName = new HashMap<>();
+  private final String catalogId;
 
   public AWSGlueCatalogSyncClient(HiveSyncConfig config, HoodieTableMetaClient metaClient) {
-    this(buildAsyncClient(config), config, metaClient);
+    this(buildAsyncClient(config), StsClient.create(), config, metaClient);
   }
 
-  AWSGlueCatalogSyncClient(GlueAsyncClient awsGlue, HiveSyncConfig config, HoodieTableMetaClient metaClient) {
+  AWSGlueCatalogSyncClient(GlueAsyncClient awsGlue, StsClient stsClient, HiveSyncConfig config, HoodieTableMetaClient metaClient) {
     super(config, metaClient);
     this.awsGlue = awsGlue;
-    this.databaseName = config.getStringOrDefault(META_SYNC_DATABASE_NAME);
+    this.databaseName = config.getStringOrDefault(GLUE_SYNC_DATABASE_NAME, GLUE_SYNC_DATABASE_NAME.getInferFunction().get().apply(config).get());
+    this.tableName = config.getStringOrDefault(GLUE_SYNC_TABLE_NAME, GLUE_SYNC_TABLE_NAME.getInferFunction().get().apply(config).get());
     this.skipTableArchive = config.getBooleanOrDefault(GlueCatalogSyncClientConfig.GLUE_SKIP_TABLE_ARCHIVE);
     this.enableMetadataTable = Boolean.toString(config.getBoolean(GLUE_METADATA_FILE_LISTING)).toUpperCase();
     this.allPartitionsReadParallelism = config.getIntOrDefault(ALL_PARTITIONS_READ_PARALLELISM);
     this.changedPartitionsReadParallelism = config.getIntOrDefault(CHANGED_PARTITIONS_READ_PARALLELISM);
     this.changeParallelism = config.getIntOrDefault(PARTITION_CHANGE_PARALLELISM);
+    GetCallerIdentityResponse identityResponse = stsClient.getCallerIdentity(GetCallerIdentityRequest.builder().build());
+    this.catalogId = config.getStringOrDefault(GlueCatalogSyncClientConfig.GLUE_CATALOG_ID, identityResponse.account());
   }
 
   private static GlueAsyncClient buildAsyncClient(HiveSyncConfig config) {
@@ -178,12 +191,23 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
     }
   }
 
+  @Override
+  public String getTableName() {
+    return this.tableName;
+  }
+
+  @Override
+  public String getDatabaseName() {
+    return this.databaseName;
+  }
+
   private List<Partition> getPartitionsSegment(Segment segment, String tableName) {
     try {
       List<Partition> partitions = new ArrayList<>();
       String nextToken = null;
       do {
         GetPartitionsResponse result = awsGlue.getPartitions(GetPartitionsRequest.builder()
+            .catalogId(catalogId)
             .databaseName(databaseName)
             .tableName(tableName)
             .excludeColumnSchema(true)
@@ -201,6 +225,10 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
     }
   }
 
+  private Table getInitialTable(String tableName) {
+    return initialTableByName.computeIfAbsent(tableName, t -> getTable(awsGlue, databaseName, t));
+  }
+
   @Override
   public List<Partition> getAllPartitions(String tableName) {
     ExecutorService executorService = Executors.newFixedThreadPool(this.allPartitionsReadParallelism, new CustomizedThreadFactory("glue-sync-all-partitions", true));
@@ -328,7 +356,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
         return PartitionInput.builder().values(partitionValues).storageDescriptor(partitionSD).build();
       }).collect(Collectors.toList());
 
-      BatchCreatePartitionRequest request = BatchCreatePartitionRequest.builder()
+      BatchCreatePartitionRequest request = BatchCreatePartitionRequest.builder().catalogId(catalogId)
           .databaseName(databaseName).tableName(table.name()).partitionInputList(partitionInputList).build();
       CompletableFuture<BatchCreatePartitionResponse> future = awsGlue.batchCreatePartition(request);
       BatchCreatePartitionResponse response = future.get();
@@ -336,7 +364,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
         if (response.errors().stream()
             .allMatch(
                 (error) -> "AlreadyExistsException".equals(error.errorDetail().errorCode()))) {
-          LOG.warn("Partitions already exist in glue: " + response.errors());
+          LOG.info("Partitions already exist in glue: {}", response.errors());
         } else {
           throw new HoodieGlueSyncException("Fail to add partitions to " + tableId(databaseName, table.name())
               + " with error(s): " + response.errors());
@@ -373,7 +401,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
         return BatchUpdatePartitionRequestEntry.builder().partitionInput(partitionInput).partitionValueList(partitionValues).build();
       }).collect(Collectors.toList());
 
-      BatchUpdatePartitionRequest request = BatchUpdatePartitionRequest.builder()
+      BatchUpdatePartitionRequest request = BatchUpdatePartitionRequest.builder().catalogId(catalogId)
               .databaseName(databaseName).tableName(table.name()).entries(updatePartitionEntries).build();
       CompletableFuture<BatchUpdatePartitionResponse> future = awsGlue.batchUpdatePartition(request);
 
@@ -409,10 +437,11 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
       ).collect(Collectors.toList());
 
       BatchDeletePartitionRequest batchDeletePartitionRequest = BatchDeletePartitionRequest.builder()
-          .databaseName(databaseName)
-          .tableName(tableName)
-          .partitionsToDelete(partitionValueLists)
-          .build();
+            .catalogId(catalogId)
+            .databaseName(databaseName)
+            .tableName(tableName)
+            .partitionsToDelete(partitionValueLists)
+            .build();
       CompletableFuture<BatchDeletePartitionResponse> future = awsGlue.batchDeletePartition(batchDeletePartitionRequest);
 
       BatchDeletePartitionResponse response = future.get();
@@ -447,7 +476,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
 
   private String getTableDoc() {
     try {
-      return new TableSchemaResolver(metaClient).getTableAvroSchema(true).getDoc();
+      return tableSchemaResolver.getTableAvroSchema(true).getDoc();
     } catch (Exception e) {
       throw new HoodieGlueSyncException("Failed to get schema's doc from storage : ", e);
     }
@@ -456,7 +485,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
   @Override
   public List<FieldSchema> getStorageFieldSchemas() {
     try {
-      return new TableSchemaResolver(metaClient).getTableAvroSchema(true)
+      return tableSchemaResolver.getTableAvroSchema(true)
           .getFields()
           .stream()
           .map(f -> new FieldSchema(f.name(), f.schema().getType().getName(), f.doc()))
@@ -499,6 +528,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
           .build();
 
       UpdateTableRequest request = UpdateTableRequest.builder()
+          .catalogId(catalogId)
           .databaseName(databaseName)
           .tableInput(updatedTableInput)
           .build();
@@ -532,6 +562,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
           .build();
 
       UpdateTableRequest request = UpdateTableRequest.builder()
+          .catalogId(catalogId)
           .databaseName(databaseName)
           .skipArchive(skipTableArchive)
           .tableInput(updatedTableInput)
@@ -658,14 +689,15 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
           .build();
 
       CreateTableRequest request = CreateTableRequest.builder()
+              .catalogId(catalogId)
               .databaseName(databaseName)
               .tableInput(tableInput)
               .build();
 
       CreateTableResponse response = awsGlue.createTable(request).get();
-      LOG.info("Created table " + tableId(databaseName, tableName) + " : " + response);
+      LOG.info("Created table {} : {}", tableId(databaseName, tableName), response);
     } catch (AlreadyExistsException e) {
-      LOG.warn("Table " + tableId(databaseName, tableName) + " already exists.", e);
+      LOG.warn("Table {} already exists.", tableId(databaseName, tableName), e);
     } catch (Exception e) {
       throw new HoodieGlueSyncException("Fail to create " + tableId(databaseName, tableName), e);
     }
@@ -684,14 +716,14 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
     if (!config.getBooleanOrDefault(META_SYNC_PARTITION_INDEX_FIELDS_ENABLE)) {
       // deactivate indexing if enabled
       if (getPartitionIndexEnable(tableName)) {
-        LOG.warn("Deactivating partition indexing");
+        LOG.info("Deactivating partition indexing");
         updatePartitionIndexEnable(tableName, false);
       }
       // also drop all existing indexes
       GetPartitionIndexesRequest indexesRequest = GetPartitionIndexesRequest.builder().databaseName(databaseName).tableName(tableName).build();
       GetPartitionIndexesResponse existingIdxsResp = awsGlue.getPartitionIndexes(indexesRequest).get();
       for (PartitionIndexDescriptor idsToDelete : existingIdxsResp.partitionIndexDescriptorList()) {
-        LOG.warn("Dropping partition index: {}", idsToDelete.indexName());
+        LOG.info("Dropping partition index: {}", idsToDelete.indexName());
         DeletePartitionIndexRequest idxToDelete = DeletePartitionIndexRequest.builder()
                 .databaseName(databaseName).tableName(tableName).indexName(idsToDelete.indexName()).build();
         awsGlue.deletePartitionIndex(idxToDelete).get();
@@ -699,7 +731,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
     } else {
       // activate indexing usage if disabled
       if (!getPartitionIndexEnable(tableName)) {
-        LOG.warn("Activating partition indexing");
+        LOG.info("Activating partition indexing");
         updatePartitionIndexEnable(tableName, true);
       }
 
@@ -724,7 +756,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
           indexesChanges = true;
           DeletePartitionIndexRequest idxToDelete = DeletePartitionIndexRequest.builder()
                   .databaseName(databaseName).tableName(tableName).indexName(existingIdx.indexName()).build();
-          LOG.warn("Dropping irrelevant index: " + existingIdx.indexName());
+          LOG.info("Dropping irrelevant index: {}", existingIdx.indexName());
           awsGlue.deletePartitionIndex(idxToDelete).get();
         }
       }
@@ -746,7 +778,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
           PartitionIndex newIdx = PartitionIndex.builder()
                   .indexName(newIdxName)
                   .keys(neededIdx).build();
-          LOG.warn("Creating new partition index: " + newIdxName);
+          LOG.info("Creating new partition index: {}", newIdxName);
           CreatePartitionIndexRequest creationRequest = CreatePartitionIndexRequest.builder()
                   .databaseName(databaseName).tableName(tableName).partitionIndex(newIdx).build();
           awsGlue.createPartitionIndex(creationRequest).get();
@@ -762,7 +794,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
                                        .map(idx -> Arrays.stream(idx.split(";"))
                                                          .collect(Collectors.toList())).collect(Collectors.toList());
     if (indexes.size() > PARTITION_INDEX_MAX_NUMBER) {
-      LOG.warn(String.format("Only considering first %s indexes", PARTITION_INDEX_MAX_NUMBER));
+      LOG.warn("Only considering first {} indexes", PARTITION_INDEX_MAX_NUMBER);
       return indexes.subList(0, PARTITION_INDEX_MAX_NUMBER);
     }
     return indexes;
@@ -790,7 +822,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
     try {
       // GlueMetastoreClient returns partition keys separate from Columns, hence get both and merge to
       // get the Schema of the table.
-      Table table = getTable(awsGlue, databaseName, tableName);
+      Table table = getInitialTable(tableName);
       Map<String, String> partitionKeysMap =
           table.partitionKeys().stream().collect(Collectors.toMap(Column::name, f -> f.type().toUpperCase()));
 
@@ -809,14 +841,19 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
   @Override
   public boolean tableExists(String tableName) {
     GetTableRequest request = GetTableRequest.builder()
+        .catalogId(catalogId)
         .databaseName(databaseName)
         .name(tableName)
         .build();
     try {
-      return Objects.nonNull(awsGlue.getTable(request).get().table());
+      Table table = awsGlue.getTable(request).get().table();
+      if (table != null) {
+        initialTableByName.put(tableName, table);
+      }
+      return Objects.nonNull(table);
     } catch (ExecutionException e) {
       if (e.getCause() instanceof EntityNotFoundException) {
-        LOG.warn("Table not found: " + tableId(databaseName, tableName), e);
+        LOG.info("Table not found: {}.{}", databaseName, tableName, e);
         return false;
       } else {
         throw new HoodieGlueSyncException("Fail to get table: " + tableId(databaseName, tableName), e);
@@ -828,12 +865,15 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
 
   @Override
   public boolean databaseExists(String databaseName) {
-    GetDatabaseRequest request = GetDatabaseRequest.builder().name(databaseName).build();
+    GetDatabaseRequest request = GetDatabaseRequest.builder()
+        .catalogId(catalogId)
+        .name(databaseName)
+        .build();
     try {
       return Objects.nonNull(awsGlue.getDatabase(request).get().database());
     } catch (ExecutionException e) {
       if (e.getCause() instanceof EntityNotFoundException) {
-        LOG.warn("Database not found: " + databaseName, e);
+        LOG.info("Database not found: {}", databaseName, e);
         return false;
       } else {
         throw new HoodieGlueSyncException("Fail to check if database exists " + databaseName, e);
@@ -849,6 +889,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
       return;
     }
     CreateDatabaseRequest request = CreateDatabaseRequest.builder()
+            .catalogId(catalogId)
             .databaseInput(DatabaseInput.builder()
             .name(databaseName)
             .description("Automatically created by " + this.getClass().getName())
@@ -858,9 +899,10 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
     ).build();
     try {
       CreateDatabaseResponse result = awsGlue.createDatabase(request).get();
-      LOG.info("Successfully created database in AWS Glue: " + result.toString());
+      tagResource(String.format(GLUE_DATABASE_ARN_FORMAT, awsGlue.serviceClientConfiguration().region(), catalogId, databaseName));
+      LOG.info("Successfully created database in AWS Glue: {}", result.toString());
     } catch (AlreadyExistsException e) {
-      LOG.warn("AWS Glue Database " + databaseName + " already exists", e);
+      LOG.info("AWS Glue Database {} already exists", databaseName, e);
     } catch (Exception e) {
       throw new HoodieGlueSyncException("Fail to create database " + databaseName, e);
     }
@@ -869,8 +911,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
   @Override
   public Option<String> getLastCommitTimeSynced(String tableName) {
     try {
-      Table table = getTable(awsGlue, databaseName, tableName);
-      return Option.ofNullable(table.parameters().getOrDefault(HOODIE_LAST_COMMIT_TIME_SYNC, null));
+      return Option.ofNullable(getInitialTable(tableName).parameters().getOrDefault(HOODIE_LAST_COMMIT_TIME_SYNC, null));
     } catch (Exception e) {
       throw new HoodieGlueSyncException("Fail to get last sync commit time for " + tableId(databaseName, tableName), e);
     }
@@ -880,8 +921,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
   public Option<String> getLastCommitCompletionTimeSynced(String tableName) {
     // Get the last commit completion time from the TBLproperties
     try {
-      Table table = getTable(awsGlue, databaseName, tableName);
-      return Option.ofNullable(table.parameters().getOrDefault(HOODIE_LAST_COMMIT_COMPLETION_TIME_SYNC, null));
+      return Option.ofNullable(getInitialTable(tableName).parameters().getOrDefault(HOODIE_LAST_COMMIT_COMPLETION_TIME_SYNC, null));
     } catch (Exception e) {
       throw new HoodieGlueSyncException("Failed to get the last commit completion time synced from the table " + tableName, e);
     }
@@ -910,7 +950,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
         throw new HoodieGlueSyncException("Fail to update last sync commit time for " + tableId(databaseName, tableName), e);
       }
     } else {
-      LOG.warn("No commit in active timeline.");
+      LOG.info("No commit in active timeline.");
     }
     try {
       // as a side effect, we also refresh the partition indexes if needed
@@ -927,6 +967,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
   @Override
   public void dropTable(String tableName) {
     DeleteTableRequest deleteTableRequest = DeleteTableRequest.builder()
+        .catalogId(catalogId)
         .databaseName(databaseName)
         .name(tableName)
         .build();
@@ -999,7 +1040,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
         boolean different = serdeProperties.entrySet().stream().anyMatch(e ->
                 !existingSerdeProperties.containsKey(e.getKey()) || !existingSerdeProperties.get(e.getKey()).equals(e.getValue()));
         if (!different) {
-          LOG.debug("Table " + tableName + " serdeProperties already up to date, skip update serde properties.");
+          LOG.debug("Table {} serdeProperties already up to date, skip update serde properties.", tableName);
           return false;
         }
       }
@@ -1045,8 +1086,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
   @Override
   public String getTableLocation(String tableName) {
     try {
-      Table table = getTable(awsGlue, databaseName, tableName);
-      return table.storageDescriptor().location();
+      return getInitialTable(tableName).storageDescriptor().location();
     } catch (Exception e) {
       throw new HoodieGlueSyncException("Fail to get base path for the table " + tableId(databaseName, tableName), e);
     }
@@ -1073,8 +1113,9 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
     MATERIALIZED_VIEW
   }
 
-  private static Table getTable(GlueAsyncClient awsGlue, String databaseName, String tableName) throws HoodieGlueSyncException {
+  private Table getTable(GlueAsyncClient awsGlue, String databaseName, String tableName) throws HoodieGlueSyncException {
     GetTableRequest request = GetTableRequest.builder()
+        .catalogId(catalogId)
         .databaseName(databaseName)
         .name(tableName)
         .build();
@@ -1087,7 +1128,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
     }
   }
 
-  private static boolean updateTableParameters(GlueAsyncClient awsGlue, String databaseName, String tableName, Map<String, String> updatingParams, boolean skipTableArchive) {
+  private boolean updateTableParameters(GlueAsyncClient awsGlue, String databaseName, String tableName, Map<String, String> updatingParams, boolean skipTableArchive) {
     if (isNullOrEmpty(updatingParams)) {
       return false;
     }
@@ -1114,6 +1155,7 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
           .build();
 
       UpdateTableRequest request =  UpdateTableRequest.builder().databaseName(databaseName)
+          .catalogId(catalogId)
           .tableInput(updatedTableInput)
           .skipArchive(skipTableArchive)
           .build();
@@ -1123,4 +1165,31 @@ public class AWSGlueCatalogSyncClient extends HoodieSyncClient {
       throw new HoodieGlueSyncException("Fail to update params for table " + tableId(databaseName, tableName) + ": " + updatingParams, e);
     }
   }
+
+  private void tagResource(String resourceArn) {
+    Map<String, String> resourceTags = parseResourceTags(config.getStringOrDefault(GLUE_SYNC_RESOURCE_TAGS, ""));
+    if (resourceTags.isEmpty()) {
+      return;
+    }
+    TagResourceRequest tagRequest = TagResourceRequest.builder()
+        .resourceArn(resourceArn)
+        .tagsToAdd(resourceTags)
+        .build();
+    awsGlue.tagResource(tagRequest).join();
+  }
+
+  private Map<String, String> parseResourceTags(String resourceTagKeyValues) {
+    Map<String, String> tags = new HashMap<>();
+    if (resourceTagKeyValues == null || resourceTagKeyValues.trim().isEmpty()) {
+      return tags;
+    }
+    String[] tagPairs = resourceTagKeyValues.split(",");
+    for (String tagPair : tagPairs) {
+      String[] keyValue = tagPair.split(":", 2);
+      if (keyValue.length == 2) {
+        tags.put(keyValue[0].trim(), keyValue[1].trim());
+      }
+    }
+    return tags;
+  }
 }
