@@ -19,11 +19,14 @@
 package org.apache.hudi.cli.commands;
 
 import org.apache.hudi.cli.functional.CLIFunctionalTestHarness;
+import org.apache.hudi.cli.testutils.ShellEvaluationResultUtil;
 import org.apache.hudi.table.HoodieTable;
 
 import org.junit.jupiter.api.Tag;
 import org.junit.jupiter.api.Test;
-import org.springframework.shell.core.CommandResult;
+import org.springframework.beans.factory.annotation.Autowired;
+import org.springframework.boot.test.context.SpringBootTest;
+import org.springframework.shell.Shell;
 
 import static org.junit.jupiter.api.Assertions.assertAll;
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -34,19 +37,23 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
  * Test class for {@link org.apache.hudi.cli.commands.UtilsCommand}.
  */
 @Tag("functional")
+@SpringBootTest(properties = {"spring.shell.interactive.enabled=false", "spring.shell.command.script.enabled=false"})
 public class TestUtilsCommand extends CLIFunctionalTestHarness {
 
+  @Autowired
+  private Shell shell;
+
   /**
    * Test case for success load class.
    */
   @Test
   public void testLoadClass() {
     String name = HoodieTable.class.getName();
-    CommandResult cr = shell().executeCommand(String.format("utils loadClass --class %s", name));
+    Object result = shell.evaluate(() -> String.format("utils loadClass --class %s", name));
     assertAll("Command runs success",
-        () -> assertTrue(cr.isSuccess()),
-        () -> assertNotNull(cr.getResult().toString()),
-        () -> assertTrue(cr.getResult().toString().startsWith("file:")));
+        () -> assertTrue(ShellEvaluationResultUtil.isSuccess(result)),
+        () -> assertNotNull(result.toString()),
+        () -> assertTrue(result.toString().startsWith("file:")));
   }
 
   /**
@@ -55,12 +62,12 @@ public class TestUtilsCommand extends CLIFunctionalTestHarness {
   @Test
   public void testLoadClassNotFound() {
     String name = "test.class.NotFound";
-    CommandResult cr = shell().executeCommand(String.format("utils loadClass --class %s", name));
+    Object result = shell.evaluate(() -> String.format("utils loadClass --class %s", name));
 
     assertAll("Command runs success",
-        () -> assertTrue(cr.isSuccess()),
-        () -> assertNotNull(cr.getResult().toString()),
-        () -> assertEquals(cr.getResult().toString(), String.format("Class %s not found!", name)));
+        () -> assertTrue(ShellEvaluationResultUtil.isSuccess(result)),
+        () -> assertNotNull(result.toString()),
+        () -> assertEquals(result.toString(), String.format("Class %s not found!", name)));
   }
 
   /**
@@ -69,11 +76,11 @@ public class TestUtilsCommand extends CLIFunctionalTestHarness {
   @Test
   public void testLoadClassNull() {
     String name = "";
-    CommandResult cr = shell().executeCommand(String.format("utils loadClass --class %s", name));
+    Object result = shell.evaluate(() -> String.format("utils loadClass --class %s", name));
 
     assertAll("Command runs success",
-        () -> assertTrue(cr.isSuccess()),
-        () -> assertNotNull(cr.getResult().toString()),
-        () -> assertEquals("Class to be loaded can not be null!", cr.getResult().toString()));
+        () -> assertTrue(ShellEvaluationResultUtil.isSuccess(result)),
+        () -> assertNotNull(result.toString()),
+        () -> assertEquals("Class to be loaded can not be null!", result.toString()));
   }
 }
