@@ -40,10 +40,9 @@ import java.util.List;
 import java.util.Map;
 import java.util.UUID;
 
+import static org.apache.hudi.common.testutils.FileCreateUtilsLegacy.baseFileName;
 import static org.apache.hudi.common.testutils.HoodieTestUtils.COMMIT_METADATA_SER_DE;
 import static org.apache.hudi.common.testutils.HoodieTestUtils.INSTANT_FILE_NAME_GENERATOR;
-import static org.apache.hudi.common.table.timeline.TimelineMetadataUtils.serializeCommitMetadata;
-import static org.apache.hudi.common.testutils.FileCreateUtilsLegacy.baseFileName;
 import static org.apache.hudi.common.util.CollectionUtils.createImmutableList;
 
 /**
@@ -95,7 +94,7 @@ public class HoodieTestCommitMetadataGenerator extends HoodieTestDataGenerator {
     for (String name : commitFileNames) {
       HoodieCommitMetadata commitMetadata =
               generateCommitMetadata(basePath, commitTime, fileId1, fileId2, writes, updates, extraMetadata, true);
-      createFileWithMetadata(basePath, configuration, name, serializeCommitMetadata(COMMIT_METADATA_SER_DE, commitMetadata).get());
+      createFileWithMetadata(basePath, configuration, name, commitMetadata);
     }
   }
 
@@ -110,14 +109,14 @@ public class HoodieTestCommitMetadataGenerator extends HoodieTestDataGenerator {
     for (String name : commitFileNames) {
       HoodieCommitMetadata commitMetadata =
           generateCommitMetadata(basePath, commitTime, fileId1, fileId2, writes, updates, extraMetadata, setDefaultFileId);
-      createFileWithMetadata(basePath, configuration, name, serializeCommitMetadata(COMMIT_METADATA_SER_DE, commitMetadata).get());
+      createFileWithMetadata(basePath, configuration, name, commitMetadata);
     }
   }
 
-  static void createFileWithMetadata(String basePath, StorageConfiguration<?> configuration, String name, byte[] content) throws IOException {
+  static <T> void createFileWithMetadata(String basePath, StorageConfiguration<?> configuration, String name, T metadata) throws IOException {
     Path commitFilePath = new Path(basePath + "/" + HoodieTableMetaClient.METAFOLDER_NAME + "/" + name);
     try (OutputStream os = HadoopFSUtils.getFs(basePath, configuration).create(commitFilePath, true)) {
-      os.write(content);
+      COMMIT_METADATA_SER_DE.getInstantWriter(metadata).get().writeToStream(os);
     }
   }
 
