@@ -33,13 +33,13 @@ import java.io.Serializable;
  * This class allows clients to start and end transactions. Anything done between a start and end transaction is
  * guaranteed to be atomic.
  */
-public class TransactionManager implements Serializable {
+public class TransactionManager implements Serializable, AutoCloseable {
 
   protected static final Logger LOG = LoggerFactory.getLogger(TransactionManager.class);
   protected final LockManager lockManager;
   protected final boolean isLockRequired;
-  protected Option<HoodieInstant> currentTxnOwnerInstant = Option.empty();
-  private Option<HoodieInstant> lastCompletedTxnOwnerInstant = Option.empty();
+  protected Option<HoodieInstant> changeActionInstant = Option.empty();
+  private Option<HoodieInstant> lastCompletedActionInstant = Option.empty();
 
   public TransactionManager(HoodieWriteConfig config, HoodieStorage storage) {
     this(new LockManager(config, storage), config.isLockRequired());
@@ -50,39 +50,40 @@ public class TransactionManager implements Serializable {
     this.isLockRequired = isLockRequired;
   }
 
-  public void beginTransaction(Option<HoodieInstant> newTxnOwnerInstant,
-                               Option<HoodieInstant> lastCompletedTxnOwnerInstant) {
+  public void beginStateChange(Option<HoodieInstant> changeActionInstant,
+                               Option<HoodieInstant> lastCompletedActionInstant) {
     if (isLockRequired) {
-      LOG.info("Transaction starting for " + newTxnOwnerInstant
-          + " with latest completed transaction instant " + lastCompletedTxnOwnerInstant);
+      LOG.info("State change starting for {} with latest completed action instant {}",
+          changeActionInstant, lastCompletedActionInstant);
       lockManager.lock();
-      reset(currentTxnOwnerInstant, newTxnOwnerInstant, lastCompletedTxnOwnerInstant);
-      LOG.info("Transaction started for " + newTxnOwnerInstant
-          + " with latest completed transaction instant " + lastCompletedTxnOwnerInstant);
+      reset(this.changeActionInstant, changeActionInstant, lastCompletedActionInstant);
+      LOG.info("State change started for {} with latest completed action instant {}",
+          changeActionInstant, lastCompletedActionInstant);
     }
   }
 
-  public void endTransaction(Option<HoodieInstant> currentTxnOwnerInstant) {
+  public void endStateChange(Option<HoodieInstant> changeActionInstant) {
     if (isLockRequired) {
-      LOG.info("Transaction ending with transaction owner " + currentTxnOwnerInstant);
-      if (reset(currentTxnOwnerInstant, Option.empty(), Option.empty())) {
+      LOG.info("State change ending for action instant {}", changeActionInstant);
+      if (reset(changeActionInstant, Option.empty(), Option.empty())) {
         lockManager.unlock();
-        LOG.info("Transaction ended with transaction owner " + currentTxnOwnerInstant);
+        LOG.info("State change ended for action instant {}", changeActionInstant);
       }
     }
   }
 
   protected synchronized boolean reset(Option<HoodieInstant> callerInstant,
-                                       Option<HoodieInstant> newTxnOwnerInstant,
-                                       Option<HoodieInstant> lastCompletedTxnOwnerInstant) {
-    if (!this.currentTxnOwnerInstant.isPresent() || this.currentTxnOwnerInstant.get().equals(callerInstant.get())) {
-      this.currentTxnOwnerInstant = newTxnOwnerInstant;
-      this.lastCompletedTxnOwnerInstant = lastCompletedTxnOwnerInstant;
+                                       Option<HoodieInstant> changeActionInstant,
+                                       Option<HoodieInstant> lastCompletedActionInstant) {
+    if (!this.changeActionInstant.isPresent() || this.changeActionInstant.get().equals(callerInstant.get())) {
+      this.changeActionInstant = changeActionInstant;
+      this.lastCompletedActionInstant = lastCompletedActionInstant;
       return true;
     }
     return false;
   }
 
+  @Override
   public void close() {
     if (isLockRequired) {
       lockManager.close();
@@ -95,11 +96,11 @@ public class TransactionManager implements Serializable {
   }
 
   public Option<HoodieInstant> getLastCompletedTransactionOwner() {
-    return lastCompletedTxnOwnerInstant;
+    return lastCompletedActionInstant;
   }
 
   public Option<HoodieInstant> getCurrentTransactionOwner() {
-    return currentTxnOwnerInstant;
+    return changeActionInstant;
   }
 
   public boolean isLockRequired() {
