@@ -50,7 +50,7 @@ public class ZookeeperBasedImplicitBasePathLockProvider extends BaseZookeeperBas
   public static String getLockBasePath(String hudiTableBasePath) {
     // Ensure consistent format for S3 URI.
     String lockBasePath = "/tmp/" + HashID.generateXXHashAsString(s3aToS3(hudiTableBasePath), HashID.Size.BITS_64);
-    LOG.info(String.format("The Zookeeper lock key for the base path %s is %s", hudiTableBasePath, lockBasePath));
+    LOG.info("The Zookeeper lock key for the base path {} is {}", hudiTableBasePath, lockBasePath);
     return lockBasePath;
   }
 
