@@ -49,7 +49,7 @@ import static org.apache.hudi.common.table.timeline.InstantComparison.compareTim
  * HoodieBackedTableMetadataWriter for tables with version 6. The class derives most of the functionality from HoodieBackedTableMetadataWriter
  * and overrides some behaviour to make it compatible for version 6.
  */
-public abstract class HoodieBackedTableMetadataWriterTableVersionSix<I> extends HoodieBackedTableMetadataWriter<I> {
+public abstract class HoodieBackedTableMetadataWriterTableVersionSix<I, O> extends HoodieBackedTableMetadataWriter<I, O> {
 
   private static final int PARTITION_INITIALIZATION_TIME_SUFFIX = 10;
 
@@ -148,9 +148,9 @@ public abstract class HoodieBackedTableMetadataWriterTableVersionSix<I> extends
 
     if (!pendingInstants.isEmpty()) {
       checkNumDeltaCommits(metadataMetaClient, dataWriteConfig.getMetadataConfig().getMaxNumDeltacommitsWhenPending());
-      LOG.info(String.format(
-          "Cannot compact metadata table as there are %d inflight instants in data table before latest deltacommit in metadata table: %s. Inflight instants in data table: %s",
-          pendingInstants.size(), latestDeltaCommitTimeInMetadataTable, Arrays.toString(pendingInstants.toArray())));
+      LOG.info(
+          "Cannot compact metadata table as there are {} inflight instants in data table before latest deltacommit in metadata table: {}. Inflight instants in data table: {}",
+          pendingInstants.size(), latestDeltaCommitTimeInMetadataTable, Arrays.toString(pendingInstants.toArray()));
       return false;
     }
 
@@ -161,8 +161,8 @@ public abstract class HoodieBackedTableMetadataWriterTableVersionSix<I> extends
     Option<HoodieInstant> pendingCompactionInstant =
         metadataMetaClient.getActiveTimeline().filterPendingCompactionTimeline().firstInstant();
     if (pendingLogCompactionInstant.isPresent() || pendingCompactionInstant.isPresent()) {
-      LOG.warn(String.format("Not scheduling compaction or logCompaction, since a pending compaction instant %s or logCompaction %s instant is present",
-          pendingCompactionInstant, pendingLogCompactionInstant));
+      LOG.info("Not scheduling compaction or logCompaction, since a pending compaction instant {} or logCompaction {} instant is present",
+          pendingCompactionInstant, pendingLogCompactionInstant);
       return false;
     }
     return true;
@@ -255,7 +255,7 @@ public abstract class HoodieBackedTableMetadataWriterTableVersionSix<I> extends
    * deltacommit.
    */
   @Override
-  void compactIfNecessary(BaseHoodieWriteClient<?,I,?,?> writeClient, Option<String> latestDeltaCommitTimeOpt) {
+  void compactIfNecessary(BaseHoodieWriteClient<?,I,?,O> writeClient, Option<String> latestDeltaCommitTimeOpt) {
     // Trigger compaction with suffixes based on the same instant time. This ensures that any future
     // delta commits synced over will not have an instant time lesser than the last completed instant on the
     // metadata table.
@@ -269,7 +269,7 @@ public abstract class HoodieBackedTableMetadataWriterTableVersionSix<I> extends
       LOG.info("Compaction with same {} time is already present in the timeline.", compactionInstantTime);
     } else if (writeClient.scheduleCompactionAtInstant(compactionInstantTime, Option.empty())) {
       LOG.info("Compaction is scheduled for timestamp {}", compactionInstantTime);
-      writeClient.compact(compactionInstantTime);
+      writeClient.compact(compactionInstantTime, true);
     } else if (metadataWriteConfig.isLogCompactionEnabled()) {
       // Schedule and execute log compaction with suffixes based on the same instant time. This ensures that any future
       // delta commits synced over will not have an instant time lesser than the last completed instant on the
@@ -279,19 +279,19 @@ public abstract class HoodieBackedTableMetadataWriterTableVersionSix<I> extends
         LOG.info("Log compaction with same {} time is already present in the timeline.", logCompactionInstantTime);
       } else if (writeClient.scheduleLogCompactionAtInstant(logCompactionInstantTime, Option.empty())) {
         LOG.info("Log compaction is scheduled for timestamp {}", logCompactionInstantTime);
-        writeClient.logCompact(logCompactionInstantTime);
+        writeClient.logCompact(logCompactionInstantTime, true);
       }
     }
   }
 
   @Override
-  String createCleanInstantTime(String instantTime) {
-    return createCleanTimestamp(instantTime);
+  protected void executeClean(BaseHoodieWriteClient writeClient, String instantTime) {
+    writeClient.clean(createCleanTimestamp(instantTime));
   }
 
   @Override
   String createRestoreInstantTime() {
-    return createRestoreTimestamp(writeClient.createNewInstantTime(false));
+    return createRestoreTimestamp(getWriteClient().createNewInstantTime(false));
   }
 
   /**
