@@ -18,7 +18,12 @@
 
 package org.apache.hudi.table.action.clean;
 
+import org.apache.hudi.common.config.EnumDescription;
+import org.apache.hudi.common.config.EnumFieldDescription;
+
+@EnumDescription("Controls when cleaning is scheduled.")
 public enum CleaningTriggerStrategy {
     // trigger cleaning when reach n commits
+    @EnumFieldDescription("Trigger the cleaning service every N commits, determined by `hoodie.clean.trigger.max.commits`.")
     NUM_COMMITS
 }
