@@ -29,7 +29,6 @@ import org.apache.hudi.common.data.HoodieData;
 import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.engine.TaskContextSupplier;
 import org.apache.hudi.common.model.HoodieCommitMetadata;
-import org.apache.hudi.common.model.HoodieFileGroupId;
 import org.apache.hudi.common.model.HoodieKey;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.model.HoodieWriteStat;
@@ -73,7 +72,6 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
 import static org.apache.hudi.config.HoodieWriteConfig.WRITE_STATUS_STORAGE_LEVEL_VALUE;
@@ -86,9 +84,10 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
   protected final Option<Map<String, String>> extraMetadata;
   protected final WriteOperationType operationType;
   protected final TaskContextSupplier taskContextSupplier;
-  protected final Option<TransactionManager> txnManagerOption;
-  protected final Option<Pair<HoodieInstant, Map<String, String>>> lastCompletedTxn;
-  protected final Set<String> pendingInflightAndRequestedInstants;
+
+  protected Option<TransactionManager> txnManagerOption;
+  protected Option<Pair<HoodieInstant, Map<String, String>>> lastCompletedTxn = Option.empty();
+  protected Set<String> pendingInflightAndRequestedInstants = Collections.emptySet();
 
   public BaseCommitActionExecutor(HoodieEngineContext context, HoodieWriteConfig config,
                                   HoodieTable<T, I, K, O> table, String instantTime, WriteOperationType operationType,
@@ -97,9 +96,15 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
     this.operationType = operationType;
     this.extraMetadata = extraMetadata;
     this.taskContextSupplier = context.getTaskContextSupplier();
-    // TODO : Remove this once we refactor and move out autoCommit method from here, since the TxnManager is held in {@link BaseHoodieWriteClient}.
-    this.txnManagerOption = config.shouldAutoCommit()
-        ? Option.of(new TransactionManager(config, table.getStorage())) : Option.empty();
+    this.txnManagerOption = Option.empty();
+    initializeLastCompletedTnxAndPendingInstants();
+    if (!table.getStorageLayout().writeOperationSupported(operationType)) {
+      throw new UnsupportedOperationException("Executor " + this.getClass().getSimpleName()
+          + " is not compatible with table layout " + table.getStorageLayout().getClass().getSimpleName());
+    }
+  }
+
+  private void initializeLastCompletedTnxAndPendingInstants() {
     if (this.txnManagerOption.isPresent() && this.txnManagerOption.get().isLockRequired()) {
       // these txn metadata are only needed for auto commit when optimistic concurrent control is also enabled
       this.lastCompletedTxn = TransactionUtils.getLastCompletedTxnInstantAndMetadata(table.getMetaClient());
@@ -109,10 +114,6 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
       this.lastCompletedTxn = Option.empty();
       this.pendingInflightAndRequestedInstants = Collections.emptySet();
     }
-    if (!table.getStorageLayout().writeOperationSupported(operationType)) {
-      throw new UnsupportedOperationException("Executor " + this.getClass().getSimpleName()
-          + " is not compatible with table layout " + table.getStorageLayout().getClass().getSimpleName());
-    }
   }
 
   public abstract HoodieWriteMetadata<O> execute(I inputRecords);
@@ -127,7 +128,7 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
    * are unknown across batches Inserts (which are new parquet files) are rolled back based on commit time. // TODO :
    * Create a new WorkloadProfile metadata file instead of using HoodieCommitMetadata
    */
-  void saveWorkloadProfileMetadataToInflight(WorkloadProfile profile, String instantTime)
+  protected void saveWorkloadProfileMetadataToInflight(WorkloadProfile profile, String instantTime)
       throws HoodieCommitException {
     try {
       HoodieCommitMetadata metadata = new HoodieCommitMetadata();
@@ -183,15 +184,13 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
     throw new HoodieIOException("Precommit validation not implemented for all engines yet");
   }
 
-  protected void commitOnAutoCommit(HoodieWriteMetadata result) {
-    // validate commit action before committing result
-    runPrecommitValidators(result);
-    if (config.shouldAutoCommit()) {
-      LOG.info("Auto commit enabled: Committing " + instantTime);
-      autoCommit(result);
-    } else {
-      LOG.info("Auto commit disabled for " + instantTime);
+  protected void completeCommit(HoodieWriteMetadata result) {
+    if (!this.txnManagerOption.isPresent()) {
+      this.txnManagerOption = Option.of(new TransactionManager(config, table.getStorage()));
+      initializeLastCompletedTnxAndPendingInstants();
     }
+    autoCommit(result);
+    LOG.info("Completed commit for " + instantTime);
   }
 
   protected void autoCommit(HoodieWriteMetadata<O> result) {
@@ -200,7 +199,7 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
         getCommitActionType(), instantTime));
     ValidationUtils.checkState(this.txnManagerOption.isPresent(), "The transaction manager has not been initialized");
     TransactionManager txnManager = this.txnManagerOption.get();
-    txnManager.beginTransaction(inflightInstant,
+    txnManager.beginStateChange(inflightInstant,
         lastCompletedTxn.isPresent() ? Option.of(lastCompletedTxn.get().getLeft()) : Option.empty());
     try {
       setCommitMetadata(result);
@@ -209,7 +208,7 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
           result.getCommitMetadata(), config, txnManager.getLastCompletedTransactionOwner(), false, pendingInflightAndRequestedInstants);
       commit(result);
     } finally {
-      txnManager.endTransaction(inflightInstant);
+      txnManager.endStateChange(inflightInstant);
     }
   }
 
@@ -235,7 +234,8 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
       // cannot serialize maps with null values
       metadata.getExtraMetadata().entrySet().removeIf(entry -> entry.getValue() == null);
       activeTimeline.saveAsComplete(false,
-          table.getMetaClient().createNewInstant(State.INFLIGHT, actionType, instantTime), Option.of(metadata));
+          table.getMetaClient().createNewInstant(State.INFLIGHT, actionType, instantTime), Option.of(metadata),
+          completedInstant -> table.getMetaClient().getTableFormat().commit(metadata, completedInstant, table.getContext(), table.getMetaClient(), table.getViewManager()));
       LOG.info("Committed " + instantTime);
       result.setCommitMetadata(Option.of(metadata));
       // update cols to Index as applicable
@@ -293,9 +293,6 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
     ClusteringUtils.transitionClusteringOrReplaceRequestedToInflight(instant, Option.empty(), table.getActiveTimeline());
     table.getMetaClient().reloadActiveTimeline();
 
-    // Disable auto commit. Strategy is only expected to write data in new files.
-    config.setValue(HoodieWriteConfig.AUTO_COMMIT_ENABLE, Boolean.FALSE.toString());
-
     Option<Schema> schema;
     try {
       schema = new TableSchemaResolver(table.getMetaClient()).getTableAvroSchemaIfPresent(false);
@@ -310,16 +307,14 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
     HoodieData<WriteStatus> writeStatusList = writeMetadata.getWriteStatuses();
     HoodieData<WriteStatus> statuses = updateIndex(writeStatusList, writeMetadata);
     statuses.persist(config.getString(WRITE_STATUS_STORAGE_LEVEL_VALUE), context, HoodieData.HoodieDataCacheKey.of(config.getBasePath(), instantTime));
-    // triggers clustering.
-    writeMetadata.setWriteStats(statuses.map(WriteStatus::getStat).collectAsList());
-    writeMetadata.setPartitionToReplaceFileIds(getPartitionToReplacedFileIds(clusteringPlan, writeMetadata));
-    commitOnAutoCommit(writeMetadata);
-    if (!writeMetadata.getCommitMetadata().isPresent()) {
-      LOG.info("Found empty commit metadata for clustering with instant time " + instantTime);
-      HoodieCommitMetadata commitMetadata = CommitUtils.buildMetadata(writeMetadata.getWriteStats().get(), writeMetadata.getPartitionToReplaceFileIds(),
-          extraMetadata, operationType, schema.get().toString(), getCommitActionType());
-      writeMetadata.setCommitMetadata(Option.of(commitMetadata));
-    }
+
+    writeMetadata.setWriteStatuses(statuses);
+
+    LOG.debug("Create place holder commit metadata for clustering with instant time " + instantTime);
+    HoodieCommitMetadata commitMetadata = CommitUtils.buildMetadata(Collections.emptyList(), Collections.emptyMap(),
+        extraMetadata, operationType, schema.get().toString(), getCommitActionType());
+    writeMetadata.setCommitMetadata(Option.of(commitMetadata));
+
     return writeMetadata;
   }
 
@@ -332,17 +327,6 @@ public abstract class BaseCommitActionExecutor<T, I, K, O, R>
     return statuses;
   }
 
-  private Map<String, List<String>> getPartitionToReplacedFileIds(HoodieClusteringPlan clusteringPlan, HoodieWriteMetadata<HoodieData<WriteStatus>> writeMetadata) {
-    Set<HoodieFileGroupId> newFilesWritten = writeMetadata.getWriteStats().get().stream()
-        .map(s -> new HoodieFileGroupId(s.getPartitionPath(), s.getFileId())).collect(Collectors.toSet());
-
-    return ClusteringUtils.getFileGroupsFromClusteringPlan(clusteringPlan)
-        .filter(fg -> "org.apache.hudi.client.clustering.run.strategy.SparkSingleFileSortExecutionStrategy"
-            .equals(config.getClusteringExecutionStrategyClass())
-            || !newFilesWritten.contains(fg))
-        .collect(Collectors.groupingBy(HoodieFileGroupId::getPartitionPath, Collectors.mapping(HoodieFileGroupId::getFileId, Collectors.toList())));
-  }
-
   /**
    * Validate actions taken by clustering. In the first implementation, we validate at least one new file is written.
    * But we can extend this to add more validation. E.g. number of records read = number of records written etc.
