@@ -145,14 +145,14 @@ public class RunIndexActionExecutor<T, I, K, O> extends BaseActionExecutor<T, I,
                 "Could not get metadata writer to run index action for instant: %s", instantTime)))) {
           // this will only build index upto base instant as generated by the plan, we will be doing catchup later
           String indexUptoInstant = indexPartitionInfos.get(0).getIndexUptoInstant();
-          LOG.info("Starting Index Building with base instant: " + indexUptoInstant);
+          LOG.info("Starting Index Building with base instant: {}", indexUptoInstant);
           HoodieTimer timer = HoodieTimer.start();
           metadataWriter.buildMetadataPartitions(context, indexPartitionInfos, indexInstant.requestedTime());
           metrics.ifPresent(m -> m.updateMetrics(HoodieMetadataMetrics.INITIALIZE_STR, timer.endTimer()));
 
           // get remaining instants to catchup
           List<HoodieInstant> instantsToCatchup = getInstantsToCatchup(indexUptoInstant);
-          LOG.info("Total remaining instants to index: " + instantsToCatchup.size());
+          LOG.info("Total remaining instants to index: {}", instantsToCatchup.size());
 
           // reconcile with metadata table timeline
           String metadataBasePath = getMetadataTableBasePath(table.getMetaClient().getBasePath().toString());
@@ -266,13 +266,13 @@ public class RunIndexActionExecutor<T, I, K, O> extends BaseActionExecutor<T, I,
                                             HoodieIndexCommitMetadata indexCommitMetadata) throws IOException {
     try {
       // update the table config and timeline in a lock as there could be another indexer running
-      txnManager.beginTransaction(Option.of(indexInstant), Option.empty());
+      txnManager.beginStateChange(Option.of(indexInstant), Option.empty());
       updateMetadataPartitionsTableConfig(table.getMetaClient(),
           finalIndexPartitionInfos.stream().map(HoodieIndexPartitionInfo::getMetadataPartitionPath).collect(Collectors.toSet()));
       table.getActiveTimeline().saveAsComplete(false, instantGenerator.createNewInstant(HoodieInstant.State.INFLIGHT, INDEXING_ACTION, indexInstant.requestedTime()),
           Option.of(indexCommitMetadata));
     } finally {
-      txnManager.endTransaction(Option.of(indexInstant));
+      txnManager.endStateChange(Option.of(indexInstant));
     }
   }
 
