@@ -21,7 +21,6 @@ package org.apache.hudi.table.action.index;
 
 import org.apache.hudi.avro.model.HoodieIndexPartitionInfo;
 import org.apache.hudi.avro.model.HoodieIndexPlan;
-import org.apache.hudi.client.transaction.TransactionManager;
 import org.apache.hudi.common.config.HoodieMetadataConfig;
 import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
@@ -72,8 +71,6 @@ public class ScheduleIndexActionExecutor<T, I, K, O> extends BaseActionExecutor<
 
   private final List<String> partitionPaths;
 
-  private final TransactionManager txnManager;
-
   public ScheduleIndexActionExecutor(HoodieEngineContext context,
                                      HoodieWriteConfig config,
                                      HoodieTable<T, I, K, O> table,
@@ -83,7 +80,6 @@ public class ScheduleIndexActionExecutor<T, I, K, O> extends BaseActionExecutor<
     super(context, config, table, instantTime);
     this.partitionIndexTypes = partitionIndexTypes;
     this.partitionPaths = partitionPaths;
-    this.txnManager = new TransactionManager(config, table.getStorage());
   }
 
   @Override
@@ -93,21 +89,39 @@ public class ScheduleIndexActionExecutor<T, I, K, O> extends BaseActionExecutor<
     Set<String> indexesInflightOrCompleted = getInflightAndCompletedMetadataPartitions(table.getMetaClient().getTableConfig());
     InstantGenerator instantGenerator = table.getMetaClient().getInstantGenerator();
 
-    Set<String> requestedPartitions = partitionIndexTypes.stream().map(MetadataPartitionType::getPartitionPath).collect(Collectors.toSet());
+    HoodieMetadataConfig metadataConfig = config.getMetadataConfig();
+    Set<String> requestedPartitions = partitionIndexTypes.stream().map(p -> {
+      if (MetadataPartitionType.EXPRESSION_INDEX.equals(p)) {
+        return getSecondaryOrExpressionIndexName(metadataConfig::getExpressionIndexName, PARTITION_NAME_EXPRESSION_INDEX_PREFIX, metadataConfig.getExpressionIndexColumn());
+      } else if (MetadataPartitionType.SECONDARY_INDEX.equals(p)) {
+        return getSecondaryOrExpressionIndexName(metadataConfig::getSecondaryIndexName, PARTITION_NAME_SECONDARY_INDEX_PREFIX, metadataConfig.getSecondaryIndexColumn());
+      }
+      return p.getPartitionPath();
+    }).collect(Collectors.toSet());
     requestedPartitions.addAll(partitionPaths);
     requestedPartitions.removeAll(indexesInflightOrCompleted);
 
     if (!requestedPartitions.isEmpty()) {
-      LOG.warn("Following partitions already exist or inflight: {}. Going to schedule indexing of only these partitions: {}", indexesInflightOrCompleted, requestedPartitions);
+      LOG.info("Some index partitions already exist: {}. Scheduling indexing of only these remaining partitions: {}",
+          indexesInflightOrCompleted, requestedPartitions);
     } else {
-      LOG.error("All requested index types are inflight or completed: {}", partitionIndexTypes);
+      LOG.info("All requested index partitions exist (either inflight or built): {}", partitionIndexTypes);
       return Option.empty();
     }
     List<MetadataPartitionType> finalPartitionsToIndex = partitionIndexTypes.stream()
-        .filter(p -> requestedPartitions.contains(p.getPartitionPath())).collect(Collectors.toList());
+        .filter(p -> {
+          String partitionName;
+          if (MetadataPartitionType.EXPRESSION_INDEX.equals(p)) {
+            partitionName = getSecondaryOrExpressionIndexName(metadataConfig::getExpressionIndexName, PARTITION_NAME_EXPRESSION_INDEX_PREFIX, metadataConfig.getExpressionIndexColumn());
+          } else if (MetadataPartitionType.SECONDARY_INDEX.equals(p)) {
+            partitionName = getSecondaryOrExpressionIndexName(metadataConfig::getSecondaryIndexName, PARTITION_NAME_SECONDARY_INDEX_PREFIX, metadataConfig.getSecondaryIndexColumn());
+          } else {
+            partitionName = p.getPartitionPath();
+          }
+          return requestedPartitions.contains(partitionName);
+        }).collect(Collectors.toList());
     final HoodieInstant indexInstant = instantGenerator.getIndexRequestedInstant(instantTime);
     try {
-      this.txnManager.beginTransaction(Option.of(indexInstant), Option.empty());
       // get last completed instant
       Option<HoodieInstant> indexUptoInstant = table.getActiveTimeline().getContiguousCompletedWriteTimeline().lastInstant();
       if (indexUptoInstant.isPresent()) {
@@ -124,8 +138,6 @@ public class ScheduleIndexActionExecutor<T, I, K, O> extends BaseActionExecutor<
       LOG.error("Could not initialize file groups", e);
       // abort gracefully
       abort(indexInstant);
-    } finally {
-      this.txnManager.endTransaction(Option.of(indexInstant));
     }
 
     return Option.empty();
