@@ -20,10 +20,12 @@
 package org.apache.hudi.table.action.restore;
 
 import org.apache.hudi.avro.model.HoodieRollbackMetadata;
+import org.apache.hudi.client.transaction.TransactionManager;
 import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.util.ClusteringUtils;
+import org.apache.hudi.common.util.Option;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieRollbackException;
 import org.apache.hudi.table.HoodieTable;
@@ -45,9 +47,17 @@ public class CopyOnWriteRestoreActionExecutor<T, I, K, O>
         && !ClusteringUtils.isClusteringOrReplaceCommitAction(instantToRollback.getAction())) {
       throw new HoodieRollbackException("Unsupported action in rollback instant:" + instantToRollback);
     }
-    table.getMetaClient().reloadActiveTimeline();
-    String newInstantTime = table.getMetaClient().createNewInstantTime();
-    table.scheduleRollback(context, newInstantTime, instantToRollback, false, false, true);
+    String newInstantTime;
+    try (TransactionManager transactionManager = new TransactionManager(config, table.getStorage())) {
+      table.getMetaClient().reloadActiveTimeline();
+      transactionManager.beginStateChange(Option.empty(), Option.empty());
+      try {
+        newInstantTime = table.getMetaClient().createNewInstantTime(false);
+        table.scheduleRollback(context, newInstantTime, instantToRollback, false, false, true);
+      } finally {
+        transactionManager.endStateChange(Option.empty());
+      }
+    }
     table.getMetaClient().reloadActiveTimeline();
     CopyOnWriteRollbackActionExecutor rollbackActionExecutor = new CopyOnWriteRollbackActionExecutor(
         context,
