@@ -26,6 +26,7 @@ import org.apache.hudi.common.HoodieRollbackStat;
 import org.apache.hudi.common.bootstrap.index.BootstrapIndex;
 import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.model.HoodieTableType;
+import org.apache.hudi.common.model.PartitionBucketIndexHashingConfig;
 import org.apache.hudi.common.table.HoodieTableVersion;
 import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
@@ -35,6 +36,7 @@ import org.apache.hudi.common.table.timeline.TimelineUtils;
 import org.apache.hudi.common.util.ClusteringUtils;
 import org.apache.hudi.common.util.HoodieTimer;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieIOException;
@@ -230,6 +232,7 @@ public abstract class BaseRollbackActionExecutor<T, I, K, O> extends BaseActionE
     }
 
     backupRollbackInstantsIfNeeded();
+    rollbackHashingConfigIfNecessary(instantToRollback);
 
     try {
       List<HoodieRollbackStat> stats = executeRollback(hoodieRollbackPlan);
@@ -243,6 +246,16 @@ public abstract class BaseRollbackActionExecutor<T, I, K, O> extends BaseActionE
     }
   }
 
+  /**
+   * try to delete hashing config during rollback if using partition level bucket index.
+   * @param instantToRollback
+   */
+  private void rollbackHashingConfigIfNecessary(HoodieInstant instantToRollback) {
+    if (StringUtils.nonEmpty(config.getBucketIndexPartitionExpression()) && instantToRollback.getAction().equals(HoodieTimeline.REPLACE_COMMIT_ACTION)) {
+      PartitionBucketIndexHashingConfig.rollbackHashingConfig(instantToRollback, table.getMetaClient());
+    }
+  }
+
   /**
    * Execute rollback and fetch rollback stats.
    * @param instantToRollback instant to be rolled back.
@@ -257,7 +270,7 @@ public abstract class BaseRollbackActionExecutor<T, I, K, O> extends BaseActionE
     boolean enableLocking = (!skipLocking && !skipTimelinePublish);
     try {
       if (enableLocking) {
-        this.txnManager.beginTransaction(Option.of(inflightInstant), Option.empty());
+        this.txnManager.beginStateChange(Option.of(inflightInstant), Option.empty());
       }
 
       // If publish the rollback to the timeline, we first write the rollback metadata to metadata table
@@ -275,12 +288,13 @@ public abstract class BaseRollbackActionExecutor<T, I, K, O> extends BaseActionE
         // NOTE: no need to lock here, since !skipTimelinePublish is always true,
         // when skipLocking is false, txnManager above-mentioned should lock it.
         // when skipLocking is true, the caller should have already held the lock.
-        table.getActiveTimeline().transitionRollbackInflightToComplete(false, inflightInstant, rollbackMetadata);
+        table.getActiveTimeline().transitionRollbackInflightToComplete(false, inflightInstant, rollbackMetadata,
+            completedInstant -> table.getMetaClient().getTableFormat().completedRollback(completedInstant, table.getContext(), table.getMetaClient(), table.getViewManager()));
         LOG.info("Rollback of Commits " + rollbackMetadata.getCommitsRollback() + " is complete");
       }
     } finally {
       if (enableLocking) {
-        this.txnManager.endTransaction(Option.of(inflightInstant));
+        this.txnManager.endStateChange(Option.of(inflightInstant));
       }
     }
   }
@@ -296,7 +310,7 @@ public abstract class BaseRollbackActionExecutor<T, I, K, O> extends BaseActionE
                                                    HoodieInstant instantToBeDeleted) {
     // Remove the rolled back inflight commits
     if (deleteInstant) {
-      LOG.info("Deleting instant=" + instantToBeDeleted);
+      LOG.info("Deleting instant={}", instantToBeDeleted);
       activeTimeline.deletePending(instantToBeDeleted);
       if (instantToBeDeleted.isInflight() && !table.getMetaClient().getTimelineLayoutVersion().isNullVersion()) {
         // Delete corresponding requested instant
@@ -304,9 +318,9 @@ public abstract class BaseRollbackActionExecutor<T, I, K, O> extends BaseActionE
             instantToBeDeleted.requestedTime());
         activeTimeline.deletePending(instantToBeDeleted);
       }
-      LOG.info("Deleted pending commit " + instantToBeDeleted);
+      LOG.info("Deleted pending commit {}", instantToBeDeleted);
     } else {
-      LOG.warn("Rollback finished without deleting inflight instant file. Instant=" + instantToBeDeleted);
+      LOG.info("Rollback finished without deleting inflight instant file. Instant={}", instantToBeDeleted);
     }
   }
 
@@ -343,10 +357,10 @@ public abstract class BaseRollbackActionExecutor<T, I, K, O> extends BaseActionE
     for (HoodieInstant instant : instantsToBackup) {
       try {
         activeTimeline.copyInstant(instant, backupDir);
-        LOG.info(String.format("Copied instant %s to backup dir %s during rollback at %s", instant, backupDir, instantTime));
+        LOG.info("Copied instant {} to backup dir {} during rollback at {}", instant, backupDir, instantTime);
       } catch (HoodieIOException e) {
         // Ignoring error in backing up
-        LOG.warn("Failed to backup rollback instant: " + e.getMessage());
+        LOG.warn("Failed to backup rollback instant", e);
       }
     }
   }
