@@ -24,10 +24,13 @@ import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.exception.HoodieRollbackException;
 import org.apache.hudi.table.HoodieTable;
+import org.apache.hudi.table.action.rollback.RestoreInstantComparatorFactory;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.util.Comparator;
+
 public class SavepointHelpers {
 
   private static final Logger LOG = LoggerFactory.getLogger(SavepointHelpers.class);
@@ -36,26 +39,29 @@ public class SavepointHelpers {
     HoodieInstant savePoint = table.getMetaClient().createNewInstant(HoodieInstant.State.COMPLETED, HoodieTimeline.SAVEPOINT_ACTION, savepointTime);
     boolean isSavepointPresent = table.getCompletedSavepointTimeline().containsInstant(savePoint);
     if (!isSavepointPresent) {
-      LOG.warn("No savepoint present " + savepointTime);
+      LOG.warn("No savepoint present {}", savepointTime);
       return;
     }
 
     table.getActiveTimeline().revertToInflight(savePoint);
     table.getActiveTimeline().deleteInflight(table.getMetaClient().createNewInstant(HoodieInstant.State.INFLIGHT, HoodieTimeline.SAVEPOINT_ACTION,
         savepointTime));
-    LOG.info("Savepoint " + savepointTime + " deleted");
+    LOG.info("Savepoint {} deleted", savepointTime);
   }
 
   public static void validateSavepointRestore(HoodieTable table, String savepointTime) {
     // Make sure the restore was successful
     table.getMetaClient().reloadActiveTimeline();
-    Option<HoodieInstant> lastInstant = table.getActiveTimeline()
+    // Validate that the restore has returned the timeline to the anticipated state
+    Comparator<HoodieInstant> instantComparator = RestoreInstantComparatorFactory.createComparator(table.getMetaClient());
+    Option<HoodieInstant> lastInstant = Option.fromJavaOptional(table.getActiveTimeline()
         .getWriteTimeline()
         .filterCompletedAndCompactionInstants()
-        .lastInstant();
+        .getInstantsAsStream()
+        .max(instantComparator));
     ValidationUtils.checkArgument(lastInstant.isPresent());
     ValidationUtils.checkArgument(lastInstant.get().requestedTime().equals(savepointTime),
-        savepointTime + " is not the last commit after restoring to savepoint, last commit was "
+        () -> savepointTime + " is not the last commit after restoring to savepoint, last commit was "
             + lastInstant.get().requestedTime());
   }
 
