@@ -27,7 +27,6 @@ import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
 import org.apache.hudi.common.table.timeline.HoodieArchivedTimeline;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
-import org.apache.hudi.common.table.timeline.TimelineLayout;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.storage.StoragePath;
@@ -90,20 +89,18 @@ public final class RepairUtils {
 
     switch (instant.getAction()) {
       case COMMIT_ACTION:
-      case DELTA_COMMIT_ACTION:
-        TimelineLayout layout = TimelineLayout.fromVersion(timeline.getTimelineLayoutVersion());
-        final HoodieCommitMetadata commitMetadata =
-            layout.getCommitMetadataSerDe().deserialize(instant,
-                timeline.getInstantDetails(instant).get(), HoodieCommitMetadata.class);
+      case DELTA_COMMIT_ACTION: {
+        final HoodieCommitMetadata commitMetadata = timeline.readCommitMetadata(instant);
         return Option.of(commitMetadata.getPartitionToWriteStats().values().stream().flatMap(List::stream)
             .map(HoodieWriteStat::getPath).collect(Collectors.toSet()));
+      }
       case REPLACE_COMMIT_ACTION:
-      case CLUSTERING_ACTION:
+      case CLUSTERING_ACTION: {
         final HoodieReplaceCommitMetadata replaceCommitMetadata =
-            HoodieReplaceCommitMetadata.fromBytes(
-                timeline.getInstantDetails(instant).get(), HoodieReplaceCommitMetadata.class);
+            timeline.readReplaceCommitMetadata(instant);
         return Option.of(replaceCommitMetadata.getPartitionToWriteStats().values().stream().flatMap(List::stream)
             .map(HoodieWriteStat::getPath).collect(Collectors.toSet()));
+      }
       default:
         return Option.empty();
     }
