@@ -37,7 +37,9 @@ public class HoodieConsistentBucketLayout extends HoodieStorageLayout {
       WriteOperationType.UPSERT,
       WriteOperationType.UPSERT_PREPPED,
       WriteOperationType.INSERT_OVERWRITE,
+      WriteOperationType.INSERT_OVERWRITE_TABLE,
       WriteOperationType.DELETE,
+      WriteOperationType.DELETE_PREPPED,
       WriteOperationType.COMPACT,
       WriteOperationType.DELETE_PARTITION,
       WriteOperationType.CLUSTER
