@@ -26,7 +26,6 @@ import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.BootstrapIndexType;
 import org.apache.hudi.common.model.DefaultHoodieRecordPayload;
-import org.apache.hudi.common.model.HoodieTableType;
 import org.apache.hudi.common.model.OverwriteWithLatestAvroPayload;
 import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
@@ -44,7 +43,6 @@ import org.apache.hudi.common.table.timeline.versioning.v2.ArchivedTimelineLoade
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.common.util.ValidationUtils;
-import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.exception.HoodieIOException;
@@ -81,13 +79,14 @@ import static org.apache.hudi.common.table.timeline.HoodieTimeline.DELTA_COMMIT_
 import static org.apache.hudi.common.table.timeline.HoodieTimeline.REPLACE_COMMIT_ACTION;
 import static org.apache.hudi.common.table.timeline.TimelineLayout.TIMELINE_LAYOUT_V1;
 import static org.apache.hudi.common.table.timeline.TimelineLayout.TIMELINE_LAYOUT_V2;
+import static org.apache.hudi.keygen.KeyGenUtils.getComplexKeygenErrorMessage;
+import static org.apache.hudi.keygen.KeyGenUtils.isComplexKeyGeneratorWithSingleRecordKeyField;
 import static org.apache.hudi.metadata.MetadataPartitionType.BLOOM_FILTERS;
 import static org.apache.hudi.metadata.MetadataPartitionType.COLUMN_STATS;
 import static org.apache.hudi.metadata.MetadataPartitionType.FILES;
 import static org.apache.hudi.metadata.MetadataPartitionType.RECORD_INDEX;
 import static org.apache.hudi.table.upgrade.UpgradeDowngradeUtils.EIGHT_TO_SIX_TIMELINE_ACTION_MAP;
 import static org.apache.hudi.table.upgrade.UpgradeDowngradeUtils.convertCompletionTimeToEpoch;
-import static org.apache.hudi.table.upgrade.UpgradeDowngradeUtils.rollbackFailedWritesAndCompact;
 
 /**
  * Version 7 is going to be placeholder version for bridge release 0.16.0.
@@ -99,13 +98,15 @@ public class EightToSevenDowngradeHandler implements DowngradeHandler {
   private static final Set<String> SUPPORTED_METADATA_PARTITION_PATHS = getSupportedMetadataPartitionPaths();
 
   @Override
-  public Pair<Map<ConfigProperty, String>, List<ConfigProperty>> downgrade(HoodieWriteConfig config, HoodieEngineContext context, String instantTime, SupportsUpgradeDowngrade upgradeDowngradeHelper) {
+  public UpgradeDowngrade.TableConfigChangeSet downgrade(HoodieWriteConfig config, HoodieEngineContext context, String instantTime, SupportsUpgradeDowngrade upgradeDowngradeHelper) {
     final HoodieTable table = upgradeDowngradeHelper.getTable(config, context);
     Map<ConfigProperty, String> tablePropsToAdd = new HashMap<>();
-    // Rollback and run compaction in one step
-    rollbackFailedWritesAndCompact(table, context, config, upgradeDowngradeHelper, HoodieTableType.MERGE_ON_READ.equals(table.getMetaClient().getTableType()), HoodieTableVersion.EIGHT);
 
     HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder().setConf(context.getStorageConf().newInstance()).setBasePath(config.getBasePath()).build();
+    if (config.enableComplexKeygenValidation()
+        && isComplexKeyGeneratorWithSingleRecordKeyField(metaClient.getTableConfig())) {
+      throw new HoodieUpgradeDowngradeException(getComplexKeygenErrorMessage("downgrade"));
+    }
     // Handle timeline downgrade:
     //  - Rename instants in active timeline to old format for table version 6
     //  - Convert LSM timeline format to archived timeline for table version 6
@@ -133,7 +134,7 @@ public class EightToSevenDowngradeHandler implements DowngradeHandler {
     // downgrade table properties
     downgradePartitionFields(config, metaClient.getTableConfig(), tablePropsToAdd);
     unsetInitialVersion(metaClient.getTableConfig(), tablePropsToAdd);
-    List<ConfigProperty> tablePropsToRemove = new ArrayList<>();
+    Set<ConfigProperty> tablePropsToRemove = new HashSet<>();
     tablePropsToRemove.addAll(unsetRecordMergeMode(config, metaClient.getTableConfig(), tablePropsToAdd));
     tablePropsToRemove.add(HoodieTableConfig.RECORD_MERGE_STRATEGY_ID);
     downgradeKeyGeneratorType(metaClient.getTableConfig(), tablePropsToAdd);
@@ -145,7 +146,7 @@ public class EightToSevenDowngradeHandler implements DowngradeHandler {
       downgradeMetadataPartitions(context, metaClient.getStorage(), metaClient, tablePropsToAdd);
       UpgradeDowngradeUtils.updateMetadataTableVersion(context, HoodieTableVersion.SEVEN, metaClient);
     }
-    return Pair.of(tablePropsToAdd, tablePropsToRemove);
+    return new UpgradeDowngrade.TableConfigChangeSet(tablePropsToAdd, tablePropsToRemove);
   }
 
   static void downgradePartitionFields(HoodieWriteConfig config,
@@ -227,9 +228,10 @@ public class EightToSevenDowngradeHandler implements DowngradeHandler {
             flusher);
       }
     } catch (Exception e) {
-      LOG.warn("Failed to downgrade LSM timeline to old archived format");
       if (config.isFailOnTimelineArchivingEnabled()) {
         throw new HoodieException("Failed to downgrade LSM timeline to old archived format", e);
+      } else {
+        LOG.warn("Failed to downgrade LSM timeline to old archived format", e);
       }
     }
   }
@@ -243,6 +245,7 @@ public class EightToSevenDowngradeHandler implements DowngradeHandler {
     private final int batchSize;
     private final StoragePath archivePath;
     private final HoodieTableMetaClient metaClient;
+    private final Object lock = new Object();
 
     public ArchiveEntryFlusher(HoodieTableMetaClient metaClient, TimelineArchiverV1 archiverV1, int batchSize, StoragePath archivePath) {
       this.metaClient = metaClient;
@@ -254,24 +257,28 @@ public class EightToSevenDowngradeHandler implements DowngradeHandler {
 
     @Override
     public void accept(String s, GenericRecord archiveEntry) {
-      if (buffer.size() >= batchSize) {
-        archiverV1.flushArchiveEntries(new ArrayList<>(buffer), archivePath);
-        buffer.clear();
-      } else {
-        try {
-          GenericRecord legacyArchiveEntry = MetadataConversionUtils.createMetaWrapper(metaClient, archiveEntry);
-          buffer.add(legacyArchiveEntry);
-        } catch (IOException e) {
-          throw new HoodieException("Convert lsm archive entry to legacy error", e);
+      synchronized (lock) {
+        if (buffer.size() >= batchSize) {
+          archiverV1.flushArchiveEntries(new ArrayList<>(buffer), archivePath);
+          buffer.clear();
+        } else {
+          try {
+            GenericRecord legacyArchiveEntry = MetadataConversionUtils.createMetaWrapper(metaClient, archiveEntry);
+            buffer.add(legacyArchiveEntry);
+          } catch (IOException e) {
+            throw new HoodieException("Convert lsm archive entry to legacy error", e);
+          }
         }
       }
     }
 
     @Override
     public void close() {
-      if (!buffer.isEmpty()) {
-        archiverV1.flushArchiveEntries(new ArrayList<>(buffer), this.archivePath);
-        buffer.clear();
+      synchronized (lock) {
+        if (!buffer.isEmpty()) {
+          archiverV1.flushArchiveEntries(new ArrayList<>(buffer), this.archivePath);
+          buffer.clear();
+        }
       }
     }
   }
@@ -342,12 +349,14 @@ public class EightToSevenDowngradeHandler implements DowngradeHandler {
     // Get base path for metadata table.
     StoragePath metadataTableBasePath =
         HoodieTableMetadata.getMetadataTableBasePath(metaClient.getBasePath());
+    HoodieTableMetaClient metadataMetaClient =
+        HoodieTableMetaClient.builder()
+            .setBasePath(metadataTableBasePath.toUri().toString())
+            .setConf(hoodieStorage.getConf())
+            .build();
 
     // Fetch metadata partition paths.
-    List<String> metadataPartitions = FSUtils.getAllPartitionPaths(context,
-        hoodieStorage,
-        metadataTableBasePath,
-        false);
+    List<String> metadataPartitions = FSUtils.getAllPartitionPaths(context, metadataMetaClient, false);
 
     // Delete partitions.
     List<String> validPartitionPaths = deleteMetadataPartition(context, metaClient, metadataPartitions);
