@@ -506,27 +506,26 @@ public class ParquetRowDataWriter {
     private void doWrite(ArrayData arrayData) {
       recordConsumer.startGroup();
       if (arrayData.size() > 0) {
-        final String repeatedGroup = "array";
+        // align with Spark And Avro regarding the standard mode array type, see:
+        // https://github.com/apache/parquet-format/blob/master/LogicalTypes.md#lists
+        //
+        // <list-repetition> group <name> (LIST) {
+        //   repeated group list {
+        //     <element-repetition> <element-type> element;
+        //   }
+        // }
+        final String repeatedGroup = "list";
+        final String elementField = "element";
         recordConsumer.startField(repeatedGroup, 0);
-        if (elementWriter instanceof RowWriter) {
-          for (int i = 0; i < arrayData.size(); i++) {
-            if (!arrayData.isNullAt(i)) {
-              // Only creates the element field if the current array element is not null.
-              elementWriter.write(arrayData, i);
-            }
-          }
-        } else {
-          final String elementField = "element";
-          for (int i = 0; i < arrayData.size(); i++) {
-            recordConsumer.startGroup();
-            if (!arrayData.isNullAt(i)) {
-              // Only creates the element field if the current array element is not null.
-              recordConsumer.startField(elementField, 0);
-              elementWriter.write(arrayData, i);
-              recordConsumer.endField(elementField, 0);
-            }
-            recordConsumer.endGroup();
+        for (int i = 0; i < arrayData.size(); i++) {
+          recordConsumer.startGroup();
+          if (!arrayData.isNullAt(i)) {
+            // Only creates the element field if the current array element is not null.
+            recordConsumer.startField(elementField, 0);
+            elementWriter.write(arrayData, i);
+            recordConsumer.endField(elementField, 0);
           }
+          recordConsumer.endGroup();
         }
         recordConsumer.endField(repeatedGroup, 0);
       }
