@@ -18,25 +18,26 @@
 
 package org.apache.hudi.table.action.commit;
 
-import org.apache.avro.Schema;
 import org.apache.hudi.client.WriteStatus;
 import org.apache.hudi.common.config.TypedProperties;
 import org.apache.hudi.common.data.HoodieListData;
 import org.apache.hudi.common.engine.HoodieEngineContext;
+import org.apache.hudi.common.engine.HoodieReaderContext;
 import org.apache.hudi.common.model.HoodieKey;
-import org.apache.hudi.common.model.HoodieOperation;
 import org.apache.hudi.common.model.HoodieRecord;
-import org.apache.hudi.common.model.HoodieRecordMerger;
 import org.apache.hudi.common.model.WriteOperationType;
-import org.apache.hudi.exception.HoodieException;
+import org.apache.hudi.common.table.read.BufferedRecordMerger;
+import org.apache.hudi.common.table.read.DeleteContext;
+import org.apache.hudi.common.util.CollectionUtils;
 import org.apache.hudi.exception.HoodieUpsertException;
 import org.apache.hudi.index.HoodieIndex;
 import org.apache.hudi.table.HoodieTable;
 import org.apache.hudi.table.action.HoodieWriteMetadata;
 
-import java.io.IOException;
+import org.apache.avro.Schema;
+
 import java.time.Duration;
-import java.time.Instant;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
@@ -51,7 +52,7 @@ import java.util.stream.Collectors;
  * <p>Computing the records batch locations all at a time is a pressure to the engine,
  * we should avoid that in streaming system.
  */
-public class FlinkWriteHelper<T, R> extends BaseWriteHelper<T, List<HoodieRecord<T>>,
+public class FlinkWriteHelper<T, R> extends BaseWriteHelper<T, Iterator<HoodieRecord<T>>,
     List<HoodieKey>, List<WriteStatus>, R> {
 
   private FlinkWriteHelper() {
@@ -67,15 +68,12 @@ public class FlinkWriteHelper<T, R> extends BaseWriteHelper<T, List<HoodieRecord
   }
 
   @Override
-  public HoodieWriteMetadata<List<WriteStatus>> write(String instantTime, List<HoodieRecord<T>> inputRecords, HoodieEngineContext context,
-                                                      HoodieTable<T, List<HoodieRecord<T>>, List<HoodieKey>, List<WriteStatus>> table, boolean shouldCombine, int configuredShuffleParallelism,
-                                                      BaseCommitActionExecutor<T, List<HoodieRecord<T>>, List<HoodieKey>, List<WriteStatus>, R> executor, WriteOperationType operationType) {
+  public HoodieWriteMetadata<List<WriteStatus>> write(String instantTime, Iterator<HoodieRecord<T>> inputRecords, HoodieEngineContext context,
+                                                      HoodieTable<T, Iterator<HoodieRecord<T>>, List<HoodieKey>, List<WriteStatus>> table, boolean shouldCombine, int configuredShuffleParallelism,
+                                                      BaseCommitActionExecutor<T, Iterator<HoodieRecord<T>>, List<HoodieKey>, List<WriteStatus>, R> executor, WriteOperationType operationType) {
     try {
-      Instant lookupBegin = Instant.now();
-      Duration indexLookupDuration = Duration.between(lookupBegin, Instant.now());
-
       HoodieWriteMetadata<List<WriteStatus>> result = executor.execute(inputRecords);
-      result.setIndexLookupDuration(indexLookupDuration);
+      result.setIndexLookupDuration(Duration.ZERO);
       return result;
     } catch (Throwable e) {
       if (e instanceof HoodieUpsertException) {
@@ -86,37 +84,30 @@ public class FlinkWriteHelper<T, R> extends BaseWriteHelper<T, List<HoodieRecord
   }
 
   @Override
-  protected List<HoodieRecord<T>> tag(List<HoodieRecord<T>> dedupedRecords, HoodieEngineContext context, HoodieTable<T, List<HoodieRecord<T>>, List<HoodieKey>, List<WriteStatus>> table) {
-    return table.getIndex().tagLocation(HoodieListData.eager(dedupedRecords), context, table).collectAsList();
+  protected Iterator<HoodieRecord<T>> tag(Iterator<HoodieRecord<T>> dedupedRecords, HoodieEngineContext context, HoodieTable<T, Iterator<HoodieRecord<T>>, List<HoodieKey>, List<WriteStatus>> table) {
+    return table.getIndex()
+        .tagLocation(HoodieListData.eager(CollectionUtils.toList(dedupedRecords)), context, table)
+        .collectAsList().iterator();
   }
 
   @Override
-  public List<HoodieRecord<T>> deduplicateRecords(
-      List<HoodieRecord<T>> records, HoodieIndex<?, ?> index, int parallelism, String schemaStr, TypedProperties props, HoodieRecordMerger merger) {
+  public Iterator<HoodieRecord<T>> deduplicateRecords(Iterator<HoodieRecord<T>> records,
+                                                      HoodieIndex<?, ?> index,
+                                                      int parallelism,
+                                                      String schemaStr,
+                                                      TypedProperties props,
+                                                      BufferedRecordMerger<T> recordMerger,
+                                                      HoodieReaderContext<T> readerContext,
+                                                      String[] orderingFieldNames) {
     // If index used is global, then records are expected to differ in their partitionPath
-    Map<Object, List<HoodieRecord<T>>> keyedRecords = records.stream()
+    Map<Object, List<HoodieRecord<T>>> keyedRecords = CollectionUtils.toStream(records)
         .collect(Collectors.groupingBy(record -> record.getKey().getRecordKey()));
 
     // caution that the avro schema is not serializable
     final Schema schema = new Schema.Parser().parse(schemaStr);
-    return keyedRecords.values().stream().map(x -> x.stream().reduce((rec1, rec2) -> {
-      HoodieRecord<T> reducedRecord;
-      try {
-        // Precombine do not need schema and do not return null
-        reducedRecord =  merger.merge(rec1, schema, rec2, schema, props).get().getLeft();
-      } catch (IOException e) {
-        throw new HoodieException(String.format("Error to merge two records, %s, %s", rec1, rec2), e);
-      }
-      // we cannot allow the user to change the key or partitionPath, since that will affect
-      // everything
-      // so pick it from one of the records.
-      boolean choosePrev = rec1.getData() == reducedRecord.getData();
-      HoodieKey reducedKey = choosePrev ? rec1.getKey() : rec2.getKey();
-      HoodieOperation operation = choosePrev ? rec1.getOperation() : rec2.getOperation();
-      HoodieRecord<T> hoodieRecord = reducedRecord.newInstance(reducedKey, operation);
-      // reuse the location from the first record.
-      hoodieRecord.setCurrentLocation(rec1.getCurrentLocation());
-      return hoodieRecord;
-    }).orElse(null)).filter(Objects::nonNull).collect(Collectors.toList());
+    DeleteContext deleteContext = DeleteContext.fromRecordSchema(props, schema);
+    return keyedRecords.values().stream().map(x -> x.stream().reduce((previous, next) ->
+      reduceRecords(props, recordMerger, orderingFieldNames, previous, next, schema, readerContext.getRecordContext(), deleteContext)
+    ).orElse(null)).filter(Objects::nonNull).iterator();
   }
 }
