@@ -113,7 +113,7 @@ public class JavaDeleteHelper<R> extends
         deleteExecutor.saveWorkloadProfileMetadataToInflight(new WorkloadProfile(Pair.of(new HashMap<>(), new WorkloadStat())), instantTime);
         result = new HoodieWriteMetadata<>();
         result.setWriteStatuses(Collections.EMPTY_LIST);
-        deleteExecutor.commitOnAutoCommit(result);
+        deleteExecutor.runPrecommitValidators(result);
       }
       return result;
     } catch (Throwable e) {
