@@ -18,7 +18,6 @@
 
 package org.apache.hudi.client;
 
-import org.apache.hudi.avro.model.HoodieClusteringPlan;
 import org.apache.hudi.client.embedded.EmbeddedTimelineService;
 import org.apache.hudi.client.utils.SparkReleaseResources;
 import org.apache.hudi.client.utils.SparkValidatorUtils;
@@ -28,16 +27,11 @@ import org.apache.hudi.common.model.HoodieCommitMetadata;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.model.HoodieWriteStat;
 import org.apache.hudi.common.model.WriteOperationType;
-import org.apache.hudi.common.util.ClusteringUtils;
 import org.apache.hudi.common.util.Option;
-import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.common.util.VisibleForTesting;
-import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.data.HoodieJavaRDD;
-import org.apache.hudi.exception.HoodieClusteringException;
-import org.apache.hudi.execution.bulkinsert.BulkInsertSortMode;
 import org.apache.hudi.metadata.HoodieTableMetadata;
 import org.apache.hudi.storage.StorageConfiguration;
 import org.apache.hudi.table.HoodieSparkTable;
@@ -50,8 +44,6 @@ import java.util.Collections;
 import java.util.List;
 import java.util.stream.Collectors;
 
-import static org.apache.hudi.config.HoodieClusteringConfig.PLAN_STRATEGY_SORT_COLUMNS;
-
 public class SparkRDDTableServiceClient<T> extends BaseHoodieTableServiceClient<HoodieData<HoodieRecord<T>>, HoodieData<WriteStatus>, JavaRDD<WriteStatus>> {
 
   private final StreamingMetadataWriteHandler streamingMetadataWriteHandler;
@@ -102,30 +94,12 @@ public class SparkRDDTableServiceClient<T> extends BaseHoodieTableServiceClient<
       String instantTime,
       WriteOperationType writeOperationType) {
     if (isStreamingWriteToMetadataEnabled(table)) {
-      boolean enforceCoalesceWithRepartition = writeOperationType == WriteOperationType.CLUSTER; // for other table services, enforceCoalesceWithRepartition will be false.
-      if (enforceCoalesceWithRepartition) {
-        enforceCoalesceWithRepartition = computeEnforceCoalesceWithRepartitionForClustering(table, instantTime);
-      }
       writeMetadata.setWriteStatuses(streamingMetadataWriteHandler.streamWriteToMetadataTable(table, writeMetadata.getWriteStatuses(), instantTime,
-          enforceCoalesceWithRepartition, config.getMetadataConfig().getStreamingWritesCoalesceDivisorForDataTableWrites()));
+          config.getMetadataConfig().getStreamingWritesCoalesceDivisorForDataTableWrites()));
     }
     return writeMetadata;
   }
 
-  private boolean computeEnforceCoalesceWithRepartitionForClustering(HoodieTable table, String instantTime) {
-    // check clustering plan for sort columns. only if there are no sort columns, then fallback to sort mode from write config.
-    HoodieClusteringPlan clusteringPlan = ClusteringUtils.getClusteringPlan(
-            table.getMetaClient(), ClusteringUtils.getRequestedClusteringInstant(instantTime, table.getActiveTimeline(), table.getInstantGenerator()).get())
-        .map(Pair::getRight).orElseThrow(() -> new HoodieClusteringException(
-            "Unable to read clustering plan for instant: " + instantTime));
-    if (clusteringPlan.getStrategy().getStrategyParams().containsKey(PLAN_STRATEGY_SORT_COLUMNS.key())
-        && !StringUtils.isNullOrEmpty(clusteringPlan.getStrategy().getStrategyParams().get(PLAN_STRATEGY_SORT_COLUMNS.key()))) {
-      // sorting enabled.
-      return false;
-    }
-    return config.getBulkInsertSortMode() == BulkInsertSortMode.NONE;
-  }
-
   @Override
   protected void writeToMetadataTable(HoodieTable table, String instantTime, HoodieCommitMetadata metadata, List<HoodieWriteStat> partialMetadataWriteStats) {
     if (isStreamingWriteToMetadataEnabled(table)) {
