@@ -23,6 +23,7 @@ import org.apache.hudi.common.data.HoodieData;
 import org.apache.hudi.common.data.HoodiePairData;
 import org.apache.hudi.common.function.SerializableBiFunction;
 import org.apache.hudi.common.function.SerializableFunction;
+import org.apache.hudi.common.function.SerializablePairPredicate;
 import org.apache.hudi.common.function.SerializablePairFunction;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.ImmutablePair;
@@ -89,6 +90,11 @@ public class HoodieJavaPairRDD<K, V> implements HoodiePairData<K, V> {
     pairRDDData.unpersist();
   }
 
+  @Override
+  public void unpersistWithDependencies() {
+    HoodieSparkRDDUtils.unpersistRDDWithDependencies(pairRDDData.rdd());
+  }
+
   @Override
   public HoodieData<K> keys() {
     return HoodieJavaRDD.of(pairRDDData.keys());
@@ -131,7 +137,7 @@ public class HoodieJavaPairRDD<K, V> implements HoodiePairData<K, V> {
   }
 
   public <W> HoodiePairData<K, W> flatMapValues(SerializableFunction<V, Iterator<W>> func) {
-    return HoodieJavaPairRDD.of(pairRDDData.flatMapValues(func::apply));
+    return HoodieJavaPairRDD.of(pairRDDData.flatMapValues(iter -> CloseableIteratorListener.addListener(func.apply(iter))));
   }
 
   @Override
@@ -155,6 +161,11 @@ public class HoodieJavaPairRDD<K, V> implements HoodiePairData<K, V> {
     return HoodieJavaPairRDD.of(pairRDDData.union(HoodieJavaPairRDD.getJavaPairRDD(other)));
   }
 
+  @Override
+  public HoodiePairData<K, V> filter(SerializablePairPredicate<K, V> filter) {
+    return HoodieJavaPairRDD.of(pairRDDData.filter(p -> filter.call(p._1, p._2)));
+  }
+
   @Override
   public <W> HoodiePairData<K, Pair<V, W>> join(HoodiePairData<K, W> other) {
     return HoodieJavaPairRDD.of(JavaPairRDD.fromJavaRDD(
@@ -187,4 +198,5 @@ public class HoodieJavaPairRDD<K, V> implements HoodiePairData<K, V> {
       return pairRDDData.getNumPartitions();
     }
   }
+
 }
