@@ -108,6 +108,11 @@ public class HoodieJavaRDD<T> implements HoodieData<T> {
     rddData.unpersist();
   }
 
+  @Override
+  public void unpersistWithDependencies() {
+    HoodieSparkRDDUtils.unpersistRDDWithDependencies(rddData.rdd());
+  }
+
   @Override
   public boolean isEmpty() {
     return rddData.isEmpty();
@@ -150,21 +155,21 @@ public class HoodieJavaRDD<T> implements HoodieData<T> {
 
   @Override
   public <O> HoodieData<O> mapPartitions(SerializableFunction<Iterator<T>, Iterator<O>> func, boolean preservesPartitioning) {
-    return HoodieJavaRDD.of(rddData.mapPartitions(func::apply, preservesPartitioning));
+    return HoodieJavaRDD.of(rddData.mapPartitions(iter -> CloseableIteratorListener.addListener(func.apply(iter)), preservesPartitioning));
   }
 
   @Override
   public <O> HoodieData<O> flatMap(SerializableFunction<T, Iterator<O>> func) {
     // NOTE: Unrolling this lambda into a method reference results in [[ClassCastException]]
     //       due to weird interop b/w Scala and Java
-    return HoodieJavaRDD.of(rddData.flatMap(e -> func.apply(e)));
+    return HoodieJavaRDD.of(rddData.flatMap(e -> CloseableIteratorListener.addListener(func.apply(e))));
   }
 
   @Override
   public <K, V> HoodiePairData<K, V> flatMapToPair(SerializableFunction<T, Iterator<? extends Pair<K, V>>> func) {
     return HoodieJavaPairRDD.of(
         rddData.flatMapToPair(e ->
-            new MappingIterator<>(func.apply(e), p -> new Tuple2<>(p.getKey(), p.getValue()))));
+            new MappingIterator<>(CloseableIteratorListener.addListener(func.apply(e)), p -> new Tuple2<>(p.getKey(), p.getValue()))));
   }
 
   @Override
@@ -204,4 +209,9 @@ public class HoodieJavaRDD<T> implements HoodieData<T> {
   public HoodieData<T> repartition(int parallelism) {
     return HoodieJavaRDD.of(rddData.repartition(parallelism));
   }
+
+  @Override
+  public HoodieData<T> coalesce(int parallelism) {
+    return HoodieJavaRDD.of(rddData.coalesce(parallelism));
+  }
 }
