@@ -171,7 +171,7 @@ public class BulkInsertDataInternalWriterHelper {
 
   public void close() throws IOException {
     for (HoodieRowCreateHandle rowCreateHandle : handles.values()) {
-      LOG.info("Closing bulk insert file " + rowCreateHandle.getFileName());
+      LOG.info("Closing bulk insert file {}", rowCreateHandle.getFileName());
       writeStatusList.add(rowCreateHandle.close());
     }
     handles.clear();
@@ -200,13 +200,13 @@ public class BulkInsertDataInternalWriterHelper {
         close();
       }
 
-      LOG.info("Creating new file for partition path " + partitionPath);
+      LOG.info("Creating new file for partition path {}", partitionPath);
       HoodieRowCreateHandle rowCreateHandle = createHandle(partitionPath);
       handles.put(partitionPath, rowCreateHandle);
     } else if (!handles.get(partitionPath).canWrite()) {
       // even if there is a handle to the partition path, it could have reached its max size threshold. So, we close the handle here and
       // create a new one.
-      LOG.info("Rolling max-size file for partition path " + partitionPath);
+      LOG.info("Rolling max-size file for partition path {}", partitionPath);
       writeStatusList.add(handles.remove(partitionPath).close());
       HoodieRowCreateHandle rowCreateHandle = createHandle(partitionPath);
       handles.put(partitionPath, rowCreateHandle);
