@@ -23,7 +23,6 @@ import org.apache.hudi.common.model.HoodieKey;
 import org.apache.hudi.common.model.HoodieRecordLocation;
 import org.apache.hudi.common.model.WriteOperationType;
 import org.apache.hudi.common.util.Option;
-import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieException;
@@ -34,7 +33,6 @@ import org.apache.hudi.table.WorkloadProfile;
 import org.apache.hudi.table.WorkloadStat;
 
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -114,30 +112,9 @@ public class SparkBucketIndexPartitioner<T> extends
   }
 
   @Override
-  public BucketInfo getBucketInfo(int bucketNumber) {
-    String partitionPath = partitionPaths.get(bucketNumber / numBuckets);
-    String bucketId = BucketIdentifier.bucketIdStr(bucketNumber % numBuckets);
-    // Insert overwrite always generates new bucket file id
-    if (isOverwrite) {
-      ValidationUtils.checkArgument(!isNonBlockingConcurrencyControl,
-          "Insert overwrite is not supported with non-blocking concurrency control");
-      return new BucketInfo(BucketType.INSERT, BucketIdentifier.newBucketFileIdPrefix(bucketId), partitionPath);
-    }
-    Option<String> fileIdOption = Option.fromJavaOptional(updatePartitionPathFileIds
-        .getOrDefault(partitionPath, Collections.emptySet()).stream()
-        .filter(e -> e.startsWith(bucketId))
-        .findFirst());
-    if (fileIdOption.isPresent()) {
-      return new BucketInfo(BucketType.UPDATE, fileIdOption.get(), partitionPath);
-    } else {
-      // Always write into log file instead of base file if using NB-CC
-      if (isNonBlockingConcurrencyControl) {
-        String fileId = BucketIdentifier.newBucketFileIdForNBCC(bucketNumber);
-        return new BucketInfo(BucketType.UPDATE, fileId, partitionPath);
-      }
-      String fileIdPrefix = BucketIdentifier.newBucketFileIdPrefix(bucketId);
-      return new BucketInfo(BucketType.INSERT, fileIdPrefix, partitionPath);
-    }
+  public SparkBucketInfoGetter getSparkBucketInfoGetter() {
+    return new SparkBucketIndexBucketInfoGetter(numBuckets, partitionPaths,
+        updatePartitionPathFileIds, isOverwrite, isNonBlockingConcurrencyControl);
   }
 
   @Override
@@ -152,7 +129,7 @@ public class SparkBucketIndexPartitioner<T> extends
     Option<HoodieRecordLocation> location = keyLocation._2;
     int bucketId = location.isPresent()
         ? BucketIdentifier.bucketIdFromFileId(location.get().getFileId())
-        : BucketIdentifier.getBucketId(keyLocation._1, indexKeyField, numBuckets);
+        : BucketIdentifier.getBucketId(keyLocation._1.getRecordKey(), indexKeyField, numBuckets);
     return partitionPathOffset.get(partitionPath) + bucketId;
   }
 }
