@@ -23,6 +23,7 @@ import org.apache.hudi.common.model.FileSlice;
 import org.apache.hudi.common.model.HoodieBaseFile;
 import org.apache.hudi.common.model.HoodieLogFile;
 import org.apache.hudi.common.model.HoodieRecordLocation;
+import org.apache.hudi.common.model.WriteOperationType;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.config.HoodieWriteConfig;
@@ -46,8 +47,8 @@ import java.util.stream.Collectors;
 public class SparkUpsertDeltaCommitPartitioner<T> extends UpsertPartitioner<T> {
 
   public SparkUpsertDeltaCommitPartitioner(WorkloadProfile profile, HoodieSparkEngineContext context, HoodieTable table,
-                                           HoodieWriteConfig config) {
-    super(profile, context, table, config);
+                                           HoodieWriteConfig config, WriteOperationType operationType) {
+    super(profile, context, table, config, operationType);
   }
 
   @Override
@@ -72,13 +73,13 @@ public class SparkUpsertDeltaCommitPartitioner<T> extends UpsertPartitioner<T> {
       if (smallFileSlice.getBaseFile().isPresent()) {
         HoodieBaseFile baseFile = smallFileSlice.getBaseFile().get();
         sf.location = new HoodieRecordLocation(baseFile.getCommitTime(), baseFile.getFileId());
-        sf.sizeBytes = getTotalFileSize(smallFileSlice);
+        sf.sizeBytes = smallFileSlice.getTotalFileSizeAsParquetFormat(config.getLogFileToParquetCompressionRatio());
         smallFileLocations.add(sf);
       } else {
         HoodieLogFile logFile = smallFileSlice.getLogFiles().findFirst().get();
-        sf.location = new HoodieRecordLocation(logFile.getBaseCommitTime(),
+        sf.location = new HoodieRecordLocation(logFile.getDeltaCommitTime(),
             logFile.getFileId());
-        sf.sizeBytes = getTotalFileSize(smallFileSlice);
+        sf.sizeBytes = smallFileSlice.getTotalFileSizeAsParquetFormat(config.getLogFileToParquetCompressionRatio());
         smallFileLocations.add(sf);
       }
     }
@@ -91,7 +92,7 @@ public class SparkUpsertDeltaCommitPartitioner<T> extends UpsertPartitioner<T> {
     // pending compaction
     if (table.getIndex().canIndexLogFiles()) {
       return table.getSliceView()
-              .getLatestFileSlicesBeforeOrOn(partitionPath, latestCommitInstant.getTimestamp(), false)
+              .getLatestFileSlicesBeforeOrOn(partitionPath, latestCommitInstant.requestedTime(), false)
               .filter(this::isSmallFile)
               .collect(Collectors.toList());
     }
@@ -103,7 +104,7 @@ public class SparkUpsertDeltaCommitPartitioner<T> extends UpsertPartitioner<T> {
     // If we cannot index log files, then we choose the smallest parquet file in the partition and add inserts to
     // it. Doing this overtime for a partition, we ensure that we handle small file issues
     return table.getSliceView()
-          .getLatestFileSlicesBeforeOrOn(partitionPath, latestCommitInstant.getTimestamp(), false)
+          .getLatestFileSlicesBeforeOrOn(partitionPath, latestCommitInstant.requestedTime(), false)
           .filter(
               fileSlice ->
                   // NOTE: We can not pad slices with existing log-files w/o compacting these,
@@ -120,31 +121,9 @@ public class SparkUpsertDeltaCommitPartitioner<T> extends UpsertPartitioner<T> {
         .collect(Collectors.toList());
   }
 
-  private long getTotalFileSize(FileSlice fileSlice) {
-    if (!fileSlice.getBaseFile().isPresent()) {
-      return convertLogFilesSizeToExpectedParquetSize(fileSlice.getLogFiles().collect(Collectors.toList()));
-    } else {
-      return fileSlice.getBaseFile().get().getFileSize()
-          + convertLogFilesSizeToExpectedParquetSize(fileSlice.getLogFiles().collect(Collectors.toList()));
-    }
-  }
-
   private boolean isSmallFile(FileSlice fileSlice) {
-    long totalSize = getTotalFileSize(fileSlice);
+    long totalSize = fileSlice.getTotalFileSizeAsParquetFormat(config.getLogFileToParquetCompressionRatio());
     return totalSize < config.getParquetMaxFileSize();
   }
 
-  // TODO (NA) : Make this static part of utility
-  public long convertLogFilesSizeToExpectedParquetSize(List<HoodieLogFile> hoodieLogFiles) {
-    long totalSizeOfLogFiles =
-        hoodieLogFiles.stream()
-            .map(HoodieLogFile::getFileSize)
-            .filter(size -> size > 0)
-            .reduce(Long::sum)
-            .orElse(0L);
-    // Here we assume that if there is no base parquet file, all log files contain only inserts.
-    // We can then just get the parquet equivalent size of these log files, compare that with
-    // {@link config.getParquetMaxFileSize()} and decide if there is scope to insert more rows
-    return (long) (totalSizeOfLogFiles * config.getLogFileToParquetCompressionRatio());
-  }
 }
