@@ -19,6 +19,7 @@
 package org.apache.hudi.client;
 
 import org.apache.hudi.common.model.HoodieAvroIndexedRecord;
+import org.apache.hudi.common.model.HoodieAvroPayload;
 import org.apache.hudi.common.model.HoodieAvroRecord;
 import org.apache.hudi.common.model.HoodieKey;
 import org.apache.hudi.common.model.HoodieRecord;
@@ -26,20 +27,21 @@ import org.apache.hudi.common.model.HoodieRecordLocation;
 import org.apache.hudi.common.table.view.FileSystemViewStorageConfig;
 import org.apache.hudi.common.testutils.HoodieTestUtils;
 import org.apache.hudi.common.testutils.InProcessTimeGenerator;
-import org.apache.hudi.common.testutils.RawTripTestPayload;
+import org.apache.hudi.common.util.JsonUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieUpsertException;
 import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.io.CreateHandleFactory;
-import org.apache.hudi.io.HoodieMergeHandle;
 import org.apache.hudi.io.HoodieWriteHandle;
+import org.apache.hudi.io.HoodieWriteMergeHandle;
 import org.apache.hudi.io.storage.HoodieIOFactory;
 import org.apache.hudi.storage.StoragePath;
 import org.apache.hudi.table.HoodieSparkTable;
 import org.apache.hudi.testutils.HoodieSparkClientTestHarness;
 
 import org.apache.avro.Schema;
+import org.apache.avro.generic.GenericData;
 import org.apache.avro.generic.GenericRecord;
 import org.apache.hadoop.fs.Path;
 import org.apache.parquet.io.InvalidRecordException;
@@ -53,12 +55,15 @@ import java.io.IOException;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 
 import static org.apache.hudi.common.testutils.HoodieTestUtils.INSTANT_FILE_NAME_GENERATOR;
+import static org.apache.hudi.common.testutils.HoodieTestUtils.createSimpleRecord;
+import static org.apache.hudi.common.testutils.HoodieTestUtils.extractPartitionFromTimeField;
 import static org.apache.hudi.common.testutils.SchemaTestUtil.getSchemaFromResource;
 import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
 import static org.junit.jupiter.api.Assertions.assertThrows;
@@ -79,23 +84,17 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     cleanupResources();
   }
 
-  private WriteStatus prepareFirstRecordCommit(List<String> recordsStrs) throws IOException {
+  private WriteStatus prepareFirstRecordCommit(List<HoodieRecord> insertRecords) throws IOException {
     // Create a bunch of records with an old version of schema
     final HoodieWriteConfig config = makeHoodieClientConfig("/exampleSchema.avsc");
     final HoodieSparkTable table = HoodieSparkTable.create(config, context);
     final List<WriteStatus> statuses = jsc.parallelize(Arrays.asList(1)).map(x -> {
-      List<HoodieRecord> insertRecords = new ArrayList<>();
-      for (String recordStr : recordsStrs) {
-        RawTripTestPayload rowChange = new RawTripTestPayload(recordStr);
-        insertRecords
-            .add(new HoodieAvroRecord(new HoodieKey(rowChange.getRowKey(), rowChange.getPartitionPath()), rowChange));
-      }
       Map<String, HoodieRecord> insertRecordMap = insertRecords.stream()
           .collect(Collectors.toMap(r -> r.getRecordKey(), Function.identity()));
       HoodieWriteHandle<?,?,?,?> createHandle = new CreateHandleFactory<>(false)
           .create(config, "100", table, insertRecords.get(0).getPartitionPath(), "f1-0", supplier);
       for (HoodieRecord record : insertRecordMap.values()) {
-        createHandle.write(record, createHandle.getWriterSchemaWithMetaFields(), createHandle.getConfig().getProps());
+        createHandle.write(record, createHandle.getWriterSchema(), createHandle.getConfig().getProps());
       }
       return createHandle.close().get(0);
     }).collect();
@@ -106,33 +105,23 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     return statuses.get(0);
   }
 
-  private List<String> generateMultipleRecordsForExampleSchema() {
-    List<String> recordsStrs = new ArrayList<>();
-    String recordStr1 = "{\"_row_key\":\"8eb5b87a-1feh-4edd-87b4-6ec96dc405a0\","
-        + "\"time\":\"2016-01-31T03:16:41.415Z\",\"number\":12}";
-    String recordStr2 = "{\"_row_key\":\"8eb5b87b-1feu-4edd-87b4-6ec96dc405a0\","
-        + "\"time\":\"2016-01-31T03:20:41.415Z\",\"number\":100}";
-    String recordStr3 = "{\"_row_key\":\"8eb5b87c-1fej-4edd-87b4-6ec96dc405a0\","
-        + "\"time\":\"2016-01-31T03:16:41.415Z\",\"number\":15}";
-    recordsStrs.add(recordStr1);
-    recordsStrs.add(recordStr2);
-    recordsStrs.add(recordStr3);
-    return recordsStrs;
+  private List<HoodieRecord> generateMultipleRecordsForExampleSchema() {
+    List<HoodieRecord> records = new ArrayList<>();
+    records.add(createSimpleRecord("8eb5b87a-1feh-4edd-87b4-6ec96dc405a0", "2016-01-31T03:16:41.415Z", 12));
+    records.add(createSimpleRecord("8eb5b87b-1feu-4edd-87b4-6ec96dc405a0", "2016-01-31T03:20:41.415Z", 100));
+    records.add(createSimpleRecord("8eb5b87c-1fej-4edd-87b4-6ec96dc405a0", "2016-01-31T03:16:41.415Z", 15));
+    return records;
   }
 
-  private List<String> generateOneRecordForExampleSchema() {
-    List<String> recordsStrs = new ArrayList<>();
-    String recordStr = "{\"_row_key\":\"8eb5b87c-1fej-4edd-87b4-6ec96dc405a0\","
-        + "\"time\":\"2016-01-31T03:16:41.415Z\",\"number\":15}";
-    recordsStrs.add(recordStr);
-    return recordsStrs;
+  private List<HoodieRecord> generateOneRecordForExampleSchema() {
+    return Collections.singletonList(createSimpleRecord("8eb5b87c-1fej-4edd-87b4-6ec96dc405a0", "2016-01-31T03:16:41.415Z", 15));
   }
 
   private void assertSchemaEvolutionOnUpdateResult(WriteStatus insertResult, HoodieSparkTable updateTable,
                                                    List<HoodieRecord> updateRecords, String assertMsg, boolean isAssertThrow, Class expectedExceptionType) {
     jsc.parallelize(Arrays.asList(1)).map(x -> {
       Executable executable = () -> {
-        HoodieMergeHandle mergeHandle = new HoodieMergeHandle(updateTable.getConfig(), "101", updateTable,
+        HoodieWriteMergeHandle mergeHandle = new HoodieWriteMergeHandle(updateTable.getConfig(), "101", updateTable,
             updateRecords.iterator(), updateRecords.get(0).getPartitionPath(), insertResult.getFileId(), supplier, Option.empty());
         List<GenericRecord> oldRecords = HoodieIOFactory.getIOFactory(updateTable.getStorage())
             .getFileFormatUtils(updateTable.getBaseFileFormat())
@@ -154,15 +143,17 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     }).collect();
   }
 
-  private List<HoodieRecord> buildUpdateRecords(String recordStr, String insertFileId) throws IOException {
-    List<HoodieRecord> updateRecords = new ArrayList<>();
-    RawTripTestPayload rowChange = new RawTripTestPayload(recordStr);
-    HoodieRecord record =
-        new HoodieAvroRecord(new HoodieKey(rowChange.getRowKey(), rowChange.getPartitionPath()), rowChange);
+  private List<HoodieRecord> buildUpdateRecords(String recordStr, String insertFileId, String schema) throws IOException {
+    Schema avroSchema = new Schema.Parser().parse(schema);
+    GenericRecord data = new GenericData.Record(avroSchema);
+    Map<String, Object> json = JsonUtils.getObjectMapper().readValue(recordStr, Map.class);
+    json.forEach(data::put);
+    String key = json.get("_row_key").toString();
+    String partition = extractPartitionFromTimeField(json.get("time").toString());
+    HoodieRecord record = new HoodieAvroRecord<>(new HoodieKey(key, partition), new HoodieAvroPayload(Option.of(data)));
     record.setCurrentLocation(new HoodieRecordLocation("101", insertFileId));
     record.seal();
-    updateRecords.add(record);
-    return updateRecords;
+    return Collections.singletonList(record);
   }
 
   @Test
@@ -175,7 +166,7 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     // New content with values for the newly added field
     String recordStr = "{\"_row_key\":\"8eb5b87a-1feh-4edd-87b4-6ec96dc405a0\","
         + "\"time\":\"2016-01-31T03:16:41.415Z\",\"number\":12,\"added_field\":1}";
-    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId());
+    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId(), config.getSchema());
     String assertMsg = "UpdateFunction could not read records written with exampleSchema.avsc using the "
         + "exampleEvolvedSchema.avsc";
     assertSchemaEvolutionOnUpdateResult(insertResult, table, updateRecords, assertMsg, false, null);
@@ -190,7 +181,7 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     final HoodieSparkTable table = HoodieSparkTable.create(config, context);
     String recordStr = "{\"_row_key\":\"8eb5b87a-1feh-4edd-87b4-6ec96dc405a0\","
         + "\"time\":\"2016-01-31T03:16:41.415Z\",\"added_field\":1,\"number\":12}";
-    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId());
+    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId(), config.getSchema());
     String assertMsg = "UpdateFunction could not read records written with exampleSchema.avsc using the "
         + "exampleEvolvedSchemaChangeOrder.avsc as column order change";
     assertSchemaEvolutionOnUpdateResult(insertResult, table, updateRecords, assertMsg, false, null);
@@ -205,7 +196,7 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     final HoodieSparkTable table = HoodieSparkTable.create(config, context);
     String recordStr = "{\"_row_key\":\"8eb5b87a-1feh-4edd-87b4-6ec96dc405a0\","
         + "\"time\":\"2016-01-31T03:16:41.415Z\"}";
-    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId());
+    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId(), config.getSchema());
     String assertMsg = "UpdateFunction when delete column, Parquet/Avro schema mismatch: Avro field 'xxx' not found";
     assertSchemaEvolutionOnUpdateResult(insertResult, table, updateRecords, assertMsg, true, InvalidRecordException.class);
   }
@@ -219,7 +210,7 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     final HoodieSparkTable table = HoodieSparkTable.create(config, context);
     String recordStr = "{\"_row_key\":\"8eb5b87a-1feh-4edd-87b4-6ec96dc405a0\","
         + "\"time\":\"2016-01-31T03:16:41.415Z\",\"number\":12,\"added_field\":1}";
-    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId());
+    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId(), config.getSchema());
     String assertMsg = "UpdateFunction could not read records written with exampleSchema.avsc using the "
         + "exampleEvolvedSchemaColumnRequire.avsc, because old records do not have required column added_field";
     assertSchemaEvolutionOnUpdateResult(insertResult, table, updateRecords, assertMsg, true, HoodieUpsertException.class);
@@ -234,7 +225,7 @@ public class TestUpdateSchemaEvolution extends HoodieSparkClientTestHarness impl
     final HoodieSparkTable table = HoodieSparkTable.create(config, context);
     String recordStr = "{\"_row_key\":\"8eb5b87a-1feh-4edd-87b4-6ec96dc405a0\","
         + "\"time\":\"2016-01-31T03:16:41.415Z\",\"number\":\"12\"}";
-    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId());
+    List<HoodieRecord> updateRecords = buildUpdateRecords(recordStr, insertResult.getFileId(), config.getSchema());
     String assertMsg = "UpdateFunction when change column type, org.apache.parquet.avro.AvroConverters$FieldUTF8Converter";
     assertSchemaEvolutionOnUpdateResult(insertResult, table, updateRecords, assertMsg, true, ParquetDecodingException.class);
   }
