@@ -61,7 +61,7 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
 public class TestTimelineServerBasedWriteMarkers extends TestWriteMarkersBase {
 
   private static final Logger LOG = LoggerFactory.getLogger(TestTimelineServerBasedWriteMarkers.class);
-  protected static int DEFAULT_READ_TIMEOUT_SECS = 60;
+  protected static final int DEFAULT_READ_TIMEOUT_SECS = 60;
 
   TimelineService timelineService = null;
 
@@ -99,7 +99,7 @@ public class TestTimelineServerBasedWriteMarkers extends TestWriteMarkersBase {
     assertIterableEquals(expectedMarkers, allMarkers);
     // Verifies the marker type file
     StoragePath markerTypeFilePath = new StoragePath(markerFolderPath, MarkerUtils.MARKER_TYPE_FILENAME);
-    assertTrue(MarkerUtils.doesMarkerTypeFileExist(storage, markerFolderPath.toString()));
+    assertTrue(MarkerUtils.doesMarkerTypeFileExist(storage, markerFolderPath));
     InputStream inputStream = storage.open(markerTypeFilePath);
     assertEquals(MarkerType.TIMELINE_SERVER_BASED.toString(),
         FileIOUtils.readAsUTFString(inputStream));
@@ -149,7 +149,6 @@ public class TestTimelineServerBasedWriteMarkers extends TestWriteMarkersBase {
       TimelineServiceTestHarness.Builder builder = TimelineServiceTestHarness.newBuilder();
       builder.withNumberOfSimulatedConnectionFailures(numberOfSimulatedConnectionFailures);
       timelineService = builder.build(
-          hoodieEngineContext,
           (Configuration) storage.getConf().unwrap(),
           TimelineService.Config.builder().serverPort(0).enableMarkerRequests(true).build(),
           FileSystemViewManager.createViewManager(
