@@ -46,6 +46,7 @@ import org.apache.hudi.timeline.service.TimelineService;
 
 import org.apache.avro.generic.GenericRecord;
 import org.apache.hadoop.conf.Configuration;
+import org.apache.spark.HoodieSparkKryoRegistrar;
 import org.apache.spark.SparkConf;
 import org.apache.spark.SparkContext;
 import org.apache.spark.api.java.JavaSparkContext;
@@ -92,7 +93,7 @@ public class HoodieClientTestUtils {
    */
   public static SparkConf getSparkConfForTest(String appName) {
     SparkConf sparkConf = new SparkConf().setAppName(appName)
-        .setMaster("local[8]")
+        .setMaster("local[8,1]")
         .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
         .set("spark.kryo.registrator", "org.apache.spark.HoodieSparkKryoRegistrar")
         .set("spark.sql.shuffle.partitions", "4")
@@ -118,7 +119,7 @@ public class HoodieClientTestUtils {
     } else {
       sparkConf.set("spark.ui.enabled", "false");
     }
-
+    HoodieSparkKryoRegistrar.register(sparkConf);
     return SparkRDDReadClient.addHoodieSupport(sparkConf);
   }
   
@@ -153,7 +154,7 @@ public class HoodieClientTestUtils {
     try {
       HashMap<String, String> paths =
           getLatestFileIDsToFullPath(basePath, commitTimeline, Arrays.asList(commitInstant));
-      LOG.info("Path :" + paths.values());
+      LOG.info("Path :{}", paths.values());
       Dataset<Row> unFilteredRows = null;
       if (HoodieTableConfig.BASE_FILE_FORMAT.defaultValue().equals(HoodieFileFormat.PARQUET)) {
         unFilteredRows = sqlContext.read().parquet(paths.values().toArray(new String[paths.size()]));
@@ -281,12 +282,12 @@ public class HoodieClientTestUtils {
           .withFileSystemViewConfig(FileSystemViewStorageConfig.newBuilder()
               .withRemoteServerPort(timelineServicePort).build())
           .build();
-      TimelineService timelineService = new TimelineService(context, HadoopFSUtils.getStorageConf(),
+      TimelineService timelineService = new TimelineService(HadoopFSUtils.getStorageConf(),
           TimelineService.Config.builder().enableMarkerRequests(true)
               .serverPort(config.getViewStorageConfig().getRemoteViewServerPort()).build(),
           FileSystemViewManager.createViewManager(context, config.getMetadataConfig(), config.getViewStorageConfig(), config.getCommonConfig()));
       timelineService.startService();
-      LOG.info("Timeline service server port: " + timelineServicePort);
+      LOG.info("Timeline service server port: {}", timelineServicePort);
       return timelineService;
     } catch (Exception ex) {
       throw new RuntimeException(ex);
