@@ -22,6 +22,7 @@ import org.apache.hudi.common.util.Either;
 import org.apache.hudi.common.util.Option;
 
 import org.apache.avro.AvroRuntimeException;
+import org.apache.avro.LogicalTypes;
 import org.apache.avro.Schema;
 import org.apache.avro.Schema.Field;
 import org.apache.avro.Schema.Type;
@@ -522,15 +523,46 @@ public class AvroSchemaCompatibility {
 
     private SchemaCompatibilityResult checkFixedSize(final Schema reader, final Schema writer,
                                                      final Deque<LocationInfo> locations) {
-      SchemaCompatibilityResult result = SchemaCompatibilityResult.compatible();
       int actual = reader.getFixedSize();
       int expected = writer.getFixedSize();
       if (actual != expected) {
         String message = String.format("Fixed size field '%s' expected: %d, found: %d", getLocationName(locations, reader.getType()), expected, actual);
-        result = SchemaCompatibilityResult.incompatible(SchemaIncompatibilityType.FIXED_SIZE_MISMATCH, reader, writer,
+        return SchemaCompatibilityResult.incompatible(SchemaIncompatibilityType.FIXED_SIZE_MISMATCH, reader, writer,
             message, asList(locations));
       }
-      return result;
+      return checkDecimalWidening(reader, writer, locations);
+    }
+
+    private SchemaCompatibilityResult checkDecimalWidening(final Schema reader, final Schema writer,
+                                                           final Deque<LocationInfo> locations) {
+      boolean isReaderDecimal = reader.getLogicalType() instanceof LogicalTypes.Decimal;
+      boolean isWriterDecimal = writer.getLogicalType() instanceof LogicalTypes.Decimal;
+      if (!isReaderDecimal && !isWriterDecimal) {
+        return SchemaCompatibilityResult.compatible();
+      }
+
+      if (!isReaderDecimal || !isWriterDecimal) {
+        String message = String.format("Decimal field '%s' expected: %s, found: %s", getLocationName(locations, reader.getType()), writer, reader);
+        return SchemaCompatibilityResult.incompatible(SchemaIncompatibilityType.DECIMAL_MISMATCH, reader, writer,
+            message, asList(locations));
+      }
+
+      int readerScale = ((LogicalTypes.Decimal) reader.getLogicalType()).getScale();
+      int writerScale = ((LogicalTypes.Decimal) writer.getLogicalType()).getScale();
+      int readerPrecision = ((LogicalTypes.Decimal) reader.getLogicalType()).getPrecision();
+      int writerPrecision = ((LogicalTypes.Decimal) writer.getLogicalType()).getPrecision();
+      if (readerScale == writerScale && readerPrecision == writerPrecision) {
+        return SchemaCompatibilityResult.compatible();
+      }
+
+      if (((readerPrecision - readerScale) < (writerPrecision - writerScale)) || (readerScale < writerScale)) {
+        String message = String.format("Decimal field '%s' evolution is lossy. Existing precision: %d, scale: %d, Incoming precision: %d, scale: %d",
+            getLocationName(locations, reader.getType()), writerPrecision, writerScale, readerPrecision, readerScale);
+        return SchemaCompatibilityResult.incompatible(SchemaIncompatibilityType.DECIMAL_MISMATCH, reader, writer,
+            message, asList(locations));
+      }
+
+      return SchemaCompatibilityResult.compatible();
     }
 
     private SchemaCompatibilityResult checkSchemaNames(final Schema reader, final Schema writer,
@@ -588,12 +620,12 @@ public class AvroSchemaCompatibility {
     /**
      * Used internally to tag a reader/writer schema pair and prevent recursion.
      */
-    RECURSION_IN_PROGRESS;
+    RECURSION_IN_PROGRESS
   }
 
   public enum SchemaIncompatibilityType {
     NAME_MISMATCH, FIXED_SIZE_MISMATCH, MISSING_ENUM_SYMBOLS, READER_FIELD_MISSING_DEFAULT_VALUE, TYPE_MISMATCH,
-    MISSING_UNION_BRANCH;
+    DECIMAL_MISMATCH, MISSING_UNION_BRANCH
   }
 
   /**
