@@ -26,18 +26,22 @@ package org.apache.hudi.common.config;
  */
 public class ConfigGroups {
   /**
-   * Config group names.
+   * Config group names. Please add the description of each group in
+   * {@link ConfigGroups#getDescription}.
    */
   public enum Names {
+    TABLE_CONFIG("Hudi Table Config"),
     ENVIRONMENT_CONFIG("Environment Config"),
     SPARK_DATASOURCE("Spark Datasource Configs"),
     FLINK_SQL("Flink Sql Configs"),
     WRITE_CLIENT("Write Client Configs"),
+    READER("Reader Configs"),
     META_SYNC("Metastore and Catalog Sync Configs"),
     METRICS("Metrics Configs"),
     RECORD_PAYLOAD("Record Payload Config"),
     KAFKA_CONNECT("Kafka Connect Configs"),
-    AWS("Amazon Web Services Configs");
+    AWS("Amazon Web Services Configs"),
+    HUDI_STREAMER("Hudi Streamer Configs");
 
     public final String name;
 
@@ -51,6 +55,11 @@ public class ConfigGroups {
         "Index Configs",
         "Configurations that control indexing behavior, "
             + "which tags incoming records as either inserts or updates to older records."),
+    KEY_GENERATOR(
+        "Key Generator Configs",
+        "Hudi maintains keys (record key + partition path) for uniquely identifying a "
+            + "particular record. These configs allow developers to setup the Key generator class "
+            + "that extracts these out of incoming records."),
     LOCK(
         "Lock Configs",
         "Configurations that control locking mechanisms required for concurrency control "
@@ -60,6 +69,12 @@ public class ConfigGroups {
         "Commit Callback Configs",
         "Configurations controlling callback behavior into HTTP endpoints, to push "
             + "notifications on commits on hudi tables."),
+    SCHEMA_PROVIDER(
+        "Hudi Streamer Schema Provider Configs",
+        "Configurations that control the schema provider for Hudi Streamer."),
+    DELTA_STREAMER_SOURCE(
+        "Hudi Streamer Source Configs",
+        "Configurations controlling the behavior of reading source data."),
     NONE(
         "None",
         "No subgroup. This description should be hidden.");
@@ -80,9 +95,12 @@ public class ConfigGroups {
   public static String getDescription(Names names) {
     String description;
     switch (names) {
+      case TABLE_CONFIG:
+        description = "Basic Hudi Table configuration parameters.";
+        break;
       case ENVIRONMENT_CONFIG:
         description = "Hudi supports passing configurations via a configuration file "
-            + "`hudi-default.conf` in which each line consists of a key and a value "
+            + "`hudi-defaults.conf` in which each line consists of a key and a value "
             + "separated by whitespace or = sign. For example:\n"
             + "```\n"
             + "hoodie.datasource.hive_sync.mode               jdbc\n"
@@ -93,9 +111,12 @@ public class ConfigGroups {
             + "job configurations/tunings, so all the jobs on your cluster can utilize it. "
             + "It also works with Spark SQL DML/DDL, and helps avoid having to pass configs "
             + "inside the SQL statements.\n\n"
-            + "By default, Hudi would load the configuration file under `/etc/hudi/conf` "
-            + "directory. You can specify a different configuration directory location by "
-            + "setting the `HUDI_CONF_DIR` environment variable.";
+            + "Hudi always loads the configuration file under default directory "
+            + "`file:/etc/hudi/conf`, if exists, to set the default configs. "
+            + "Besides, you can specify another configuration "
+            + "directory location by setting the `HUDI_CONF_DIR` environment variable. "
+            + "The configs stored in `HUDI_CONF_DIR/hudi-defaults.conf` are loaded, "
+            + "overriding any configs already set by the config file in the default directory.";
         break;
       case SPARK_DATASOURCE:
         description = "These configs control the Hudi Spark Datasource, "
@@ -127,7 +148,7 @@ public class ConfigGroups {
             + "on both datasource and WriteClient levels.";
         break;
       case METRICS:
-        description = "These set of configs are used to enable monitoring and reporting of key"
+        description = "These set of configs are used to enable monitoring and reporting of key "
             + "Hudi stats and metrics.";
         break;
       case KAFKA_CONNECT:
@@ -136,6 +157,10 @@ public class ConfigGroups {
       case AWS:
         description = "Configurations specific to Amazon Web Services.";
         break;
+      case HUDI_STREAMER:
+        description = "These set of configs are used for Hudi Streamer utility which provides "
+            + "the way to ingest from different sources such as DFS or Kafka.";
+        break;
       default:
         description = "Please fill in the description for Config Group Name: " + names.name;
         break;
