@@ -160,6 +160,14 @@ public class HoodieStorageConfig extends HoodieConfig {
       .withDocumentation("Control whether to write bloom filter or not. Default true. "
           + "We can set to false in non bloom index cases for CPU resource saving.");
 
+  public static final ConfigProperty<Boolean> WRITE_UTC_TIMEZONE = ConfigProperty
+      .key("hoodie.parquet.write.utc-timezone.enabled")
+      .defaultValue(true)
+      .markAdvanced()
+      .sinceVersion("1.1.0")
+      .withDocumentation("Use UTC timezone or local timezone to the conversion between epoch"
+              + " time and LocalDateTime. Default value is utc timezone for forward compatibility.");
+
   public static final ConfigProperty<String> HFILE_COMPRESSION_ALGORITHM_NAME = ConfigProperty
       .key("hoodie.hfile.compression.algorithm")
       .defaultValue("GZ")
@@ -178,7 +186,8 @@ public class HoodieStorageConfig extends HoodieConfig {
       .defaultValue(String.valueOf(0.35))
       .markAdvanced()
       .withDocumentation("Expected additional compression as records move from log files to parquet. Used for merge_on_read "
-          + "table to send inserts into log files & control the size of compacted parquet file.");
+          + "table to send inserts into log files & control the size of compacted parquet file."
+          + "When encoding log blocks in parquet format, increase this value for a more accurate estimation");
 
   // Configs that control the bloom filter that is written to the file footer
   public static final ConfigProperty<String> BLOOM_FILTER_TYPE = ConfigProperty
@@ -237,6 +246,14 @@ public class HoodieStorageConfig extends HoodieConfig {
           + "and it is loaded at runtime. This is only required when trying to "
           + "override the existing write context when `hoodie.datasource.write.row.writer.enable=true`.");
 
+  public static final ConfigProperty<String> HOODIE_PARQUET_FLINK_ROW_DATA_WRITE_SUPPORT_CLASS = ConfigProperty
+      .key("hoodie.parquet.flink.rowdata.write.support.class")
+      .defaultValue("org.apache.hudi.io.storage.row.HoodieRowDataParquetWriteSupport")
+      .markAdvanced()
+      .sinceVersion("1.1.0")
+      .withDocumentation("Provided write support class should extend HoodieRowDataParquetWriteSupport class "
+          + "and it is loaded at runtime. This is only required when trying to override the existing write support.");
+
   public static final ConfigProperty<String> HOODIE_STORAGE_CLASS = ConfigProperty
       .key("hoodie.storage.class")
       .defaultValue("org.apache.hudi.storage.hadoop.HoodieHadoopStorage")
@@ -368,6 +385,10 @@ public class HoodieStorageConfig extends HoodieConfig {
     super();
   }
 
+  public String getBloomFilterType() {
+    return getString(BLOOM_FILTER_TYPE);
+  }
+
   public static HoodieStorageConfig.Builder newBuilder() {
     return new Builder();
   }
@@ -493,6 +514,60 @@ public class HoodieStorageConfig extends HoodieConfig {
       return this;
     }
 
+    public Builder withRowDataWriteSupport(String rowDataWriteSupportClassName) {
+      storageConfig.setValue(HOODIE_PARQUET_FLINK_ROW_DATA_WRITE_SUPPORT_CLASS, rowDataWriteSupportClassName);
+      return this;
+    }
+
+    public Builder withWriteUtcTimezone(boolean writeUtcTimezone) {
+      storageConfig.setValue(WRITE_UTC_TIMEZONE, String.valueOf(writeUtcTimezone));
+      return this;
+    }
+
+    /**
+     * Sets the bloom filter type for the configuration.
+     *
+     * @param bloomFilterType The bloom filter type (SIMPLE or DYNAMIC_V0)
+     * @return this builder instance for method chaining
+     */
+    public Builder withBloomFilterType(String bloomFilterType) {
+      storageConfig.setValue(BLOOM_FILTER_TYPE, bloomFilterType);
+      return this;
+    }
+
+    /**
+     * Sets the number of entries to be stored in the bloom filter.
+     *
+     * @param numEntries The number of entries for the bloom filter
+     * @return this builder instance for method chaining
+     */
+    public Builder withBloomFilterNumEntries(int numEntries) {
+      storageConfig.setValue(BLOOM_FILTER_NUM_ENTRIES_VALUE, String.valueOf(numEntries));
+      return this;
+    }
+
+    /**
+     * Sets the false positive probability (FPP) for the bloom filter.
+     *
+     * @param fpp The false positive probability as a double
+     * @return this builder instance for method chaining
+     */
+    public Builder withBloomFilterFpp(double fpp) {
+      storageConfig.setValue(BLOOM_FILTER_FPP_VALUE, String.valueOf(fpp));
+      return this;
+    }
+
+    /**
+     * Sets the maximum number of entries for dynamic bloom filter.
+     *
+     * @param maxEntries The maximum number of entries for dynamic bloom filter
+     * @return this builder instance for method chaining
+     */
+    public Builder withBloomFilterDynamicMaxEntries(int maxEntries) {
+      storageConfig.setValue(BLOOM_FILTER_DYNAMIC_MAX_ENTRIES, String.valueOf(maxEntries));
+      return this;
+    }
+
     public HoodieStorageConfig build() {
       storageConfig.setDefaults(HoodieStorageConfig.class.getName());
       return storageConfig;
