@@ -22,6 +22,7 @@ package org.apache.hudi.common.data;
 import org.apache.hudi.common.function.SerializableBiFunction;
 import org.apache.hudi.common.function.SerializableFunction;
 import org.apache.hudi.common.function.SerializablePairFunction;
+import org.apache.hudi.common.function.SerializablePairPredicate;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
 
@@ -55,6 +56,13 @@ public interface HoodiePairData<K, V> extends Serializable {
    */
   void unpersist();
 
+  /**
+   * Un-persists this data and all its upstream dependencies recursively.
+   * This method traverses the lineage graph and unpersists any cached RDDs
+   * that this data depends on.
+   */
+  void unpersistWithDependencies();
+
   /**
    * Returns a {@link HoodieData} holding the key from every corresponding pair
    */
@@ -134,6 +142,26 @@ public interface HoodiePairData<K, V> extends Serializable {
    */
   HoodiePairData<K, V> union(HoodiePairData<K, V> other);
 
+  /**
+   * Filters key-value pairs of this {@link HoodiePairData} container based on provided predicate
+   *
+   * @param filter predicate function that takes a key and value as input and returns true if the pair should be kept
+   * @return filtered {@link HoodiePairData} containing only pairs that satisfy the predicate
+   */
+  HoodiePairData<K, V> filter(SerializablePairPredicate<K, V> filter);
+
+  /**
+   * Performs an inner join of this dataset against {@code other}.
+   *
+   * For each element (k, v) in this, the resulting {@link HoodiePairData} will contain all
+   * pairs {@code (k, (v, Some(w)))} for every {@code w} in the {@code other},
+   *
+   * @param other the other {@link HoodiePairData}
+   * @param <W>   value type of the other {@link HoodiePairData}
+   * @return containing the result of the left outer join
+   */
+  <W> HoodiePairData<K, Pair<V, W>> join(HoodiePairData<K, W> other);
+
   /**
    * Collects results of the underlying collection into a {@link List<Pair<K, V>>}
    *
