@@ -22,6 +22,9 @@ import org.apache.hudi.common.util.Option;
 
 import java.util.function.Supplier;
 
+/**
+ * Supplier of task context using local Java engine.
+ */
 public final class LocalTaskContextSupplier extends TaskContextSupplier {
   @Override
   public Supplier<Integer> getPartitionIdSupplier() {
@@ -42,4 +45,14 @@ public final class LocalTaskContextSupplier extends TaskContextSupplier {
   public Option<String> getProperty(EngineProperty prop) {
     return Option.empty();
   }
+
+  @Override
+  public Supplier<Integer> getTaskAttemptNumberSupplier() {
+    return () -> -1;
+  }
+
+  @Override
+  public Supplier<Integer> getStageAttemptNumberSupplier() {
+    return () -> -1;
+  }
 }
