@@ -19,29 +19,38 @@
 package org.apache.hudi.common.model;
 
 import org.apache.hudi.avro.HoodieAvroUtils;
+import org.apache.hudi.common.util.Option;
 import org.apache.hudi.exception.HoodieException;
 
+import com.esotericsoftware.kryo.Kryo;
+import com.esotericsoftware.kryo.KryoSerializable;
+import com.esotericsoftware.kryo.io.Input;
+import com.esotericsoftware.kryo.io.Output;
 import org.apache.avro.Schema;
 import org.apache.avro.generic.GenericRecord;
+import org.apache.avro.generic.IndexedRecord;
 
+import java.io.IOException;
 import java.io.Serializable;
 import java.util.Properties;
 
 /**
  * Base class for all AVRO record based payloads, that can be ordered based on a field.
  */
-public abstract class BaseAvroPayload implements Serializable {
+public abstract class BaseAvroPayload implements Serializable, KryoSerializable {
   /**
    * Avro data extracted from the source converted to bytes.
    */
-  protected final byte[] recordBytes;
+  private byte[] recordBytes;
 
   /**
    * For purposes of preCombining.
    */
-  protected final Comparable orderingVal;
+  protected Comparable orderingVal;
 
-  protected final boolean isDeletedRecord;
+  protected boolean isDeletedRecord;
+
+  private transient GenericRecord record;
 
   /**
    * Instantiate {@link BaseAvroPayload}.
@@ -50,7 +59,8 @@ public abstract class BaseAvroPayload implements Serializable {
    * @param orderingVal {@link Comparable} to be used in pre combine.
    */
   public BaseAvroPayload(GenericRecord record, Comparable orderingVal) {
-    this.recordBytes = record != null ? HoodieAvroUtils.avroToBytes(record) : new byte[0];
+    this.record = record;
+    this.recordBytes = null; // only initialized when needed
     this.orderingVal = orderingVal;
     this.isDeletedRecord = record == null || isDeleteRecord(record);
 
@@ -96,6 +106,56 @@ public abstract class BaseAvroPayload implements Serializable {
   }
 
   public byte[] getRecordBytes() {
+    if (recordBytes == null) {
+      if (record == null) {
+        recordBytes = new byte[0];
+      } else {
+        recordBytes = HoodieAvroUtils.avroToBytes(record);
+      }
+    }
     return recordBytes;
   }
+
+  public Option<IndexedRecord> getIndexedRecord(Schema schema, Properties properties) throws IOException {
+    return getRecord(schema);
+  }
+
+  protected boolean isEmptyRecord() {
+    if (recordBytes == null) {
+      return record == null;
+    }
+    return recordBytes.length == 0;
+  }
+
+  protected Option<IndexedRecord> getRecord(Schema schema) throws IOException {
+    if (record != null) {
+      if (record.getSchema() == schema) {
+        return Option.of(record);
+      }
+      // if the schema does not match, we need to deserialize with the proper schema to match legacy behavior
+      recordBytes = getRecordBytes();
+    }
+    if (recordBytes == null || recordBytes.length == 0) {
+      return Option.empty();
+    }
+    record = SerializableIndexedRecord.fromAvroBytes(schema, recordBytes);
+    return Option.of(record);
+  }
+
+  @Override
+  public void write(Kryo kryo, Output output) {
+    byte[] bytes = getRecordBytes();
+    output.writeInt(bytes.length);
+    output.writeBytes(bytes);
+    kryo.writeClassAndObject(output, orderingVal);
+    output.writeBoolean(isDeletedRecord);
+  }
+
+  @Override
+  public void read(Kryo kryo, Input input) {
+    int length = input.readInt();
+    this.recordBytes = input.readBytes(length);
+    this.orderingVal = (Comparable) kryo.readClassAndObject(input);
+    this.isDeletedRecord = input.readBoolean();
+  }
 }
