@@ -18,6 +18,7 @@
 
 package org.apache.hudi.common.model;
 
+import org.apache.hudi.common.table.read.DeleteContext;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.keygen.BaseKeyGenerator;
@@ -58,7 +59,7 @@ public class HoodieEmptyRecord<T> extends HoodieRecord<T> {
   }
 
   @Override
-  public Comparable<?> doGetOrderingValue(Schema recordSchema, Properties props) {
+  public Comparable<?> doGetOrderingValue(Schema recordSchema, Properties props, String[] orderingFields) {
     return orderingVal;
   }
 
@@ -98,6 +99,11 @@ public class HoodieEmptyRecord<T> extends HoodieRecord<T> {
     throw new UnsupportedOperationException();
   }
 
+  @Override
+  public Object getColumnValueAsJava(Schema recordSchema, String column, Properties props) {
+    return null;
+  }
+
   @Override
   public HoodieRecord joinWith(HoodieRecord other, Schema targetSchema) {
     throw new UnsupportedOperationException();
@@ -119,7 +125,7 @@ public class HoodieEmptyRecord<T> extends HoodieRecord<T> {
   }
 
   @Override
-  public boolean isDelete(Schema recordSchema, Properties props) throws IOException {
+  protected boolean checkIsDelete(DeleteContext deleteContext, Properties props) {
     return true;
   }
 
@@ -151,7 +157,7 @@ public class HoodieEmptyRecord<T> extends HoodieRecord<T> {
   }
 
   @Override
-  public ByteArrayOutputStream getAvroBytes(Schema recordSchema, Properties props) throws IOException {
+  public ByteArrayOutputStream getAvroBytes(Schema recordSchema, Properties props) {
     return new ByteArrayOutputStream(0);
   }
 
@@ -183,4 +189,11 @@ public class HoodieEmptyRecord<T> extends HoodieRecord<T> {
     // NOTE: [[EmptyRecord]]'s payload is always null
     return null;
   }
+
+  @Override
+  public Object convertColumnValueForLogicalType(Schema fieldSchema,
+                                                 Object fieldValue,
+                                                 boolean keepConsistentLogicalTimestamp) {
+    return null;
+  }
 }
