@@ -109,6 +109,13 @@ public enum HoodieOperation {
     return operation == UPDATE_BEFORE;
   }
 
+  /**
+   * Returns whether the operation is UPDATE_BEFORE.
+   */
+  public static boolean isUpdateBefore(String operation) {
+    return isUpdateBefore(fromName(operation));
+  }
+
   /**
    * Returns whether the operation is UPDATE_AFTER.
    */
