@@ -151,7 +151,7 @@ public class HoodiePartitionMetadata {
           storage.deleteFile(tmpPath);
         }
       } catch (IOException ioe) {
-        LOG.warn("Error trying to clean up temporary files for " + partitionPath, ioe);
+        LOG.info("Error trying to clean up temporary files for {}", partitionPath, ioe);
       }
     }
   }
@@ -181,7 +181,7 @@ public class HoodiePartitionMetadata {
       format = Option.empty();
       return true;
     } catch (Throwable t) {
-      LOG.debug("Unable to read partition meta properties file for partition " + partitionPath);
+      LOG.debug("Unable to read partition meta properties file for partition {}", partitionPath);
       return false;
     }
   }
@@ -199,7 +199,7 @@ public class HoodiePartitionMetadata {
         format = Option.of(reader.getFormat());
         return true;
       } catch (Throwable t) {
-        LOG.debug("Unable to read partition metadata " + metafilePath.getName() + " for partition " + partitionPath);
+        LOG.debug("Unable to read partition metadata {} for partition {}", metafilePath.getName(), partitionPath);
       }
     }
     return false;
@@ -215,7 +215,7 @@ public class HoodiePartitionMetadata {
       }
       return Option.of(props.getProperty(COMMIT_TIME_KEY));
     } catch (IOException ioe) {
-      LOG.warn("Error fetch Hoodie partition metadata for " + partitionPath, ioe);
+      LOG.warn("Error fetch Hoodie partition metadata for {}", partitionPath, ioe);
       return Option.empty();
     }
   }
