@@ -18,9 +18,11 @@
 
 package org.apache.hudi.common.model;
 
+import org.apache.hudi.common.table.read.DeleteContext;
 import org.apache.hudi.common.util.CollectionUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
+import org.apache.hudi.exception.HoodieIOException;
 import org.apache.hudi.keygen.BaseKeyGenerator;
 
 import com.esotericsoftware.kryo.Kryo;
@@ -158,7 +160,9 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
    */
   protected Option<Map<String, String>> metaData;
 
-  private transient Comparable<?> orderingValue;
+  protected transient Comparable<?> orderingValue;
+
+  protected Boolean isDelete;
 
   public HoodieRecord(HoodieKey key, T data) {
     this(key, data, null, Option.empty());
@@ -175,6 +179,18 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
     this.metaData = metaData;
   }
 
+  public HoodieRecord(HoodieKey key, T data, HoodieOperation operation, Boolean isDelete, Option<Map<String, String>> metaData) {
+    this.key = key;
+    this.data = data;
+    this.currentLocation = null;
+    this.newLocation = null;
+    this.sealed = false;
+    this.ignoreIndexUpdate = false;
+    this.operation = operation;
+    this.isDelete = isDelete;
+    this.metaData = metaData;
+  }
+
   public HoodieRecord(
       HoodieKey key,
       T data,
@@ -213,14 +229,34 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
     return operation;
   }
 
-  public Comparable<?> getOrderingValue(Schema recordSchema, Properties props) {
+  /**
+   * Get ordering value for the record from the cached variable, or extracting from the record if not cached.
+   *
+   * @param recordSchema Avro schema for the record
+   * @param props Properties containing the necessary configurations
+   * @param orderingFields name of the ordering fields
+   * @return The ordering value for the record
+   */
+  public Comparable<?> getOrderingValue(Schema recordSchema, Properties props, String[] orderingFields) {
     if (orderingValue == null) {
-      orderingValue = doGetOrderingValue(recordSchema, props);
+      orderingValue = doGetOrderingValue(recordSchema, props, orderingFields);
     }
     return orderingValue;
   }
 
-  protected abstract Comparable<?> doGetOrderingValue(Schema recordSchema, Properties props);
+  /**
+   * Extracting the ordering value from the record.
+   *
+   * @param recordSchema Avro schema for the record
+   * @param props Properties containing the necessary configurations
+   * @param orderingFields name of the ordering fields
+   * @return The ordering value for the record
+   */
+  protected abstract Comparable<?> doGetOrderingValue(Schema recordSchema, Properties props, String[] orderingFields);
+
+  public Comparable<?> getOrderingValueAsJava(Schema recordSchema, Properties props, String[] orderingFields) {
+    return getOrderingValue(recordSchema, props, orderingFields);
+  }
 
   public T getData() {
     if (data == null) {
@@ -282,6 +318,9 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
    */
   public void setIgnoreIndexUpdate(boolean ignoreFlag) {
     this.ignoreIndexUpdate = ignoreFlag;
+    if (ignoreFlag) {
+      this.operation = HoodieOperation.UPDATE_BEFORE;
+    }
   }
 
   public boolean getIgnoreIndexUpdate() {
@@ -375,6 +414,9 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
     //       implementation
     writeRecordPayload(data, kryo, output);
     kryo.writeObjectOrNull(output, ignoreIndexUpdate, Boolean.class);
+    kryo.writeObjectOrNull(output, isDelete, Boolean.class);
+    kryo.writeClassAndObject(output, orderingValue);
+    kryo.writeClassAndObject(output, metaData == null ? null : metaData.orElse(null));
   }
 
   /**
@@ -391,17 +433,40 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
     //       implementation
     this.data = readRecordPayload(kryo, input);
     this.ignoreIndexUpdate = kryo.readObjectOrNull(input, Boolean.class);
+    this.isDelete = kryo.readObjectOrNull(input, Boolean.class);
+    this.orderingValue = (Comparable<?>) kryo.readClassAndObject(input);
+    this.metaData = Option.ofNullable((Map<String, String>) kryo.readClassAndObject(input));
 
     // NOTE: We're always seal object after deserialization
     this.sealed = true;
   }
 
+  /**
+   * This method converts a value for a column with certain Avro Logical data types that require special handling.
+   * <p>
+   * E.g., Logical Date Type is converted to actual Date value instead of Epoch Integer which is how it is
+   * represented/stored in parquet.
+   * <p>
+   * E.g., Decimal Data Type is converted to actual decimal value instead of bytes/fixed which is how it is
+   * represented/stored in parquet.
+   */
+  public abstract Object convertColumnValueForLogicalType(
+      Schema fieldSchema, Object fieldValue, boolean keepConsistentLogicalTimestamp);
+
   /**
    * Get column in record to support RDDCustomColumnsSortPartitioner
    * @return
    */
   public abstract Object[] getColumnValues(Schema recordSchema, String[] columns, boolean consistentLogicalTimestampEnabled);
 
+  /**
+   * Get column as Java type in record to collect index stats: col_stats, secondary index, etc.
+   * The Java type is required to keep the value engine agnostic.
+   *
+   * @return the column value
+   */
+  public abstract Object getColumnValueAsJava(Schema recordSchema, String column, Properties props);
+
   /**
    * Support bootstrap.
    */
@@ -414,6 +479,18 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
    */
   public abstract HoodieRecord prependMetaFields(Schema recordSchema, Schema targetSchema, MetadataValues metadataValues, Properties props);
 
+  /**
+   * Update a specific metadata field with given value.
+   *
+   * @param recordSchema the schema for the record
+   * @param ordinal the ordinal for the target medata field
+   * @param value the new value for the target metadata field
+   * @return the new HoodieRecord with updated metadata value
+   */
+  public HoodieRecord updateMetaField(Schema recordSchema, int ordinal, String value) {
+    throw new UnsupportedOperationException("updateMetaField is not supported yet for: " + this.getClass().getSimpleName());
+  }
+
   /**
    * Support schema evolution.
    */
@@ -423,7 +500,22 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
     return rewriteRecordWithNewSchema(recordSchema, props, newSchema, Collections.emptyMap());
   }
 
-  public abstract boolean isDelete(Schema recordSchema, Properties props) throws IOException;
+  public boolean isDelete(DeleteContext deleteContext, Properties props) {
+    if (isDelete == null) {
+      try {
+        isDelete = checkIsDelete(deleteContext, props);
+      } catch (IOException e) {
+        throw new HoodieIOException("Failed to check DELETE record.", e);
+      }
+    }
+    return isDelete;
+  }
+
+  /**
+   * Check whether the record is DELETE by the data content.
+   * todo: all subclass
+   */
+  protected abstract boolean checkIsDelete(DeleteContext deleteContext, Properties props) throws IOException;
 
   /**
    * Is EmptyRecord. Generated by ExpressionPayload.
@@ -490,6 +582,6 @@ public abstract class HoodieRecord<T> implements HoodieRecordCompatibilityInterf
   }
 
   public enum HoodieRecordType {
-    AVRO, SPARK, HIVE
+    AVRO, SPARK, HIVE, FLINK
   }
 }
