@@ -22,11 +22,12 @@ import org.apache.hudi.ApiMaturityLevel;
 import org.apache.hudi.PublicAPIClass;
 import org.apache.hudi.common.config.RecordMergeMode;
 import org.apache.hudi.common.config.TypedProperties;
+import org.apache.hudi.common.engine.RecordContext;
 import org.apache.hudi.common.model.HoodieRecord.HoodieRecordType;
 import org.apache.hudi.common.table.HoodieTableConfig;
+import org.apache.hudi.common.table.HoodieTableVersion;
+import org.apache.hudi.common.table.read.BufferedRecord;
 import org.apache.hudi.common.util.Option;
-import org.apache.hudi.common.util.StringUtils;
-import org.apache.hudi.common.util.collection.Pair;
 
 import org.apache.avro.Schema;
 
@@ -52,6 +53,9 @@ public interface HoodieRecordMerger extends Serializable {
   // Always chooses the most recently written record
   String COMMIT_TIME_BASED_MERGE_STRATEGY_UUID = "ce9acb64-bde0-424c-9b91-f6ebba25356d";
 
+  // Uses customized merge strategy to merge records
+  String CUSTOM_MERGE_STRATEGY_UUID = "1897ef5f-18bc-4557-939c-9d6a8afd1519";
+
   // Use avro payload to merge records
   String PAYLOAD_BASED_MERGE_STRATEGY_UUID = "00000000-0000-0000-0000-000000000000";
 
@@ -60,8 +64,14 @@ public interface HoodieRecordMerger extends Serializable {
    * It'd be associative operation: f(a, f(b, c)) = f(f(a, b), c) (which we can translate as having 3 versions A, B, C
    * of the single record, both orders of operations applications have to yield the same result)
    * This method takes only full records for merging.
+   *
+   * @param older         Older record in terms of commit time ordering.
+   * @param newer         Newer record in terms of commit time ordering.
+   * @param recordContext The record context for accessing and manipulating the records.
+   * @param props         Additional properties for configuring the merging behavior.
+   * @return The merged record and schema. The record is expected to be non-null. If the record represents a deletion, the operation must be set as {@link HoodieOperation#DELETE}.
    */
-  Option<Pair<HoodieRecord, Schema>> merge(HoodieRecord older, Schema oldSchema, HoodieRecord newer, Schema newSchema, TypedProperties props) throws IOException;
+  <T> BufferedRecord merge(BufferedRecord<T> older, BufferedRecord<T> newer, RecordContext<T> recordContext, TypedProperties props) throws IOException;
 
   /**
    * Merges records which can contain partial updates, i.e., only subset of fields and values are
@@ -108,53 +118,31 @@ public interface HoodieRecordMerger extends Serializable {
    * ts | price | tags
    * 16 |  2.8  | fruit,juicy
    *
-   * @param older        Older record.
-   * @param oldSchema    Schema of the older record.
-   * @param newer        Newer record.
-   * @param newSchema    Schema of the newer record.
-   * @param readerSchema Reader schema containing all the fields to read. This is used to maintain
-   *                     the ordering of the fields of the merged record.
-   * @param props        Configuration in {@link TypedProperties}.
-   * @return The merged record and schema.
+   * @param older         Older record.
+   * @param newer         Newer record.
+   * @param readerSchema  Reader schema containing all the fields to read. This is used to maintain
+   *                      the ordering of the fields of the merged record.
+   * @param recordContext the record context for accessing and manipulating the records.
+   * @param props         Configuration in {@link TypedProperties}.
+   * @return The merged record and schema. The record is expected to be non-null. If the record represents a deletion, the operation must be set as {@link HoodieOperation#DELETE}.
    * @throws IOException upon merging error.
    */
-  default Option<Pair<HoodieRecord, Schema>> partialMerge(HoodieRecord older, Schema oldSchema, HoodieRecord newer, Schema newSchema, Schema readerSchema, TypedProperties props) throws IOException {
-    throw new UnsupportedOperationException("Partial merging logic is not implemented.");
-  }
-
-  /**
-   * In some cases a business logic does some checks before flushing a merged record to the disk.
-   * This method does the check, and when false is returned, it means the merged record should not
-   * be flushed.
-   *
-   * @param record the merged record.
-   * @param schema the schema of the merged record.
-   * @return a boolean variable to indicate if the merged record should be returned or not.
-   *
-   * <p> This interface is experimental and might be evolved in the future.
-   **/
-  default boolean shouldFlush(HoodieRecord record, Schema schema, TypedProperties props) throws IOException {
-    return true;
-  }
-
-  /**
-   * Merges two records with the same key in full outer merge fashion i.e. all fields from both records are included.
-   */
-  default List<Pair<HoodieRecord, Schema>> fullOuterMerge(HoodieRecord older, Schema oldSchema, HoodieRecord newer, Schema newSchema, TypedProperties props) throws IOException {
-    throw new UnsupportedOperationException("Partial merging logic is not implemented.");
+  default <T> BufferedRecord<T> partialMerge(BufferedRecord<T> older, BufferedRecord<T> newer, Schema readerSchema, RecordContext<T> recordContext, TypedProperties props) throws IOException {
+    throw new UnsupportedOperationException("Partial merging logic is not implemented by " + this.getClass().getName());
   }
 
   /**
    * If false, whenever we have log files, we will need to read all columns
    * If true, mor merging can be done without all columns. The columns required can be configured
    * by overriding getMandatoryFieldsForMerging
+   * EventTime based merging and CommitTime based merging are projection compatible
    */
   default boolean isProjectionCompatible() {
     return false;
   }
 
   /**
-   * Returns a list of fields required for mor merging. The default implementation will return the recordkey field and the precombine
+   * Returns a list of fields required for mor merging. The default implementation will return the recordKey field and the ordering fields.
    */
   default String[] getMandatoryFieldsForMerging(Schema dataSchema, HoodieTableConfig cfg, TypedProperties properties) {
     ArrayList<String> requiredFields = new ArrayList<>();
@@ -168,10 +156,8 @@ public interface HoodieRecordMerger extends Serializable {
       }
     }
 
-    String preCombine = cfg.getPreCombineField();
-    if (!StringUtils.isNullOrEmpty(preCombine)) {
-      requiredFields.add(preCombine);
-    }
+    List<String> orderingFields = cfg.getOrderingFields();
+    requiredFields.addAll(orderingFields);
     return requiredFields.toArray(new String[0]);
   }
 
@@ -186,9 +172,8 @@ public interface HoodieRecordMerger extends Serializable {
    */
   String getMergingStrategy();
 
-  static String getRecordMergeStrategyId(RecordMergeMode mergeMode,
-                                         String payloadClassName,
-                                         String recordMergeStrategyId) {
+  static String getRecordMergeStrategyId(RecordMergeMode mergeMode, String payloadClassName,
+                                         String recordMergeStrategyId, HoodieTableVersion tableVersion) {
     switch (mergeMode) {
       case COMMIT_TIME_ORDERING:
         return COMMIT_TIME_BASED_MERGE_STRATEGY_UUID;
@@ -196,13 +181,44 @@ public interface HoodieRecordMerger extends Serializable {
         return EVENT_TIME_BASED_MERGE_STRATEGY_UUID;
       case CUSTOM:
       default:
-        if (nonEmpty(recordMergeStrategyId)) {
-          return recordMergeStrategyId;
-        }
-        if (nonEmpty(payloadClassName)) {
-          return PAYLOAD_BASED_MERGE_STRATEGY_UUID;
-        }
-        return null;
+        return getCustomRecordMergeStrategyId(payloadClassName, recordMergeStrategyId, tableVersion);
     }
   }
+
+  static String getCustomRecordMergeStrategyId(String payloadClassName, String recordMergeStrategyId, HoodieTableVersion tableVersion) {
+    if (tableVersion.greaterThanOrEquals(HoodieTableVersion.EIGHT)) {
+      // For table version 8, we give preference to input recordMergeStrategyId over payload based strategy
+      if (nonEmpty(recordMergeStrategyId)) {
+        return recordMergeStrategyId;
+      } else if (nonEmpty(payloadClassName)) {
+        return PAYLOAD_BASED_MERGE_STRATEGY_UUID;
+      }
+      return null;
+    } else {
+      // For table version 6, we give preference to payload based strategy over input recordMergeStrategyId
+      if (nonEmpty(payloadClassName)) {
+        return PAYLOAD_BASED_MERGE_STRATEGY_UUID;
+      } else if (nonEmpty(recordMergeStrategyId)) {
+        return recordMergeStrategyId;
+      }
+      return null;
+    }
+  }
+
+  /**
+   * If the new record is a commit time ordered delete, it will always be used regardless of the ordering value of the old record.
+   * If the old record was a commit time ordered delete, the newer record will be returned because it occurred after that delete and ordering time comparison is not needed.
+   */
+  static <T> boolean isCommitTimeOrderingDelete(BufferedRecord<T> oldRecord, BufferedRecord<T> newRecord) {
+    return newRecord.isCommitTimeOrderingDelete() || oldRecord.isCommitTimeOrderingDelete();
+  }
+
+  /**
+   * Returns the max ordering value with the given two records.
+   */
+  static <T> Comparable maxOrderingValue(BufferedRecord<T> oldRecord, BufferedRecord<T> newRecord) {
+    Comparable oldOrderingVal = oldRecord.getOrderingValue();
+    Comparable newOrderingVal = newRecord.getOrderingValue();
+    return oldOrderingVal.compareTo(newOrderingVal) > 0 ? oldOrderingVal : newOrderingVal;
+  }
 }
