@@ -18,6 +18,8 @@
 
 package org.apache.hudi.common.table;
 
+import org.apache.hudi.common.HoodieTableFormat;
+import org.apache.hudi.common.NativeTableFormat;
 import org.apache.hudi.common.bootstrap.index.hfile.HFileBootstrapIndex;
 import org.apache.hudi.common.config.ConfigClassProperty;
 import org.apache.hudi.common.config.ConfigGroups;
@@ -26,15 +28,22 @@ import org.apache.hudi.common.config.HoodieConfig;
 import org.apache.hudi.common.config.OrderedProperties;
 import org.apache.hudi.common.config.RecordMergeMode;
 import org.apache.hudi.common.config.TypedProperties;
+import org.apache.hudi.common.model.AWSDmsAvroPayload;
 import org.apache.hudi.common.model.BootstrapIndexType;
 import org.apache.hudi.common.model.DefaultHoodieRecordPayload;
+import org.apache.hudi.common.model.EventTimeAvroPayload;
 import org.apache.hudi.common.model.HoodieFileFormat;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.model.HoodieRecordMerger;
 import org.apache.hudi.common.model.HoodieRecordPayload;
 import org.apache.hudi.common.model.HoodieTableType;
 import org.apache.hudi.common.model.HoodieTimelineTimeZone;
+import org.apache.hudi.common.model.OverwriteNonDefaultsWithLatestAvroPayload;
 import org.apache.hudi.common.model.OverwriteWithLatestAvroPayload;
+import org.apache.hudi.common.model.PartialUpdateAvroPayload;
+import org.apache.hudi.common.model.debezium.DebeziumConstants;
+import org.apache.hudi.common.model.debezium.MySqlDebeziumAvroPayload;
+import org.apache.hudi.common.model.debezium.PostgresDebeziumAvroPayload;
 import org.apache.hudi.common.table.cdc.HoodieCDCSupplementalLoggingMode;
 import org.apache.hudi.common.table.timeline.HoodieInstantTimeGenerator;
 import org.apache.hudi.common.table.timeline.versioning.TimelineLayoutVersion;
@@ -67,10 +76,13 @@ import java.lang.reflect.Field;
 import java.lang.reflect.Modifier;
 import java.time.Instant;
 import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
+import java.util.ServiceLoader;
 import java.util.Set;
 import java.util.function.BiConsumer;
 import java.util.function.Function;
@@ -89,11 +101,17 @@ import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAM
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_OUTPUT_TIMEZONE_FORMAT;
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_TIMEZONE_FORMAT;
 import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_TYPE_FIELD;
+import static org.apache.hudi.common.model.AWSDmsAvroPayload.DELETE_OPERATION_VALUE;
+import static org.apache.hudi.common.model.AWSDmsAvroPayload.OP_FIELD;
+import static org.apache.hudi.common.model.DefaultHoodieRecordPayload.DELETE_KEY;
+import static org.apache.hudi.common.model.DefaultHoodieRecordPayload.DELETE_MARKER;
 import static org.apache.hudi.common.model.HoodieRecordMerger.COMMIT_TIME_BASED_MERGE_STRATEGY_UUID;
+import static org.apache.hudi.common.model.HoodieRecordMerger.CUSTOM_MERGE_STRATEGY_UUID;
 import static org.apache.hudi.common.model.HoodieRecordMerger.EVENT_TIME_BASED_MERGE_STRATEGY_UUID;
 import static org.apache.hudi.common.model.HoodieRecordMerger.PAYLOAD_BASED_MERGE_STRATEGY_UUID;
 import static org.apache.hudi.common.util.ConfigUtils.fetchConfigs;
 import static org.apache.hudi.common.util.ConfigUtils.recoverIfNeeded;
+import static org.apache.hudi.common.util.StringUtils.EMPTY_STRING;
 import static org.apache.hudi.common.util.StringUtils.getUTF8Bytes;
 import static org.apache.hudi.common.util.StringUtils.isNullOrEmpty;
 import static org.apache.hudi.common.util.StringUtils.nonEmpty;
@@ -113,6 +131,19 @@ public class HoodieTableConfig extends HoodieConfig {
   public static final String HOODIE_PROPERTIES_FILE_BACKUP = "hoodie.properties.backup";
   public static final String HOODIE_WRITE_TABLE_NAME_KEY = "hoodie.datasource.write.table.name";
   public static final String HOODIE_TABLE_NAME_KEY = "hoodie.table.name";
+  public static final String PARTIAL_UPDATE_UNAVAILABLE_VALUE = "hoodie.write.partial.update.unavailable.value";
+  public static final String DEBEZIUM_UNAVAILABLE_VALUE = "__debezium_unavailable_value";
+  // This prefix is used to set merging related properties.
+  // A reader might need to read some writer properties to function as expected,
+  // and Hudi stores properties with this prefix so the reader parses these properties to fetch any custom property.
+  public static final String RECORD_MERGE_PROPERTY_PREFIX = "hoodie.record.merge.property.";
+
+  public static final Set<String> BUILTIN_MERGE_STRATEGIES = Collections.unmodifiableSet(
+      new HashSet<>(Arrays.asList(
+          COMMIT_TIME_BASED_MERGE_STRATEGY_UUID,
+          CUSTOM_MERGE_STRATEGY_UUID,
+          EVENT_TIME_BASED_MERGE_STRATEGY_UUID,
+          PAYLOAD_BASED_MERGE_STRATEGY_UUID)));
 
   public static final ConfigProperty<String> DATABASE_NAME = ConfigProperty
       .key("hoodie.database.name")
@@ -144,12 +175,24 @@ public class HoodieTableConfig extends HoodieConfig {
           + " to identify what upgrade/downgrade paths happened on the table. This is only configured "
           + "when the table is initially setup.");
 
-  // TODO: is this this called precombine in 1.0. ..
+  /**
+   * @deprecated Use {@link #ORDERING_FIELDS} instead
+   */
+  @Deprecated
   public static final ConfigProperty<String> PRECOMBINE_FIELD = ConfigProperty
       .key("hoodie.table.precombine.field")
       .noDefaultValue()
-      .withDocumentation("Field used in preCombining before actual write. By default, when two records have the same key value, "
-          + "the largest value for the precombine field determined by Object.compareTo(..), is picked.");
+      .withDocumentation("Comma separated fields used in preCombining before actual write. By default, when two records have the same key value, "
+          + "the largest value for the precombine field determined by Object.compareTo(..), is picked. If there are multiple fields configured, "
+          + "comparison is made on the first field. If the first field values are same, comparison is made on the second field and so on.");
+
+  public static final ConfigProperty<String> ORDERING_FIELDS = ConfigProperty
+      .key("hoodie.table.ordering.fields")
+      .noDefaultValue()
+      .withAlternatives(PRECOMBINE_FIELD.key())
+      .withDocumentation("Comma separated fields used in records merging comparison. By default, when two records have the same key value, "
+          + "the largest value for the ordering field determined by Object.compareTo(..), is picked. If there are multiple fields configured, "
+          + "comparison is made on the first field. If the first field values are same, comparison is made on the second field and so on.");
 
   public static final ConfigProperty<String> PARTITION_FIELDS = ConfigProperty
       .key("hoodie.table.partition.fields")
@@ -197,11 +240,16 @@ public class HoodieTableConfig extends HoodieConfig {
       .key("hoodie.timeline.layout.version")
       .noDefaultValue()
       .withDocumentation("Version of timeline used, by the table.");
-  
+
+  public static final ConfigProperty<String> TABLE_FORMAT = ConfigProperty
+      .key("hoodie.table.format")
+      .defaultValue(NativeTableFormat.TABLE_FORMAT)
+      .withDocumentation("Table format name used when writing to the table.");
+
   public static final ConfigProperty<RecordMergeMode> RECORD_MERGE_MODE = ConfigProperty
       .key("hoodie.record.merge.mode")
       .defaultValue((RecordMergeMode) null,
-          "COMMIT_TIME_ORDERING if precombine is not set; EVENT_TIME_ORDERING if precombine is set")
+          "COMMIT_TIME_ORDERING if ordering field is not set; EVENT_TIME_ORDERING if ordering field is set")
       .sinceVersion("1.0.0")
       .withDocumentation(RecordMergeMode.class);
 
@@ -212,8 +260,16 @@ public class HoodieTableConfig extends HoodieConfig {
       .withDocumentation("Payload class to use for performing merges, compactions, i.e merge delta logs with current base file and then "
           + " produce a new base file.");
 
+  public static final ConfigProperty<String> LEGACY_PAYLOAD_CLASS_NAME = ConfigProperty
+      .key("hoodie.table.legacy.payload.class")
+      .noDefaultValue()
+      .sinceVersion("1.1.0")
+      .withDocumentation("Payload class to indicate the payload class that is used to create the table and is not used anymore.");
+
   // This is the default payload class used by Hudi 0.x releases (table version 6 and below)
-  public static final String DEFAULT_PAYLOAD_CLASS_NAME = DefaultHoodieRecordPayload.class.getName();
+  public static final String getDefaultPayloadClassName() {
+    return DefaultHoodieRecordPayload.class.getName();
+  }
 
   public static final ConfigProperty<String> RECORD_MERGE_STRATEGY_ID = ConfigProperty
       .key("hoodie.record.merge.strategy.id")
@@ -303,6 +359,12 @@ public class HoodieTableConfig extends HoodieConfig {
       .sinceVersion("1.0.0")
       .withDocumentation("When set to true, the table can support reading and writing multiple base file formats.");
 
+  public static final ConfigProperty<String> PARTIAL_UPDATE_MODE = ConfigProperty
+      .key("hoodie.table.partial.update.mode")
+      .noDefaultValue()
+      .sinceVersion("1.1.0")
+      .withDocumentation("This property when set, will define how two versions of the record will be merged together when records are partially formed");
+
   public static final ConfigProperty<String> URL_ENCODE_PARTITIONING = KeyGeneratorOptions.URL_ENCODE_PARTITIONING;
   public static final ConfigProperty<String> HIVE_STYLE_PARTITIONING_ENABLE = KeyGeneratorOptions.HIVE_STYLE_PARTITIONING_ENABLE;
 
@@ -400,61 +462,17 @@ public class HoodieTableConfig extends HoodieConfig {
     return new HoodieTableConfig(storage, metaPath);
   }
 
-  private HoodieTableConfig(HoodieStorage storage, StoragePath metaPath) {
-    this(storage, metaPath, null, null, null, false);
-  }
-
-  public HoodieTableConfig(HoodieStorage storage, StoragePath metaPath, RecordMergeMode recordMergeMode, String payloadClassName,
-                           String recordMergeStrategyId) {
-    this(storage, metaPath, recordMergeMode, payloadClassName, recordMergeStrategyId, true);
-  }
-
-  public HoodieTableConfig(HoodieStorage storage, StoragePath metaPath, RecordMergeMode recordMergeMode, String payloadClassName,
-                           String recordMergeStrategyId, boolean autoUpdate) {
+  public HoodieTableConfig(HoodieStorage storage, StoragePath metaPath) {
     super();
     StoragePath propertyPath = new StoragePath(metaPath, HOODIE_PROPERTIES_FILE);
     LOG.info("Loading table properties from " + propertyPath);
     try {
       this.props = fetchConfigs(storage, metaPath, HOODIE_PROPERTIES_FILE, HOODIE_PROPERTIES_FILE_BACKUP, MAX_READ_RETRIES, READ_RETRY_DELAY_MSEC);
-      if (autoUpdate) {
-        autoUpdateHoodieProperties(storage, metaPath, recordMergeMode, payloadClassName, recordMergeStrategyId);
-      }
     } catch (IOException e) {
       throw new HoodieIOException("Could not load properties from " + propertyPath, e);
     }
   }
 
-  private void autoUpdateHoodieProperties(HoodieStorage storage, StoragePath metaPath,
-                                          RecordMergeMode recordMergeMode, String payloadClassName,
-                                          String recordMergeStrategyId) {
-    StoragePath propertyPath = new StoragePath(metaPath, HOODIE_PROPERTIES_FILE);
-    try {
-      boolean needStore = false;
-      if (contains(PAYLOAD_CLASS_NAME) && payloadClassName != null
-          && !getString(PAYLOAD_CLASS_NAME).equals(payloadClassName)) {
-        setValue(PAYLOAD_CLASS_NAME, payloadClassName);
-        needStore = true;
-      }
-      if (contains(RECORD_MERGE_MODE) && recordMergeMode != null
-          && !recordMergeMode.equals(RecordMergeMode.getValue(getString(RECORD_MERGE_MODE)))) {
-        setValue(RECORD_MERGE_MODE, recordMergeMode.name());
-        needStore = true;
-      }
-      if (contains(RECORD_MERGE_STRATEGY_ID) && recordMergeStrategyId != null
-          && !getString(RECORD_MERGE_STRATEGY_ID).equals(recordMergeStrategyId)) {
-        setValue(RECORD_MERGE_STRATEGY_ID, recordMergeStrategyId);
-        needStore = true;
-      }
-      if (needStore) {
-        try (OutputStream outputStream = storage.create(propertyPath)) {
-          storeProperties(props, outputStream);
-        }
-      }
-    } catch (IOException e) {
-      throw new HoodieIOException("Could not store properties in " + propertyPath, e);
-    }
-  }
-
   private static Properties getOrderedPropertiesWithTableChecksum(Properties props) {
     Properties orderedProps = new OrderedProperties(props);
     orderedProps.put(TABLE_CHECKSUM.key(), String.valueOf(generateChecksum(props)));
@@ -466,10 +484,11 @@ public class HoodieTableConfig extends HoodieConfig {
    *
    * @param props        - properties to be written
    * @param outputStream - output stream to which properties will be written
+   * @param propertyPath - Path of the file where properties would be stored
    * @return return the table checksum
    * @throws IOException
    */
-  private static String storeProperties(Properties props, OutputStream outputStream) throws IOException {
+  private static String storeProperties(Properties props, OutputStream outputStream, StoragePath propertyPath) throws IOException {
     final String checksum;
     if (isValidChecksum(props)) {
       checksum = props.getProperty(TABLE_CHECKSUM.key());
@@ -480,6 +499,7 @@ public class HoodieTableConfig extends HoodieConfig {
       checksum = propsWithChecksum.getProperty(TABLE_CHECKSUM.key());
       props.setProperty(TABLE_CHECKSUM.key(), checksum);
     }
+    LOG.info("Created properties file at " + propertyPath);
     return checksum;
   }
 
@@ -500,7 +520,8 @@ public class HoodieTableConfig extends HoodieConfig {
     recoverIfNeeded(storage, cfgPath, backupCfgPath);
   }
 
-  private static void modify(HoodieStorage storage, StoragePath metadataFolder, Properties modifyProps, BiConsumer<Properties, Properties> modifyFn) {
+  private static void modify(HoodieStorage storage, StoragePath metadataFolder, Properties modifyProps, BiConsumer<Properties, Properties> propsToUpdate,
+                             Set<String> propsToDelete) {
     StoragePath cfgPath = new StoragePath(metadataFolder, HOODIE_PROPERTIES_FILE);
     StoragePath backupCfgPath = new StoragePath(metadataFolder, HOODIE_PROPERTIES_FILE_BACKUP);
     try {
@@ -512,17 +533,18 @@ public class HoodieTableConfig extends HoodieConfig {
 
       // 2. backup the existing properties.
       try (OutputStream out = storage.create(backupCfgPath, false)) {
-        storeProperties(props, out);
+        storeProperties(props, out, backupCfgPath);
       }
 
       // 3. delete the properties file, reads will go to the backup, until we are done.
-      storage.deleteFile(cfgPath);
+      deleteFile(storage, cfgPath);
 
       // 4. Upsert and save back.
       String checksum;
       try (OutputStream out = storage.create(cfgPath, true)) {
-        modifyFn.accept(props, modifyProps);
-        checksum = storeProperties(props, out);
+        propsToUpdate.accept(props, modifyProps);
+        propsToDelete.forEach(propToDelete -> props.remove(propToDelete));
+        checksum = storeProperties(props, out, cfgPath);
       }
 
       // 4. verify and remove backup.
@@ -532,31 +554,41 @@ public class HoodieTableConfig extends HoodieConfig {
         if (!props.containsKey(TABLE_CHECKSUM.key()) || !props.getProperty(TABLE_CHECKSUM.key()).equals(checksum)) {
           // delete the properties file and throw exception indicating update failure
           // subsequent writes will recover and update, reads will go to the backup until then
-          storage.deleteFile(cfgPath);
+          deleteFile(storage, cfgPath);
           throw new HoodieIOException("Checksum property missing or does not match.");
         }
       }
 
       // 5. delete the backup properties file
-      storage.deleteFile(backupCfgPath);
+      deleteFile(storage, backupCfgPath);
     } catch (IOException e) {
       throw new HoodieIOException("Error updating table configs.", e);
     }
   }
 
+  private static void deleteFile(HoodieStorage storage, StoragePath cfgPath) throws IOException {
+    storage.deleteFile(cfgPath);
+    LOG.info("Deleted properties file at " + cfgPath);
+  }
+
   /**
    * Upserts the table config with the set of properties passed in. We implement a fail-safe backup protocol
    * here for safely updating with recovery and also ensuring the table config continues to be readable.
    */
   public static void update(HoodieStorage storage, StoragePath metadataFolder,
                             Properties updatedProps) {
-    modify(storage, metadataFolder, updatedProps, ConfigUtils::upsertProperties);
+    modify(storage, metadataFolder, updatedProps, ConfigUtils::upsertProperties, Collections.EMPTY_SET);
+  }
+
+  public static void updateAndDeleteProps(HoodieStorage storage, StoragePath metadataFolder,
+                                          Properties updatedProps, Set<String> propstoDelete) {
+    modify(storage, metadataFolder, updatedProps, ConfigUtils::upsertProperties, propstoDelete);
   }
 
   public static void delete(HoodieStorage storage, StoragePath metadataFolder, Set<String> deletedProps) {
     Properties props = new Properties();
     deletedProps.forEach(p -> props.setProperty(p, ""));
-    modify(storage, metadataFolder, props, ConfigUtils::deleteProperties);
+    modify(storage, metadataFolder, props, ConfigUtils::deleteProperties, Collections.EMPTY_SET);
   }
 
   /**
@@ -597,7 +629,7 @@ public class HoodieTableConfig extends HoodieConfig {
       hoodieConfig.setDefaultValue(DROP_PARTITION_COLUMNS);
 
       dropInvalidConfigs(hoodieConfig);
-      storeProperties(hoodieConfig.getProps(), outputStream);
+      storeProperties(hoodieConfig.getProps(), outputStream, propertyPath);
     }
   }
 
@@ -719,6 +751,20 @@ public class HoodieTableConfig extends HoodieConfig {
         : Option.empty();
   }
 
+  public HoodieTableFormat getTableFormat(TimelineLayoutVersion layoutVersion) {
+    String tableFormat = getStringOrDefault(TABLE_FORMAT);
+    if (!tableFormat.equals(NativeTableFormat.TABLE_FORMAT)) {
+      ServiceLoader<HoodieTableFormat> loader = ServiceLoader.load(HoodieTableFormat.class);
+      for (HoodieTableFormat tableFormatImpl : loader) {
+        if (getString(TABLE_FORMAT).equals(tableFormatImpl.getName())) {
+          tableFormatImpl.init(props);
+          return tableFormatImpl;
+        }
+      }
+    }
+    return new NativeTableFormat(layoutVersion);
+  }
+
   /**
    * @return the hoodie.table.version from hoodie.properties file.
    */
@@ -755,19 +801,155 @@ public class HoodieTableConfig extends HoodieConfig {
     return HoodieRecordPayload.getPayloadClassName(this);
   }
 
+  /**
+   * Read the payload class if present for HoodieRecords from the table properties.
+   */
+  public Option<String> getPayloadClassIfPresent() {
+    return HoodieRecordPayload.getPayloadClassNameIfPresent(this.getProps());
+  }
+
+  public String getLegacyPayloadClass() {
+    return getStringOrDefault(LEGACY_PAYLOAD_CLASS_NAME, EMPTY_STRING);
+  }
+
   public String getRecordMergeStrategyId() {
     return getString(RECORD_MERGE_STRATEGY_ID);
   }
 
   /**
-   * Infers the merging behavior based on what the user sets (or doesn't set).
-   * Validates that the user has not set an illegal combination of configs
+   * Handle table config creation logic when creating a table for Table Version 9,
+   * which is based on the logic of table version < 9, and then tuned for version 9 logic.
+   * This approach fits the same behavior of upgrade from 8 to 9.
    */
-  public static Triple<RecordMergeMode, String, String> inferCorrectMergingBehavior(RecordMergeMode recordMergeMode,
-                                                                                    String payloadClassName,
-                                                                                    String recordMergeStrategyId,
-                                                                                    String orderingFieldName,
-                                                                                    HoodieTableVersion tableVersion) {
+  public static Map<String, String> inferMergingConfigsForV9TableCreation(RecordMergeMode recordMergeMode,
+                                                                          String payloadClassName,
+                                                                          String recordMergeStrategyId,
+                                                                          String orderingFieldName,
+                                                                          HoodieTableVersion tableVersion) {
+    Map<String, String> reconciledConfigs = new HashMap<>();
+    if (tableVersion.lesserThan(HoodieTableVersion.NINE)) {
+      throw new HoodieIOException("Unsupported flow for table versions less than 9");
+    }
+
+    // Step 1: Infer merging configs based on input information.
+    // This step is important since it provides the same configs before we do table upgrade.
+    // Then additional logic for table version 9 could be verified.
+    Triple<RecordMergeMode, String, String> inferredConfigs = inferMergingConfigsForPreV9Table(
+        recordMergeMode, payloadClassName, recordMergeStrategyId, orderingFieldName, tableVersion);
+    recordMergeMode = inferredConfigs.getLeft();
+    recordMergeStrategyId = inferredConfigs.getRight();
+
+    // Step 2: Handle Version 9 specific logic.
+    if (recordMergeMode != CUSTOM && StringUtils.isNullOrEmpty(payloadClassName)) {
+      // CASE 0: For commit time and event time based table, only merge mode is set.
+      //   `StringUtils.isNullOrEmpty(payloadClassName)` is added since
+      //   `DefaultHoodieRecordPayload`, `OverwriteWithLatestAvroPayload` and `EventTimeAvroPayload`
+      //   have the commit/event time merge mode through `inferMergingConfigsForPreV9Table`.
+      //   NOTE: Payload class / strategy id should NOT be set for this case.
+      reconciledConfigs.put(RECORD_MERGE_MODE.key(), recordMergeMode.name());
+    } else if (!BUILTIN_MERGE_STRATEGIES.contains(recordMergeStrategyId)
+        || StringUtils.isNullOrEmpty(payloadClassName)) {
+      // CASE 1: For tables using non-builtin custom merger
+      //           (payload class is inferred in `inferMergingConfigsForPreV9Table`),
+      //         or for tables using builtin custom merger without using a payload class,
+      //         in this case, merge mode and strategy id should be set.
+      //   NOTE: Payload class should NOT be set for these cases.
+      reconciledConfigs.put(RECORD_MERGE_MODE.key(), recordMergeMode.name());
+      reconciledConfigs.put(RECORD_MERGE_STRATEGY_ID.key(), recordMergeStrategyId);
+    } else {
+      // For tables using payload classes in custom merge mode.
+      //   CASE 2: Custom payload class. We set these properties explicitly.
+      if (!PayloadGroupings.getPayloadsUnderDeprecation().contains(payloadClassName)) {
+        reconciledConfigs.put(RECORD_MERGE_MODE.key(), CUSTOM.toString());
+        reconciledConfigs.put(PAYLOAD_CLASS_NAME.key(), payloadClassName);
+        reconciledConfigs.put(RECORD_MERGE_STRATEGY_ID.key(), PAYLOAD_BASED_MERGE_STRATEGY_UUID);
+      } else { // CASE 3: Payload classes are under deprecation.
+        // Standard merging configs.
+        handleMergeModeConfigs(payloadClassName, reconciledConfigs);
+        // Partial update mode config.
+        handlePartialUpdateModeConfigs(payloadClassName, reconciledConfigs);
+        // Additional custom merge properties.s
+        // Certain payloads are migrated to non payload way from 1.1 Hudi binary and the reader might need certain properties for the
+        // merge to function as expected. Handing such special cases here.
+        handlePayloadAdhocConfigs(payloadClassName, reconciledConfigs);
+      }
+    }
+    return reconciledConfigs;
+  }
+
+  private static void handleMergeModeConfigs(String payloadClassName, Map<String, String> reconciledConfigs) {
+    // NOTE: We use LEGACY_PAYLOAD_CLASS_NAME instead of PAYLOAD_CLASS_NAME here.
+    if (PayloadGroupings.getEventTimeOrderingPayloads().contains(payloadClassName)) {
+      reconciledConfigs.put(RECORD_MERGE_MODE.key(), EVENT_TIME_ORDERING.name());
+      reconciledConfigs.put(LEGACY_PAYLOAD_CLASS_NAME.key(), payloadClassName);
+    } else {
+      reconciledConfigs.put(RECORD_MERGE_MODE.key(), COMMIT_TIME_ORDERING.name());
+      reconciledConfigs.put(LEGACY_PAYLOAD_CLASS_NAME.key(), payloadClassName);
+    }
+  }
+
+  private static void handlePartialUpdateModeConfigs(String payloadClassName, Map<String, String> reconciledConfigs) {
+    // Certain payloads are migrated to non payload way from 1.1 Hudi binary.
+    // Hence we need to set the right value for partial update mode for some of the cases.
+    if (payloadClassName.equals(PartialUpdateAvroPayload.class.getName())
+        || payloadClassName.equals(OverwriteNonDefaultsWithLatestAvroPayload.class.getName())) {
+      reconciledConfigs.put(PARTIAL_UPDATE_MODE.key(), PartialUpdateMode.IGNORE_DEFAULTS.name());
+    } else if (payloadClassName.equals(PostgresDebeziumAvroPayload.class.getName())) {
+      reconciledConfigs.put(PARTIAL_UPDATE_MODE.key(), PartialUpdateMode.FILL_UNAVAILABLE.name());
+    }
+  }
+
+  private static void handlePayloadAdhocConfigs(String payloadClassName, Map<String, String> reconciledConfigs) {
+    // Certain payloads are migrated to non payload way from 1.1 Hudi binary and the reader might need certain properties for the
+    // merge to function as expected. Handing such special cases here.
+    if (payloadClassName.equals(PostgresDebeziumAvroPayload.class.getName())) {
+      reconciledConfigs.put(RECORD_MERGE_PROPERTY_PREFIX + PARTIAL_UPDATE_UNAVAILABLE_VALUE, DEBEZIUM_UNAVAILABLE_VALUE);
+      reconciledConfigs.put(RECORD_MERGE_PROPERTY_PREFIX + DELETE_KEY, DebeziumConstants.FLATTENED_OP_COL_NAME);
+      reconciledConfigs.put(RECORD_MERGE_PROPERTY_PREFIX + DELETE_MARKER, DebeziumConstants.DELETE_OP);
+      reconciledConfigs.put(ORDERING_FIELDS.key(), DebeziumConstants.FLATTENED_LSN_COL_NAME);
+    } else if (payloadClassName.equals(MySqlDebeziumAvroPayload.class.getName())) {
+      reconciledConfigs.put(RECORD_MERGE_PROPERTY_PREFIX + DELETE_KEY, DebeziumConstants.FLATTENED_OP_COL_NAME);
+      reconciledConfigs.put(RECORD_MERGE_PROPERTY_PREFIX + DELETE_MARKER, DebeziumConstants.DELETE_OP);
+      reconciledConfigs.put(ORDERING_FIELDS.key(), DebeziumConstants.FLATTENED_FILE_COL_NAME + "," + DebeziumConstants.FLATTENED_POS_COL_NAME);
+    } else if (payloadClassName.equals(AWSDmsAvroPayload.class.getName())) {
+      reconciledConfigs.put(RECORD_MERGE_PROPERTY_PREFIX + DELETE_KEY, OP_FIELD);
+      reconciledConfigs.put(RECORD_MERGE_PROPERTY_PREFIX + DELETE_MARKER, DELETE_OPERATION_VALUE);
+    }
+  }
+
+  /**
+   * To be invoked for table creation flows or writer flows.
+   * @return the merging configs to use.
+   */
+  public static Triple<RecordMergeMode, String, String> inferMergingConfigsForWrites(RecordMergeMode recordMergeMode,
+                                                                                     String payloadClassName,
+                                                                                     String recordMergeStrategyId,
+                                                                                     String orderingFieldNamesAsString,
+                                                                                     HoodieTableVersion tableVersion) {
+    if (tableVersion.greaterThanOrEquals(HoodieTableVersion.NINE)) {
+      Map<String, String> inferredMergingConfigs = inferMergingConfigsForV9TableCreation(
+          recordMergeMode, payloadClassName, recordMergeStrategyId, orderingFieldNamesAsString, tableVersion);
+      checkArgument(inferredMergingConfigs.containsKey(HoodieTableConfig.RECORD_MERGE_MODE.key()));
+      return Triple.of(
+          RecordMergeMode.valueOf(inferredMergingConfigs.get(RECORD_MERGE_MODE.key())),
+          inferredMergingConfigs.getOrDefault(PAYLOAD_CLASS_NAME.key(), null),
+          inferredMergingConfigs.getOrDefault(RECORD_MERGE_STRATEGY_ID.key(), null));
+    }
+    // For table version <= 8.
+    return inferMergingConfigsForPreV9Table(
+        recordMergeMode, payloadClassName, recordMergeStrategyId, orderingFieldNamesAsString, tableVersion);
+  }
+
+  /**
+   * Infers the merging behavior based on what the user sets (or doesn't set).
+   * Validates that the user has not set an illegal combination of configs.
+   * This function infers basic merging properties used by table version <= 8.
+   */
+  public static Triple<RecordMergeMode, String, String> inferMergingConfigsForPreV9Table(RecordMergeMode recordMergeMode,
+                                                                                         String payloadClassName,
+                                                                                         String recordMergeStrategyId,
+                                                                                         String orderingFieldNamesAsString,
+                                                                                         HoodieTableVersion tableVersion) {
     RecordMergeMode inferredRecordMergeMode;
     String inferredPayloadClassName;
     String inferredRecordMergeStrategyId;
@@ -778,7 +960,7 @@ public class HoodieTableConfig extends HoodieConfig {
       // use the default merge mode determined by whether the ordering field name is set.
       inferredRecordMergeMode = recordMergeMode != null
           ? recordMergeMode
-          : (isNullOrEmpty(orderingFieldName) ? COMMIT_TIME_ORDERING : EVENT_TIME_ORDERING);
+          : (isNullOrEmpty(orderingFieldNamesAsString) ? COMMIT_TIME_ORDERING : EVENT_TIME_ORDERING);
     } else {
       // Infer the merge mode from either the payload class or record merge strategy ID
       RecordMergeMode modeBasedOnPayload = inferRecordMergeModeFromPayloadClass(payloadClassName);
@@ -801,7 +983,9 @@ public class HoodieTableConfig extends HoodieConfig {
         inferredRecordMergeMode = modeBasedOnPayload != null ? modeBasedOnPayload : modeBasedOnStrategyId;
       }
     }
-    if (recordMergeMode != null) {
+
+    if (recordMergeMode != null && (tableVersion.lesserThan(HoodieTableVersion.NINE)
+        || !PayloadGroupings.getPayloadsUnderDeprecation().contains(payloadClassName))) {
       checkArgument(inferredRecordMergeMode == recordMergeMode,
           String.format("Configured record merge mode (%s) is inconsistent with payload class (%s) "
                   + "or record merge strategy ID (%s) configured. Please revisit the configs.",
@@ -810,14 +994,14 @@ public class HoodieTableConfig extends HoodieConfig {
 
     // Check ordering field name based on record merge mode
     if (inferredRecordMergeMode == COMMIT_TIME_ORDERING) {
-      if (nonEmpty(orderingFieldName)) {
-        LOG.warn("The precombine or ordering field ({}) is specified. COMMIT_TIME_ORDERING "
-            + "merge mode does not use precombine or ordering field anymore.", orderingFieldName);
+      if (nonEmpty(orderingFieldNamesAsString)) {
+        LOG.warn("The ordering field ({}) is specified. COMMIT_TIME_ORDERING "
+            + "merge mode does not use ordering field anymore.", orderingFieldNamesAsString);
       }
     } else if (inferredRecordMergeMode == EVENT_TIME_ORDERING) {
-      if (isNullOrEmpty(orderingFieldName)) {
-        LOG.warn("The precombine or ordering field is not specified. EVENT_TIME_ORDERING "
-            + "merge mode requires precombine or ordering field to be set for getting the "
+      if (isNullOrEmpty(orderingFieldNamesAsString)) {
+        LOG.warn("The ordering field is not specified. EVENT_TIME_ORDERING "
+            + "merge mode requires ordering field to be set for getting the "
             + "event time. Using commit time-based ordering now.");
       }
     }
@@ -841,19 +1025,21 @@ public class HoodieTableConfig extends HoodieConfig {
       }
       // TODO(HUDI-8925): remove this once the payload class name is no longer required
       if (isNullOrEmpty(inferredPayloadClassName)) {
-        inferredPayloadClassName = DEFAULT_PAYLOAD_CLASS_NAME;
+        inferredPayloadClassName = getDefaultPayloadClassName();
       }
     }
 
     return Triple.of(inferredRecordMergeMode, inferredPayloadClassName, inferredRecordMergeStrategyId);
   }
 
-  static RecordMergeMode inferRecordMergeModeFromPayloadClass(String payloadClassName) {
+  public static RecordMergeMode inferRecordMergeModeFromPayloadClass(String payloadClassName) {
     if (isNullOrEmpty(payloadClassName)) {
       return null;
     }
-    if (DefaultHoodieRecordPayload.class.getName().equals(payloadClassName)) {
-      // DefaultHoodieRecordPayload matches with EVENT_TIME_ORDERING.
+
+    if (DefaultHoodieRecordPayload.class.getName().equals(payloadClassName)
+        || EventTimeAvroPayload.class.getName().equals(payloadClassName)) {
+      // DefaultHoodieRecordPayload and EventTimeAvroPayload match with EVENT_TIME_ORDERING.
       return EVENT_TIME_ORDERING;
     } else if (payloadClassName.equals(OverwriteWithLatestAvroPayload.class.getName())) {
       // OverwriteWithLatestAvroPayload matches with COMMIT_TIME_ORDERING.
@@ -876,8 +1062,14 @@ public class HoodieTableConfig extends HoodieConfig {
     }
   }
 
-  public String getPreCombineField() {
-    return getString(PRECOMBINE_FIELD);
+  public List<String> getOrderingFields() {
+    return getOrderingFieldsStr()
+        .map(orderingFieldsStr -> Arrays.stream(orderingFieldsStr.split(",")).filter(StringUtils::nonEmpty).collect(Collectors.toList()))
+        .orElse(Collections.emptyList());
+  }
+
+  public Option<String> getOrderingFieldsStr() {
+    return Option.ofNullable(getString(ORDERING_FIELDS));
   }
 
   public Option<String[]> getRecordKeyFields() {
@@ -886,7 +1078,7 @@ public class HoodieTableConfig extends HoodieConfig {
       return Option.empty();
     } else {
       return Option.of(Arrays.stream(keyFieldsValue.split(","))
-          .filter(p -> p.length() > 0).collect(Collectors.toList()).toArray(new String[] {}));
+          .filter(p -> !p.isEmpty()).collect(Collectors.toList()).toArray(new String[] {}));
     }
   }
 
@@ -1047,6 +1239,19 @@ public class HoodieTableConfig extends HoodieConfig {
             CONFIG_VALUES_DELIMITER));
   }
 
+  public Option<PartialUpdateMode> getPartialUpdateMode() {
+    if (getTableVersion().greaterThanOrEquals(HoodieTableVersion.NINE)) {
+      if (contains(PARTIAL_UPDATE_MODE)) {
+        return Option.of(PartialUpdateMode.valueOf(getString(PARTIAL_UPDATE_MODE)));
+      } else {
+        return Option.empty();
+      }
+    } else {
+      // For table version <= 8, partial update is not supported.
+      return Option.empty();
+    }
+  }
+
   /**
    * @returns the index definition path.
    */
@@ -1097,7 +1302,7 @@ public class HoodieTableConfig extends HoodieConfig {
     setValue(TABLE_METADATA_PARTITIONS, partitions.stream().sorted().collect(Collectors.joining(CONFIG_VALUES_DELIMITER)));
     setValue(TABLE_METADATA_PARTITIONS_INFLIGHT, partitionsInflight.stream().sorted().collect(Collectors.joining(CONFIG_VALUES_DELIMITER)));
     update(metaClient.getStorage(), metaClient.getMetaPath(), getProps());
-    LOG.info(String.format("MDT %s partition %s has been %s", metaClient.getBasePath(), partitionPath, enabled ? "enabled" : "disabled"));
+    LOG.info("MDT {} partition {} has been {}", metaClient.getBasePath(), partitionPath, enabled ? "enabled" : "disabled");
   }
 
   /**
@@ -1115,7 +1320,7 @@ public class HoodieTableConfig extends HoodieConfig {
 
     setValue(TABLE_METADATA_PARTITIONS_INFLIGHT, partitionsInflight.stream().sorted().collect(Collectors.joining(CONFIG_VALUES_DELIMITER)));
     update(metaClient.getStorage(), metaClient.getMetaPath(), getProps());
-    LOG.info(String.format("MDT %s partitions %s have been set to inflight", metaClient.getBasePath(), partitionPaths));
+    LOG.info("MDT {} partitions {} have been set to inflight", metaClient.getBasePath(), partitionPaths);
   }
 
   public void setMetadataPartitionsInflight(HoodieTableMetaClient metaClient, MetadataPartitionType... partitionTypes) {
@@ -1140,6 +1345,21 @@ public class HoodieTableConfig extends HoodieConfig {
     return Option.empty();
   }
 
+  public Map<String, String> getTableMergeProperties() {
+    Map<String, String> configs = ConfigUtils.extractWithPrefix(this.props, RECORD_MERGE_PROPERTY_PREFIX);
+    if (getTableVersion().lesserThan(HoodieTableVersion.NINE)) {
+      // Convert legacy payload properties do delete key and delete marker properties
+      if (getPayloadClass().equals(AWSDmsAvroPayload.class.getName())) {
+        configs.put(DELETE_KEY, OP_FIELD);
+        configs.put(DELETE_MARKER, DELETE_OPERATION_VALUE);
+      } else if (getPayloadClass().equals(MySqlDebeziumAvroPayload.class.getName()) || getPayloadClass().equals(PostgresDebeziumAvroPayload.class.getName())) {
+        configs.put(DELETE_KEY, DebeziumConstants.FLATTENED_OP_COL_NAME);
+        configs.put(DELETE_MARKER, DebeziumConstants.DELETE_OP);
+      }
+    }
+    return configs;
+  }
+
   public Map<String, String> propsMap() {
     return props.entrySet().stream()
         .collect(Collectors.toMap(e -> String.valueOf(e.getKey()), e -> String.valueOf(e.getValue())));
@@ -1170,11 +1390,13 @@ public class HoodieTableConfig extends HoodieConfig {
    */
   @Deprecated
   public static final String HOODIE_TABLE_VERSION_PROP_NAME = VERSION.key();
+
   /**
-   * @deprecated Use {@link #PRECOMBINE_FIELD} and its methods.
+   * @deprecated Use {@link #ORDERING_FIELDS} and its methods.
    */
   @Deprecated
   public static final String HOODIE_TABLE_PRECOMBINE_FIELD = PRECOMBINE_FIELD.key();
+
   /**
    * @deprecated Use {@link #BASE_FILE_FORMAT} and its methods.
    */
@@ -1240,4 +1462,33 @@ public class HoodieTableConfig extends HoodieConfig {
    */
   @Deprecated
   public static final String DEFAULT_ARCHIVELOG_FOLDER = TIMELINE_HISTORY_PATH.defaultValue();
+
+  private static class PayloadGroupings {
+    public static final Set<String> PAYLOADS_UNDER_DEPRECATION = Collections.unmodifiableSet(
+        new HashSet<>(Arrays.asList(
+            AWSDmsAvroPayload.class.getName(),
+            DefaultHoodieRecordPayload.class.getName(),
+            EventTimeAvroPayload.class.getName(),
+            MySqlDebeziumAvroPayload.class.getName(),
+            OverwriteNonDefaultsWithLatestAvroPayload.class.getName(),
+            OverwriteWithLatestAvroPayload.class.getName(),
+            PartialUpdateAvroPayload.class.getName(),
+            PostgresDebeziumAvroPayload.class.getName())));
+
+    public static final Set<String> EVENT_TIME_ORDERING_PAYLOADS = Collections.unmodifiableSet(
+        new HashSet<>(Arrays.asList(
+            DefaultHoodieRecordPayload.class.getName(),
+            EventTimeAvroPayload.class.getName(),
+            MySqlDebeziumAvroPayload.class.getName(),
+            PartialUpdateAvroPayload.class.getName(),
+            PostgresDebeziumAvroPayload.class.getName())));
+
+    private static Set<String> getPayloadsUnderDeprecation() {
+      return PAYLOADS_UNDER_DEPRECATION;
+    }
+
+    private static Set<String> getEventTimeOrderingPayloads() {
+      return EVENT_TIME_ORDERING_PAYLOADS;
+    }
+  }
 }
