@@ -104,8 +104,6 @@ public abstract class AbstractHoodieLogRecordScanner {
   private final Option<String> partitionPathFieldOpt;
   // Partition name override
   private final Option<String> partitionNameOverrideOpt;
-  // Pre-combining field
-  protected final String preCombineField;
   // Stateless component for merging records
   protected final HoodieRecordMerger recordMerger;
   private final TypedProperties payloadProps;
@@ -175,11 +173,12 @@ public abstract class AbstractHoodieLogRecordScanner {
     // load class from the payload fully qualified class name
     HoodieTableConfig tableConfig = this.hoodieTableMetaClient.getTableConfig();
     this.payloadClassFQN = tableConfig.getPayloadClass();
-    this.preCombineField = tableConfig.getPreCombineField();
+    String orderingFieldsStr = tableConfig.getOrderingFieldsStr().orElse(null);
     // Log scanner merge log with precombine
     TypedProperties props = new TypedProperties();
-    if (this.preCombineField != null) {
-      props.setProperty(HoodiePayloadProps.PAYLOAD_ORDERING_FIELD_PROP_KEY, this.preCombineField);
+    if (orderingFieldsStr != null) {
+      props.setProperty(HoodiePayloadProps.PAYLOAD_ORDERING_FIELD_PROP_KEY, orderingFieldsStr);
+      props.setProperty(HoodieTableConfig.ORDERING_FIELDS.key(), orderingFieldsStr);
     }
     this.tableVersion = tableConfig.getTableVersion();
     this.payloadProps = props;
