@@ -28,6 +28,7 @@ import org.apache.hudi.exception.HoodieIOException;
 import org.apache.hudi.io.SeekableDataInputStream;
 import org.apache.hudi.storage.HoodieStorage;
 
+import org.apache.avro.AvroTypeException;
 import org.apache.avro.Schema;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -61,7 +62,7 @@ public abstract class HoodieDataBlock extends HoodieLogBlock {
 
   // TODO rebase records/content to leverage Either to warrant
   //      that they are mutex (used by read/write flows respectively)
-  private final Option<List<HoodieRecord>> records;
+  protected Option<List<HoodieRecord>> records;
 
   /**
    * Key field's name w/in the record's schema
@@ -145,16 +146,38 @@ public abstract class HoodieDataBlock extends HoodieLogBlock {
   }
 
   /**
-   * Returns all the records iterator contained w/in this block.
+   * Returns an iterator over all the records contained within this block.
+   * This method uses a default buffer size of 0, which means it will read
+   * the entire Data Block content at once.
+   *
+   * @param type The type of HoodieRecord.
+   * @param <T>  The type parameter for HoodieRecord.
+   * @return A ClosableIterator over HoodieRecord<T>.
    */
   public final <T> ClosableIterator<HoodieRecord<T>> getRecordIterator(HoodieRecordType type) {
+    return getRecordIterator(type, 0);
+  }
+
+  /**
+   * Returns an iterator over all the records contained within this block.
+   *
+   * @param type       The type of HoodieRecord.
+   * @param bufferSize The size of the buffer for streaming read.
+   *                   A bufferSize less than or equal to 0 means that streaming read is disabled and
+   *                   the entire block content will be read at once.
+   *                   A bufferSize greater than 0 enables streaming read with the specified buffer size.
+   * @param <T>        The type parameter for HoodieRecord.
+   * @return A ClosableIterator over HoodieRecord<T>.
+   * @throws HoodieIOException If there is an error reading records from the block payload.
+   */
+  public final <T> ClosableIterator<HoodieRecord<T>> getRecordIterator(HoodieRecordType type, int bufferSize) {
     if (records.isPresent()) {
       // TODO need convert record type
       return list2Iterator(unsafeCast(records.get()));
     }
     try {
       // in case records are absent, read content lazily and then convert to IndexedRecords
-      return readRecordsFromBlockPayload(type);
+      return readRecordsFromBlockPayload(type, bufferSize);
     } catch (IOException io) {
       throw new HoodieIOException("Unable to convert content bytes to records", io);
     }
@@ -173,6 +196,23 @@ public abstract class HoodieDataBlock extends HoodieLogBlock {
    * @throws IOException in case of failures encountered when reading/parsing records
    */
   public final <T> ClosableIterator<HoodieRecord<T>> getRecordIterator(List<String> keys, boolean fullKey, HoodieRecordType type) throws IOException {
+    return getRecordIterator(keys, fullKey, type, 0);
+  }
+
+  /**
+   * Batch get of keys of interest. Implementation can choose to either do full scan and return matched entries or
+   * do a seek based parsing and return matched entries.
+   *
+   * @param keys keys of interest.
+   * @param bufferSize The size of the buffer for streaming read.
+   *                   A bufferSize less than or equal to 0 means that streaming read is disabled and
+   *                   the entire block content will be read at once.
+   *                   A bufferSize greater than 0 enables streaming read with the specified buffer size.
+   * @param <T>        The type parameter for HoodieRecord.
+   * @return List of IndexedRecords for the keys of interest.
+   * @throws IOException in case of failures encountered when reading/parsing records
+   */
+  public final <T> ClosableIterator<HoodieRecord<T>> getRecordIterator(List<String> keys, boolean fullKey, HoodieRecordType type, int bufferSize) throws IOException {
     boolean fullScan = keys.isEmpty();
     if (enablePointLookups && !fullScan) {
       return lookupRecords(keys, fullKey);
@@ -180,7 +220,7 @@ public abstract class HoodieDataBlock extends HoodieLogBlock {
 
     // Otherwise, we fetch all the records and filter out all the records, but the
     // ones requested
-    ClosableIterator<HoodieRecord<T>> allRecords = getRecordIterator(type);
+    ClosableIterator<HoodieRecord<T>> allRecords = getRecordIterator(type, bufferSize);
     if (fullScan) {
       return allRecords;
     }
@@ -220,19 +260,13 @@ public abstract class HoodieDataBlock extends HoodieLogBlock {
    * @param <T>           The type of engine-specific record representation to return.
    * @return An iterator containing the records of interest in specified type.
    */
-  public final <T> ClosableIterator<T> getEngineRecordIterator(HoodieReaderContext<T> readerContext, List<String> keys, boolean fullKey) {
+  public final <T> ClosableIterator<T> getEngineRecordIterator(HoodieReaderContext<T> readerContext, List<String> keys, boolean fullKey) throws IOException {
     boolean fullScan = keys.isEmpty();
-
-    // Otherwise, we fetch all the records and filter out all the records, but the
-    // ones requested
-    ClosableIterator<T> allRecords = getEngineRecordIterator(readerContext);
-    if (fullScan) {
-      return allRecords;
+    if (!fullScan) {
+      return lookupEngineRecords(keys, fullKey);
+    } else {
+      throw new IllegalStateException("Unexpected code reached. Expected to be called only with keySpec defined for non FILES partition in Metadata table");
     }
-
-    HashSet<String> keySet = new HashSet<>(keys);
-    return FilteringEngineRecordIterator.getInstance(allRecords, keySet, fullKey, record ->
-        Option.of(readerContext.getValue(record, readerSchema, keyFieldName).toString()));
   }
 
   protected <T> ClosableIterator<HoodieRecord<T>> readRecordsFromBlockPayload(HoodieRecordType type) throws IOException {
@@ -249,6 +283,26 @@ public abstract class HoodieDataBlock extends HoodieLogBlock {
     }
   }
 
+  /**
+   * Reads records from the block payload using a specified buffer size.
+   * This method attempts to read serialized records from the block payload, leveraging a buffer size for streaming reads.
+   * If the buffer size is less than or equal to 0, it reads the entire block content at once.
+   *
+   * @param bufferSize The size of the buffer for streaming read.
+   *                   A bufferSize less than or equal to 0 means that streaming read is disabled and
+   *                   the entire block content will be read at once.
+   *                   A bufferSize greater than 0 enables streaming read with the specified buffer size.
+   * @return A ClosableIterator over HoodieRecord<T>.
+   * @throws IOException If there is an error reading or deserializing the records.
+   */
+  protected <T> ClosableIterator<HoodieRecord<T>> readRecordsFromBlockPayload(HoodieRecordType type, int bufferSize) throws IOException {
+    if (getContent().isPresent() || bufferSize <= 0) {
+      return readRecordsFromBlockPayload(type);
+    }
+
+    return deserializeRecords(getInputStreamSupplier().get(), getBlockContentLocation().get(), type, bufferSize);
+  }
+
   protected <T> ClosableIterator<T> readRecordsFromBlockPayload(HoodieReaderContext<T> readerContext) throws IOException {
     if (readBlockLazily && !getContent().isPresent()) {
       // read log block contents from disk
@@ -269,10 +323,32 @@ public abstract class HoodieDataBlock extends HoodieLogBlock {
     );
   }
 
+  protected <T> ClosableIterator<T> lookupEngineRecords(List<String> keys, boolean fullKey) throws IOException {
+    throw new UnsupportedOperationException(
+        String.format("Point lookups are not supported by this Data block type (%s)", getBlockType())
+    );
+  }
+
   protected abstract ByteArrayOutputStream serializeRecords(List<HoodieRecord> records, HoodieStorage storage) throws IOException;
 
   protected abstract <T> ClosableIterator<HoodieRecord<T>> deserializeRecords(byte[] content, HoodieRecordType type) throws IOException;
 
+  /**
+   * Streaming deserialization of records.
+   *
+   * @param inputStream The input stream from which to read the records.
+   * @param contentLocation The location within the input stream where the content starts.
+   * @param bufferSize The size of the buffer to use for reading the records.
+   * @return A ClosableIterator over HoodieRecord<T>.
+   * @throws IOException If there is an error reading or deserializing the records.
+   */
+  protected abstract <T> ClosableIterator<HoodieRecord<T>> deserializeRecords(
+      SeekableDataInputStream inputStream,
+      HoodieLogBlockContentLocation contentLocation,
+      HoodieRecordType type,
+      int bufferSize
+  ) throws IOException;
+
   /**
    * Deserializes the content bytes of the data block to the records in engine-specific representation.
    *
@@ -296,7 +372,16 @@ public abstract class HoodieDataBlock extends HoodieLogBlock {
 
   protected Schema getSchemaFromHeader() {
     String schemaStr = getLogBlockHeader().get(HeaderMetadataType.SCHEMA);
-    SCHEMA_MAP.computeIfAbsent(schemaStr, (schemaString) -> new Schema.Parser().parse(schemaString));
+    SCHEMA_MAP.computeIfAbsent(schemaStr,
+        (schemaString) -> {
+          try {
+            return new Schema.Parser().parse(schemaStr);
+          } catch (AvroTypeException e) {
+            // Archived commits from earlier hudi versions fail the schema check
+            // So we retry in this one specific instance.
+            return new Schema.Parser().setValidateDefaults(false).parse(schemaStr);
+          }
+        });
     return SCHEMA_MAP.get(schemaStr);
   }
 
