@@ -18,15 +18,16 @@
 
 package org.apache.hudi.common.table.timeline;
 
+import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.timeline.HoodieInstant.State;
 import org.apache.hudi.common.util.CollectionUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.exception.HoodieException;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
+import org.apache.hudi.exception.HoodieIOException;
 
+import java.io.IOException;
 import java.io.InputStream;
 import java.io.Serializable;
 import java.security.MessageDigest;
@@ -55,8 +56,6 @@ import static org.apache.hudi.common.util.StringUtils.getUTF8Bytes;
  */
 public abstract class BaseHoodieTimeline implements HoodieTimeline {
 
-  private static final Logger LOG = LoggerFactory.getLogger(BaseHoodieTimeline.class);
-
   private static final long serialVersionUID = 1L;
 
   private static final String HASHING_ALGORITHM = "SHA-256";
@@ -115,6 +114,14 @@ public abstract class BaseHoodieTimeline implements HoodieTimeline {
     clearState();
   }
 
+  protected List<HoodieInstant> getInstantsFromFileSystem(HoodieTableMetaClient metaClient, Set<String> includedExtensions, boolean applyLayoutFilters) {
+    try {
+      return metaClient.scanHoodieInstantsFromFileSystem(metaClient.getTimelinePath(), includedExtensions, applyLayoutFilters);
+    } catch (IOException e) {
+      throw new HoodieIOException("Failed to scan metadata", e);
+    }
+  }
+
   @Override
   public HoodieTimeline filterInflights() {
     return factory.createDefaultTimeline(getInstantsAsStream().filter(HoodieInstant::isInflight), getInstantReader());
@@ -256,7 +263,7 @@ public abstract class BaseHoodieTimeline implements HoodieTimeline {
   @Override
   public HoodieTimeline findInstantsInRangeByCompletionTime(String startTs, String endTs) {
     return factory.createDefaultTimeline(
-        getInstantsAsStream().filter(s -> s.getCompletionTime() != null && InstantComparison.isInClosedRange(s.getCompletionTime(), startTs, endTs)),
+        getInstantsAsStream().filter(s -> s.getCompletionTime() != null && InstantComparison.isInRange(s.getCompletionTime(), startTs, endTs)),
         getInstantReader());
   }
 
@@ -494,6 +501,11 @@ public abstract class BaseHoodieTimeline implements HoodieTimeline {
         .sorted(instantComparator.completionTimeOrderedComparator());
   }
 
+  @Override
+  public Stream<HoodieInstant> getReverseOrderedInstantsByCompletionTime() {
+    return getInstantsAsStream().sorted(instantComparator.completionTimeOrderedComparator().reversed());
+  }
+
   @Override
   public boolean isBeforeTimelineStarts(String instant) {
     Option<HoodieInstant> firstNonSavepointCommit = getFirstNonSavepointCommit();
