@@ -19,12 +19,12 @@
 package org.apache.hudi.io.storage;
 
 import org.apache.hudi.common.config.HoodieConfig;
-import org.apache.hudi.common.config.HoodieReaderConfig;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieFileFormat;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.storage.HoodieStorage;
 import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.StoragePathInfo;
 
 import org.apache.avro.Schema;
 
@@ -77,6 +77,20 @@ public class HoodieFileReaderFactory {
     }
   }
 
+  public HoodieFileReader getFileReader(HoodieConfig hoodieConfig, StoragePathInfo pathInfo, HoodieFileFormat format,
+                                        Option<Schema> schemaOption) throws IOException {
+    switch (format) {
+      case PARQUET:
+        return newParquetFileReader(pathInfo.getPath());
+      case HFILE:
+        return newHFileFileReader(hoodieConfig, pathInfo, schemaOption);
+      case ORC:
+        return newOrcFileReader(pathInfo.getPath());
+      default:
+        throw new UnsupportedOperationException(format + " format not supported yet.");
+    }
+  }
+
   public HoodieFileReader getContentReader(HoodieConfig hoodieConfig, StoragePath path, HoodieFileFormat format,
                                            HoodieStorage storage, byte[] content,
                                            Option<Schema> schemaOption) throws IOException {
@@ -97,6 +111,11 @@ public class HoodieFileReaderFactory {
     throw new UnsupportedOperationException();
   }
 
+  protected HoodieFileReader newHFileFileReader(HoodieConfig hoodieConfig, StoragePathInfo pathInfo,
+                                                Option<Schema> schemaOption) throws IOException {
+    throw new UnsupportedOperationException();
+  }
+
   protected HoodieFileReader newHFileFileReader(HoodieConfig hoodieConfig, StoragePath path,
                                                 HoodieStorage storage, byte[] content, Option<Schema> schemaOption)
       throws IOException {
@@ -114,7 +133,4 @@ public class HoodieFileReaderFactory {
     throw new UnsupportedOperationException();
   }
 
-  protected static boolean isUseNativeHFileReaderEnabled(HoodieConfig hoodieConfig) {
-    return hoodieConfig.getBooleanOrDefault(HoodieReaderConfig.USE_NATIVE_HFILE_READER);
-  }
 }
\ No newline at end of file
