@@ -24,6 +24,34 @@ import org.apache.hudi.avro.model.DateWrapper;
 import org.apache.hudi.avro.model.DecimalWrapper;
 import org.apache.hudi.avro.model.DoubleWrapper;
 import org.apache.hudi.avro.model.FloatWrapper;
+import org.apache.hudi.avro.model.HoodieActionInstant;
+import org.apache.hudi.avro.model.HoodieBootstrapFilePartitionInfo;
+import org.apache.hudi.avro.model.HoodieBootstrapIndexInfo;
+import org.apache.hudi.avro.model.HoodieBootstrapPartitionMetadata;
+import org.apache.hudi.avro.model.HoodieCleanFileInfo;
+import org.apache.hudi.avro.model.HoodieCleanMetadata;
+import org.apache.hudi.avro.model.HoodieCleanPartitionMetadata;
+import org.apache.hudi.avro.model.HoodieCleanerPlan;
+import org.apache.hudi.avro.model.HoodieCommitMetadata;
+import org.apache.hudi.avro.model.HoodieCompactionOperation;
+import org.apache.hudi.avro.model.HoodieCompactionPlan;
+import org.apache.hudi.avro.model.HoodieCompactionStrategy;
+import org.apache.hudi.avro.model.HoodieDeleteRecordList;
+import org.apache.hudi.avro.model.HoodieFSPermission;
+import org.apache.hudi.avro.model.HoodieFileStatus;
+import org.apache.hudi.avro.model.HoodieIndexPlan;
+import org.apache.hudi.avro.model.HoodieMergeArchiveFilePlan;
+import org.apache.hudi.avro.model.HoodiePath;
+import org.apache.hudi.avro.model.HoodieReplaceCommitMetadata;
+import org.apache.hudi.avro.model.HoodieRequestedReplaceMetadata;
+import org.apache.hudi.avro.model.HoodieRestoreMetadata;
+import org.apache.hudi.avro.model.HoodieRestorePlan;
+import org.apache.hudi.avro.model.HoodieRollbackMetadata;
+import org.apache.hudi.avro.model.HoodieRollbackPartitionMetadata;
+import org.apache.hudi.avro.model.HoodieRollbackPlan;
+import org.apache.hudi.avro.model.HoodieSavepointMetadata;
+import org.apache.hudi.avro.model.HoodieSavepointPartitionMetadata;
+import org.apache.hudi.avro.model.HoodieWriteStat;
 import org.apache.hudi.avro.model.IntWrapper;
 import org.apache.hudi.avro.model.LocalDateWrapper;
 import org.apache.hudi.avro.model.LongWrapper;
@@ -53,6 +81,9 @@ import org.apache.avro.io.BinaryDecoder;
 import org.apache.avro.io.BinaryEncoder;
 import org.apache.avro.io.DecoderFactory;
 import org.apache.avro.io.EncoderFactory;
+import org.apache.avro.specific.SpecificData;
+import org.apache.avro.specific.SpecificRecord;
+import org.apache.avro.specific.SpecificRecordBase;
 import org.apache.avro.util.Utf8;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Test;
@@ -60,8 +91,6 @@ import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.Arguments;
 import org.junit.jupiter.params.provider.MethodSource;
 import org.junit.jupiter.params.provider.ValueSource;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
@@ -84,11 +113,11 @@ import java.util.Random;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
-import static org.apache.hudi.avro.AvroSchemaUtils.resolveNullableSchema;
+import static org.apache.hudi.avro.AvroSchemaUtils.getNonNullTypeFromUnion;
 import static org.apache.hudi.avro.HoodieAvroUtils.getNestedFieldSchemaFromWriteSchema;
 import static org.apache.hudi.avro.HoodieAvroUtils.sanitizeName;
-import static org.apache.hudi.avro.HoodieAvroUtils.unwrapAvroValueWrapper;
-import static org.apache.hudi.avro.HoodieAvroUtils.wrapValueIntoAvro;
+import static org.apache.hudi.avro.HoodieAvroWrapperUtils.unwrapAvroValueWrapper;
+import static org.apache.hudi.avro.HoodieAvroWrapperUtils.wrapValueIntoAvro;
 import static org.apache.hudi.common.util.StringUtils.getUTF8Bytes;
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -103,8 +132,6 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
  */
 public class TestHoodieAvroUtils {
 
-  private static final Logger LOG = LoggerFactory.getLogger(TestHoodieAvroUtils.class);
-
   private static final String EVOLVED_SCHEMA = "{\"type\": \"record\",\"name\": \"testrec1\",\"fields\": [ "
       + "{\"name\": \"timestamp\",\"type\": \"double\"},{\"name\": \"_row_key\", \"type\": \"string\"},"
       + "{\"name\": \"non_pii_col\", \"type\": \"string\"},"
@@ -125,6 +152,17 @@ public class TestHoodieAvroUtils {
       + "{\"name\": \"pii_col\", \"type\": \"string\", \"column_category\": \"user_profile\"}], "
       + "\"custom_schema_property\": \"custom_schema_property_value\"}";
 
+  private static final String EXAMPLE_SCHEMA_WITH_META_FIELDS = "{\"type\": \"record\",\"name\": \"testrec\",\"fields\": [ "
+      + "{\"name\": \"_hoodie_commit_time\",\"type\": \"string\"},"
+      + "{\"name\": \"_hoodie_commit_seqno\",\"type\": \"string\"},"
+      + "{\"name\": \"_hoodie_record_key\",\"type\": \"string\"},"
+      + "{\"name\": \"_hoodie_partition_path\",\"type\": \"string\"},"
+      + "{\"name\": \"_hoodie_file_name\",\"type\": \"string\"},"
+      + "{\"name\": \"timestamp\",\"type\": \"double\"},"
+      + "{\"name\": \"_row_key\", \"type\": \"string\"},"
+      + "{\"name\": \"non_pii_col\", \"type\": \"string\"},"
+      + "{\"name\": \"pii_col\", \"type\": \"string\", \"column_category\": \"user_profile\"}]}";
+
   private static final int NUM_FIELDS_IN_EXAMPLE_SCHEMA = 4;
 
   private static final String SCHEMA_WITH_METADATA_FIELD = "{\"type\": \"record\",\"name\": \"testrec2\",\"fields\": [ "
@@ -161,14 +199,14 @@ public class TestHoodieAvroUtils {
       + "{\"name\":\"firstname\",\"type\":\"string\"},"
       + "{\"name\":\"lastname\",\"type\":\"string\"},"
       + "{\"name\":\"student\",\"type\":{\"name\":\"student\",\"type\":\"record\",\"fields\":["
-      + "{\"name\":\"firstname\",\"type\":[\"null\" ,\"string\"],\"default\": null},{\"name\":\"lastname\",\"type\":[\"null\" ,\"string\"],\"default\": null}]}}]}";
+      + "{\"name\":\"firstnameNested\",\"type\":[\"null\" ,\"string\"],\"default\": null},{\"name\":\"lastnameNested\",\"type\":[\"null\" ,\"string\"],\"default\": null}]}}]}";
 
   private static final String SCHEMA_WITH_NESTED_FIELD_RENAMED =
       "{\"name\":\"MyClass\",\"type\":\"record\",\"namespace\":\"com.acme.avro\",\"fields\":["
       + "{\"name\":\"fn\",\"type\":\"string\"},"
       + "{\"name\":\"ln\",\"type\":\"string\"},"
       + "{\"name\":\"ss\",\"type\":{\"name\":\"ss\",\"type\":\"record\",\"fields\":["
-      + "{\"name\":\"fn\",\"type\":[\"null\" ,\"string\"],\"default\": null},{\"name\":\"ln\",\"type\":[\"null\" ,\"string\"],\"default\": null}]}}]}";
+      + "{\"name\":\"fnn\",\"type\":[\"null\" ,\"string\"],\"default\": null},{\"name\":\"lnn\",\"type\":[\"null\" ,\"string\"],\"default\": null}]}}]}";
 
   public static final String SCHEMA_WITH_AVRO_TYPES_STR = "{\"name\":\"TestRecordAvroTypes\",\"type\":\"record\",\"fields\":["
       // Primitive types
@@ -179,6 +217,10 @@ public class TestHoodieAvroUtils {
       + "{\"name\":\"doubleField\",\"type\":\"double\"},"
       + "{\"name\":\"bytesField\",\"type\":\"bytes\"},"
       + "{\"name\":\"stringField\",\"type\":\"string\"},"
+      + "{\"name\":\"secondLevelField\",\"type\":[\"null\", {\"name\":\"secondLevelField\",\"type\":\"record\",\"fields\":["
+      + "{\"name\":\"firstname\",\"type\":[\"null\",\"string\"],\"default\":null},"
+      + "{\"name\":\"lastname\",\"type\":[\"null\",\"string\"],\"default\":null}"
+      + "]}],\"default\":null},"
       // Logical types
       + "{\"name\":\"decimalField\",\"type\":\"bytes\",\"logicalType\":\"decimal\",\"precision\":20,\"scale\":5},"
       + "{\"name\":\"timeMillisField\",\"type\":\"int\",\"logicalType\":\"time-millis\"},"
@@ -189,7 +231,6 @@ public class TestHoodieAvroUtils {
       + "{\"name\":\"localTimestampMicrosField\",\"type\":\"long\",\"logicalType\":\"local-timestamp-micros\"}"
       + "]}";
 
-  private static final Schema SCHEMA_WITH_NESTED_FIELD = new Schema.Parser().parse(SCHEMA_WITH_NESTED_FIELD_STR);
   private static final Schema SCHEMA_WITH_AVRO_TYPES = new Schema.Parser().parse(SCHEMA_WITH_AVRO_TYPES_STR);
 
   // Define schema with a nested field containing a union type
@@ -218,12 +259,26 @@ public class TestHoodieAvroUtils {
   public static String SCHEMA_WITH_NESTED_FIELD_LARGE_STR = "{\"name\":\"MyClass\",\"type\":\"record\",\"namespace\":\"com.acme.avro\",\"fields\":["
       + "{\"name\":\"firstname\",\"type\":\"string\"},"
       + "{\"name\":\"lastname\",\"type\":\"string\"},"
-      + "{\"name\":\"nested_field\",\"type\":" + SCHEMA_WITH_AVRO_TYPES_STR + "},"
+      + "{\"name\":\"nested_field\",\"type\":[\"null\"," + SCHEMA_WITH_AVRO_TYPES_STR + "],\"default\":null},"
       + "{\"name\":\"student\",\"type\":{\"name\":\"student\",\"type\":\"record\",\"fields\":["
       + "{\"name\":\"firstname\",\"type\":[\"null\" ,\"string\"],\"default\": null},{\"name\":\"lastname\",\"type\":[\"null\" ,\"string\"],\"default\": null}]}}]}";
 
   public static Schema SCHEMA_WITH_NESTED_FIELD_LARGE = new Schema.Parser().parse(SCHEMA_WITH_NESTED_FIELD_LARGE_STR);
 
+  @Test
+  void testAddMetaFields() {
+    // validate null schema does not throw errors
+    assertNull(HoodieAvroUtils.addMetadataFields(null));
+    Schema nullSchema = Schema.create(Schema.Type.NULL);
+    assertEquals(nullSchema, HoodieAvroUtils.addMetadataFields(nullSchema));
+    // test with non-null schema
+    Schema schema = HoodieAvroUtils.addMetadataFields(new Schema.Parser().parse(EXAMPLE_SCHEMA));
+    assertEquals(NUM_FIELDS_IN_EXAMPLE_SCHEMA + HoodieRecord.HOODIE_META_COLUMNS.size(), schema.getFields().size());
+    for (String metaCol : HoodieRecord.HOODIE_META_COLUMNS) {
+      assertNotNull(schema.getField(metaCol));
+    }
+  }
+
   @Test
   public void testPropsPresent() {
     Schema schema = HoodieAvroUtils.addMetadataFields(new Schema.Parser().parse(EXAMPLE_SCHEMA));
@@ -288,6 +343,46 @@ public class TestHoodieAvroUtils {
     assertNull(rec1.get("nullable_field_wo_default"));
   }
 
+  @Test
+  public void testJoinedGenericRecord() {
+    GenericRecord rec = new GenericData.Record(new Schema.Parser().parse(EXAMPLE_SCHEMA));
+    rec.put("_row_key", "key1");
+    rec.put("non_pii_col", "val1");
+    rec.put("pii_col", "val2");
+    rec.put("timestamp", 3.5);
+
+    GenericRecord rec1 = new JoinedGenericRecord(rec, 5, new Schema.Parser().parse(EXAMPLE_SCHEMA_WITH_META_FIELDS));
+    assertNull(rec1.get("_hoodie_commit_time"));
+    assertNull(rec1.get("_hoodie_record_key"));
+
+    assertEquals(rec.get("_row_key"), rec1.get("_row_key"));
+    assertEquals(rec.get("_row_key"), rec1.get(6));
+    assertEquals(rec.get("non_pii_col"), rec1.get("non_pii_col"));
+    assertEquals(rec.get("non_pii_col"), rec1.get(7));
+    assertEquals(rec.get("pii_col"), rec1.get("pii_col"));
+    assertEquals(rec.get("pii_col"), rec1.get(8));
+    assertEquals(rec.get("timestamp"), rec1.get("timestamp"));
+    assertEquals(rec.get("timestamp"), rec1.get(5));
+
+    // lets add meta field values and validate
+    rec1.put(0, "commitTime1");
+    rec1.put(1, "commitSecNo1");
+    rec1.put(2, "recKey1");
+    rec1.put(3, "pPath1");
+    rec1.put(4, "fileName");
+
+    assertEquals("commitTime1", rec1.get(0));
+    assertEquals("commitTime1", rec1.get(HoodieRecord.COMMIT_TIME_METADATA_FIELD));
+    assertEquals("commitSecNo1", rec1.get(1));
+    assertEquals("commitSecNo1", rec1.get(HoodieRecord.COMMIT_SEQNO_METADATA_FIELD));
+    assertEquals("recKey1", rec1.get(2));
+    assertEquals("recKey1", rec1.get(HoodieRecord.RECORD_KEY_METADATA_FIELD));
+    assertEquals("pPath1", rec1.get(3));
+    assertEquals("pPath1", rec1.get(HoodieRecord.PARTITION_PATH_METADATA_FIELD));
+    assertEquals("fileName", rec1.get(4));
+    assertEquals("fileName", rec1.get(HoodieRecord.FILENAME_METADATA_FIELD));
+  }
+
   @Test
   public void testNonNullableFieldWithoutDefault() {
     GenericRecord rec = new GenericData.Record(new Schema.Parser().parse(EXAMPLE_SCHEMA));
@@ -437,30 +532,30 @@ public class TestHoodieAvroUtils {
 
     // test get student
     GenericRecord studentRecord = new GenericData.Record(rec.getSchema().getField("student").schema());
-    studentRecord.put("firstname", "person");
+    studentRecord.put("firstnameNested", "person");
     rec.put("student", studentRecord);
     assertEquals(studentRecord, HoodieAvroUtils.getNestedFieldVal(rec, "student", false, false));
 
     // test get student.fake_key
-    assertEquals("student.fake_key(Part -fake_key) field not found in record. Acceptable fields were :[firstname, lastname]",
+    assertEquals("student.fake_key(Part -fake_key) field not found in record. Acceptable fields were :[firstnameNested, lastnameNested]",
         assertThrows(HoodieException.class, () ->
             HoodieAvroUtils.getNestedFieldVal(rec, "student.fake_key", false, false)).getMessage());
 
     // test get student.firstname
-    assertEquals("person", HoodieAvroUtils.getNestedFieldVal(rec, "student.firstname", false, false));
+    assertEquals("person", HoodieAvroUtils.getNestedFieldVal(rec, "student.firstnameNested", false, false));
 
     // test get student.lastname(null)
-    assertNull(HoodieAvroUtils.getNestedFieldVal(rec, "student.lastname", false, false));
+    assertNull(HoodieAvroUtils.getNestedFieldVal(rec, "student.lastnameNested", false, false));
 
     // test get student.firstname.fake_key
-    assertEquals("Cannot find a record at part value :firstname",
+    assertEquals("Cannot find a record at part value :firstnameNested",
         assertThrows(HoodieException.class, () ->
-            HoodieAvroUtils.getNestedFieldVal(rec, "student.firstname.fake_key", false, false)).getMessage());
+            HoodieAvroUtils.getNestedFieldVal(rec, "student.firstnameNested.fake_key", false, false)).getMessage());
 
     // test get student.lastname(null).fake_key
-    assertEquals("Cannot find a record at part value :lastname",
+    assertEquals("Cannot find a record at part value :lastnameNested",
         assertThrows(HoodieException.class, () ->
-            HoodieAvroUtils.getNestedFieldVal(rec, "student.lastname.fake_key", false, false)).getMessage());
+            HoodieAvroUtils.getNestedFieldVal(rec, "student.lastnameNested.fake_key", false, false)).getMessage());
   }
 
   @Test
@@ -505,12 +600,12 @@ public class TestHoodieAvroUtils {
     rec3.put("firstname", "person1");
     rec3.put("lastname", "person2");
     GenericRecord studentRecord = new GenericData.Record(rec3.getSchema().getField("student").schema());
-    studentRecord.put("firstname", "person1");
-    studentRecord.put("lastname", "person2");
+    studentRecord.put("firstnameNested", "person1");
+    studentRecord.put("lastnameNested", "person2");
     rec3.put("student", studentRecord);
 
-    assertEquals(Schema.create(Schema.Type.STRING), getNestedFieldSchemaFromWriteSchema(rec3.getSchema(), "student.firstname"));
-    assertEquals(Schema.create(Schema.Type.STRING), getNestedFieldSchemaFromWriteSchema(nestedSchema, "student.firstname"));
+    assertEquals(Schema.create(Schema.Type.STRING), getNestedFieldSchemaFromWriteSchema(rec3.getSchema(), "student.firstnameNested"));
+    assertEquals(Schema.create(Schema.Type.STRING), getNestedFieldSchemaFromWriteSchema(nestedSchema, "student.firstnameNested"));
   }
 
   @Test
@@ -547,8 +642,8 @@ public class TestHoodieAvroUtils {
     rec3.put("firstname", "person1");
     rec3.put("lastname", "person2");
     GenericRecord studentRecord = new GenericData.Record(rec3.getSchema().getField("student").schema());
-    studentRecord.put("firstname", "person1");
-    studentRecord.put("lastname", "person2");
+    studentRecord.put("firstnameNested", "person3");
+    studentRecord.put("lastnameNested", "person4");
     rec3.put("student", studentRecord);
 
     Schema nestedSchemaRename = new Schema.Parser().parse(SCHEMA_WITH_NESTED_FIELD_RENAMED);
@@ -556,10 +651,51 @@ public class TestHoodieAvroUtils {
     colRenames.put("fn", "firstname");
     colRenames.put("ln", "lastname");
     colRenames.put("ss", "student");
-    colRenames.put("ss.fn", "firstname");
-    colRenames.put("ss.ln", "lastname");
+    colRenames.put("ss.fnn", "firstnameNested");
+    colRenames.put("ss.lnn", "lastnameNested");
     GenericRecord studentRecordRename = HoodieAvroUtils.rewriteRecordWithNewSchema(rec3, nestedSchemaRename, colRenames);
-    Assertions.assertEquals(GenericData.get().validate(nestedSchemaRename, studentRecordRename), true);
+    Assertions.assertTrue(GenericData.get().validate(nestedSchemaRename, studentRecordRename));
+    Assertions.assertEquals("person1", studentRecordRename.get("fn"));
+    Assertions.assertEquals("person2",  studentRecordRename.get("ln"));
+    Assertions.assertEquals("person3", ((GenericRecord) studentRecordRename.get("ss")).get("fnn"));
+    Assertions.assertEquals("person4",  ((GenericRecord) studentRecordRename.get("ss")).get("lnn"));
+  }
+
+  @Test
+  public void testReWriteAvroRecordWithSwappedNames() {
+    Schema origNestedSchema = new Schema.Parser().parse(SCHEMA_WITH_NESTED_FIELD_STR);
+    GenericRecord rec = new GenericData.Record(origNestedSchema);
+    rec.put("firstname", "John");
+    rec.put("lastname",  "Doe");
+
+    GenericRecord student = new GenericData.Record(
+        origNestedSchema.getField("student").schema());
+    student.put("firstnameNested", "Albert");
+    student.put("lastnameNested",  "Einstein");
+    rec.put("student", student);
+
+    final String SCHEMA_WITH_SWAPPED_NAMES =
+        "{\"name\":\"MyClass\",\"type\":\"record\",\"namespace\":\"com.acme.avro\",\"fields\":["
+            +   "{\"name\":\"fn\",\"type\":\"string\"},"
+            +   "{\"name\":\"firstname\",\"type\":\"string\"},"
+            +   "{\"name\":\"student\",\"type\":{\"name\":\"student\",\"type\":\"record\",\"fields\":["
+            +     "{\"name\":\"fnn\",\"type\":[\"null\",\"string\"],\"default\":null},"
+            +     "{\"name\":\"firstnameNested\",\"type\":[\"null\",\"string\"],\"default\":null}]}}]}";
+
+    Schema swappedSchema = new Schema.Parser().parse(SCHEMA_WITH_SWAPPED_NAMES);
+
+    Map<String, String> renames = new HashMap<>();
+    renames.put("fn", "firstname");
+    renames.put("firstname", "lastname");
+    renames.put("student.fnn", "firstnameNested");
+    renames.put("student.firstnameNested", "lastnameNested");
+
+    GenericRecord rewritten = HoodieAvroUtils.rewriteRecordWithNewSchema(rec, swappedSchema, renames);
+    Assertions.assertTrue(GenericData.get().validate(swappedSchema, rewritten));
+    Assertions.assertEquals("Albert", ((GenericRecord) rewritten.get("student")).get("fnn"));
+    Assertions.assertEquals("Einstein",  ((GenericRecord) rewritten.get("student")).get("firstnameNested"));
+    Assertions.assertEquals("John", rewritten.get("fn"));
+    Assertions.assertEquals("Doe",  rewritten.get("firstname"));
   }
 
   @Test
@@ -640,6 +776,8 @@ public class TestHoodieAvroUtils {
     BinaryDecoder decoder = DecoderFactory.get().binaryDecoder(data, 0, data.length, null);
     GenericRecord deserializedRecord = reader.read(null, decoder);
     Map<String, Object> fieldValueMapping = deserializedRecord.getSchema().getFields().stream()
+        // filtering out nested record field
+        .filter(field -> !field.schema().getType().equals(Schema.Type.UNION))
         .collect(Collectors.toMap(
             Schema.Field::name,
             field -> deserializedRecord.get(field.name())
@@ -832,8 +970,18 @@ public class TestHoodieAvroUtils {
   void testCreateFullName() {
     String result = HoodieAvroUtils.createFullName(new ArrayDeque<>(Arrays.asList("a", "b", "c")));
     String resultSingle = HoodieAvroUtils.createFullName(new ArrayDeque<>(Collections.singletonList("a")));
+    String resultEmpty = HoodieAvroUtils.createFullName(new ArrayDeque<>());
     assertEquals("c.b.a", result);
     assertEquals("a", resultSingle);
+    assertEquals("", resultEmpty);
+  }
+
+  @Test
+  public void testCreateNamePrefix() {
+    assertNull(HoodieAvroUtils.createNamePrefix(true, new ArrayDeque<>(Collections.singletonList("field1"))));
+    assertEquals("field1", HoodieAvroUtils.createNamePrefix(false, new ArrayDeque<>(Collections.singletonList("field1"))));
+    assertNull(HoodieAvroUtils.createNamePrefix(false, new ArrayDeque<>()));
+    assertEquals("parent.child", HoodieAvroUtils.createNamePrefix(false, new ArrayDeque<>(Arrays.asList("child", "parent"))));
   }
 
   @ParameterizedTest
@@ -928,6 +1076,351 @@ public class TestHoodieAvroUtils {
   public void testGetSchemaForFieldNested(String colName, Schema.Type schemaType) {
     Pair<String, Schema.Field> actualColNameAndSchemaFile = HoodieAvroUtils.getSchemaForField(SCHEMA_WITH_NESTED_FIELD_LARGE, colName);
     assertEquals(colName, actualColNameAndSchemaFile.getKey());
-    assertEquals(schemaType, resolveNullableSchema(actualColNameAndSchemaFile.getValue().schema()).getType());
+    assertEquals(schemaType, getNonNullTypeFromUnion(actualColNameAndSchemaFile.getValue().schema()).getType());
+  }
+
+  public static Stream<Arguments> getExpectedSchemaForFields() {
+    // Projection of two nested fields. secondLevelField is entirely projected since both its fields are included
+    List<String> fields1 = Arrays.asList("nested_field.secondLevelField.firstname", "nested_field.secondLevelField.lastname");
+    // Expected schema - top level field and one nested field
+    String expectedSchema1 =
+        "{\n"
+            + "  \"type\": \"record\",\n"
+            + "  \"name\": \"MyClass\",\n"
+            + "  \"doc\": \"\",\n"
+            + "  \"namespace\": \"com.acme.avro\",\n"
+            + "  \"fields\": [\n"
+            + "    { \"name\": \"nested_field\", \"type\": [\"null\", {\n"
+            + "      \"type\": \"record\",\n"
+            + "      \"name\": \"TestRecordAvroTypes\",\n"
+            + "      \"fields\": [\n"
+            + "        { \"name\": \"secondLevelField\", \"type\": [\"null\", {\n"
+            + "          \"type\": \"record\",\n"
+            + "          \"name\": \"secondLevelField\",\n"
+            + "          \"fields\": [\n"
+            + "            { \"name\": \"firstname\", \"type\": [\"null\", \"string\"], \"default\": null },\n"
+            + "            { \"name\": \"lastname\", \"type\": [\"null\", \"string\"], \"default\": null }\n"
+            + "          ]\n"
+            + "        }], \"default\": null }\n"
+            + "      ]\n"
+            + "    }], \"default\": null }\n"
+            + "  ]\n"
+            + "}";
+
+    // Projection of first level nested field and top level field which contains the nested field
+    // Also include the nested field twice
+    // Expected schema - top level field
+    List<String> fields2 = Arrays.asList("nested_field.secondLevelField.lastname", "nested_field",
+        "nested_field.secondLevelField.lastname");
+    String expectedSchema2 =
+        "{\n"
+            + "  \"type\": \"record\",\n"
+            + "  \"name\": \"MyClass\",\n"
+            + "  \"doc\": \"\",\n"
+            + "  \"namespace\": \"com.acme.avro\",\n"
+            + "  \"fields\": [\n"
+            + "    { \"name\": \"nested_field\", \"type\": [\"null\", " + SCHEMA_WITH_AVRO_TYPES_STR + "], \"default\": null }\n"
+            + "  ]\n"
+            + "}";
+
+    // Projection of non overlapping nested field and top level field with nested fields
+    // Expected schema - top level field and one nested field
+    List<String> fields3 = Arrays.asList("student.lastname", "nested_field");
+    String expectedSchema3 =
+        "{\n"
+            + "  \"type\": \"record\",\n"
+            + "  \"name\": \"MyClass\",\n"
+            + "  \"doc\": \"\",\n"
+            + "  \"namespace\": \"com.acme.avro\",\n"
+            + "  \"fields\": [\n"
+            + "    { \"name\": \"nested_field\", \"type\": [\"null\", " + SCHEMA_WITH_AVRO_TYPES_STR + "], \"default\": null },\n"
+            + "    { \"name\": \"student\", \"type\": {\n"
+            + "      \"type\": \"record\",\n"
+            + "      \"name\": \"student\",\n"
+            + "      \"fields\": [\n"
+            + "        { \"name\": \"lastname\", \"type\": [\"null\", \"string\"], \"default\": null }\n"
+            + "      ]\n"
+            + "    }}\n"
+            + "  ]\n"
+            + "}";
+
+    // Projection of two nested fields
+    // Expected schema - two nested fields
+    List<String> fields4 = Arrays.asList("student.lastname", "nested_field.secondLevelField.lastname");
+    String expectedSchema4 =
+        "{\n"
+            + "  \"type\": \"record\",\n"
+            + "  \"name\": \"MyClass\",\n"
+            + "  \"doc\": \"\",\n"
+            + "  \"namespace\": \"com.acme.avro\",\n"
+            + "  \"fields\": [\n"
+            + "    { \"name\": \"nested_field\", \"type\": [\"null\", {\n"
+            + "      \"type\": \"record\",\n"
+            + "      \"name\": \"TestRecordAvroTypes\",\n"
+            + "      \"fields\": [\n"
+            + "        { \"name\": \"secondLevelField\", \"type\": [\"null\", {\n"
+            + "          \"type\": \"record\",\n"
+            + "          \"name\": \"secondLevelField\",\n"
+            + "          \"fields\": [\n"
+            + "            { \"name\": \"lastname\", \"type\": [\"null\", \"string\"], \"default\": null }\n"
+            + "          ]\n"
+            + "        }], \"default\": null }\n"
+            + "      ]\n"
+            + "    }], \"default\": null },\n"
+            + "    { \"name\": \"student\", \"type\": {\n"
+            + "      \"type\": \"record\",\n"
+            + "      \"name\": \"student\",\n"
+            + "      \"namespace\": \"com.acme.avro\","
+            + "      \"fields\": [\n"
+            + "        { \"name\": \"lastname\", \"type\": [\"null\", \"string\"], \"default\": null }\n"
+            + "      ]\n"
+            + "    }}\n"
+            + "  ]\n"
+            + "}";
+
+    // Projection of top level field and nested field column
+    List<String> fields5 = Arrays.asList("firstname", "nested_field.secondLevelField.lastname", "nested_field.longField");
+    // Expected schema - top level field and one nested field
+    String expectedSchema5 =
+        "{\n"
+            + "  \"type\": \"record\",\n"
+            + "  \"name\": \"MyClass\",\n"
+            + "  \"doc\": \"\",\n"
+            + "  \"namespace\": \"com.acme.avro\",\n"
+            + "  \"fields\": [\n"
+            + "    { \"name\": \"firstname\", \"type\": \"string\" },\n"
+            + "    { \"name\": \"nested_field\", \"type\": [\"null\", {\n"
+            + "      \"type\": \"record\",\n"
+            + "      \"name\": \"TestRecordAvroTypes\",\n"
+            + "      \"fields\": [\n"
+            + "        { \"name\": \"longField\", \"type\": \"long\" },\n"
+            + "        { \"name\": \"secondLevelField\", \"type\": [\"null\", {\n"
+            + "          \"type\": \"record\",\n"
+            + "          \"name\": \"secondLevelField\",\n"
+            + "          \"fields\": [\n"
+            + "            { \"name\": \"lastname\", \"type\": [\"null\", \"string\"], \"default\": null }\n"
+            + "          ]\n"
+            + "        }], \"default\": null }\n"
+            + "      ]\n"
+            + "    }], \"default\": null }\n"
+            + "  ]\n"
+            + "}";
+
+    Object[][] data = new Object[][] {
+        {fields1, expectedSchema1},
+        {fields2, expectedSchema2},
+        {fields3, expectedSchema3},
+        {fields4, expectedSchema4},
+        {fields5, expectedSchema5}};
+    return Stream.of(data).map(Arguments::of);
+  }
+
+  @ParameterizedTest
+  @MethodSource("getExpectedSchemaForFields")
+  public void testProjectSchemaWithNullableAndNestedFields(List<String> projectedFields, String expectedSchemaStr) {
+    Schema expectedSchema = Schema.parse(expectedSchemaStr);
+    Schema projectedSchema = HoodieAvroUtils.projectSchema(SCHEMA_WITH_NESTED_FIELD_LARGE, projectedFields);
+    assertEquals(expectedSchema, projectedSchema);
+    assertTrue(AvroSchemaUtils.isSchemaCompatible(projectedSchema, expectedSchema, false));
+  }
+
+  private static Stream<Arguments> recordNeedsRewriteForExtendedAvroTypePromotion() {
+    Schema decimal1 = LogicalTypes.decimal(12, 2).addToSchema(Schema.create(Schema.Type.BYTES));
+    Schema decimal2 = LogicalTypes.decimal(10, 2).addToSchema(Schema.create(Schema.Type.BYTES));
+    Schema dateSchema = LogicalTypes.date().addToSchema(Schema.create(Schema.Type.INT));
+    Schema doubleSchema = Schema.create(Schema.Type.DOUBLE);
+    Schema intSchema = Schema.create(Schema.Type.INT);
+    Schema longSchema = Schema.create(Schema.Type.LONG);
+    Schema floatSchema = Schema.create(Schema.Type.FLOAT);
+    Schema stringSchema = Schema.create(Schema.Type.STRING);
+
+    Schema recordSchema1 = Schema.createRecord("record1", null, "com.example", false,
+        Arrays.asList(new Schema.Field("decimalField", decimal1, null, null),
+            new Schema.Field("doubleField", doubleSchema, null, null)));
+
+    Schema recordSchema2 = Schema.createRecord("record2", null, "com.example2", false,
+        Arrays.asList(new Schema.Field("decimalField", decimal1, null, null),
+            new Schema.Field("doubleField", doubleSchema, null, null)));
+
+    return Stream.of(
+        Arguments.of(intSchema, longSchema, false),
+        Arguments.of(intSchema, floatSchema, false),
+        Arguments.of(longSchema, intSchema, true),
+        Arguments.of(longSchema, floatSchema, false),
+        Arguments.of(decimal1, decimal2, true),
+        Arguments.of(doubleSchema, decimal1, true),
+        Arguments.of(decimal1, doubleSchema, true),
+        Arguments.of(intSchema, stringSchema, true),
+        Arguments.of(longSchema, doubleSchema, false),
+        Arguments.of(intSchema, doubleSchema, false),
+        Arguments.of(longSchema, stringSchema, true),
+        Arguments.of(floatSchema, stringSchema, true),
+        Arguments.of(doubleSchema, stringSchema, true),
+        Arguments.of(decimal1, stringSchema, true),
+        Arguments.of(stringSchema, decimal2, true),
+        Arguments.of(stringSchema, intSchema, true),
+        Arguments.of(floatSchema, doubleSchema, true),
+        Arguments.of(doubleSchema, floatSchema, true),
+        Arguments.of(recordSchema1, recordSchema2, false),
+        Arguments.of(dateSchema, stringSchema, true)
+    );
+  }
+
+  @ParameterizedTest
+  @MethodSource
+  void recordNeedsRewriteForExtendedAvroTypePromotion(Schema writerSchema, Schema readerSchema, boolean expected) {
+    boolean result = HoodieAvroUtils.recordNeedsRewriteForExtendedAvroTypePromotion(writerSchema, readerSchema);
+    assertEquals(expected, result);
+  }
+
+  /**
+   * Utility class for generating random GenericRecord instances.
+   */
+  private static class AvroTestUtils {
+    private static final Random RANDOM = new Random(42);
+
+    /**
+     * Generate a list of random GenericRecord instances
+     */
+    public static List<GenericRecord> generateRandomRecords(Schema schema, int count) {
+      List<GenericRecord> records = new ArrayList<>();
+      for (int i = 0; i < count; i++) {
+        records.add(generateRandomRecord(schema));
+      }
+      return records;
+    }
+
+    /**
+     * Generate a random GenericRecord for the given schema
+     */
+    public static GenericRecord generateRandomRecord(Schema schema) {
+      GenericRecord record = new GenericData.Record(schema);
+      for (Schema.Field field : schema.getFields()) {
+        Object value = generateRandomValue(field.schema(), field.defaultVal());
+        record.put(field.pos(), value);
+      }
+      return record;
+    }
+
+    /**
+     * Generate a random value for the given schema type
+     */
+    private static Object generateRandomValue(Schema schema, Object defaultValue) {
+      // CASE 1: Handle default value
+      if (defaultValue != null
+          && !(defaultValue instanceof JsonProperties.Null)
+          && RANDOM.nextBoolean()) {
+        return defaultValue;
+      }
+      // Handle Union type.
+      Schema actualSchema = schema;
+      try {
+        actualSchema = getNonNullTypeFromUnion(schema);
+      } catch (Exception e) {
+        // If we can't resolve the schema, just use the original
+        // Op.
+      }
+      // CASE 2: Handle different types
+      switch (actualSchema.getType()) {
+        case NULL:
+          return null;
+        case BOOLEAN:
+          return RANDOM.nextBoolean();
+        case INT:
+          return RANDOM.nextInt(1000);
+        case LONG:
+          return RANDOM.nextLong() % 1000000L;
+        case FLOAT:
+          return RANDOM.nextFloat() * 100f;
+        case DOUBLE:
+          return RANDOM.nextDouble() * 1000.0;
+        case STRING:
+          return "test_string_" + RANDOM.nextInt(1000);
+        case BYTES:
+          byte[] bytes = new byte[RANDOM.nextInt(10) + 1];
+          RANDOM.nextBytes(bytes);
+          return ByteBuffer.wrap(bytes);
+        case RECORD:
+          return generateRandomRecord(actualSchema);
+        case ENUM:
+          List<String> symbols = actualSchema.getEnumSymbols();
+          return new GenericData.EnumSymbol(actualSchema, symbols.get(RANDOM.nextInt(symbols.size())));
+        case ARRAY:
+          List<Object> array = new ArrayList<>();
+          int arraySize = RANDOM.nextInt(3) + 1;
+          for (int i = 0; i < arraySize; i++) {
+            array.add(generateRandomValue(actualSchema.getElementType(), null));
+          }
+          return array;
+        case MAP:
+          Map<String, Object> map = new HashMap<>();
+          int mapSize = RANDOM.nextInt(3) + 1;
+          for (int i = 0; i < mapSize; i++) {
+            map.put("key_" + i, generateRandomValue(actualSchema.getValueType(), null));
+          }
+          return map;
+        case FIXED:
+          byte[] fixedBytes = new byte[actualSchema.getFixedSize()];
+          RANDOM.nextBytes(fixedBytes);
+          return new GenericData.Fixed(actualSchema, fixedBytes);
+        default:
+          return null;
+      }
+    }
+  }
+
+  /**
+   * Test convertToSpecificRecord with multiple random records for each type
+   */
+  @ParameterizedTest
+  @MethodSource("provideAvroModelClasses")
+  void testConvertToSpecificRecordMultipleRecords(Class<? extends SpecificRecord> recordClass) {
+    Schema schema = SpecificData.get().getSchema(recordClass);
+    List<GenericRecord> genericRecords = AvroTestUtils.generateRandomRecords(schema, 3);
+    for (GenericRecord genericRecord : genericRecords) {
+      Class<? extends SpecificRecordBase> specificRecordBaseClass
+          = (Class<? extends SpecificRecordBase>) recordClass;
+      SpecificRecord specificRecord =
+          HoodieAvroUtils.convertToSpecificRecord(specificRecordBaseClass, genericRecord);
+      assertEquals(recordClass, specificRecord.getClass());
+      GenericRecord copied = (GenericRecord) GenericData.get().deepCopy(schema, specificRecord);
+      assertEquals(genericRecord, copied);
+    }
+  }
+
+  /**
+   * Provide all the Avro model classes to test
+   */
+  static Stream<Arguments> provideAvroModelClasses() {
+    return Stream.of(
+        Arguments.of(HoodieRollbackPartitionMetadata.class),
+        Arguments.of(HoodieSavepointPartitionMetadata.class),
+        Arguments.of(HoodieWriteStat.class),
+        Arguments.of(HoodieCleanPartitionMetadata.class),
+        Arguments.of(HoodieCleanFileInfo.class),
+        Arguments.of(HoodieActionInstant.class),
+        Arguments.of(HoodieCompactionStrategy.class),
+        Arguments.of(HoodieCompactionOperation.class),
+        Arguments.of(HoodieFSPermission.class),
+        Arguments.of(HoodiePath.class),
+        Arguments.of(HoodieFileStatus.class),
+        Arguments.of(HoodieBootstrapFilePartitionInfo.class),
+        Arguments.of(HoodieCompactionPlan.class),
+        Arguments.of(HoodieCleanerPlan.class),
+        Arguments.of(HoodieCleanMetadata.class),
+        Arguments.of(HoodieReplaceCommitMetadata.class),
+        Arguments.of(HoodieSavepointMetadata.class),
+        Arguments.of(HoodieMergeArchiveFilePlan.class),
+        Arguments.of(HoodieRollbackMetadata.class),
+        Arguments.of(HoodieBootstrapPartitionMetadata.class),
+        Arguments.of(HoodieBootstrapIndexInfo.class),
+        Arguments.of(HoodieIndexPlan.class),
+        Arguments.of(HoodieRequestedReplaceMetadata.class),
+        Arguments.of(HoodieRestoreMetadata.class),
+        Arguments.of(HoodieRestorePlan.class),
+        Arguments.of(HoodieRollbackPlan.class),
+        Arguments.of(HoodieDeleteRecordList.class),
+        Arguments.of(HoodieCommitMetadata.class)
+    );
   }
 }
