@@ -86,7 +86,7 @@ public class TestHoodieLogFile {
                                  HoodieLogFile hoodieLogFile,
                                  long fileLength, String suffix) {
     assertEquals(fileId, hoodieLogFile.getFileId());
-    assertEquals(baseCommitTime, hoodieLogFile.getBaseCommitTime());
+    assertEquals(baseCommitTime, hoodieLogFile.getDeltaCommitTime());
     assertEquals(logVersion, hoodieLogFile.getLogVersion());
     assertEquals(writeToken, hoodieLogFile.getLogWriteToken());
     assertEquals(fileExtension, hoodieLogFile.getFileExtension());
