@@ -19,7 +19,9 @@
 
 package org.apache.hudi.common.util;
 
-import org.apache.hudi.common.model.HoodieColumnRangeMetadata;
+import org.apache.hudi.metadata.HoodieIndexVersion;
+import org.apache.hudi.stats.HoodieColumnRangeMetadata;
+import org.apache.hudi.stats.ValueMetadata;
 
 import org.junit.jupiter.api.Test;
 
@@ -36,14 +38,15 @@ public class TestBaseFileUtils {
 
   @Test
   public void testGetColumnRangeInPartition() {
+    HoodieIndexVersion indexVersion = HoodieIndexVersion.V1;
     // Step 1: Set Up Test Data
     HoodieColumnRangeMetadata<Comparable> fileColumnRange1 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file1", COLUMN_NAME, 1, 5, 0, 10, 100, 200);
+        "path/to/file1", COLUMN_NAME, 1, 5, 0, 10, 100, 200, ValueMetadata.V1EmptyMetadata.get());
     HoodieColumnRangeMetadata<Comparable> fileColumnRange2 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file2", COLUMN_NAME, 3, 8, 1, 15, 120, 250);
+        "path/to/file2", COLUMN_NAME, 3, 8, 1, 15, 120, 250, ValueMetadata.V1EmptyMetadata.get());
     List<HoodieColumnRangeMetadata<Comparable>> fileColumnRanges = Arrays.asList(fileColumnRange1, fileColumnRange2);
     // Step 2: Call the Method
-    HoodieColumnRangeMetadata<Comparable> result = FileFormatUtils.getColumnRangeInPartition(PARTITION_PATH, fileColumnRanges, Collections.emptyMap());
+    HoodieColumnRangeMetadata<Comparable> result = FileFormatUtils.getColumnRangeInPartition(PARTITION_PATH, COLUMN_NAME, fileColumnRanges, Collections.emptyMap(), indexVersion);
     // Step 3: Assertions
     assertEquals(PARTITION_PATH, result.getFilePath());
     assertEquals(COLUMN_NAME, result.getColumnName());
@@ -57,15 +60,16 @@ public class TestBaseFileUtils {
 
   @Test
   public void testGetColumnRangeInPartitionWithNullMinMax() {
+    HoodieIndexVersion indexVersion = HoodieIndexVersion.V1;
     // Step 1: Set Up Test Data
     HoodieColumnRangeMetadata<Comparable> fileColumnRange1 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file1", COLUMN_NAME, 1, null, 0, 10, 100, 200);
+        "path/to/file1", COLUMN_NAME, 1, null, 0, 10, 100, 200, ValueMetadata.V1EmptyMetadata.get());
     HoodieColumnRangeMetadata<Comparable> fileColumnRange2 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file2", COLUMN_NAME, null, 8, 1, 15, 120, 250);
+        "path/to/file2", COLUMN_NAME, null, 8, 1, 15, 120, 250, ValueMetadata.V1EmptyMetadata.get());
 
     List<HoodieColumnRangeMetadata<Comparable>> fileColumnRanges = Arrays.asList(fileColumnRange1, fileColumnRange2);
     // Step 2: Call the Method
-    HoodieColumnRangeMetadata<Comparable> result = FileFormatUtils.getColumnRangeInPartition(PARTITION_PATH, fileColumnRanges, Collections.emptyMap());
+    HoodieColumnRangeMetadata<Comparable> result = FileFormatUtils.getColumnRangeInPartition(PARTITION_PATH, COLUMN_NAME, fileColumnRanges, Collections.emptyMap(), indexVersion);
     // Step 3: Assertions
     assertEquals(PARTITION_PATH, result.getFilePath());
     assertEquals(COLUMN_NAME, result.getColumnName());
@@ -79,14 +83,15 @@ public class TestBaseFileUtils {
 
   @Test
   public void testGetColumnRangeInPartitionWithDifferentColumnNameThrowsException() {
+    HoodieIndexVersion indexVersion = HoodieIndexVersion.V1;
     // Step 1: Set Up Test Data
     HoodieColumnRangeMetadata<Comparable> fileColumnRange1 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file1", "columnName1", 1, null, 0, 10, 100, 200);
+        "path/to/file1", "columnName1", 1, null, 0, 10, 100, 200, ValueMetadata.V1EmptyMetadata.get());
     HoodieColumnRangeMetadata<Comparable> fileColumnRange2 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file2", "columnName2", null, 8, 1, 15, 120, 250);
+        "path/to/file2", "columnName2", null, 8, 1, 15, 120, 250, ValueMetadata.V1EmptyMetadata.get());
     List<HoodieColumnRangeMetadata<Comparable>> fileColumnRanges = Arrays.asList(fileColumnRange1, fileColumnRange2);
     // Step 2: Call the Method
-    assertThrows(IllegalArgumentException.class, () -> FileFormatUtils.getColumnRangeInPartition(PARTITION_PATH, fileColumnRanges,
-        Collections.emptyMap()));
+    assertThrows(IllegalArgumentException.class, () -> FileFormatUtils.getColumnRangeInPartition(PARTITION_PATH, COLUMN_NAME, fileColumnRanges,
+        Collections.emptyMap(), indexVersion));
   }
 }
