@@ -18,11 +18,11 @@
 
 package org.apache.hudi.examples.quickstart;
 
+import org.apache.hudi.common.model.HoodieTableType;
+
 import org.apache.flink.test.util.AbstractTestBase;
 import org.apache.flink.types.Row;
-import org.apache.hudi.common.model.HoodieTableType;
 import org.junit.jupiter.api.BeforeEach;
-import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.EnumSource;
@@ -46,7 +46,6 @@ public class TestHoodieFlinkQuickstart extends AbstractTestBase {
   @TempDir
   File tempFile;
 
-  @Disabled
   @ParameterizedTest
   @EnumSource(value = HoodieTableType.class)
   void testHoodieFlinkQuickstart(HoodieTableType tableType) throws Exception {
