@@ -21,12 +21,11 @@ package org.apache.hudi.examples.quickstart;
 import org.apache.hudi.QuickstartUtils;
 import org.apache.hudi.common.model.HoodieAvroPayload;
 import org.apache.hudi.common.model.WriteOperationType;
-import org.apache.hudi.config.HoodieWriteConfig;
+import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.examples.common.HoodieExampleDataGenerator;
 import org.apache.hudi.examples.common.HoodieExampleSparkUtils;
 import org.apache.hudi.keygen.constant.KeyGeneratorOptions;
 
-import org.apache.spark.SparkConf;
 import org.apache.spark.api.java.JavaSparkContext;
 import org.apache.spark.api.java.function.Function;
 import org.apache.spark.sql.Dataset;
@@ -53,9 +52,8 @@ public final class HoodieSparkQuickstart {
     String tableName = args[1];
 
     SparkSession spark = HoodieExampleSparkUtils.defaultSparkSession("Hudi Spark basic example");
-    SparkConf sparkConf = HoodieExampleSparkUtils.defaultSparkConf("hoodie-client-example");
 
-    try (JavaSparkContext jsc = new JavaSparkContext(sparkConf)) {
+    try (JavaSparkContext jsc = new JavaSparkContext(spark.sparkContext())) {
       runQuickstart(jsc, spark, tableName, tablePath);
     }
   }
@@ -117,7 +115,7 @@ public final class HoodieSparkQuickstart {
 
     df.write().format("hudi")
         .options(QuickstartUtils.getQuickstartWriteConfigs())
-        .option(HoodieWriteConfig.PRECOMBINE_FIELD_NAME.key(), "ts")
+        .option(HoodieTableConfig.ORDERING_FIELDS.key(), "ts")
         .option(KeyGeneratorOptions.RECORDKEY_FIELD_NAME.key(), "uuid")
         .option(KeyGeneratorOptions.PARTITIONPATH_FIELD_NAME.key(), "partitionpath")
         .option(TBL_NAME.key(), tableName)
@@ -138,7 +136,7 @@ public final class HoodieSparkQuickstart {
     df.write().format("hudi")
         .options(QuickstartUtils.getQuickstartWriteConfigs())
         .option("hoodie.datasource.write.operation", WriteOperationType.INSERT_OVERWRITE.name())
-        .option(HoodieWriteConfig.PRECOMBINE_FIELD_NAME.key(), "ts")
+        .option(HoodieTableConfig.ORDERING_FIELDS.key(), "ts")
         .option(KeyGeneratorOptions.RECORDKEY_FIELD_NAME.key(), "uuid")
         .option(KeyGeneratorOptions.PARTITIONPATH_FIELD_NAME.key(), "partitionpath")
         .option(TBL_NAME.key(), tableName)
@@ -185,7 +183,7 @@ public final class HoodieSparkQuickstart {
     Dataset<Row> df = spark.read().json(jsc.parallelize(updates, 1));
     df.write().format("hudi")
         .options(QuickstartUtils.getQuickstartWriteConfigs())
-        .option(HoodieWriteConfig.PRECOMBINE_FIELD_NAME.key(), "ts")
+        .option(HoodieTableConfig.ORDERING_FIELDS.key(), "ts")
         .option(KeyGeneratorOptions.RECORDKEY_FIELD_NAME.key(), "uuid")
         .option(KeyGeneratorOptions.PARTITIONPATH_FIELD_NAME.key(), "partitionpath")
         .option(TBL_NAME.key(), tableName)
@@ -206,7 +204,7 @@ public final class HoodieSparkQuickstart {
 
     df.write().format("hudi")
         .options(QuickstartUtils.getQuickstartWriteConfigs())
-        .option(HoodieWriteConfig.PRECOMBINE_FIELD_NAME.key(), "ts")
+        .option(HoodieTableConfig.ORDERING_FIELDS.key(), "ts")
         .option(KeyGeneratorOptions.RECORDKEY_FIELD_NAME.key(), "uuid")
         .option(KeyGeneratorOptions.PARTITIONPATH_FIELD_NAME.key(), "partitionpath")
         .option(TBL_NAME.key(), tableName)
@@ -223,7 +221,7 @@ public final class HoodieSparkQuickstart {
     Dataset<Row> df = spark.emptyDataFrame();
     df.write().format("hudi")
         .options(QuickstartUtils.getQuickstartWriteConfigs())
-        .option(HoodieWriteConfig.PRECOMBINE_FIELD_NAME.key(), "ts")
+        .option(HoodieTableConfig.ORDERING_FIELDS.key(), "ts")
         .option(KeyGeneratorOptions.RECORDKEY_FIELD_NAME.key(), "uuid")
         .option(KeyGeneratorOptions.PARTITIONPATH_FIELD_NAME.key(), "partitionpath")
         .option(TBL_NAME.key(), tableName)
