@@ -66,7 +66,7 @@ public class HoodieWriteClientExample {
 
   private static final Logger LOG = LoggerFactory.getLogger(HoodieWriteClientExample.class);
 
-  private static final String TABLE_TYPE = HoodieTableType.COPY_ON_WRITE.name();
+  private static String tableType = HoodieTableType.MERGE_ON_READ.name();
 
   public static void main(String[] args) throws Exception {
     if (args.length < 2) {
@@ -87,7 +87,7 @@ public class HoodieWriteClientExample {
       FileSystem fs = HadoopFSUtils.getFs(tablePath, jsc.hadoopConfiguration());
       if (!fs.exists(path)) {
         HoodieTableMetaClient.newTableBuilder()
-            .setTableType(TABLE_TYPE)
+            .setTableType(tableType)
             .setTableName(tableName)
             .setPayloadClass(HoodieAvroPayload.class)
             .initTable(HadoopFSUtils.getStorageConfWithCopy(jsc.hadoopConfiguration()), tablePath);
@@ -129,8 +129,7 @@ public class HoodieWriteClientExample {
         client.delete(deleteRecords, newCommitTime);
 
         // Delete by partition
-        newCommitTime = client.startCommit();
-        client.startCommitWithTime(newCommitTime, HoodieTimeline.REPLACE_COMMIT_ACTION);
+        newCommitTime = client.startCommit(HoodieTimeline.REPLACE_COMMIT_ACTION);
         LOG.info("Starting commit " + newCommitTime);
         // The partition where the data needs to be deleted
         List<String> partitionList = toBeDeleted.stream().map(s -> s.getPartitionPath()).distinct().collect(Collectors.toList());
@@ -139,10 +138,10 @@ public class HoodieWriteClientExample {
         client.deletePartitions(deleteList, newCommitTime);
 
         // compaction
-        if (HoodieTableType.valueOf(TABLE_TYPE) == HoodieTableType.MERGE_ON_READ) {
+        if (HoodieTableType.valueOf(tableType) == HoodieTableType.MERGE_ON_READ) {
           Option<String> instant = client.scheduleCompaction(Option.empty());
           HoodieWriteMetadata<JavaRDD<WriteStatus>> compactionMetadata = client.compact(instant.get());
-          client.commitCompaction(instant.get(), compactionMetadata.getCommitMetadata().get(), Option.empty());
+          client.commitCompaction(instant.get(), compactionMetadata, Option.empty());
         }
       }
     }
