@@ -20,6 +20,7 @@ package org.apache.hudi.examples.quickstart;
 
 import org.apache.hudi.client.SparkRDDReadClient;
 import org.apache.hudi.client.common.HoodieSparkEngineContext;
+import org.apache.hudi.testutils.SparkClientFunctionalTestHarness;
 import org.apache.hudi.testutils.providers.SparkProvider;
 
 import org.apache.spark.HoodieSparkKryoRegistrar$;
@@ -56,6 +57,11 @@ public class TestHoodieSparkQuickstart implements SparkProvider {
     return spark;
   }
 
+  @Override
+  public SparkConf conf() {
+    return conf(SparkClientFunctionalTestHarness.getSparkSqlConf());
+  }
+
   @Override
   public SQLContext sqlContext() {
     return sqlContext;
