@@ -23,6 +23,7 @@ import org.apache.hudi.client.transaction.BucketIndexConcurrentFileWritesConflic
 import org.apache.hudi.client.transaction.ConflictResolutionStrategy;
 import org.apache.hudi.client.transaction.SimpleConcurrentFileWritesConflictResolutionStrategy;
 import org.apache.hudi.common.config.HoodieCommonConfig;
+import org.apache.hudi.common.config.HoodieStorageConfig;
 import org.apache.hudi.common.model.DefaultHoodieRecordPayload;
 import org.apache.hudi.common.model.HoodieFailedWritesCleaningPolicy;
 import org.apache.hudi.common.model.WriteConcurrencyMode;
@@ -30,20 +31,26 @@ import org.apache.hudi.common.model.WriteOperationType;
 import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.table.cdc.HoodieCDCSupplementalLoggingMode;
 import org.apache.hudi.common.table.timeline.TimelineUtils.HollowCommitHandling;
+import org.apache.hudi.common.util.Option;
+import org.apache.hudi.common.util.ReflectionUtils;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.config.HoodieCleanConfig;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.index.HoodieIndex;
+import org.apache.hudi.index.bucket.partition.PartitionBucketIndexUtils;
 import org.apache.hudi.keygen.constant.KeyGeneratorOptions;
 import org.apache.hudi.sink.overwrite.PartitionOverwriteMode;
 import org.apache.hudi.table.format.FilePathUtils;
+import org.apache.hudi.table.format.HoodieFlinkIOFactory;
 
+import org.apache.flink.api.common.functions.Partitioner;
 import org.apache.flink.configuration.ConfigOption;
 import org.apache.flink.configuration.Configuration;
 
 import java.lang.reflect.Field;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
@@ -54,11 +61,19 @@ import static org.apache.hudi.common.config.HoodieCommonConfig.INCREMENTAL_READ_
  * Tool helping to resolve the flink options {@link FlinkOptions}.
  */
 public class OptionsResolver {
+
+  /**
+   * Returns whether the current runtime mode is adaptive batch execution.
+   */
+  public static boolean isIncrementalJobGraph(Configuration conf) {
+    return conf.get(FlinkOptions.WRITE_INCREMENTAL_JOB_GRAPH_GENERATION);
+  }
+
   /**
    * Returns whether insert clustering is allowed with given configuration {@code conf}.
    */
   public static boolean insertClustering(Configuration conf) {
-    return isCowTable(conf) && isInsertOperation(conf) && conf.getBoolean(FlinkOptions.INSERT_CLUSTER);
+    return isCowTable(conf) && isInsertOperation(conf) && conf.get(FlinkOptions.INSERT_CLUSTER);
   }
 
   /**
@@ -67,14 +82,23 @@ public class OptionsResolver {
   public static boolean isAppendMode(Configuration conf) {
     // 1. inline clustering is supported for COW table;
     // 2. async clustering is supported for both COW and MOR table
-    return isInsertOperation(conf) && ((isCowTable(conf) && !conf.getBoolean(FlinkOptions.INSERT_CLUSTER)) || isMorTable(conf));
+    return isInsertOperation(conf) && ((isCowTable(conf) && !conf.get(FlinkOptions.INSERT_CLUSTER)) || isMorTable(conf));
+  }
+
+  /**
+   * Returns whether current index is partition level simple bucket index based on given configuration {@code conf}.
+   */
+  public static Boolean isPartitionLevelSimpleBucketIndex(Configuration conf) {
+    HoodieIndex.BucketIndexEngineType engineType = OptionsResolver.getBucketEngineType(conf);
+    return engineType.equals(HoodieIndex.BucketIndexEngineType.SIMPLE)
+        && PartitionBucketIndexUtils.isPartitionSimpleBucketIndex(HadoopConfigurations.getHadoopConf(conf), conf.get(FlinkOptions.PATH));
   }
 
   /**
    * Returns whether the table operation is 'insert'.
    */
   public static boolean isInsertOperation(Configuration conf) {
-    WriteOperationType operationType = WriteOperationType.fromValue(conf.getString(FlinkOptions.OPERATION));
+    WriteOperationType operationType = WriteOperationType.fromValue(conf.get(FlinkOptions.OPERATION));
     return operationType == WriteOperationType.INSERT;
   }
 
@@ -82,7 +106,7 @@ public class OptionsResolver {
    * Returns whether the table operation is 'upsert'.
    */
   public static boolean isUpsertOperation(Configuration conf) {
-    WriteOperationType operationType = WriteOperationType.fromValue(conf.getString(FlinkOptions.OPERATION));
+    WriteOperationType operationType = WriteOperationType.fromValue(conf.get(FlinkOptions.OPERATION));
     return operationType == WriteOperationType.UPSERT;
   }
 
@@ -90,7 +114,7 @@ public class OptionsResolver {
    * Returns whether the table operation is 'bulk_insert'.
    */
   public static boolean isBulkInsertOperation(Configuration conf) {
-    WriteOperationType operationType = WriteOperationType.fromValue(conf.getString(FlinkOptions.OPERATION));
+    WriteOperationType operationType = WriteOperationType.fromValue(conf.get(FlinkOptions.OPERATION));
     return operationType == WriteOperationType.BULK_INSERT;
   }
 
@@ -98,7 +122,7 @@ public class OptionsResolver {
    * Returns whether it is a MERGE_ON_READ table.
    */
   public static boolean isMorTable(Configuration conf) {
-    return conf.getString(FlinkOptions.TABLE_TYPE)
+    return conf.get(FlinkOptions.TABLE_TYPE)
         .toUpperCase(Locale.ROOT)
         .equals(FlinkOptions.TABLE_TYPE_MERGE_ON_READ);
   }
@@ -115,7 +139,7 @@ public class OptionsResolver {
    * Returns whether it is a COPY_ON_WRITE table.
    */
   public static boolean isCowTable(Configuration conf) {
-    return conf.getString(FlinkOptions.TABLE_TYPE)
+    return conf.get(FlinkOptions.TABLE_TYPE)
         .toUpperCase(Locale.ROOT)
         .equals(FlinkOptions.TABLE_TYPE_COPY_ON_WRITE);
   }
@@ -124,23 +148,23 @@ public class OptionsResolver {
    * Returns whether the payload clazz is {@link DefaultHoodieRecordPayload}.
    */
   public static boolean isDefaultHoodieRecordPayloadClazz(Configuration conf) {
-    return conf.getString(FlinkOptions.PAYLOAD_CLASS_NAME).contains(DefaultHoodieRecordPayload.class.getSimpleName());
+    return conf.get(FlinkOptions.PAYLOAD_CLASS_NAME).contains(DefaultHoodieRecordPayload.class.getSimpleName());
   }
 
   /**
-   * Returns the preCombine field
+   * Returns the ordering fields as comma separated string
    * or null if the value is set as {@link FlinkOptions#NO_PRE_COMBINE}.
    */
-  public static String getPreCombineField(Configuration conf) {
-    final String preCombineField = conf.getString(FlinkOptions.PRECOMBINE_FIELD);
-    return preCombineField.equals(FlinkOptions.NO_PRE_COMBINE) ? null : preCombineField;
+  public static String getOrderingFieldsStr(Configuration conf) {
+    final String orderingFields = conf.get(FlinkOptions.ORDERING_FIELDS);
+    return orderingFields.equals(FlinkOptions.NO_PRE_COMBINE) ? null : orderingFields;
   }
 
   /**
    * Returns whether the compaction strategy is based on elapsed delta time.
    */
   public static boolean isDeltaTimeCompaction(Configuration conf) {
-    final String strategy = conf.getString(FlinkOptions.COMPACTION_TRIGGER_STRATEGY).toLowerCase(Locale.ROOT);
+    final String strategy = conf.get(FlinkOptions.COMPACTION_TRIGGER_STRATEGY).toLowerCase(Locale.ROOT);
     return FlinkOptions.TIME_ELAPSED.equals(strategy) || FlinkOptions.NUM_OR_TIME.equals(strategy);
   }
 
@@ -155,7 +179,7 @@ public class OptionsResolver {
    * Returns whether the table index is bucket index.
    */
   public static boolean isBucketIndexType(Configuration conf) {
-    return conf.getString(FlinkOptions.INDEX_TYPE).equalsIgnoreCase(HoodieIndex.IndexType.BUCKET.name());
+    return conf.get(FlinkOptions.INDEX_TYPE).equalsIgnoreCase(HoodieIndex.IndexType.BUCKET.name());
   }
 
   /**
@@ -198,9 +222,9 @@ public class OptionsResolver {
    * @return true if the source should emit changes.
    */
   public static boolean emitChangelog(Configuration conf) {
-    return conf.getBoolean(FlinkOptions.READ_AS_STREAMING) && conf.getBoolean(FlinkOptions.CHANGELOG_ENABLED)
-        || conf.getBoolean(FlinkOptions.READ_AS_STREAMING) && conf.getBoolean(FlinkOptions.CDC_ENABLED)
-        || isIncrementalQuery(conf) && conf.getBoolean(FlinkOptions.CDC_ENABLED);
+    return conf.get(FlinkOptions.READ_AS_STREAMING) && conf.get(FlinkOptions.CHANGELOG_ENABLED)
+        || conf.get(FlinkOptions.READ_AS_STREAMING) && conf.get(FlinkOptions.CDC_ENABLED)
+        || isIncrementalQuery(conf) && conf.get(FlinkOptions.CDC_ENABLED);
   }
 
   /**
@@ -209,7 +233,7 @@ public class OptionsResolver {
    * @return true if the source is read as streaming with changelog mode enabled.
    */
   public static boolean emitDeletes(Configuration conf) {
-    return conf.getBoolean(FlinkOptions.READ_AS_STREAMING) && conf.getBoolean(FlinkOptions.CHANGELOG_ENABLED);
+    return conf.get(FlinkOptions.READ_AS_STREAMING) && conf.get(FlinkOptions.CHANGELOG_ENABLED);
   }
 
   /**
@@ -219,7 +243,7 @@ public class OptionsResolver {
    */
   public static boolean needsAsyncCompaction(Configuration conf) {
     return OptionsResolver.isMorTable(conf)
-        && conf.getBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED);
+        && conf.get(FlinkOptions.COMPACTION_ASYNC_ENABLED);
   }
 
   /**
@@ -229,7 +253,7 @@ public class OptionsResolver {
    */
   public static boolean needsScheduleCompaction(Configuration conf) {
     return OptionsResolver.isMorTable(conf)
-        && conf.getBoolean(FlinkOptions.COMPACTION_SCHEDULE_ENABLED);
+        && conf.get(FlinkOptions.COMPACTION_SCHEDULE_ENABLED) && !isAppendMode(conf);
   }
 
   /**
@@ -238,7 +262,7 @@ public class OptionsResolver {
    * @param conf The flink configuration.
    */
   public static boolean needsAsyncClustering(Configuration conf) {
-    return isInsertOperation(conf) && conf.getBoolean(FlinkOptions.CLUSTERING_ASYNC_ENABLED);
+    return isInsertOperation(conf) && conf.get(FlinkOptions.CLUSTERING_ASYNC_ENABLED);
   }
 
   /**
@@ -247,10 +271,10 @@ public class OptionsResolver {
    * @param conf The flink configuration.
    */
   public static boolean needsScheduleClustering(Configuration conf) {
-    if (!conf.getBoolean(FlinkOptions.CLUSTERING_SCHEDULE_ENABLED)) {
+    if (!conf.get(FlinkOptions.CLUSTERING_SCHEDULE_ENABLED)) {
       return false;
     }
-    WriteOperationType operationType = WriteOperationType.fromValue(conf.getString(FlinkOptions.OPERATION));
+    WriteOperationType operationType = WriteOperationType.fromValue(conf.get(FlinkOptions.OPERATION));
     if (OptionsResolver.isConsistentHashingBucketIndexType(conf)) {
       // Write pipelines for table with consistent bucket index would detect whether clustering service occurs,
       // and automatically adjust the partitioner and write function if clustering service happens.
@@ -266,23 +290,23 @@ public class OptionsResolver {
    * Returns whether the clustering sort is enabled.
    */
   public static boolean sortClusteringEnabled(Configuration conf) {
-    return !StringUtils.isNullOrEmpty(conf.getString(FlinkOptions.CLUSTERING_SORT_COLUMNS));
+    return !StringUtils.isNullOrEmpty(conf.get(FlinkOptions.CLUSTERING_SORT_COLUMNS));
   }
 
   /**
    * Returns whether the operation is INSERT OVERWRITE (table or partition).
    */
   public static boolean isInsertOverwrite(Configuration conf) {
-    return conf.getString(FlinkOptions.OPERATION).equalsIgnoreCase(WriteOperationType.INSERT_OVERWRITE_TABLE.value())
-        || conf.getString(FlinkOptions.OPERATION).equalsIgnoreCase(WriteOperationType.INSERT_OVERWRITE.value());
+    return conf.get(FlinkOptions.OPERATION).equalsIgnoreCase(WriteOperationType.INSERT_OVERWRITE_TABLE.value())
+        || conf.get(FlinkOptions.OPERATION).equalsIgnoreCase(WriteOperationType.INSERT_OVERWRITE.value());
   }
 
   /**
    * Returns whether the operation is INSERT OVERWRITE dynamic partition.
    */
   public static boolean overwriteDynamicPartition(Configuration conf) {
-    return conf.getString(FlinkOptions.OPERATION).equalsIgnoreCase(WriteOperationType.INSERT_OVERWRITE.value())
-        || conf.getString(FlinkOptions.WRITE_PARTITION_OVERWRITE_MODE).equalsIgnoreCase(PartitionOverwriteMode.DYNAMIC.name());
+    return conf.get(FlinkOptions.OPERATION).equalsIgnoreCase(WriteOperationType.INSERT_OVERWRITE.value())
+        || conf.get(FlinkOptions.WRITE_PARTITION_OVERWRITE_MODE).equalsIgnoreCase(PartitionOverwriteMode.DYNAMIC.name());
   }
 
   /**
@@ -304,14 +328,14 @@ public class OptionsResolver {
    * Returns the read commits limit or -1 if not specified.
    */
   public static int getReadCommitsLimit(Configuration conf) {
-    return conf.getInteger(FlinkOptions.READ_COMMITS_LIMIT, -1);
+    return conf.getOptional(FlinkOptions.READ_COMMITS_LIMIT).orElse(-1);
   }
 
   /**
    * Returns the supplemental logging mode.
    */
   public static HoodieCDCSupplementalLoggingMode getCDCSupplementalLoggingMode(Configuration conf) {
-    String mode = conf.getString(FlinkOptions.SUPPLEMENTAL_LOGGING_MODE).toUpperCase();
+    String mode = conf.get(FlinkOptions.SUPPLEMENTAL_LOGGING_MODE).toUpperCase();
     return HoodieCDCSupplementalLoggingMode.valueOf(mode);
   }
 
@@ -319,7 +343,7 @@ public class OptionsResolver {
    * Returns whether comprehensive schema evolution enabled.
    */
   public static boolean isSchemaEvolutionEnabled(Configuration conf) {
-    return conf.getBoolean(HoodieCommonConfig.SCHEMA_EVOLUTION_ENABLE.key(), HoodieCommonConfig.SCHEMA_EVOLUTION_ENABLE.defaultValue());
+    return Boolean.parseBoolean(conf.getString(HoodieCommonConfig.SCHEMA_EVOLUTION_ENABLE.key(), HoodieCommonConfig.SCHEMA_EVOLUTION_ENABLE.defaultValue().toString()));
   }
 
   /**
@@ -333,15 +357,15 @@ public class OptionsResolver {
    * Returns whether consistent value will be generated for a logical timestamp type column.
    */
   public static boolean isConsistentLogicalTimestampEnabled(Configuration conf) {
-    return conf.getBoolean(KeyGeneratorOptions.KEYGENERATOR_CONSISTENT_LOGICAL_TIMESTAMP_ENABLED.key(),
-        Boolean.parseBoolean(KeyGeneratorOptions.KEYGENERATOR_CONSISTENT_LOGICAL_TIMESTAMP_ENABLED.defaultValue()));
+    return Boolean.parseBoolean(conf.getString(KeyGeneratorOptions.KEYGENERATOR_CONSISTENT_LOGICAL_TIMESTAMP_ENABLED.key(),
+        KeyGeneratorOptions.KEYGENERATOR_CONSISTENT_LOGICAL_TIMESTAMP_ENABLED.defaultValue()));
   }
 
   /**
    * Returns whether the writer txn should be guarded by lock.
    */
   public static boolean isLockRequired(Configuration conf) {
-    return conf.getBoolean(FlinkOptions.METADATA_ENABLED) || isMultiWriter(conf);
+    return conf.get(FlinkOptions.METADATA_ENABLED) || isMultiWriter(conf);
   }
 
   /**
@@ -373,7 +397,7 @@ public class OptionsResolver {
    * once it is disabled, the reader would infer the changes based on the file slice dependencies.
    */
   public static boolean readCDCFromChangelog(Configuration conf) {
-    return conf.getBoolean(FlinkOptions.READ_CDC_FROM_CHANGELOG);
+    return conf.get(FlinkOptions.READ_CDC_FROM_CHANGELOG);
   }
 
   /**
@@ -390,14 +414,14 @@ public class OptionsResolver {
    * Returns the index type.
    */
   public static HoodieIndex.IndexType getIndexType(Configuration conf) {
-    return HoodieIndex.IndexType.valueOf(conf.getString(FlinkOptions.INDEX_TYPE).toUpperCase());
+    return HoodieIndex.IndexType.valueOf(conf.get(FlinkOptions.INDEX_TYPE).toUpperCase());
   }
 
   /**
    * Returns the index key field.
    */
   public static String getIndexKeyField(Configuration conf) {
-    return conf.getString(FlinkOptions.INDEX_KEY_FIELD, conf.getString(FlinkOptions.RECORD_KEY_FIELD));
+    return conf.getString(FlinkOptions.INDEX_KEY_FIELD.key(), conf.get(FlinkOptions.RECORD_KEY_FIELD));
   }
 
   /**
@@ -416,11 +440,31 @@ public class OptionsResolver {
         : new SimpleConcurrentFileWritesConflictResolutionStrategy();
   }
 
+  /**
+   * Returns the IO related options as a map.
+   */
+  public static Map<String, String> getIOOptions(Configuration conf) {
+    Map<String, String> confMap = new HashMap<>();
+    // file writer options
+    confMap.put(HoodieStorageConfig.HOODIE_IO_FACTORY_CLASS.key(), HoodieFlinkIOFactory.class.getName());
+    confMap.put(HoodieStorageConfig.WRITE_UTC_TIMEZONE.key(), conf.get(FlinkOptions.WRITE_UTC_TIMEZONE).toString());
+
+    // file reader options
+    confMap.put(HoodieCommonConfig.SCHEMA_EVOLUTION_ENABLE.key(),
+        conf.getString(HoodieCommonConfig.SCHEMA_EVOLUTION_ENABLE.key(), HoodieCommonConfig.SCHEMA_EVOLUTION_ENABLE.defaultValue().toString()));
+    confMap.put(FlinkOptions.READ_UTC_TIMEZONE.key(), conf.get(FlinkOptions.READ_UTC_TIMEZONE).toString());
+
+    confMap.put(FlinkOptions.PARTITION_PATH_FIELD.key(), conf.get(FlinkOptions.PARTITION_PATH_FIELD));
+    confMap.put(FlinkOptions.PARTITION_DEFAULT_NAME.key(), conf.get(FlinkOptions.PARTITION_DEFAULT_NAME));
+    confMap.put(FlinkOptions.HIVE_STYLE_PARTITIONING.key(), conf.get(FlinkOptions.HIVE_STYLE_PARTITIONING).toString());
+    return confMap;
+  }
+
   /**
    * Returns whether to commit even when current batch has no data, for flink defaults false
    */
   public static boolean allowCommitOnEmptyBatch(Configuration conf) {
-    return conf.getBoolean(HoodieWriteConfig.ALLOW_EMPTY_COMMIT.key(), HoodieWriteConfig.ALLOW_EMPTY_COMMIT.defaultValue());
+    return Boolean.parseBoolean(conf.getString(HoodieWriteConfig.ALLOW_EMPTY_COMMIT.key(), HoodieWriteConfig.ALLOW_EMPTY_COMMIT.defaultValue().toString()));
   }
 
   /**
@@ -447,6 +491,35 @@ public class OptionsResolver {
     return HoodieCleanConfig.FAILED_WRITES_CLEANER_POLICY.defaultValue().equalsIgnoreCase(HoodieFailedWritesCleaningPolicy.LAZY.name());
   }
 
+  /**
+   * Returns whether the writers should use blocking instant time generation.
+   */
+  public static boolean isBlockingInstantGeneration(Configuration conf) {
+    return isCowTable(conf) && isUpsertOperation(conf);
+  }
+
+  /**
+   * Returns the customized insert partitioner instance.
+   */
+  public static Option<Partitioner> getInsertPartitioner(Configuration conf) {
+    String insertPartitionerClass = conf.get(FlinkOptions.INSERT_PARTITIONER_CLASS_NAME);
+    try {
+      return StringUtils.isNullOrEmpty(insertPartitionerClass)
+          ? Option.empty()
+          : Option.of((Partitioner) ReflectionUtils.loadClass(insertPartitionerClass, conf));
+    } catch (Throwable e) {
+      throw new HoodieException("Could not create custom insert partitioner " + insertPartitionerClass, e);
+    }
+  }
+
+  /**
+   * Returns whether complex keygen encodes single record key with field name.
+   */
+  public static boolean useComplexKeygenNewEncoding(Configuration conf) {
+    return Boolean.parseBoolean(conf.getString(HoodieWriteConfig.COMPLEX_KEYGEN_NEW_ENCODING.key(),
+        HoodieWriteConfig.COMPLEX_KEYGEN_NEW_ENCODING.defaultValue().toString()));
+  }
+
   // -------------------------------------------------------------------------
   //  Utilities
   // -------------------------------------------------------------------------
@@ -474,7 +547,7 @@ public class OptionsResolver {
    * Whether the reader only consumes new commit instants.
    */
   public static boolean isOnlyConsumingNewCommits(Configuration conf) {
-    return isMorTable(conf) && conf.getBoolean(FlinkOptions.READ_STREAMING_SKIP_COMPACT) // this is only true for flink.
-        || isAppendMode(conf) && conf.getBoolean(FlinkOptions.READ_STREAMING_SKIP_CLUSTERING);
+    return isMorTable(conf) && conf.get(FlinkOptions.READ_STREAMING_SKIP_COMPACT) // this is only true for flink.
+        || isAppendMode(conf) && conf.get(FlinkOptions.READ_STREAMING_SKIP_CLUSTERING);
   }
 }
