@@ -18,7 +18,6 @@
 
 package org.apache.hudi.sink;
 
-import org.apache.hudi.adapter.OperatorCoordinatorAdapter;
 import org.apache.hudi.client.HoodieFlinkWriteClient;
 import org.apache.hudi.client.WriteStatus;
 import org.apache.hudi.common.model.HoodieTableType;
@@ -27,18 +26,21 @@ import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.util.CommitUtils;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.common.util.SerializationUtils;
 import org.apache.hudi.common.util.ValidationUtils;
-import org.apache.hudi.config.HoodieWriteConfig;
+import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.configuration.HadoopConfigurations;
 import org.apache.hudi.configuration.OptionsResolver;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.hive.HiveSyncTool;
-import org.apache.hudi.sink.event.CommitAckEvent;
+import org.apache.hudi.sink.common.AbstractStreamWriteFunction;
+import org.apache.hudi.sink.event.Correspondent;
 import org.apache.hudi.sink.event.WriteMetadataEvent;
-import org.apache.hudi.sink.meta.CkpMetadata;
-import org.apache.hudi.sink.meta.CkpMetadataFactory;
+import org.apache.hudi.sink.utils.CoordinationResponseSerDe;
+import org.apache.hudi.sink.utils.EventBuffers;
+import org.apache.hudi.sink.utils.ExplicitClassloaderThreadFactory;
 import org.apache.hudi.sink.utils.HiveSyncContext;
 import org.apache.hudi.sink.utils.NonThrownExecutor;
 import org.apache.hudi.storage.StorageConfiguration;
@@ -46,19 +48,21 @@ import org.apache.hudi.util.ClientIds;
 import org.apache.hudi.util.ClusteringUtil;
 import org.apache.hudi.util.CompactionUtil;
 import org.apache.hudi.util.FlinkWriteClients;
+import org.apache.hudi.util.StreamerUtil;
 
 import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.runtime.jobgraph.OperatorID;
+import org.apache.flink.runtime.operators.coordination.CoordinationRequest;
+import org.apache.flink.runtime.operators.coordination.CoordinationRequestHandler;
+import org.apache.flink.runtime.operators.coordination.CoordinationResponse;
 import org.apache.flink.runtime.operators.coordination.OperatorCoordinator;
 import org.apache.flink.runtime.operators.coordination.OperatorEvent;
-import org.apache.flink.runtime.operators.coordination.TaskNotRunningException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import javax.annotation.Nullable;
 
-import java.io.IOException;
 import java.io.Serializable;
 import java.util.Arrays;
 import java.util.Collection;
@@ -70,6 +74,7 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CompletionException;
+import java.util.concurrent.ThreadFactory;
 import java.util.stream.Collectors;
 
 import static org.apache.hudi.common.table.timeline.InstantComparison.GREATER_THAN_OR_EQUALS;
@@ -82,12 +87,31 @@ import static org.apache.hudi.util.StreamerUtil.initTableIfNotExists;
  * <p>This coordinator starts a new instant when a new checkpoint starts. It commits the instant when all the
  * operator tasks write the buffer successfully for a round of checkpoint.
  *
- * <p>If there is no data for a round of checkpointing, it resets the events buffer and returns early.
+ * <p>If there is no data for a round of checkpointing,
+ * the coordinator will either commit the instant or rolls back it in LAZY cleaning
+ * based on the user configuration.
  *
- * @see StreamWriteFunction for the work flow and semantics
+ * <p><h2>Insurance of Exactly Once Semantics</h2></p>
+ *
+ * <p>Task failover workflow:
+ * <pre>
+ *             task failover
+ *                  |
+ *           initially started ?
+ *            /(yes)     \(no)
+ *           /            \
+ *       has pending      send an empty
+ *     instant event ?    bootstrap event
+ *        /(yes)  \(no)
+ *       /         \
+ *    resend      no-op
+ * </pre>
+ *
+ * @see StreamWriteFunction         for the data inputs checkpointing and semantics
+ * @see AbstractStreamWriteFunction for the bootstrap event sending workflow
  */
 public class StreamWriteOperatorCoordinator
-    implements OperatorCoordinatorAdapter {
+    implements OperatorCoordinator, CoordinationRequestHandler {
   private static final Logger LOG = LoggerFactory.getLogger(StreamWriteOperatorCoordinator.class);
 
   /**
@@ -103,10 +127,10 @@ public class StreamWriteOperatorCoordinator
   /**
    * Coordinator context.
    */
-  private final Context context;
+  protected final Context context;
 
   /**
-   * Gateways for sending events to sub tasks.
+   * Gateways for sending events to sub-tasks.
    */
   private transient SubtaskGateway[] gateways;
 
@@ -126,10 +150,14 @@ public class StreamWriteOperatorCoordinator
   private volatile String instant = WriteMetadataEvent.BOOTSTRAP_INSTANT;
 
   /**
-   * Event buffer for one round of checkpointing. When all the elements are non-null and have the same
-   * write instant, then the instant succeed and we can commit it.
+   * Event buffers for checkpointing.
+   *
+   * <p>It's a map of {checkpointId -> (instant, events)}.
+   *
+   * <p>A checkpoint ack event of one instant implies
+   * that the instant write succeed then we can commit it.
    */
-  private transient WriteMetadataEvent[] eventBuffer;
+  private transient EventBuffers eventBuffers;
 
   /**
    * Task number of the operator.
@@ -137,9 +165,14 @@ public class StreamWriteOperatorCoordinator
   private final int parallelism;
 
   /**
-   * A single-thread executor to handle all the asynchronous jobs of the coordinator.
+   * A single-thread executor to handle all the write metadata events.
    */
-  private NonThrownExecutor executor;
+  protected NonThrownExecutor executor;
+
+  /**
+   * A single-thread executor to handle the instant time request.
+   */
+  protected NonThrownExecutor instantRequestExecutor;
 
   /**
    * A single-thread executor to handle asynchronous hive sync.
@@ -156,11 +189,6 @@ public class StreamWriteOperatorCoordinator
    */
   private transient TableState tableState;
 
-  /**
-   * The checkpoint metadata.
-   */
-  private CkpMetadata ckpMetadata;
-
   /**
    * The client id heartbeats.
    */
@@ -186,27 +214,37 @@ public class StreamWriteOperatorCoordinator
     // setup classloader for APIs that use reflection without taking ClassLoader param
     // reference: https://stackoverflow.com/questions/1771679/difference-between-threads-context-class-loader-and-normal-classloader
     Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
-    // initialize event buffer
-    reset();
-    this.gateways = new SubtaskGateway[this.parallelism];
-    // init table, create if not exists.
-    this.metaClient = initTableIfNotExists(this.conf);
-    // the write client must create after the table creation
-    this.writeClient = FlinkWriteClients.createWriteClient(conf);
-    this.ckpMetadata = initCkpMetadata(writeClient.getConfig(), this.conf);
-    initMetadataTable(this.writeClient);
+    initEventBufferIfNecessary();
     this.tableState = TableState.create(conf);
-    // start the executor
-    this.executor = NonThrownExecutor.builder(LOG)
-        .exceptionHook((errMsg, t) -> this.context.failJob(new HoodieException(errMsg, t)))
-        .waitForTasksFinish(true).build();
-    // start the executor if required
-    if (tableState.syncHive) {
-      initHiveSync();
-    }
-    // start client id heartbeats for optimistic concurrency control
-    if (OptionsResolver.isMultiWriter(conf)) {
-      initClientIds(conf);
+    this.gateways = new SubtaskGateway[this.parallelism];
+    try {
+      // init table, create if not exists.
+      this.metaClient = initTableIfNotExists(this.conf);
+      // the write client must create after the table creation
+      this.writeClient = FlinkWriteClients.createWriteClient(conf);
+      this.writeClient.tryUpgrade(instant, this.metaClient);
+      initMetadataTable(this.writeClient);
+      // start the executor
+      this.executor = NonThrownExecutor.builder(LOG)
+          .threadFactory(getThreadFactory("meta-event-handle"))
+          .exceptionHook((errMsg, t) -> this.context.failJob(new HoodieException(errMsg, t)))
+          .waitForTasksFinish(true).build();
+      this.instantRequestExecutor = NonThrownExecutor.builder(LOG)
+          .threadFactory(getThreadFactory("instant-request"))
+          .exceptionHook((errMsg, t) -> this.context.failJob(new HoodieException(errMsg, t)))
+          .build();
+      // start the executor if required
+      if (tableState.syncHive) {
+        initHiveSync();
+      }
+      // start client id heartbeats for optimistic concurrency control
+      if (OptionsResolver.isMultiWriter(conf)) {
+        initClientIds(conf);
+      }
+      restoreEvents();
+    } catch (Throwable throwable) {
+      LOG.error("Failed to start operator coordinator.", throwable);
+      context.failJob(throwable);
     }
   }
 
@@ -216,6 +254,9 @@ public class StreamWriteOperatorCoordinator
     if (executor != null) {
       executor.close();
     }
+    if (instantRequestExecutor != null) {
+      instantRequestExecutor.close();
+    }
     if (hiveSyncExecutor != null) {
       hiveSyncExecutor.close();
     }
@@ -224,10 +265,7 @@ public class StreamWriteOperatorCoordinator
     if (writeClient != null) {
       writeClient.close();
     }
-    this.eventBuffer = null;
-    if (this.ckpMetadata != null) {
-      this.ckpMetadata.close();
-    }
+    this.eventBuffers = null;
     if (this.clientIds != null) {
       this.clientIds.close();
     }
@@ -238,7 +276,8 @@ public class StreamWriteOperatorCoordinator
     executor.execute(
         () -> {
           try {
-            result.complete(new byte[0]);
+            byte[] eventBytes = SerializationUtils.serialize(this.eventBuffers.getAllCompletedEvents());
+            result.complete(eventBytes);
           } catch (Throwable throwable) {
             // when a checkpoint fails, throws directly.
             result.completeExceptionally(
@@ -254,21 +293,16 @@ public class StreamWriteOperatorCoordinator
   public void notifyCheckpointComplete(long checkpointId) {
     executor.execute(
         () -> {
-          // The executor thread inherits the classloader of the #notifyCheckpointComplete
-          // caller, which is a AppClassLoader.
-          Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
           // for streaming mode, commits the ever received events anyway,
           // the stream write task snapshot and flush the data buffer synchronously in sequence,
           // so a successful checkpoint subsumes the old one(follows the checkpoint subsuming contract)
-          final boolean committed = commitInstant(this.instant, checkpointId);
+          final boolean committed = commitInstants(checkpointId);
           // schedules the compaction or clustering if it is enabled in stream execution mode
           scheduleTableServices(committed);
 
           if (committed) {
             // sync Hive if is enabled
             syncHiveAsync();
-            // start new instant.
-            startInstant();
           }
         }, "commits the instant %s", this.instant
     );
@@ -276,11 +310,19 @@ public class StreamWriteOperatorCoordinator
 
   @Override
   public void resetToCheckpoint(long checkpointID, byte[] checkpointData) {
-    // no operation
+    if (checkpointData != null) {
+      initEventBufferIfNecessary();
+      this.eventBuffers.addEventsToBuffer(SerializationUtils.deserialize(checkpointData));
+    }
   }
 
   @Override
-  public void handleEventFromOperator(int i, OperatorEvent operatorEvent) {
+  public void handleEventFromOperator(int subtask, int attemptNumber, OperatorEvent operatorEvent) {
+    handleEventFromOperator(subtask, operatorEvent);
+  }
+
+  @VisibleForTesting
+  public void handleEventFromOperator(int subtask, OperatorEvent operatorEvent) {
     ValidationUtils.checkState(operatorEvent instanceof WriteMetadataEvent,
         "The coordinator can only handle WriteMetaEvent");
     WriteMetadataEvent event = (WriteMetadataEvent) operatorEvent;
@@ -302,7 +344,7 @@ public class StreamWriteOperatorCoordinator
     }
   }
 
-  @Override
+  @VisibleForTesting
   public void subtaskFailed(int i, @Nullable Throwable throwable) {
     // no operation
   }
@@ -312,17 +354,63 @@ public class StreamWriteOperatorCoordinator
     // no operation
   }
 
-  @Override
+  @VisibleForTesting
   public void subtaskReady(int i, SubtaskGateway subtaskGateway) {
     this.gateways[i] = subtaskGateway;
   }
 
+  @Override
+  public void executionAttemptFailed(int i, int attemptNumber, Throwable reason) {
+    subtaskReady(i, null);
+  }
+
+  @Override
+  public void executionAttemptReady(int i, int attemptNumber, SubtaskGateway gateway) {
+    subtaskReady(i, gateway);
+  }
+
+  @Override
+  public CompletableFuture<CoordinationResponse> handleCoordinationRequest(CoordinationRequest request) {
+    CompletableFuture<CoordinationResponse> response = new CompletableFuture<>();
+    instantRequestExecutor.execute(() -> {
+      Correspondent.InstantTimeRequest instantTimeRequest = (Correspondent.InstantTimeRequest) request;
+      long checkpointId = instantTimeRequest.getCheckpointId();
+      Pair<String, WriteMetadataEvent[]> instantTimeAndEventBuffer = this.eventBuffers.getInstantAndEventBuffer(checkpointId);
+      final String instantTime;
+      if (instantTimeAndEventBuffer == null) {
+        // wait until previous instants are committed.
+        eventBuffers.awaitAllInstantsToCompleteIfNecessary();
+        instantTime = startInstant();
+        this.eventBuffers.initNewEventBuffer(checkpointId, instantTime, this.parallelism);
+      } else {
+        instantTime = instantTimeAndEventBuffer.getLeft();
+      }
+      response.complete(CoordinationResponseSerDe.wrap(Correspondent.InstantTimeResponse.getInstance(instantTime)));
+    }, "request instant time");
+    return response;
+  }
+
   // -------------------------------------------------------------------------
   //  Utilities
   // -------------------------------------------------------------------------
 
+  private void restoreEvents() {
+    if (this.eventBuffers.nonEmpty()) {
+      final HoodieTimeline completedTimeline = this.metaClient.getActiveTimeline().filterCompletedInstants();
+      this.eventBuffers.getEventBufferStream()
+          .forEach(entry -> recommitInstant(completedTimeline, entry.getKey(), entry.getValue().getLeft(), entry.getValue().getRight()));
+      this.metaClient.reloadActiveTimeline();
+    }
+  }
+
+  private ThreadFactory getThreadFactory(String threadName) {
+    return new ExplicitClassloaderThreadFactory(threadName, context.getUserCodeClassloader());
+  }
+
   private void initHiveSync() {
-    this.hiveSyncExecutor = NonThrownExecutor.builder(LOG).waitForTasksFinish(true).build();
+    this.hiveSyncExecutor = NonThrownExecutor.builder(LOG)
+        .threadFactory(getThreadFactory("hive-sync"))
+        .waitForTasksFinish(true).build();
     this.hiveSyncContext = HiveSyncContext.create(conf, this.storageConf);
   }
 
@@ -348,14 +436,19 @@ public class StreamWriteOperatorCoordinator
     }
   }
 
-  private static void initMetadataTable(HoodieFlinkWriteClient<?> writeClient) {
-    writeClient.initMetadataTable();
+  private void scheduleTableServices(Boolean committed) {
+    // if compaction is on, schedule the compaction
+    if (tableState.scheduleCompaction) {
+      CompactionUtil.scheduleCompaction(writeClient, tableState.isDeltaTimeCompaction, committed);
+    }
+    // if clustering is on, schedule the clustering
+    if (tableState.scheduleClustering) {
+      ClusteringUtil.scheduleClustering(conf, writeClient, committed);
+    }
   }
 
-  private static CkpMetadata initCkpMetadata(HoodieWriteConfig writeConfig, Configuration conf) throws IOException {
-    CkpMetadata ckpMetadata = CkpMetadataFactory.getCkpMetadata(writeConfig, conf);
-    ckpMetadata.bootstrap();
-    return ckpMetadata;
+  private static void initMetadataTable(HoodieFlinkWriteClient<?> writeClient) {
+    writeClient.initMetadataTable();
   }
 
   private void initClientIds(Configuration conf) {
@@ -363,31 +456,15 @@ public class StreamWriteOperatorCoordinator
     this.clientIds.start();
   }
 
-  private void reset() {
-    this.eventBuffer = new WriteMetadataEvent[this.parallelism];
-  }
-
-  /**
-   * Checks the buffer is ready to commit.
-   */
-  private boolean allEventsReceived() {
-    return Arrays.stream(eventBuffer)
-        // we do not use event.isReady to check the instant
-        // because the write task may send an event eagerly for empty
-        // data set, the even may have a timestamp of last committed instant.
-        .allMatch(event -> event != null && event.isLastBatch());
-  }
-
-  private void addEventToBuffer(WriteMetadataEvent event) {
-    if (this.eventBuffer[event.getTaskID()] != null
-        && this.eventBuffer[event.getTaskID()].getInstantTime().equals(event.getInstantTime())) {
-      this.eventBuffer[event.getTaskID()].mergeWith(event);
-    } else {
-      this.eventBuffer[event.getTaskID()] = event;
+  private void initEventBufferIfNecessary() {
+    if (this.eventBuffers != null) {
+      return;
     }
+    // initialize event buffer
+    this.eventBuffers = EventBuffers.getInstance(conf);
   }
 
-  private void startInstant() {
+  private String startInstant() {
     // refresh the meta client which is reused
     metaClient.reloadActiveTimeline();
     // refresh the last txn metadata
@@ -397,79 +474,55 @@ public class StreamWriteOperatorCoordinator
     this.instant = this.writeClient.startCommit(tableState.commitAction, this.metaClient);
     this.metaClient.getActiveTimeline().transitionRequestedToInflight(tableState.commitAction, this.instant);
     this.writeClient.setWriteTimer(tableState.commitAction);
-    this.ckpMetadata.startInstant(this.instant);
     LOG.info("Create instant [{}] for table [{}] with type [{}]", this.instant,
-        this.conf.getString(FlinkOptions.TABLE_NAME), conf.getString(FlinkOptions.TABLE_TYPE));
+        this.conf.get(FlinkOptions.TABLE_NAME), conf.get(FlinkOptions.TABLE_TYPE));
+    return this.instant;
   }
 
   /**
-   * Initializes the instant.
-   *
-   * <p>Recommits the last inflight instant if the write metadata checkpoint successfully
+   * Recommits the last inflight instant if the write metadata checkpoint successfully
    * but was not committed due to some rare cases.
-   *
-   * <p>Starts a new instant, a writer can not flush data buffer
-   * until it finds a new inflight instant on the timeline.
    */
-  private void initInstant(String instant) {
+  private void recommitInstant(long checkpointId, String instant, WriteMetadataEvent[] bootstrapBuffer) {
     HoodieTimeline completedTimeline = this.metaClient.getActiveTimeline().filterCompletedInstants();
-    if (instant.equals(WriteMetadataEvent.BOOTSTRAP_INSTANT) || completedTimeline.containsInstant(instant)) {
-      // the last instant committed successfully
-      reset();
-    } else {
+    recommitInstant(completedTimeline, checkpointId, instant, bootstrapBuffer);
+  }
+
+  /**
+   * Recommits the last inflight instant if the write metadata checkpoint successfully
+   * but was not committed due to some rare cases.
+   */
+  private void recommitInstant(HoodieTimeline completedTimeline, long checkpointId, String instant, WriteMetadataEvent[] bootstrapBuffer) {
+    if (!completedTimeline.containsInstant(instant)) {
       LOG.info("Recommit instant {}", instant);
-      // Recommit should start heartbeat for lazy failed writes clean policy to avoid aborting for heartbeat expired.
+      // Recommit should start heartbeat for lazy failed writes clean policy to avoid aborting for heartbeat expired;
+      // The following up checkpoints would recommit the instant.
       if (writeClient.getConfig().getFailedWritesCleanPolicy().isLazy()) {
         writeClient.getHeartbeatClient().start(instant);
       }
-      commitInstant(instant);
+      commitInstant(checkpointId, instant, bootstrapBuffer);
     }
-    // stop the heartbeat for old instant
-    if (writeClient.getConfig().getFailedWritesCleanPolicy().isLazy() && !WriteMetadataEvent.BOOTSTRAP_INSTANT.equals(this.instant)) {
-      writeClient.getHeartbeatClient().stop(this.instant);
-    }
-    // starts a new instant
-    startInstant();
-    // upgrade downgrade
-    this.writeClient.upgradeDowngrade(this.instant, this.metaClient);
   }
 
   private void handleBootstrapEvent(WriteMetadataEvent event) {
-    this.eventBuffer[event.getTaskID()] = event;
+    if (event.getInstantTime().equals(WriteMetadataEvent.BOOTSTRAP_INSTANT)) {
+      this.eventBuffers.cleanLegacyEvents(event);
+      return;
+    }
+    WriteMetadataEvent[] eventBuffer = this.eventBuffers.getOrCreateBootstrapBuffer(event, this.parallelism);
+    eventBuffer[event.getTaskID()] = event;
     if (Arrays.stream(eventBuffer).allMatch(evt -> evt != null && evt.isBootstrap())) {
-      // start to initialize the instant.
-      final String instant = Arrays.stream(eventBuffer)
-          .filter(evt -> evt.getWriteStatuses().size() > 0)
-          .findFirst().map(WriteMetadataEvent::getInstantTime)
-          .orElse(WriteMetadataEvent.BOOTSTRAP_INSTANT);
-
-      // if currentInstant is pending && bootstrap event instant is empty
-      // reuse currentInstant, reject bootstrap
-      if (this.metaClient.reloadActiveTimeline().filterInflightsAndRequested().containsInstant(this.instant)
-              && instant.equals(WriteMetadataEvent.BOOTSTRAP_INSTANT)
-              && this.tableState.operationType == WriteOperationType.INSERT) {
-        LOG.warn("Reuse current pending Instant {} with {} operationType, "
-                + "ignoring empty bootstrap event.", this.instant, WriteOperationType.INSERT.value());
-        reset();
-
-        // send commit act event to unblock write tasks
-        sendCommitAckEvents(-1L);
-        return;
-      }
-
-      initInstant(instant);
+      // start to recommit the instant.
+      recommitInstant(event.getCheckpointId(), event.getInstantTime(), eventBuffer);
     }
   }
 
   private void handleEndInputEvent(WriteMetadataEvent event) {
-    addEventToBuffer(event);
-    if (allEventsReceived()) {
+    WriteMetadataEvent[] eventBuffer = this.eventBuffers.addEventToBuffer(event);
+    if (EventBuffers.allEventsReceived(eventBuffer)) {
       // start to commit the instant.
-      boolean committed = commitInstant(this.instant);
+      boolean committed = commitInstant(event.getCheckpointId(), event.getInstantTime(), eventBuffer);
       if (committed) {
-        // The executor thread inherits the classloader of the #handleEventFromOperator
-        // caller, which is a AppClassLoader.
-        Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
         // sync Hive synchronously if it is enabled in batch mode.
         syncHive();
         // schedules the compaction or clustering if it is enabled in batch execution mode
@@ -478,17 +531,6 @@ public class StreamWriteOperatorCoordinator
     }
   }
 
-  private void scheduleTableServices(Boolean committed) {
-    // if compaction is on, schedule the compaction
-    if (tableState.scheduleCompaction) {
-      CompactionUtil.scheduleCompaction(writeClient, tableState.isDeltaTimeCompaction, committed);
-    }
-    // if clustering is on, schedule the clustering
-    if (tableState.scheduleClustering) {
-      ClusteringUtil.scheduleClustering(conf, writeClient, committed);
-    }
-  }
-
   private void handleWriteMetaEvent(WriteMetadataEvent event) {
     // the write task does not block after checkpointing(and before it receives a checkpoint success event),
     // if it checkpoints succeed then flushes the data buffer again before this coordinator receives a checkpoint
@@ -498,39 +540,17 @@ public class StreamWriteOperatorCoordinator
         String.format("Receive an unexpected event for instant %s from task %d",
             event.getInstantTime(), event.getTaskID()));
 
-    addEventToBuffer(event);
-  }
-
-  /**
-   * The coordinator reuses the instant if there is no data for this round of checkpoint,
-   * sends the commit ack events to unblock the flushing.
-   */
-  private void sendCommitAckEvents(long checkpointId) {
-    CompletableFuture<?>[] futures = Arrays.stream(this.gateways).filter(Objects::nonNull)
-        .map(gw -> gw.sendEvent(CommitAckEvent.getInstance(checkpointId)))
-        .toArray(CompletableFuture<?>[]::new);
-    CompletableFuture.allOf(futures).whenComplete((resp, error) -> {
-      if (!sendToFinishedTasks(error)) {
-        throw new HoodieException("Error while waiting for the commit ack events to finish sending", error);
-      }
-    });
-  }
-
-  /**
-   * Decides whether the given exception is caused by sending events to FINISHED tasks.
-   *
-   * <p>Ugly impl: the exception may change in the future.
-   */
-  private static boolean sendToFinishedTasks(Throwable throwable) {
-    return throwable.getCause() instanceof TaskNotRunningException
-        || throwable.getCause().getMessage().contains("running");
+    this.eventBuffers.addEventToBuffer(event);
   }
 
   /**
    * Commits the instant.
    */
-  private boolean commitInstant(String instant) {
-    return commitInstant(instant, -1);
+  private boolean commitInstants(long checkpointId) {
+    // use < instead of <= because the write metadata event sends the last known checkpoint id which is smaller than the current one.
+    List<Boolean> result = this.eventBuffers.getEventBufferStream().filter(entry -> entry.getKey() < checkpointId)
+        .map(entry -> commitInstant(entry.getKey(), entry.getValue().getLeft(), entry.getValue().getRight())).collect(Collectors.toList());
+    return result.stream().anyMatch(i -> i);
   }
 
   /**
@@ -538,9 +558,12 @@ public class StreamWriteOperatorCoordinator
    *
    * @return true if the write statuses are committed successfully.
    */
-  private boolean commitInstant(String instant, long checkpointId) {
+  private boolean commitInstant(long checkpointId, String instant, WriteMetadataEvent[] eventBuffer) {
     if (Arrays.stream(eventBuffer).allMatch(Objects::isNull)) {
-      // The last checkpoint finished successfully.
+      // all the tasks are reset by failover, reset the while buffer and returns early.
+      this.eventBuffers.reset(checkpointId);
+      // stop the heart beat for lazy cleaning
+      writeClient.getHeartbeatClient().stop(instant);
       return false;
     }
 
@@ -550,18 +573,14 @@ public class StreamWriteOperatorCoordinator
         .flatMap(Collection::stream)
         .collect(Collectors.toList());
 
-    if (writeResults.size() == 0 && !OptionsResolver.allowCommitOnEmptyBatch(conf)) {
+    if (writeResults.isEmpty() && !OptionsResolver.allowCommitOnEmptyBatch(conf)) {
       // No data has written, reset the buffer and returns early
-      reset();
-      // Send commit ack event to the write function to unblock the flushing
-      // If this checkpoint has no inputs while the next checkpoint has inputs,
-      // the 'isConfirming' flag should be switched with the ack event.
-      if (checkpointId != -1) {
-        sendCommitAckEvents(checkpointId);
-      }
+      this.eventBuffers.reset(checkpointId);
+      // stop the heart beat for lazy cleaning
+      writeClient.getHeartbeatClient().stop(instant);
       return false;
     }
-    doCommit(instant, writeResults);
+    doCommit(checkpointId, instant, writeResults);
     return true;
   }
 
@@ -569,17 +588,19 @@ public class StreamWriteOperatorCoordinator
    * Performs the actual commit action.
    */
   @SuppressWarnings("unchecked")
-  private void doCommit(String instant, List<WriteStatus> writeResults) {
+  private void doCommit(long checkpointId, String instant, List<WriteStatus> writeResults) {
     // commit or rollback
     long totalErrorRecords = writeResults.stream().map(WriteStatus::getTotalErrorRecords).reduce(Long::sum).orElse(0L);
     long totalRecords = writeResults.stream().map(WriteStatus::getTotalRecords).reduce(Long::sum).orElse(0L);
     boolean hasErrors = totalErrorRecords > 0;
 
-    if (!hasErrors || this.conf.getBoolean(FlinkOptions.IGNORE_FAILED)) {
+    if (!hasErrors || this.conf.get(FlinkOptions.IGNORE_FAILED)) {
       HashMap<String, String> checkpointCommitMetadata = new HashMap<>();
+      StreamerUtil.addFlinkCheckpointIdIntoMetaData(conf, checkpointCommitMetadata, checkpointId);
+
       if (hasErrors) {
-        LOG.warn("Some records failed to merge but forcing commit since commitOnErrors set to true. Errors/Total="
-            + totalErrorRecords + "/" + totalRecords);
+        LOG.warn("Some records failed to merge but forcing commit since commitOnErrors set to true. Errors/Total={}/{}",
+            totalErrorRecords, totalRecords);
       }
 
       final Map<String, List<String>> partitionToReplacedFileIds = tableState.isOverwrite
@@ -588,27 +609,29 @@ public class StreamWriteOperatorCoordinator
       boolean success = writeClient.commit(instant, writeResults, Option.of(checkpointCommitMetadata),
           tableState.commitAction, partitionToReplacedFileIds);
       if (success) {
-        reset();
-        this.ckpMetadata.commitInstant(instant);
+        this.eventBuffers.reset(checkpointId);
         LOG.info("Commit instant [{}] success!", instant);
       } else {
         throw new HoodieException(String.format("Commit instant [%s] failed!", instant));
       }
     } else {
-      LOG.error("Error when writing. Errors/Total=" + totalErrorRecords + "/" + totalRecords);
-      LOG.error("The first 10 files with write errors:");
-      writeResults.stream().filter(WriteStatus::hasErrors).limit(10).forEach(ws -> {
-        if (ws.getGlobalError() != null) {
-          LOG.error("Global error for partition path {} and fileID {}: {}",
-              ws.getPartitionPath(), ws.getFileId(), ws.getGlobalError());
-        }
-        if (ws.getErrors().size() > 0) {
-          LOG.error("The first 100 records-level errors for partition path {} and fileID {}:",
-              ws.getPartitionPath(), ws.getFileId());
-          ws.getErrors().entrySet().stream().limit(100).forEach(entry -> LOG.error("Error for key: "
-              + entry.getKey() + " and Exception: " + entry.getValue().getMessage()));
-        }
-      });
+      if (LOG.isErrorEnabled()) {
+        LOG.error("Error when writing. Errors/Total={}/{}", totalErrorRecords, totalRecords);
+        LOG.error("The first 10 files with write errors:");
+        writeResults.stream().filter(WriteStatus::hasErrors).limit(10).forEach(ws -> {
+          if (ws.getGlobalError() != null) {
+            LOG.error("Global error for partition path {} and fileID {}: {}",
+                ws.getPartitionPath(), ws.getFileId(), ws.getGlobalError());
+          }
+          if (!ws.getErrors().isEmpty()) {
+            LOG.error("The first 100 records-level errors for partition path {} and fileID {}:",
+                ws.getPartitionPath(), ws.getFileId());
+            ws.getErrors().entrySet().stream().limit(100).forEach(entry ->
+                LOG.error("Error for key: {} and Exception: {}", entry.getKey(), entry.getValue().getMessage()));
+          }
+        });
+      }
+
       // Rolls back instant
       writeClient.rollback(instant);
       throw new HoodieException(String.format("Commit instant [%s] failed and rolled back !", instant));
@@ -617,7 +640,12 @@ public class StreamWriteOperatorCoordinator
 
   @VisibleForTesting
   public WriteMetadataEvent[] getEventBuffer() {
-    return eventBuffer;
+    return this.eventBuffers.getLatestEventBuffer(this.instant);
+  }
+
+  @VisibleForTesting
+  public WriteMetadataEvent[] getEventBuffer(long checkpointId) {
+    return this.eventBuffers.getEventBuffer(checkpointId);
   }
 
   @VisibleForTesting
@@ -643,6 +671,14 @@ public class StreamWriteOperatorCoordinator
     this.executor = executor;
   }
 
+  @VisibleForTesting
+  public void setInstantRequestExecutor(NonThrownExecutor executor) throws Exception {
+    if (this.instantRequestExecutor != null) {
+      this.instantRequestExecutor.close();
+    }
+    this.instantRequestExecutor = executor;
+  }
+
   // -------------------------------------------------------------------------
   //  Inner Class
   // -------------------------------------------------------------------------
@@ -686,14 +722,14 @@ public class StreamWriteOperatorCoordinator
     final boolean isDeltaTimeCompaction;
 
     private TableState(Configuration conf) {
-      this.operationType = WriteOperationType.fromValue(conf.getString(FlinkOptions.OPERATION));
+      this.operationType = WriteOperationType.fromValue(conf.get(FlinkOptions.OPERATION));
       this.commitAction = CommitUtils.getCommitActionType(this.operationType,
-          HoodieTableType.valueOf(conf.getString(FlinkOptions.TABLE_TYPE).toUpperCase(Locale.ROOT)));
+          HoodieTableType.valueOf(conf.get(FlinkOptions.TABLE_TYPE).toUpperCase(Locale.ROOT)));
       this.isOverwrite = WriteOperationType.isOverwrite(this.operationType);
       this.scheduleCompaction = OptionsResolver.needsScheduleCompaction(conf);
       this.scheduleClustering = OptionsResolver.needsScheduleClustering(conf);
-      this.syncHive = conf.getBoolean(FlinkOptions.HIVE_SYNC_ENABLED);
-      this.syncMetadata = conf.getBoolean(FlinkOptions.METADATA_ENABLED);
+      this.syncHive = conf.get(FlinkOptions.HIVE_SYNC_ENABLED);
+      this.syncMetadata = conf.get(FlinkOptions.METADATA_ENABLED);
       this.isDeltaTimeCompaction = OptionsResolver.isDeltaTimeCompaction(conf);
     }
 
