@@ -18,6 +18,7 @@
 
 package org.apache.hudi.sink.bucket;
 
+import org.apache.hudi.adapter.ProcessFunctionAdapter;
 import org.apache.hudi.client.HoodieFlinkWriteClient;
 import org.apache.hudi.client.model.HoodieFlinkInternalRow;
 import org.apache.hudi.common.fs.FSUtils;
@@ -39,7 +40,6 @@ import org.apache.flink.configuration.Configuration;
 import org.apache.flink.runtime.state.FunctionInitializationContext;
 import org.apache.flink.runtime.state.FunctionSnapshotContext;
 import org.apache.flink.streaming.api.checkpoint.CheckpointedFunction;
-import org.apache.flink.streaming.api.functions.ProcessFunction;
 import org.apache.flink.util.Collector;
 import org.apache.flink.util.Preconditions;
 import org.slf4j.Logger;
@@ -55,7 +55,7 @@ import java.util.concurrent.TimeUnit;
 /**
  * The function to tag each incoming record with a location of a file based on consistent bucket index.
  */
-public class ConsistentBucketAssignFunction extends ProcessFunction<HoodieFlinkInternalRow, HoodieFlinkInternalRow> implements CheckpointedFunction {
+public class ConsistentBucketAssignFunction extends ProcessFunctionAdapter<HoodieFlinkInternalRow, HoodieFlinkInternalRow> implements CheckpointedFunction {
 
   private static final Logger LOG = LoggerFactory.getLogger(ConsistentBucketAssignFunction.class);
 
@@ -71,13 +71,15 @@ public class ConsistentBucketAssignFunction extends ProcessFunction<HoodieFlinkI
   public ConsistentBucketAssignFunction(Configuration conf) {
     this.config = conf;
     this.indexKeyFields = Arrays.asList(OptionsResolver.getIndexKeyField(conf).split(","));
-    this.bucketNum = conf.getInteger(FlinkOptions.BUCKET_INDEX_NUM_BUCKETS);
+    this.bucketNum = conf.get(FlinkOptions.BUCKET_INDEX_NUM_BUCKETS);
   }
 
   @Override
   public void open(Configuration parameters) throws Exception {
     try {
-      this.writeClient = FlinkWriteClients.createWriteClient(this.config, getRuntimeContext());
+      // not load fs view storage config for incremental job graph, since embedded timeline server
+      // is started in write coordinator which is started after bucket assigner operator finished
+      this.writeClient = FlinkWriteClients.createWriteClient(this.config, getRuntimeContext(), !OptionsResolver.isIncrementalJobGraph(config));
       this.partitionToIdentifier = new HashMap<>();
       this.lastRefreshInstant = HoodieTimeline.INIT_INSTANT_TS;
     } catch (Throwable e) {
