@@ -23,7 +23,6 @@ import org.apache.hudi.client.WriteStatus;
 import org.apache.hudi.common.data.HoodieListData;
 import org.apache.hudi.common.model.HoodieCommitMetadata;
 import org.apache.hudi.common.util.CompactionUtils;
-import org.apache.hudi.common.util.Option;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.metrics.FlinkCompactionMetrics;
@@ -112,7 +111,7 @@ public class CompactionCommitSink extends CleanFunction<CompactionCommitEvent> {
     if (event.isFailed()
         || (event.getWriteStatuses() != null
         && event.getWriteStatuses().stream().anyMatch(writeStatus -> writeStatus.getTotalErrorRecords() > 0))) {
-      LOG.warn("Receive abnormal CompactionCommitEvent of instant {}, task ID is {},"
+      LOG.warn("Received abnormal CompactionCommitEvent of instant {}, task ID is {},"
               + " is failed: {}, error record count: {}",
           instant, event.getTaskID(), event.isFailed(), getNumErrorRecords(event));
     }
@@ -154,7 +153,7 @@ public class CompactionCommitSink extends CleanFunction<CompactionCommitEvent> {
     if (events.stream().anyMatch(CompactionCommitEvent::isFailed)) {
       try {
         // handle failure case
-        CompactionUtil.rollbackCompaction(table, instant);
+        CompactionUtil.rollbackCompaction(table, instant, writeClient.getTransactionManager());
       } finally {
         // remove commitBuffer to avoid obsolete metadata commit
         reset(instant);
@@ -184,12 +183,12 @@ public class CompactionCommitSink extends CleanFunction<CompactionCommitEvent> {
 
     long numErrorRecords = statuses.stream().map(WriteStatus::getTotalErrorRecords).reduce(Long::sum).orElse(0L);
 
-    if (numErrorRecords > 0 && !this.conf.getBoolean(FlinkOptions.IGNORE_FAILED)) {
+    if (numErrorRecords > 0 && !this.conf.get(FlinkOptions.IGNORE_FAILED)) {
       // handle failure case
       LOG.error("Got {} error records during compaction of instant {},\n"
           + "option '{}' is configured as false,"
           + "rolls back the compaction", numErrorRecords, instant, FlinkOptions.IGNORE_FAILED.key());
-      CompactionUtil.rollbackCompaction(table, instant);
+      CompactionUtil.rollbackCompaction(table, instant, writeClient.getTransactionManager());
       this.compactionMetrics.markCompactionRolledBack();
       return;
     }
@@ -198,13 +197,13 @@ public class CompactionCommitSink extends CleanFunction<CompactionCommitEvent> {
         table, instant, HoodieListData.eager(statuses), writeClient.getConfig().getSchema());
 
     // commit the compaction
-    this.writeClient.commitCompaction(instant, metadata, Option.empty());
+    this.writeClient.completeCompaction(metadata, table, instant);
 
     this.compactionMetrics.updateCommitMetrics(instant, metadata);
     this.compactionMetrics.markCompactionCompleted();
 
     // Whether to clean up the old log file when compaction
-    if (!conf.getBoolean(FlinkOptions.CLEAN_ASYNC_ENABLED) && !isCleaning) {
+    if (!conf.get(FlinkOptions.CLEAN_ASYNC_ENABLED) && !isCleaning) {
       this.writeClient.clean();
     }
   }
