@@ -35,6 +35,7 @@ import org.slf4j.LoggerFactory;
 
 import java.io.Serializable;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 import java.util.Objects;
 import java.util.stream.Collectors;
@@ -177,6 +178,11 @@ public class ExpressionPredicates {
      * @return A filter predicate of parquet file.
      */
     FilterPredicate filter();
+
+    /**
+     * List of columns that are referenced by this filter.
+     */
+    List<String> references();
   }
 
   /**
@@ -217,7 +223,9 @@ public class ExpressionPredicates {
       if (literalObject instanceof Serializable) {
         this.literal = (Serializable) literalObject;
       } else {
-        LOG.warn("Encountered a non-serializable literal. " + "Cannot push predicate with value literal [{}] into FileInputFormat. " + "This is a bug and should be reported.", valueLiteral);
+        LOG.warn("Encountered a non-serializable literal. "
+            + "Cannot push predicate with value literal [{}] into FileInputFormat. "
+            + "This is a bug and should be reported.", valueLiteral);
         this.literal = null;
       }
       return this;
@@ -229,6 +237,11 @@ public class ExpressionPredicates {
       return toParquetPredicate(getFunctionDefinition(), literalType, columnName, convertedLiteral);
     }
 
+    @Override
+    public List<String> references() {
+      return Collections.singletonList(columnName);
+    }
+
     /**
      * Returns function definition of predicate.
      *
@@ -408,8 +421,7 @@ public class ExpressionPredicates {
 
     private static final long serialVersionUID = 1L;
 
-    private static final Logger LOG = LoggerFactory.getLogger(ExpressionEvaluators.In.class);
-
+    private static final Logger LOG = LoggerFactory.getLogger(In.class);
 
     private static final int IN_PREDICATE_LIMIT = 200;
 
@@ -438,7 +450,9 @@ public class ExpressionPredicates {
         if (literalObject instanceof Serializable) {
           return (Serializable) literalObject;
         } else {
-          LOG.warn("Encountered a non-serializable literal. " + "Cannot push predicate with value literal [{}] into FileInputFormat. " + "This is a bug and should be reported.", valueLiteral);
+          LOG.warn("Encountered a non-serializable literal. "
+              + "Cannot push predicate with value literal [{}] into FileInputFormat. "
+              + "This is a bug and should be reported.", valueLiteral);
           return null;
         }
       }).collect(Collectors.toList());
@@ -482,6 +496,11 @@ public class ExpressionPredicates {
     public FilterPredicate filter() {
       return null;
     }
+
+    @Override
+    public List<String> references() {
+      return Collections.emptyList();
+    }
   }
 
   /**
@@ -520,6 +539,11 @@ public class ExpressionPredicates {
       return not(filterPredicate);
     }
 
+    @Override
+    public List<String> references() {
+      return predicate.references();
+    }
+
     @Override
     public String toString() {
       return "NOT(" + predicate.toString() + ")";
@@ -563,6 +587,11 @@ public class ExpressionPredicates {
       return and(filterPredicate0, filterPredicate1);
     }
 
+    @Override
+    public List<String> references() {
+      return Arrays.stream(predicates).map(Predicate::references).flatMap(List::stream).distinct().collect(Collectors.toList());
+    }
+
     @Override
     public String toString() {
       return "AND(" + Arrays.toString(predicates) + ")";
@@ -606,6 +635,11 @@ public class ExpressionPredicates {
       return or(filterPredicate0, filterPredicate1);
     }
 
+    @Override
+    public List<String> references() {
+      return Arrays.stream(predicates).map(Predicate::references).flatMap(List::stream).distinct().collect(Collectors.toList());
+    }
+
     @Override
     public String toString() {
       return "OR(" + Arrays.toString(predicates) + ")";
