@@ -114,9 +114,10 @@ public class FlinkStreamerConfig extends Configuration {
       + "By default `SIMPLE`.")
   public String keygenType = KeyGeneratorType.SIMPLE.name();
 
-  @Parameter(names = {"--source-ordering-field"}, description = "Field within source record to decide how"
-      + " to break ties between records with same key in input data. Default: 'ts' holding unix timestamp of record.")
-  public String sourceOrderingField = "ts";
+  @Parameter(names = {"--source-ordering-fields", "--source-ordering-field"}, description = "Field within source record to decide how"
+      + " to break ties between records with same key in input data. Default: 'ts' holding unix timestamp of record. "
+      + "Option --source-ordering-field is deprecated, please use --source-ordering-fields instead.")
+  public String sourceOrderingFields = "ts";
 
   @Parameter(names = {"--write-table-version"}, description = "Version of table written")
   public Integer writeTableVersion = HoodieTableVersion.current().versionCode();
@@ -419,102 +420,102 @@ public class FlinkStreamerConfig extends Configuration {
     Map<String, String> propsMap = new HashMap<String, String>((Map) StreamerUtil.getProps(config));
     org.apache.flink.configuration.Configuration conf = fromMap(propsMap);
 
-    conf.setString(FlinkOptions.PATH, config.targetBasePath);
-    conf.setString(FlinkOptions.TABLE_NAME, config.targetTableName);
+    conf.set(FlinkOptions.PATH, config.targetBasePath);
+    conf.set(FlinkOptions.TABLE_NAME, config.targetTableName);
     // copy_on_write works same as COPY_ON_WRITE
-    conf.setString(FlinkOptions.TABLE_TYPE, config.tableType.toUpperCase());
-    conf.setBoolean(FlinkOptions.INSERT_CLUSTER, config.insertCluster);
-    conf.setString(FlinkOptions.OPERATION, config.operation.value());
-    conf.setString(FlinkOptions.PRECOMBINE_FIELD, config.sourceOrderingField);
-    conf.setInteger(FlinkOptions.WRITE_TABLE_VERSION, config.writeTableVersion);
-    conf.setString(FlinkOptions.PAYLOAD_CLASS_NAME, config.payloadClassName);
-    conf.setString(FlinkOptions.RECORD_MERGER_IMPLS, config.recordMergerImpls);
-    conf.setString(FlinkOptions.RECORD_MERGER_STRATEGY_ID, config.recordMergerStrategy);
-    conf.setBoolean(FlinkOptions.PRE_COMBINE, config.preCombine);
-    conf.setInteger(FlinkOptions.RETRY_TIMES, Integer.parseInt(config.instantRetryTimes));
-    conf.setLong(FlinkOptions.RETRY_INTERVAL_MS, Long.parseLong(config.instantRetryInterval));
-    conf.setBoolean(FlinkOptions.IGNORE_FAILED, config.commitOnErrors);
-    conf.setString(FlinkOptions.RECORD_KEY_FIELD, config.recordKeyField);
-    conf.setString(FlinkOptions.PARTITION_PATH_FIELD, config.partitionPathField);
-    conf.setBoolean(FlinkOptions.METADATA_ENABLED, config.metadataEnabled);
-    conf.setInteger(FlinkOptions.METADATA_COMPACTION_DELTA_COMMITS, config.metadataCompactionDeltaCommits);
-    conf.setString(FlinkOptions.PARTITION_FORMAT, config.writePartitionFormat);
-    conf.setLong(FlinkOptions.WRITE_RATE_LIMIT, config.writeRateLimit);
-    conf.setInteger(FlinkOptions.WRITE_PARQUET_BLOCK_SIZE, config.writeParquetBlockSize);
-    conf.setInteger(FlinkOptions.WRITE_PARQUET_MAX_FILE_SIZE, config.writeParquetMaxFileSize);
-    conf.setInteger(FlinkOptions.WRITE_PARQUET_PAGE_SIZE, config.parquetPageSize);
+    conf.set(FlinkOptions.TABLE_TYPE, config.tableType.toUpperCase());
+    conf.set(FlinkOptions.INSERT_CLUSTER, config.insertCluster);
+    conf.set(FlinkOptions.OPERATION, config.operation.value());
+    conf.set(FlinkOptions.ORDERING_FIELDS, config.sourceOrderingFields);
+    conf.set(FlinkOptions.WRITE_TABLE_VERSION, config.writeTableVersion);
+    conf.set(FlinkOptions.PAYLOAD_CLASS_NAME, config.payloadClassName);
+    conf.set(FlinkOptions.RECORD_MERGER_IMPLS, config.recordMergerImpls);
+    conf.set(FlinkOptions.RECORD_MERGER_STRATEGY_ID, config.recordMergerStrategy);
+    conf.set(FlinkOptions.PRE_COMBINE, config.preCombine);
+    conf.set(FlinkOptions.RETRY_TIMES, Integer.parseInt(config.instantRetryTimes));
+    conf.set(FlinkOptions.RETRY_INTERVAL_MS, Long.parseLong(config.instantRetryInterval));
+    conf.set(FlinkOptions.IGNORE_FAILED, config.commitOnErrors);
+    conf.set(FlinkOptions.RECORD_KEY_FIELD, config.recordKeyField);
+    conf.set(FlinkOptions.PARTITION_PATH_FIELD, config.partitionPathField);
+    conf.set(FlinkOptions.METADATA_ENABLED, config.metadataEnabled);
+    conf.set(FlinkOptions.METADATA_COMPACTION_DELTA_COMMITS, config.metadataCompactionDeltaCommits);
+    conf.set(FlinkOptions.PARTITION_FORMAT, config.writePartitionFormat);
+    conf.set(FlinkOptions.WRITE_RATE_LIMIT, config.writeRateLimit);
+    conf.set(FlinkOptions.WRITE_PARQUET_BLOCK_SIZE, config.writeParquetBlockSize);
+    conf.set(FlinkOptions.WRITE_PARQUET_MAX_FILE_SIZE, config.writeParquetMaxFileSize);
+    conf.set(FlinkOptions.WRITE_PARQUET_PAGE_SIZE, config.parquetPageSize);
     if (!StringUtils.isNullOrEmpty(config.keygenClass)) {
-      conf.setString(FlinkOptions.KEYGEN_CLASS_NAME, config.keygenClass);
+      conf.set(FlinkOptions.KEYGEN_CLASS_NAME, config.keygenClass);
     } else {
-      conf.setString(FlinkOptions.KEYGEN_TYPE, config.keygenType);
+      conf.set(FlinkOptions.KEYGEN_TYPE, config.keygenType);
     }
-    conf.setInteger(FlinkOptions.INDEX_BOOTSTRAP_TASKS, config.indexBootstrapNum);
-    conf.setInteger(FlinkOptions.BUCKET_ASSIGN_TASKS, config.bucketAssignNum);
-    conf.setInteger(FlinkOptions.WRITE_TASKS, config.writeTaskNum);
-    conf.setString(FlinkOptions.PARTITION_DEFAULT_NAME, config.partitionDefaultName);
-    conf.setBoolean(FlinkOptions.INDEX_BOOTSTRAP_ENABLED, config.indexBootstrapEnabled);
-    conf.setDouble(FlinkOptions.INDEX_STATE_TTL, config.indexStateTtl);
-    conf.setBoolean(FlinkOptions.INDEX_GLOBAL_ENABLED, config.indexGlobalEnabled);
-    conf.setString(FlinkOptions.INDEX_PARTITION_REGEX, config.indexPartitionRegex);
+    conf.set(FlinkOptions.INDEX_BOOTSTRAP_TASKS, config.indexBootstrapNum);
+    conf.set(FlinkOptions.BUCKET_ASSIGN_TASKS, config.bucketAssignNum);
+    conf.set(FlinkOptions.WRITE_TASKS, config.writeTaskNum);
+    conf.set(FlinkOptions.PARTITION_DEFAULT_NAME, config.partitionDefaultName);
+    conf.set(FlinkOptions.INDEX_BOOTSTRAP_ENABLED, config.indexBootstrapEnabled);
+    conf.set(FlinkOptions.INDEX_STATE_TTL, config.indexStateTtl);
+    conf.set(FlinkOptions.INDEX_GLOBAL_ENABLED, config.indexGlobalEnabled);
+    conf.set(FlinkOptions.INDEX_PARTITION_REGEX, config.indexPartitionRegex);
     if (!StringUtils.isNullOrEmpty(config.sourceAvroSchemaPath)) {
-      conf.setString(FlinkOptions.SOURCE_AVRO_SCHEMA_PATH, config.sourceAvroSchemaPath);
+      conf.set(FlinkOptions.SOURCE_AVRO_SCHEMA_PATH, config.sourceAvroSchemaPath);
     }
-    conf.setString(FlinkOptions.SOURCE_AVRO_SCHEMA, config.sourceAvroSchema);
-    conf.setBoolean(FlinkOptions.READ_UTC_TIMEZONE, config.utcTimezone);
-    conf.setBoolean(FlinkOptions.URL_ENCODE_PARTITIONING, config.writePartitionUrlEncode);
-    conf.setBoolean(FlinkOptions.HIVE_STYLE_PARTITIONING, config.hiveStylePartitioning);
-    conf.setDouble(FlinkOptions.WRITE_TASK_MAX_SIZE, config.writeTaskMaxSize);
-    conf.setDouble(FlinkOptions.WRITE_BATCH_SIZE, config.writeBatchSize);
-    conf.setInteger(FlinkOptions.WRITE_LOG_BLOCK_SIZE, config.writeLogBlockSize);
-    conf.setLong(FlinkOptions.WRITE_LOG_MAX_SIZE, config.writeLogMaxSize);
-    conf.setInteger(FlinkOptions.WRITE_MERGE_MAX_MEMORY, config.writeMergeMaxMemory);
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, config.compactionAsyncEnabled);
-    conf.setInteger(FlinkOptions.COMPACTION_TASKS, config.compactionTasks);
-    conf.setString(FlinkOptions.COMPACTION_TRIGGER_STRATEGY, config.compactionTriggerStrategy);
-    conf.setInteger(FlinkOptions.COMPACTION_DELTA_COMMITS, config.compactionDeltaCommits);
-    conf.setInteger(FlinkOptions.COMPACTION_DELTA_SECONDS, config.compactionDeltaSeconds);
-    conf.setInteger(FlinkOptions.COMPACTION_MAX_MEMORY, config.compactionMaxMemory);
-    conf.setLong(FlinkOptions.COMPACTION_TARGET_IO, config.compactionTargetIo);
-    conf.setBoolean(FlinkOptions.CLUSTERING_ASYNC_ENABLED, config.clusteringAsyncEnabled);
-    conf.setInteger(FlinkOptions.CLUSTERING_TASKS, config.clusteringTasks);
-    conf.setInteger(FlinkOptions.CLUSTERING_DELTA_COMMITS, config.clusteringDeltaCommits);
-    conf.setString(FlinkOptions.CLUSTERING_PLAN_STRATEGY_CLASS, config.planStrategyClass);
-    conf.setString(FlinkOptions.CLUSTERING_PLAN_PARTITION_FILTER_MODE_NAME, config.planPartitionFilterMode);
-    conf.setLong(FlinkOptions.CLUSTERING_PLAN_STRATEGY_TARGET_FILE_MAX_BYTES, config.targetFileMaxBytes);
-    conf.setLong(FlinkOptions.CLUSTERING_PLAN_STRATEGY_SMALL_FILE_LIMIT, config.smallFileLimit);
-    conf.setInteger(FlinkOptions.CLUSTERING_PLAN_STRATEGY_SKIP_PARTITIONS_FROM_LATEST, config.skipFromLatestPartitions);
-    conf.setString(FlinkOptions.CLUSTERING_PLAN_STRATEGY_CLUSTER_BEGIN_PARTITION, config.clusterBeginPartition);
-    conf.setString(FlinkOptions.CLUSTERING_PLAN_STRATEGY_CLUSTER_END_PARTITION, config.clusterEndPartition);
-    conf.setString(FlinkOptions.CLUSTERING_PLAN_STRATEGY_PARTITION_REGEX_PATTERN, config.partitionRegexPattern);
-    conf.setString(FlinkOptions.CLUSTERING_PLAN_STRATEGY_PARTITION_SELECTED, config.partitionSelected);
-    conf.setString(FlinkOptions.CLUSTERING_SORT_COLUMNS, config.sortColumns);
-    conf.setInteger(FlinkOptions.WRITE_SORT_MEMORY, config.sortMemory);
-    conf.setInteger(FlinkOptions.CLUSTERING_MAX_NUM_GROUPS, config.maxNumGroups);
-    conf.setInteger(FlinkOptions.CLUSTERING_TARGET_PARTITIONS, config.targetPartitions);
-    conf.setBoolean(FlinkOptions.CLEAN_ASYNC_ENABLED, config.cleanAsyncEnabled);
-    conf.setString(FlinkOptions.CLEAN_POLICY, config.cleanPolicy);
-    conf.setInteger(FlinkOptions.CLEAN_RETAIN_COMMITS, config.cleanRetainCommits);
-    conf.setInteger(FlinkOptions.CLEAN_RETAIN_HOURS, config.cleanRetainHours);
-    conf.setInteger(FlinkOptions.CLEAN_RETAIN_FILE_VERSIONS, config.cleanRetainFileVersions);
-    conf.setInteger(FlinkOptions.ARCHIVE_MAX_COMMITS, config.archiveMaxCommits);
-    conf.setInteger(FlinkOptions.ARCHIVE_MIN_COMMITS, config.archiveMinCommits);
-    conf.setBoolean(FlinkOptions.HIVE_SYNC_ENABLED, config.hiveSyncEnabled);
-    conf.setString(FlinkOptions.HIVE_SYNC_DB, config.hiveSyncDb);
-    conf.setString(FlinkOptions.HIVE_SYNC_TABLE, config.hiveSyncTable);
-    conf.setString(FlinkOptions.HIVE_SYNC_FILE_FORMAT, config.hiveSyncFileFormat);
-    conf.setString(FlinkOptions.HIVE_SYNC_MODE, config.hiveSyncMode);
-    conf.setString(FlinkOptions.HIVE_SYNC_USERNAME, config.hiveSyncUsername);
-    conf.setString(FlinkOptions.HIVE_SYNC_PASSWORD, config.hiveSyncPassword);
-    conf.setString(FlinkOptions.HIVE_SYNC_JDBC_URL, config.hiveSyncJdbcUrl);
-    conf.setString(FlinkOptions.HIVE_SYNC_METASTORE_URIS, config.hiveSyncMetastoreUri);
-    conf.setString(FlinkOptions.HIVE_SYNC_PARTITION_FIELDS, config.hiveSyncPartitionFields);
-    conf.setString(FlinkOptions.HIVE_SYNC_PARTITION_EXTRACTOR_CLASS_NAME, config.hiveSyncPartitionExtractorClass);
-    conf.setBoolean(FlinkOptions.HIVE_SYNC_ASSUME_DATE_PARTITION, config.hiveSyncAssumeDatePartition);
-    conf.setBoolean(FlinkOptions.HIVE_SYNC_USE_JDBC, config.hiveSyncUseJdbc);
-    conf.setBoolean(FlinkOptions.HIVE_SYNC_AUTO_CREATE_DB, config.hiveSyncAutoCreateDb);
-    conf.setBoolean(FlinkOptions.HIVE_SYNC_IGNORE_EXCEPTIONS, config.hiveSyncIgnoreExceptions);
-    conf.setBoolean(FlinkOptions.HIVE_SYNC_SKIP_RO_SUFFIX, config.hiveSyncSkipRoSuffix);
-    conf.setBoolean(FlinkOptions.HIVE_SYNC_SUPPORT_TIMESTAMP, config.hiveSyncSupportTimestamp);
+    conf.set(FlinkOptions.SOURCE_AVRO_SCHEMA, config.sourceAvroSchema);
+    conf.set(FlinkOptions.READ_UTC_TIMEZONE, config.utcTimezone);
+    conf.set(FlinkOptions.URL_ENCODE_PARTITIONING, config.writePartitionUrlEncode);
+    conf.set(FlinkOptions.HIVE_STYLE_PARTITIONING, config.hiveStylePartitioning);
+    conf.set(FlinkOptions.WRITE_TASK_MAX_SIZE, config.writeTaskMaxSize);
+    conf.set(FlinkOptions.WRITE_BATCH_SIZE, config.writeBatchSize);
+    conf.set(FlinkOptions.WRITE_LOG_BLOCK_SIZE, config.writeLogBlockSize);
+    conf.set(FlinkOptions.WRITE_LOG_MAX_SIZE, config.writeLogMaxSize.longValue());
+    conf.set(FlinkOptions.WRITE_MERGE_MAX_MEMORY, config.writeMergeMaxMemory);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, config.compactionAsyncEnabled);
+    conf.set(FlinkOptions.COMPACTION_TASKS, config.compactionTasks);
+    conf.set(FlinkOptions.COMPACTION_TRIGGER_STRATEGY, config.compactionTriggerStrategy);
+    conf.set(FlinkOptions.COMPACTION_DELTA_COMMITS, config.compactionDeltaCommits);
+    conf.set(FlinkOptions.COMPACTION_DELTA_SECONDS, config.compactionDeltaSeconds);
+    conf.set(FlinkOptions.COMPACTION_MAX_MEMORY, config.compactionMaxMemory);
+    conf.set(FlinkOptions.COMPACTION_TARGET_IO, config.compactionTargetIo);
+    conf.set(FlinkOptions.CLUSTERING_ASYNC_ENABLED, config.clusteringAsyncEnabled);
+    conf.set(FlinkOptions.CLUSTERING_TASKS, config.clusteringTasks);
+    conf.set(FlinkOptions.CLUSTERING_DELTA_COMMITS, config.clusteringDeltaCommits);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_STRATEGY_CLASS, config.planStrategyClass);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_PARTITION_FILTER_MODE_NAME, config.planPartitionFilterMode);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_STRATEGY_TARGET_FILE_MAX_BYTES, config.targetFileMaxBytes);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_STRATEGY_SMALL_FILE_LIMIT, config.smallFileLimit);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_STRATEGY_SKIP_PARTITIONS_FROM_LATEST, config.skipFromLatestPartitions);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_STRATEGY_CLUSTER_BEGIN_PARTITION, config.clusterBeginPartition);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_STRATEGY_CLUSTER_END_PARTITION, config.clusterEndPartition);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_STRATEGY_PARTITION_REGEX_PATTERN, config.partitionRegexPattern);
+    conf.set(FlinkOptions.CLUSTERING_PLAN_STRATEGY_PARTITION_SELECTED, config.partitionSelected);
+    conf.set(FlinkOptions.CLUSTERING_SORT_COLUMNS, config.sortColumns);
+    conf.set(FlinkOptions.WRITE_SORT_MEMORY, config.sortMemory);
+    conf.set(FlinkOptions.CLUSTERING_MAX_NUM_GROUPS, config.maxNumGroups);
+    conf.set(FlinkOptions.CLUSTERING_TARGET_PARTITIONS, config.targetPartitions);
+    conf.set(FlinkOptions.CLEAN_ASYNC_ENABLED, config.cleanAsyncEnabled);
+    conf.set(FlinkOptions.CLEAN_POLICY, config.cleanPolicy);
+    conf.set(FlinkOptions.CLEAN_RETAIN_COMMITS, config.cleanRetainCommits);
+    conf.set(FlinkOptions.CLEAN_RETAIN_HOURS, config.cleanRetainHours);
+    conf.set(FlinkOptions.CLEAN_RETAIN_FILE_VERSIONS, config.cleanRetainFileVersions);
+    conf.set(FlinkOptions.ARCHIVE_MAX_COMMITS, config.archiveMaxCommits);
+    conf.set(FlinkOptions.ARCHIVE_MIN_COMMITS, config.archiveMinCommits);
+    conf.set(FlinkOptions.HIVE_SYNC_ENABLED, config.hiveSyncEnabled);
+    conf.set(FlinkOptions.HIVE_SYNC_DB, config.hiveSyncDb);
+    conf.set(FlinkOptions.HIVE_SYNC_TABLE, config.hiveSyncTable);
+    conf.set(FlinkOptions.HIVE_SYNC_FILE_FORMAT, config.hiveSyncFileFormat);
+    conf.set(FlinkOptions.HIVE_SYNC_MODE, config.hiveSyncMode);
+    conf.set(FlinkOptions.HIVE_SYNC_USERNAME, config.hiveSyncUsername);
+    conf.set(FlinkOptions.HIVE_SYNC_PASSWORD, config.hiveSyncPassword);
+    conf.set(FlinkOptions.HIVE_SYNC_JDBC_URL, config.hiveSyncJdbcUrl);
+    conf.set(FlinkOptions.HIVE_SYNC_METASTORE_URIS, config.hiveSyncMetastoreUri);
+    conf.set(FlinkOptions.HIVE_SYNC_PARTITION_FIELDS, config.hiveSyncPartitionFields);
+    conf.set(FlinkOptions.HIVE_SYNC_PARTITION_EXTRACTOR_CLASS_NAME, config.hiveSyncPartitionExtractorClass);
+    conf.set(FlinkOptions.HIVE_SYNC_ASSUME_DATE_PARTITION, config.hiveSyncAssumeDatePartition);
+    conf.set(FlinkOptions.HIVE_SYNC_USE_JDBC, config.hiveSyncUseJdbc);
+    conf.set(FlinkOptions.HIVE_SYNC_AUTO_CREATE_DB, config.hiveSyncAutoCreateDb);
+    conf.set(FlinkOptions.HIVE_SYNC_IGNORE_EXCEPTIONS, config.hiveSyncIgnoreExceptions);
+    conf.set(FlinkOptions.HIVE_SYNC_SKIP_RO_SUFFIX, config.hiveSyncSkipRoSuffix);
+    conf.set(FlinkOptions.HIVE_SYNC_SUPPORT_TIMESTAMP, config.hiveSyncSupportTimestamp);
     return conf;
   }
 }
