@@ -29,16 +29,15 @@ import org.apache.hudi.sink.transform.Transformer;
 import org.apache.hudi.sink.utils.Pipelines;
 import org.apache.hudi.util.AvroSchemaConverter;
 import org.apache.hudi.util.StreamerUtil;
+import org.apache.hudi.utils.StreamerUtils;
 
 import com.beust.jcommander.JCommander;
+import org.apache.flink.configuration.CheckpointingOptions;
 import org.apache.flink.configuration.Configuration;
-import org.apache.flink.formats.common.TimestampFormat;
-import org.apache.flink.formats.json.JsonRowDataDeserializationSchema;
+import org.apache.flink.configuration.StateBackendOptions;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
-import org.apache.flink.streaming.connectors.kafka.FlinkKafkaConsumer;
 import org.apache.flink.table.data.RowData;
-import org.apache.flink.table.runtime.typeutils.InternalTypeInfo;
 import org.apache.flink.table.types.logical.RowType;
 
 /**
@@ -48,7 +47,6 @@ import org.apache.flink.table.types.logical.RowType;
  */
 public class HoodieFlinkStreamer {
   public static void main(String[] args) throws Exception {
-    StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
 
     final FlinkStreamerConfig cfg = new FlinkStreamerConfig();
     JCommander cmd = new JCommander(cfg, null, args);
@@ -56,17 +54,19 @@ public class HoodieFlinkStreamer {
       cmd.usage();
       System.exit(1);
     }
+    Configuration envConf = new Configuration();
+    envConf.set(StateBackendOptions.STATE_BACKEND, cfg.stateBackend.getName());
+    envConf.set(CheckpointingOptions.CHECKPOINT_STORAGE, "filesystem");
+    if (cfg.flinkCheckPointPath != null) {
+      envConf.set(CheckpointingOptions. CHECKPOINTS_DIRECTORY, cfg.flinkCheckPointPath);
+    }
+    StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment(envConf);
     env.enableCheckpointing(cfg.checkpointInterval);
     env.getConfig().setGlobalJobParameters(cfg);
     // We use checkpoint to trigger write operation, including instant generating and committing,
     // There can only be one checkpoint at one time.
     env.getCheckpointConfig().setMaxConcurrentCheckpoints(1);
 
-    env.setStateBackend(cfg.stateBackend);
-    if (cfg.flinkCheckPointPath != null) {
-      env.getCheckpointConfig().setCheckpointStorage(cfg.flinkCheckPointPath);
-    }
-
     TypedProperties kafkaProps = DFSPropertiesConfiguration.getGlobalProps();
     kafkaProps.putAll(StreamerUtil.appendKafkaProps(cfg));
 
@@ -77,20 +77,9 @@ public class HoodieFlinkStreamer {
             .getLogicalType();
 
     long ckpTimeout = env.getCheckpointConfig().getCheckpointTimeout();
-    conf.setLong(FlinkOptions.WRITE_COMMIT_ACK_TIMEOUT, ckpTimeout);
-
-    DataStream<RowData> dataStream = env.addSource(new FlinkKafkaConsumer<>(
-            cfg.kafkaTopic,
-            new JsonRowDataDeserializationSchema(
-                rowType,
-                InternalTypeInfo.of(rowType),
-                false,
-                true,
-                TimestampFormat.ISO_8601
-            ), kafkaProps))
-        .name("kafka_source")
-        .uid("uid_kafka_source");
+    conf.set(FlinkOptions.WRITE_COMMIT_ACK_TIMEOUT, ckpTimeout);
 
+    DataStream<RowData> dataStream = StreamerUtils.createKafkaStream(env, rowType, cfg.kafkaTopic, kafkaProps);
     if (cfg.transformerClassNames != null && !cfg.transformerClassNames.isEmpty()) {
       Option<Transformer> transformer = StreamerUtil.createTransformer(cfg.transformerClassNames);
       if (transformer.isPresent()) {
@@ -100,7 +89,7 @@ public class HoodieFlinkStreamer {
 
     OptionsInference.setupSinkTasks(conf, env.getParallelism());
     OptionsInference.setupClientId(conf);
-    DataStream<Object> pipeline;
+    DataStream<RowData> pipeline;
     // Append mode
     if (OptionsResolver.isAppendMode(conf)) {
       // append mode should not compaction operator
