@@ -32,6 +32,7 @@ import org.apache.hudi.keygen.NonpartitionedAvroKeyGenerator;
 import org.apache.hudi.util.AvroSchemaConverter;
 import org.apache.hudi.util.DataTypeUtils;
 import org.apache.hudi.util.StreamerUtil;
+import org.apache.hudi.utils.CatalogUtils;
 
 import org.apache.avro.Schema;
 import org.apache.flink.annotation.VisibleForTesting;
@@ -44,11 +45,11 @@ import org.apache.flink.table.catalog.CatalogDatabaseImpl;
 import org.apache.flink.table.catalog.CatalogFunction;
 import org.apache.flink.table.catalog.CatalogPartition;
 import org.apache.flink.table.catalog.CatalogPartitionSpec;
-import org.apache.flink.table.catalog.CatalogTable;
 import org.apache.flink.table.catalog.CatalogView;
 import org.apache.flink.table.catalog.ObjectPath;
 import org.apache.flink.table.catalog.ResolvedCatalogTable;
 import org.apache.flink.table.catalog.ResolvedSchema;
+import org.apache.flink.table.catalog.UniqueConstraint;
 import org.apache.flink.table.catalog.exceptions.CatalogException;
 import org.apache.flink.table.catalog.exceptions.DatabaseAlreadyExistException;
 import org.apache.flink.table.catalog.exceptions.DatabaseNotEmptyException;
@@ -273,11 +274,11 @@ public class HoodieCatalog extends AbstractCatalog {
         builder.primaryKey(pkColumns);
       }
       final org.apache.flink.table.api.Schema schema = builder.build();
-      return CatalogTable.of(
+      return CatalogUtils.createCatalogTable(
           schema,
-          TableOptionProperties.getComment(options),
           TableOptionProperties.getPartitionColumns(options),
-          TableOptionProperties.getTableOptions(options));
+          TableOptionProperties.getTableOptions(options),
+          TableOptionProperties.getComment(options));
     } else {
       throw new TableNotExistException(getName(), tablePath);
     }
@@ -306,7 +307,7 @@ public class HoodieCatalog extends AbstractCatalog {
     final String tablePathStr = inferTablePath(catalogPathStr, tablePath);
     Map<String, String> options = applyOptionsHook(tablePathStr, catalogTable.getOptions());
     Configuration conf = Configuration.fromMap(options);
-    conf.setString(FlinkOptions.PATH, tablePathStr);
+    conf.set(FlinkOptions.PATH, tablePathStr);
     ResolvedSchema resolvedSchema = resolvedTable.getResolvedSchema();
     if (!resolvedSchema.getPrimaryKey().isPresent() && !conf.containsKey(RECORD_KEY_FIELD.key())) {
       throw new CatalogException("Primary key definition is missing");
@@ -314,7 +315,7 @@ public class HoodieCatalog extends AbstractCatalog {
     final String avroSchema = AvroSchemaConverter.convertToSchema(
         resolvedSchema.toPhysicalRowDataType().getLogicalType(),
         AvroSchemaUtils.getAvroRecordQualifiedName(tablePath.getObjectName())).toString();
-    conf.setString(FlinkOptions.SOURCE_AVRO_SCHEMA, avroSchema);
+    conf.set(FlinkOptions.SOURCE_AVRO_SCHEMA, avroSchema);
 
     // stores two copies of options:
     // - partition keys
@@ -326,38 +327,39 @@ public class HoodieCatalog extends AbstractCatalog {
     if (resolvedSchema.getPrimaryKey().isPresent()
             && !conf.containsKey(FlinkOptions.RECORD_KEY_FIELD.key())) {
       final String pkColumns = String.join(",", resolvedSchema.getPrimaryKey().get().getColumns());
-      conf.setString(RECORD_KEY_FIELD, pkColumns);
+      conf.set(RECORD_KEY_FIELD, pkColumns);
     }
 
     if (resolvedSchema.getPrimaryKey().isPresent()) {
       options.put(TableOptionProperties.PK_CONSTRAINT_NAME, resolvedSchema.getPrimaryKey().get().getName());
     }
     if (conf.containsKey(RECORD_KEY_FIELD.key())) {
-      options.put(TableOptionProperties.PK_COLUMNS, conf.getString(RECORD_KEY_FIELD));
+      options.put(TableOptionProperties.PK_COLUMNS, conf.get(RECORD_KEY_FIELD));
     }
 
     // check preCombine
-    StreamerUtil.checkPreCombineKey(conf, resolvedSchema.getColumnNames());
+    StreamerUtil.checkOrderingFields(conf, resolvedSchema.getColumnNames());
 
     if (resolvedTable.isPartitioned()) {
       final String partitions = String.join(",", resolvedTable.getPartitionKeys());
-      conf.setString(FlinkOptions.PARTITION_PATH_FIELD, partitions);
+      conf.set(FlinkOptions.PARTITION_PATH_FIELD, partitions);
       options.put(TableOptionProperties.PARTITION_COLUMNS, partitions);
 
-      final String[] pks = conf.getString(FlinkOptions.RECORD_KEY_FIELD).split(",");
+      final String[] pks = conf.get(FlinkOptions.RECORD_KEY_FIELD).split(",");
       boolean complexHoodieKey = pks.length > 1 || resolvedTable.getPartitionKeys().size() > 1;
       StreamerUtil.checkKeygenGenerator(complexHoodieKey, conf);
     } else {
       conf.setString(FlinkOptions.KEYGEN_CLASS_NAME.key(), NonpartitionedAvroKeyGenerator.class.getName());
     }
-    conf.setString(FlinkOptions.TABLE_NAME, tablePath.getObjectName());
+    conf.set(FlinkOptions.TABLE_NAME, tablePath.getObjectName());
     try {
-      StreamerUtil.initTableIfNotExists(conf);
+      HoodieTableMetaClient metaClient = StreamerUtil.initTableIfNotExists(conf);
       // prepare the non-table-options properties
       if (!StringUtils.isNullOrEmpty(resolvedTable.getComment())) {
         options.put(TableOptionProperties.COMMENT, resolvedTable.getComment());
       }
       TableOptionProperties.createProperties(tablePathStr, hadoopConf, options);
+      HoodieCatalogUtil.initPartitionBucketIndexMeta(metaClient, catalogTable);
     } catch (IOException e) {
       throw new CatalogException(String.format("Initialize table path %s exception.", tablePathStr), e);
     }
@@ -479,8 +481,8 @@ public class HoodieCatalog extends AbstractCatalog {
     }
 
     try (HoodieFlinkWriteClient<?> writeClient = HoodieCatalogUtil.createWriteClient(options, tablePathStr, tablePath, hadoopConf)) {
-      writeClient.deletePartitions(Collections.singletonList(partitionPathStr),
-              writeClient.createNewInstantTime())
+      String instantTime = writeClient.startDeletePartitionCommit();
+      writeClient.deletePartitions(Collections.singletonList(partitionPathStr), instantTime)
           .forEach(writeStatus -> {
             if (writeStatus.hasErrors()) {
               throw new HoodieMetadataException(String.format("Failed to commit metadata table records at file id %s.", writeStatus.getFileId()));
@@ -585,7 +587,7 @@ public class HoodieCatalog extends AbstractCatalog {
         HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(path, hadoopConf);
         return new TableSchemaResolver(metaClient).getTableAvroSchema(false); // change log mode is not supported now
       } catch (Throwable throwable) {
-        LOG.warn("Error while resolving the latest table schema.", throwable);
+        LOG.warn("Failed to resolve the latest table schema.", throwable);
         // ignored
       }
     }
@@ -602,10 +604,20 @@ public class HoodieCatalog extends AbstractCatalog {
 
   private void refreshTableProperties(ObjectPath tablePath, CatalogBaseTable newCatalogTable) {
     Map<String, String> options = newCatalogTable.getOptions();
+    ResolvedCatalogTable resolvedTable =  (ResolvedCatalogTable) newCatalogTable;
     final String avroSchema = AvroSchemaConverter.convertToSchema(
-        ((ResolvedCatalogTable) newCatalogTable).getResolvedSchema().toPhysicalRowDataType().getLogicalType(),
+        resolvedTable.getResolvedSchema().toPhysicalRowDataType().getLogicalType(),
         AvroSchemaUtils.getAvroRecordQualifiedName(tablePath.getObjectName())).toString();
     options.put(FlinkOptions.SOURCE_AVRO_SCHEMA.key(), avroSchema);
+    java.util.Optional<UniqueConstraint> pkConstraintOpt = resolvedTable.getResolvedSchema().getPrimaryKey();
+    if (pkConstraintOpt.isPresent()) {
+      options.put(TableOptionProperties.PK_COLUMNS, String.join(",", pkConstraintOpt.get().getColumns()));
+      options.put(TableOptionProperties.PK_CONSTRAINT_NAME, pkConstraintOpt.get().getName());
+    }
+    if (resolvedTable.isPartitioned()) {
+      final String partitions = String.join(",", resolvedTable.getPartitionKeys());
+      options.put(TableOptionProperties.PARTITION_COLUMNS, partitions);
+    }
     String tablePathStr = inferTablePath(catalogPathStr, tablePath);
     try {
       TableOptionProperties.overwriteProperties(tablePathStr, hadoopConf, options);
