@@ -28,6 +28,7 @@ import org.apache.hudi.hadoop.fs.HadoopFSUtils;
 import org.apache.hudi.storage.StoragePath;
 import org.apache.hudi.sync.common.util.SparkDataSourceTableUtils;
 import org.apache.hudi.util.AvroSchemaConverter;
+import org.apache.hudi.util.DataTypeUtils;
 
 import org.apache.avro.Schema;
 import org.apache.flink.table.catalog.CatalogTable;
@@ -89,13 +90,13 @@ public class TableOptionProperties {
 
     KEY_MAPPING.put("type", FlinkOptions.TABLE_TYPE.key());
     KEY_MAPPING.put("primaryKey", FlinkOptions.RECORD_KEY_FIELD.key());
-    KEY_MAPPING.put("preCombineField", FlinkOptions.PRECOMBINE_FIELD.key());
+    KEY_MAPPING.put("orderingFields", FlinkOptions.ORDERING_FIELDS.key());
     KEY_MAPPING.put("payloadClass", FlinkOptions.PAYLOAD_CLASS_NAME.key());
     KEY_MAPPING.put(SPARK_SOURCE_PROVIDER, CONNECTOR.key());
     KEY_MAPPING.put(FlinkOptions.KEYGEN_CLASS_NAME.key(), FlinkOptions.KEYGEN_CLASS_NAME.key());
     KEY_MAPPING.put(FlinkOptions.TABLE_TYPE.key(), "type");
     KEY_MAPPING.put(FlinkOptions.RECORD_KEY_FIELD.key(), "primaryKey");
-    KEY_MAPPING.put(FlinkOptions.PRECOMBINE_FIELD.key(), "preCombineField");
+    KEY_MAPPING.put(FlinkOptions.ORDERING_FIELDS.key(), "orderingFields");
     KEY_MAPPING.put(FlinkOptions.PAYLOAD_CLASS_NAME.key(), "payloadClass");
   }
 
@@ -196,7 +197,7 @@ public class TableOptionProperties {
       Map<String, String> properties,
       List<String> partitionKeys,
       boolean withOperationField) {
-    RowType rowType = supplementMetaFields((RowType) catalogTable.getSchema().toPhysicalRowDataType().getLogicalType(), withOperationField);
+    RowType rowType = supplementMetaFields(DataTypeUtils.toRowType(catalogTable.getUnresolvedSchema()), withOperationField);
     Schema schema = AvroSchemaConverter.convertToSchema(rowType);
     MessageType messageType = ParquetTableSchemaResolver.convertAvroSchemaToParquet(schema, hadoopConf);
     String sparkVersion = catalogTable.getOptions().getOrDefault(SPARK_VERSION, DEFAULT_SPARK_VERSION);
