@@ -210,10 +210,8 @@ public class CopyOnWriteInputFormat extends FileInputFormat<RowData> {
       if (this.minSplitSize <= blockSize) {
         minSplitSize = this.minSplitSize;
       } else {
-        if (LOG.isWarnEnabled()) {
-          LOG.warn("Minimal split size of " + this.minSplitSize + " is larger than the block size of "
-              + blockSize + ". Decreasing minimal split size to block size.");
-        }
+        LOG.warn("Minimal split size of {} is larger than the block size of {}."
+            + " Decreasing minimal split size to block size.", this.minSplitSize, blockSize);
         minSplitSize = blockSize;
       }
 
@@ -270,7 +268,6 @@ public class CopyOnWriteInputFormat extends FileInputFormat<RowData> {
     return inputSplits.toArray(new FileInputSplit[0]);
   }
 
-  @Override
   public boolean supportsMultiPaths() {
     return true;
   }
@@ -315,8 +312,8 @@ public class CopyOnWriteInputFormat extends FileInputFormat<RowData> {
         if (acceptFile(dir) && enumerateNestedFiles) {
           length += addFilesInDir(dir.getPath(), files, logExcludedFiles);
         } else {
-          if (logExcludedFiles && LOG.isDebugEnabled()) {
-            LOG.debug("Directory " + dir.getPath().toString() + " did not pass the file-filter and is excluded.");
+          if (logExcludedFiles) {
+            LOG.debug("Directory {} did not pass the file-filter and is excluded.", dir.getPath());
           }
         }
       } else {
@@ -325,8 +322,8 @@ public class CopyOnWriteInputFormat extends FileInputFormat<RowData> {
           length += dir.getLen();
           testForUnsplittable(dir);
         } else {
-          if (logExcludedFiles && LOG.isDebugEnabled()) {
-            LOG.debug("Directory " + dir.getPath().toString() + " did not pass the file-filter and is excluded.");
+          if (logExcludedFiles) {
+            LOG.debug("Directory {} did not pass the file-filter and is excluded.", dir.getPath());
           }
         }
       }
