@@ -18,17 +18,26 @@
 
 package org.apache.hudi.util;
 
+import org.apache.hudi.client.WriteStatus;
+import org.apache.hudi.client.model.CommitTimeFlinkRecordMerger;
+import org.apache.hudi.client.model.EventTimeFlinkRecordMerger;
+import org.apache.hudi.client.model.PartialUpdateFlinkRecordMerger;
 import org.apache.hudi.client.transaction.lock.FileSystemBasedLockProvider;
 import org.apache.hudi.common.config.DFSPropertiesConfiguration;
 import org.apache.hudi.common.config.HoodieMetadataConfig;
 import org.apache.hudi.common.config.HoodieTimeGeneratorConfig;
+import org.apache.hudi.common.config.RecordMergeMode;
 import org.apache.hudi.common.config.TypedProperties;
 import org.apache.hudi.common.engine.EngineType;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.DefaultHoodieRecordPayload;
+import org.apache.hudi.common.model.HoodieKey;
 import org.apache.hudi.common.model.HoodieTableType;
+import org.apache.hudi.common.model.OverwriteWithLatestAvroPayload;
+import org.apache.hudi.common.model.PartialUpdateAvroPayload;
 import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
+import org.apache.hudi.common.table.HoodieTableVersion;
 import org.apache.hudi.common.table.TableSchemaResolver;
 import org.apache.hudi.common.table.log.HoodieLogFormat;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
@@ -39,6 +48,7 @@ import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.ReflectionUtils;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.common.util.ValidationUtils;
+import org.apache.hudi.common.util.collection.Triple;
 import org.apache.hudi.config.HoodieIndexConfig;
 import org.apache.hudi.config.HoodieLockConfig;
 import org.apache.hudi.config.HoodiePayloadConfig;
@@ -77,7 +87,9 @@ import java.io.StringReader;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Date;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.Properties;
 
 import static org.apache.hudi.common.model.HoodieFileFormat.HOODIE_LOG;
@@ -87,6 +99,7 @@ import static org.apache.hudi.common.table.HoodieTableConfig.TIMELINE_HISTORY_PA
 import static org.apache.hudi.common.table.timeline.InstantComparison.GREATER_THAN_OR_EQUALS;
 import static org.apache.hudi.common.table.timeline.InstantComparison.LESSER_THAN_OR_EQUALS;
 import static org.apache.hudi.common.table.timeline.InstantComparison.compareTimestamps;
+import static org.apache.hudi.configuration.FlinkOptions.WRITE_FAIL_FAST;
 
 /**
  * Utilities for Flink stream read and write.
@@ -95,6 +108,8 @@ public class StreamerUtil {
 
   private static final Logger LOG = LoggerFactory.getLogger(StreamerUtil.class);
 
+  public static final String FLINK_CHECKPOINT_ID = "flink_checkpoint_id";
+
   public static TypedProperties appendKafkaProps(FlinkStreamerConfig config) {
     TypedProperties properties = getProps(config);
     properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, config.kafkaBootstrapServers);
@@ -158,9 +173,9 @@ public class StreamerUtil {
    */
   public static HoodiePayloadConfig getPayloadConfig(Configuration conf) {
     return HoodiePayloadConfig.newBuilder()
-        .withPayloadClass(conf.getString(FlinkOptions.PAYLOAD_CLASS_NAME))
-        .withPayloadOrderingField(conf.getString(FlinkOptions.PRECOMBINE_FIELD))
-        .withPayloadEventTimeField(conf.getString(FlinkOptions.PRECOMBINE_FIELD))
+        .withPayloadClass(conf.get(FlinkOptions.PAYLOAD_CLASS_NAME))
+        .withPayloadOrderingFields(conf.get(FlinkOptions.ORDERING_FIELDS))
+        .withPayloadEventTimeField(conf.get(FlinkOptions.ORDERING_FIELDS))
         .build();
   }
 
@@ -170,8 +185,8 @@ public class StreamerUtil {
   public static HoodieIndexConfig getIndexConfig(Configuration conf) {
     return HoodieIndexConfig.newBuilder()
         .withIndexType(OptionsResolver.getIndexType(conf))
-        .withBucketNum(String.valueOf(conf.getInteger(FlinkOptions.BUCKET_INDEX_NUM_BUCKETS)))
-        .withRecordKeyField(conf.getString(FlinkOptions.RECORD_KEY_FIELD))
+        .withBucketNum(String.valueOf(conf.get(FlinkOptions.BUCKET_INDEX_NUM_BUCKETS)))
+        .withRecordKeyField(conf.get(FlinkOptions.RECORD_KEY_FIELD))
         .withIndexKeyField(OptionsResolver.getIndexKeyField(conf))
         .withBucketIndexEngineType(OptionsResolver.getBucketEngineType(conf))
         .withEngineType(EngineType.FLINK)
@@ -185,7 +200,7 @@ public class StreamerUtil {
     if (OptionsResolver.isLockRequired(conf) && !conf.containsKey(HoodieLockConfig.LOCK_PROVIDER_CLASS_NAME.key())) {
       // configure the fs lock provider by default
       return Option.of(HoodieLockConfig.newBuilder()
-          .fromProperties(FileSystemBasedLockProvider.getLockConfig(conf.getString(FlinkOptions.PATH)))
+          .fromProperties(FileSystemBasedLockProvider.getLockConfig(conf.get(FlinkOptions.PATH)))
           .withConflictResolutionStrategy(OptionsResolver.getConflictResolutionStrategy(conf))
           .build());
     }
@@ -205,7 +220,7 @@ public class StreamerUtil {
     }
 
     return HoodieTimeGeneratorConfig.newBuilder()
-        .withPath(conf.getString(FlinkOptions.PATH))
+        .withPath(conf.get(FlinkOptions.PATH))
         .fromProperties(properties)
         .build();
   }
@@ -228,10 +243,26 @@ public class StreamerUtil {
         properties.put(option.key(), option.defaultValue());
       }
     }
-    properties.put(HoodieTableConfig.TYPE.key(), conf.getString(FlinkOptions.TABLE_TYPE));
+    properties.put(HoodieTableConfig.TYPE.key(), conf.get(FlinkOptions.TABLE_TYPE));
     return properties;
   }
 
+  public static void initTableFromClientIfNecessary(Configuration conf) {
+    // Since Flink 2.0, the adaptive execution for batch job will generate job graph incrementally
+    // for multiple stages (FLIP-469). And the write coordinator is initialized along with write
+    // operator in the final stage, so hudi table should be initialized if necessary during the plan
+    // compilation phase when adaptive execution is enabled.
+    if (OptionsResolver.isIncrementalJobGraph(conf)
+        || OptionsResolver.isPartitionLevelSimpleBucketIndex(conf)) {
+      // init table, create if not exists.
+      try {
+        StreamerUtil.initTableIfNotExists(conf);
+      } catch (IOException e) {
+        throw new HoodieException("Failed to initialize table.", e);
+      }
+    }
+  }
+
   /**
    * Initialize the table if it does not exist.
    *
@@ -251,31 +282,34 @@ public class StreamerUtil {
   public static HoodieTableMetaClient initTableIfNotExists(
       Configuration conf,
       org.apache.hadoop.conf.Configuration hadoopConf) throws IOException {
-    final String basePath = conf.getString(FlinkOptions.PATH);
+    final String basePath = conf.get(FlinkOptions.PATH);
     if (!tableExists(basePath, hadoopConf)) {
       HoodieTableMetaClient.newTableBuilder()
-          .setTableCreateSchema(conf.getString(FlinkOptions.SOURCE_AVRO_SCHEMA))
-          .setTableType(conf.getString(FlinkOptions.TABLE_TYPE))
-          .setTableName(conf.getString(FlinkOptions.TABLE_NAME))
-          .setTableVersion(conf.getInteger(FlinkOptions.WRITE_TABLE_VERSION))
-          .setDatabaseName(conf.getString(FlinkOptions.DATABASE_NAME))
-          .setRecordKeyFields(conf.getString(FlinkOptions.RECORD_KEY_FIELD, null))
-          .setPayloadClassName(conf.getString(FlinkOptions.PAYLOAD_CLASS_NAME))
-          .setPreCombineField(OptionsResolver.getPreCombineField(conf))
+          .setTableCreateSchema(conf.get(FlinkOptions.SOURCE_AVRO_SCHEMA))
+          .setTableType(conf.get(FlinkOptions.TABLE_TYPE))
+          .setTableName(conf.get(FlinkOptions.TABLE_NAME))
+          .setTableVersion(conf.get(FlinkOptions.WRITE_TABLE_VERSION))
+          .setTableFormat(conf.get(FlinkOptions.WRITE_TABLE_FORMAT))
+          .setRecordMergeMode(getMergeMode(conf))
+          .setRecordMergeStrategyId(getMergeStrategyId(conf))
+          .setPayloadClassName(getPayloadClass(conf))
+          .setDatabaseName(conf.get(FlinkOptions.DATABASE_NAME))
+          .setRecordKeyFields(conf.getString(FlinkOptions.RECORD_KEY_FIELD.key(), null))
+          .setOrderingFields(OptionsResolver.getOrderingFieldsStr(conf))
           .setArchiveLogFolder(TIMELINE_HISTORY_PATH.defaultValue())
-          .setPartitionFields(conf.getString(FlinkOptions.PARTITION_PATH_FIELD, null))
+          .setPartitionFields(conf.getString(FlinkOptions.PARTITION_PATH_FIELD.key(), null))
           .setKeyGeneratorClassProp(
               conf.getOptional(FlinkOptions.KEYGEN_CLASS_NAME).orElse(SimpleAvroKeyGenerator.class.getName()))
-          .setHiveStylePartitioningEnable(conf.getBoolean(FlinkOptions.HIVE_STYLE_PARTITIONING))
-          .setUrlEncodePartitioning(conf.getBoolean(FlinkOptions.URL_ENCODE_PARTITIONING))
-          .setCDCEnabled(conf.getBoolean(FlinkOptions.CDC_ENABLED))
-          .setCDCSupplementalLoggingMode(conf.getString(FlinkOptions.SUPPLEMENTAL_LOGGING_MODE))
+          .setHiveStylePartitioningEnable(conf.get(FlinkOptions.HIVE_STYLE_PARTITIONING))
+          .setUrlEncodePartitioning(conf.get(FlinkOptions.URL_ENCODE_PARTITIONING))
+          .setCDCEnabled(conf.get(FlinkOptions.CDC_ENABLED))
+          .setCDCSupplementalLoggingMode(conf.get(FlinkOptions.SUPPLEMENTAL_LOGGING_MODE))
           .setPopulateMetaFields(OptionsResolver.isPopulateMetaFields(conf))
           .initTable(HadoopFSUtils.getStorageConfWithCopy(hadoopConf), basePath);
       LOG.info("Table initialized under base path {}", basePath);
     } else {
       LOG.info("Table [path={}, name={}] already exists, no need to initialize the table",
-          basePath, conf.getString(FlinkOptions.TABLE_NAME));
+          basePath, conf.get(FlinkOptions.TABLE_NAME));
     }
 
     return StreamerUtil.createMetaClient(conf, hadoopConf);
@@ -284,6 +318,90 @@ public class StreamerUtil {
     // some filesystems release the handles in #close method.
   }
 
+  private static String getMergeStrategyId(Configuration conf) {
+    return conf.getString(HoodieTableConfig.RECORD_MERGE_STRATEGY_ID.key(), null);
+  }
+
+  private static String getPayloadClass(Configuration conf) {
+    if (conf.contains(FlinkOptions.RECORD_MERGER_IMPLS)) {
+      // check whether contains partial update merger class
+      String mergerClasses = conf.get(FlinkOptions.RECORD_MERGER_IMPLS);
+      if (mergerClasses.contains(PartialUpdateFlinkRecordMerger.class.getSimpleName())) {
+        return PartialUpdateAvroPayload.class.getName();
+      }
+    }
+    if (conf.contains(FlinkOptions.PAYLOAD_CLASS_NAME)) {
+      return conf.get(FlinkOptions.PAYLOAD_CLASS_NAME);
+    } else if (getMergeMode(conf) == RecordMergeMode.COMMIT_TIME_ORDERING) {
+      return OverwriteWithLatestAvroPayload.class.getName();
+    } else {
+      // payload inferred in HoodieTableConfig for EVENT_TIME_ORDERING is DefaultHoodieRecordPayload,
+      // but Flink use EventTimeAvroPayload, so return default value here if necessary
+      return conf.get(FlinkOptions.PAYLOAD_CLASS_NAME);
+    }
+  }
+
+  /**
+   * Returns the merge mode.
+   */
+  public static RecordMergeMode getMergeMode(Configuration conf) {
+    if (conf.contains(FlinkOptions.RECORD_MERGE_MODE)) {
+      return RecordMergeMode.valueOf(conf.get(FlinkOptions.RECORD_MERGE_MODE));
+    } else {
+      return null;
+    }
+  }
+
+  /**
+   * Returns the merger classes.
+   */
+  public static String getMergerClasses(Configuration conf, RecordMergeMode mergeMode, String payloadClass) {
+    switch (mergeMode) {
+      case EVENT_TIME_ORDERING:
+        if (payloadClass.contains(PartialUpdateAvroPayload.class.getSimpleName())) {
+          return PartialUpdateFlinkRecordMerger.class.getName();
+        } else {
+          return EventTimeFlinkRecordMerger.class.getName();
+        }
+      case COMMIT_TIME_ORDERING:
+        return CommitTimeFlinkRecordMerger.class.getName();
+      default:
+        return conf.get(FlinkOptions.RECORD_MERGER_IMPLS);
+    }
+  }
+
+  /**
+   * Write Flink checkpoint id as extra metadata, if write.extra.metadata.enabled is true.
+   *
+   * @param conf Flink configuration
+   * @param checkpointCommitMetadata commit metadata map
+   * @param checkpointId flink checkpoint id
+   */
+  public static void addFlinkCheckpointIdIntoMetaData(
+      Configuration conf,
+      HashMap<String, String> checkpointCommitMetadata,
+      long checkpointId) {
+    if (conf.get(FlinkOptions.WRITE_EXTRA_METADATA_ENABLED)) {
+      checkpointCommitMetadata.put(FLINK_CHECKPOINT_ID, String.valueOf(checkpointId));
+    }
+  }
+
+  /**
+   * Infers the merging behavior based on what the user sets (or doesn't set).
+   *
+   * @param conf Flink configuration
+   * @return The correct merging behaviour: <merge_mode, payload_class, merge_strategy_id>
+   */
+  public static Triple<RecordMergeMode, String, String> inferMergingBehavior(Configuration conf) {
+    String payloadClassName = getPayloadClass(conf);
+    Map<String, String> mergeConf = HoodieTableConfig.inferMergingConfigsForV9TableCreation(
+        getMergeMode(conf), payloadClassName, getMergeStrategyId(conf), OptionsResolver.getOrderingFieldsStr(conf), HoodieTableVersion.current());
+    String mergeMode = mergeConf.get(HoodieTableConfig.RECORD_MERGE_MODE.key());
+    String mergeStrategyId = mergeConf.get(HoodieTableConfig.RECORD_MERGE_STRATEGY_ID.key());
+    ValidationUtils.checkArgument(mergeMode != null, "Merge mode should not be null");
+    return Triple.of(RecordMergeMode.valueOf(mergeMode), payloadClassName, mergeStrategyId);
+  }
+
   /**
    * Returns whether the hoodie table exists under given path {@code basePath}.
    */
@@ -319,7 +437,11 @@ public class StreamerUtil {
    * Generates the bucket ID using format {partition path}_{fileID}.
    */
   public static String generateBucketKey(String partitionPath, String fileId) {
-    return partitionPath + "_" + fileId;
+    return new StringBuilder()
+        .append(partitionPath)
+        .append('_')
+        .append(fileId)
+        .toString();
   }
 
   /**
@@ -333,8 +455,8 @@ public class StreamerUtil {
   public static HoodieTableMetaClient metaClientForReader(
       Configuration conf,
       org.apache.hadoop.conf.Configuration hadoopConf) {
-    final String basePath = conf.getString(FlinkOptions.PATH);
-    if (conf.getBoolean(FlinkOptions.READ_AS_STREAMING) && !tableExists(basePath, hadoopConf)) {
+    final String basePath = conf.get(FlinkOptions.PATH);
+    if (conf.get(FlinkOptions.READ_AS_STREAMING) && !tableExists(basePath, hadoopConf)) {
       return null;
     } else {
       return createMetaClient(basePath, hadoopConf);
@@ -363,7 +485,7 @@ public class StreamerUtil {
    */
   public static HoodieTableMetaClient createMetaClient(Configuration conf, org.apache.hadoop.conf.Configuration hadoopConf) {
     return HoodieTableMetaClient.builder()
-        .setBasePath(conf.getString(FlinkOptions.PATH))
+        .setBasePath(conf.get(FlinkOptions.PATH))
         .setConf(HadoopFSUtils.getStorageConfWithCopy(hadoopConf))
         .setTimeGeneratorConfig(getTimeGeneratorConfig(conf))
         .build();
@@ -377,7 +499,7 @@ public class StreamerUtil {
     StoragePath metaPath = new StoragePath(basePath, HoodieTableMetaClient.METAFOLDER_NAME);
     try {
       if (storage.exists(new StoragePath(metaPath, HoodieTableConfig.HOODIE_PROPERTIES_FILE))) {
-        return Option.of(new HoodieTableConfig(storage, metaPath, null, null, null));
+        return Option.of(new HoodieTableConfig(storage, metaPath));
       }
     } catch (IOException e) {
       throw new HoodieIOException("Get table config error", e);
@@ -483,7 +605,7 @@ public class StreamerUtil {
    * Returns the max compaction memory in bytes with given conf.
    */
   public static long getMaxCompactionMemoryInBytes(Configuration conf) {
-    return (long) conf.getInteger(FlinkOptions.COMPACTION_MAX_MEMORY) * 1024 * 1024;
+    return (long) conf.get(FlinkOptions.COMPACTION_MAX_MEMORY) * 1024 * 1024;
   }
 
   public static Schema getTableAvroSchema(HoodieTableMetaClient metaClient, boolean includeMetadataFields) throws Exception {
@@ -500,7 +622,7 @@ public class StreamerUtil {
       HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(path, hadoopConf);
       return getTableAvroSchema(metaClient, false);
     } catch (Exception e) {
-      LOG.warn("Error while resolving the latest table schema", e);
+      LOG.error("Failed to resolve the latest table schema", e);
     }
     return null;
   }
@@ -529,18 +651,18 @@ public class StreamerUtil {
   /**
    * Validate pre_combine key.
    */
-  public static void checkPreCombineKey(Configuration conf, List<String> fields) {
-    String preCombineField = conf.get(FlinkOptions.PRECOMBINE_FIELD);
-    if (!fields.contains(preCombineField)) {
+  public static void checkOrderingFields(Configuration conf, List<String> fields) {
+    String orderingFields = conf.get(FlinkOptions.ORDERING_FIELDS);
+    if (!fields.contains(orderingFields)) {
       if (OptionsResolver.isDefaultHoodieRecordPayloadClazz(conf)) {
-        throw new HoodieValidationException("Option '" + FlinkOptions.PRECOMBINE_FIELD.key()
+        throw new HoodieValidationException("Option '" + FlinkOptions.ORDERING_FIELDS.key()
                 + "' is required for payload class: " + DefaultHoodieRecordPayload.class.getName());
       }
-      if (preCombineField.equals(FlinkOptions.PRECOMBINE_FIELD.defaultValue())) {
-        conf.setString(FlinkOptions.PRECOMBINE_FIELD, FlinkOptions.NO_PRE_COMBINE);
-      } else if (!preCombineField.equals(FlinkOptions.NO_PRE_COMBINE)) {
-        throw new HoodieValidationException("Field " + preCombineField + " does not exist in the table schema."
-                + "Please check '" + FlinkOptions.PRECOMBINE_FIELD.key() + "' option.");
+      if (orderingFields.equals(FlinkOptions.ORDERING_FIELDS.defaultValue())) {
+        conf.set(FlinkOptions.ORDERING_FIELDS, FlinkOptions.NO_PRE_COMBINE);
+      } else if (!orderingFields.equals(FlinkOptions.NO_PRE_COMBINE)) {
+        throw new HoodieValidationException("Field " + orderingFields + " does not exist in the table schema."
+                + "Please check '" + FlinkOptions.ORDERING_FIELDS.key() + "' option.");
       }
     }
   }
@@ -550,7 +672,7 @@ public class StreamerUtil {
    */
   public static void checkKeygenGenerator(boolean isComplexHoodieKey, Configuration conf) {
     if (isComplexHoodieKey && FlinkOptions.isDefaultValueDefined(conf, FlinkOptions.KEYGEN_CLASS_NAME)) {
-      conf.setString(FlinkOptions.KEYGEN_CLASS_NAME, ComplexAvroKeyGenerator.class.getName());
+      conf.set(FlinkOptions.KEYGEN_CLASS_NAME, ComplexAvroKeyGenerator.class.getName());
       LOG.info("Table option [{}] is reset to {} because record key or partition path has two or more fields",
           FlinkOptions.KEYGEN_CLASS_NAME.key(), ComplexAvroKeyGenerator.class.getName());
     }
@@ -563,8 +685,31 @@ public class StreamerUtil {
     Properties properties = new Properties();
 
     // set up metadata.enabled=true in table DDL to enable metadata listing
-    properties.put(HoodieMetadataConfig.ENABLE.key(), conf.getBoolean(FlinkOptions.METADATA_ENABLED));
+    properties.put(HoodieMetadataConfig.ENABLE.key(), conf.get(FlinkOptions.METADATA_ENABLED));
 
     return HoodieMetadataConfig.newBuilder().fromProperties(properties).build();
   }
+
+  /**
+   * Validate against the given list of write statuses.
+   *
+   * @param config          The Flink conf
+   * @param currentInstant  The current instant
+   * @param writeStatusList The write status list
+   *
+   * @throws HoodieException if the {code WRITE_FAIL_FAST} is set up as true and there are writing errors
+   */
+  public static void validateWriteStatus(
+      Configuration config,
+      String currentInstant,
+      List<WriteStatus> writeStatusList) throws HoodieException {
+    if (config.get(WRITE_FAIL_FAST)) {
+      // It will early detect the write failures in each of task to prevent data loss caused by commit failure
+      // after a checkpoint has been triggered.
+      writeStatusList.stream().filter(ws -> !ws.getErrors().isEmpty()).findFirst().map(writeStatus -> {
+        Map.Entry<HoodieKey, Throwable> entry = writeStatus.getErrors().entrySet().iterator().next();
+        throw new HoodieException(String.format("Write failure occurs with hoodie key %s at Instant [%s] in append write function", entry.getKey(), currentInstant), entry.getValue());
+      });
+    }
+  }
 }
