@@ -18,8 +18,10 @@
 
 package org.apache.hudi.sink.bulk;
 
+import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.exception.HoodieKeyException;
+import org.apache.hudi.keygen.TimestampBasedAvroKeyGenerator;
 import org.apache.hudi.keygen.constant.KeyGeneratorOptions;
 import org.apache.hudi.table.HoodieTableFactory;
 import org.apache.hudi.utils.TestConfigurations;
@@ -35,6 +37,9 @@ import org.junit.jupiter.params.provider.ValueSource;
 
 import java.sql.Timestamp;
 
+import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_INPUT_DATE_FORMAT;
+import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_OUTPUT_DATE_FORMAT;
+import static org.apache.hudi.common.config.TimestampKeyGeneratorConfig.TIMESTAMP_TYPE_FIELD;
 import static org.apache.hudi.common.util.PartitionPathEncodeUtils.DEFAULT_PARTITION_PATH;
 import static org.apache.hudi.utils.TestData.insertRow;
 import static org.hamcrest.CoreMatchers.is;
@@ -73,6 +78,39 @@ public class TestRowDataKeyGen {
     assertThat(keyGen2.getPartitionPath(rowData3), is(String.format("partition=%s", DEFAULT_PARTITION_PATH)));
   }
 
+  @ParameterizedTest
+  @ValueSource(booleans = {true, false})
+  void testSingleKeyMultiplePartitionFields(boolean encodeSingleKeyFieldValueOnly) {
+    Configuration conf = TestConfigurations.getDefaultConf("path1");
+    conf.set(FlinkOptions.RECORD_KEY_FIELD, "uuid");
+    conf.set(FlinkOptions.PARTITION_PATH_FIELD, "partition,ts");
+    conf.setString(HoodieWriteConfig.WRITE_TABLE_VERSION.key(), "8");
+    conf.setString(HoodieWriteConfig.COMPLEX_KEYGEN_NEW_ENCODING.key(), String.valueOf(encodeSingleKeyFieldValueOnly));
+    RowData rowData1 = insertRow(StringData.fromString("id1"), StringData.fromString("Danny"), 23,
+        TimestampData.fromEpochMillis(1), StringData.fromString("par1"));
+    RowDataKeyGen keyGen1 = RowDataKeyGen.instance(conf, TestConfigurations.ROW_TYPE);
+    String expectedKey = encodeSingleKeyFieldValueOnly ? "id1" : "uuid:id1";
+    assertThat(keyGen1.getRecordKey(rowData1), is(expectedKey));
+    assertThat(keyGen1.getPartitionPath(rowData1), is("par1/1970-01-01T00:00:00.001"));
+
+    // null record key and partition path
+    final RowData rowData2 = insertRow(TestConfigurations.ROW_TYPE, null, null, 23, null, null);
+    assertThrows(HoodieKeyException.class, () -> keyGen1.getRecordKey(rowData2));
+    assertThat(keyGen1.getPartitionPath(rowData2), is(String.format("%s/%s", DEFAULT_PARTITION_PATH, DEFAULT_PARTITION_PATH)));
+    // empty record key and partition path
+    final RowData rowData3 = insertRow(StringData.fromString(""), StringData.fromString(""), 23,
+        TimestampData.fromEpochMillis(1), StringData.fromString(""));
+    assertThrows(HoodieKeyException.class, () -> keyGen1.getRecordKey(rowData3));
+    assertThat(keyGen1.getPartitionPath(rowData3), is(String.format("%s/1970-01-01T00:00:00.001", DEFAULT_PARTITION_PATH)));
+
+    // hive style partitioning
+    conf.set(FlinkOptions.HIVE_STYLE_PARTITIONING, true);
+    final RowDataKeyGen keyGen2 = RowDataKeyGen.instance(conf, TestConfigurations.ROW_TYPE);
+    assertThat(keyGen2.getPartitionPath(rowData1), is(String.format("partition=%s/ts=%s", "par1", "1970-01-01T00:00:00.001")));
+    assertThat(keyGen2.getPartitionPath(rowData2), is(String.format("partition=%s/ts=%s", DEFAULT_PARTITION_PATH, DEFAULT_PARTITION_PATH)));
+    assertThat(keyGen2.getPartitionPath(rowData3), is(String.format("partition=%s/ts=%s", DEFAULT_PARTITION_PATH, "1970-01-01T00:00:00.001")));
+  }
+
   @Test
   void testComplexKeyAndPartition() {
     Configuration conf = TestConfigurations.getDefaultConf("path1");
@@ -105,7 +143,7 @@ public class TestRowDataKeyGen {
   @Test
   void testTimestampBasedKeyGenerator() {
     Configuration conf = TestConfigurations.getDefaultConf("path1");
-    conf.setString(FlinkOptions.PARTITION_PATH_FIELD, "ts");
+    conf.set(FlinkOptions.PARTITION_PATH_FIELD, "ts");
     HoodieTableFactory.setupTimestampKeygenOptions(conf, DataTypes.TIMESTAMP(3));
     final RowData rowData1 = insertRow(StringData.fromString("id1"), StringData.fromString("Danny"), 23,
         TimestampData.fromEpochMillis(7200000), StringData.fromString("par1"));
@@ -131,6 +169,18 @@ public class TestRowDataKeyGen {
     assertThat(keyGen2.getPartitionPath(rowData1), is("ts=1970010102"));
     assertThat(keyGen2.getPartitionPath(rowData2), is("ts=1970010100"));
     assertThat(keyGen2.getPartitionPath(rowData3), is("ts=1970010100"));
+
+    // TimestampType.DATE_STRING case, we use another string type `partition` column instead of `ts`
+    conf = TestConfigurations.getDefaultConf("path1");
+    conf.set(FlinkOptions.PARTITION_PATH_FIELD, "partition");
+    conf.setString(HoodieWriteConfig.KEYGENERATOR_CLASS_NAME.key(), TimestampBasedAvroKeyGenerator.class.getName());
+    conf.setString(TIMESTAMP_TYPE_FIELD.key(), TimestampBasedAvroKeyGenerator.TimestampType.DATE_STRING.name());
+    conf.setString(TIMESTAMP_INPUT_DATE_FORMAT.key(), "yyyy-MM-dd HH:mm:ss");
+    conf.setString(TIMESTAMP_OUTPUT_DATE_FORMAT.key(), "yyyy-MM-dd");
+    final RowData rowData4 = insertRow(StringData.fromString("id1"), StringData.fromString("Danny"), 23,
+        TimestampData.fromEpochMillis(7200000), StringData.fromString("2004-02-29 01:02:03"));
+    final RowDataKeyGen keyGen3 = RowDataKeyGen.instance(conf, TestConfigurations.ROW_TYPE);
+    assertThat(keyGen3.getPartitionPath(rowData4), is("2004-02-29"));
   }
 
   @ParameterizedTest
@@ -138,8 +188,8 @@ public class TestRowDataKeyGen {
   void testDateBasedKeyGenerator(String partitionFormat) {
     boolean dashed = partitionFormat.equals(FlinkOptions.PARTITION_FORMAT_DASHED_DAY);
     Configuration conf = TestConfigurations.getDefaultConf("path1", TestConfigurations.ROW_DATA_TYPE_DATE);
-    conf.setString(FlinkOptions.PARTITION_PATH_FIELD, "dt");
-    conf.setString(FlinkOptions.PARTITION_FORMAT, partitionFormat);
+    conf.set(FlinkOptions.PARTITION_PATH_FIELD, "dt");
+    conf.set(FlinkOptions.PARTITION_FORMAT, partitionFormat);
     HoodieTableFactory.setupTimestampKeygenOptions(conf, DataTypes.DATE());
     final RowData rowData1 = insertRow(TestConfigurations.ROW_TYPE_DATE,
         StringData.fromString("id1"), StringData.fromString("Danny"), 23, 1);
@@ -172,7 +222,7 @@ public class TestRowDataKeyGen {
   @Test
   void testPrimaryKeylessWrite() {
     Configuration conf = TestConfigurations.getDefaultConf("path1");
-    conf.setString(FlinkOptions.RECORD_KEY_FIELD, "");
+    conf.set(FlinkOptions.RECORD_KEY_FIELD, "");
     final RowData rowData1 = insertRow(StringData.fromString("id1"), StringData.fromString("Danny"), 23,
         TimestampData.fromEpochMillis(1), StringData.fromString("par1"));
     final int taskId = 3;
@@ -194,7 +244,7 @@ public class TestRowDataKeyGen {
   @Test
   void testRecordKeyContainsTimestamp() {
     Configuration conf = TestConfigurations.getDefaultConf("path1");
-    conf.setString(FlinkOptions.RECORD_KEY_FIELD, "uuid,ts");
+    conf.set(FlinkOptions.RECORD_KEY_FIELD, "uuid,ts");
     conf.setString(KeyGeneratorOptions.KEYGENERATOR_CONSISTENT_LOGICAL_TIMESTAMP_ENABLED.key(), "true");
     Timestamp ts = new Timestamp(1675841687000L);
     final RowData rowData1 = insertRow(StringData.fromString("id1"), StringData.fromString("Danny"), 23,
@@ -209,5 +259,4 @@ public class TestRowDataKeyGen {
     assertThat(keyGen2.getRecordKey(rowData1), is("uuid:id1,ts:1675841687000"));
 
   }
-
 }
