@@ -21,6 +21,7 @@ package org.apache.hudi.sink.compact;
 import org.apache.hudi.avro.model.HoodieCompactionPlan;
 import org.apache.hudi.client.HoodieFlinkWriteClient;
 import org.apache.hudi.client.common.HoodieFlinkEngineContext;
+import org.apache.hudi.common.config.HoodieStorageConfig;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieBaseFile;
 import org.apache.hudi.common.table.HoodieTableConfig;
@@ -43,10 +44,9 @@ import org.apache.hudi.utils.TestConfigurations;
 import org.apache.hudi.utils.TestData;
 import org.apache.hudi.utils.TestSQL;
 
-import org.apache.flink.api.common.restartstrategy.RestartStrategies;
-import org.apache.flink.api.common.time.Time;
 import org.apache.flink.api.common.typeinfo.TypeInformation;
 import org.apache.flink.configuration.Configuration;
+import org.apache.flink.configuration.RestartStrategyOptions;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.table.api.EnvironmentSettings;
 import org.apache.flink.table.api.TableEnvironment;
@@ -57,12 +57,15 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtendWith;
 import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.Arguments;
+import org.junit.jupiter.params.provider.MethodSource;
 import org.junit.jupiter.params.provider.ValueSource;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.File;
 import java.io.IOException;
+import java.time.Duration;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -72,6 +75,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
+import java.util.stream.Stream;
 
 import static org.apache.hudi.common.testutils.HoodieTestUtils.INSTANT_GENERATOR;
 import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
@@ -114,18 +118,19 @@ public class ITTestHoodieFlinkCompactor {
   File tempFile;
 
   @ParameterizedTest
-  @ValueSource(booleans = {true, false})
-  public void testHoodieFlinkCompactor(boolean enableChangelog) throws Exception {
+  @MethodSource("changedlogAndLogBlockParams")
+  public void testHoodieFlinkCompactor(boolean enableChangelog, String logBlockFormat) throws Exception {
     // Create hoodie table and insert into data.
     EnvironmentSettings settings = EnvironmentSettings.newInstance().inBatchMode().build();
     TableEnvironment tableEnv = TableEnvironmentImpl.create(settings);
     tableEnv.getConfig().getConfiguration()
-        .setInteger(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
+        .set(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
     Map<String, String> options = new HashMap<>();
     options.put(FlinkOptions.COMPACTION_SCHEDULE_ENABLED.key(), "false");
     options.put(FlinkOptions.COMPACTION_ASYNC_ENABLED.key(), "false");
     options.put(FlinkOptions.PATH.key(), tempFile.getAbsolutePath());
     options.put(FlinkOptions.TABLE_TYPE.key(), "MERGE_ON_READ");
+    options.put(HoodieStorageConfig.LOGFILE_DATA_BLOCK_FORMAT.key(), logBlockFormat);
     options.put(FlinkOptions.CHANGELOG_ENABLED.key(), enableChangelog + "");
     String hoodieTableDDL = TestConfigurations.getCreateHoodieTableDDL("t1", options);
     tableEnv.executeSql(hoodieTableDDL);
@@ -139,13 +144,16 @@ public class ITTestHoodieFlinkCompactor {
     FlinkCompactionConfig cfg = new FlinkCompactionConfig();
     cfg.path = tempFile.getAbsolutePath();
     Configuration conf = FlinkCompactionConfig.toFlinkConfig(cfg);
-    conf.setString(FlinkOptions.TABLE_TYPE.key(), "MERGE_ON_READ");
+    conf.set(FlinkOptions.TABLE_TYPE, "MERGE_ON_READ");
 
     // create metaClient
     HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(conf);
 
     // set the table name
-    conf.setString(FlinkOptions.TABLE_NAME, metaClient.getTableConfig().getTableName());
+    conf.set(FlinkOptions.TABLE_NAME, metaClient.getTableConfig().getTableName());
+
+    // set the partition fields
+    CompactionUtil.setPartitionField(conf, metaClient);
 
     // set table schema
     CompactionUtil.setAvroSchema(conf, metaClient);
@@ -187,7 +195,7 @@ public class ITTestHoodieFlinkCompactor {
 
   @ParameterizedTest
   @ValueSource(booleans = {true, false})
-  public void testHoodieFlinkCompactorWithUpgradeAndDowngrade(boolean upgrade) throws Exception {
+  void testHoodieFlinkCompactorWithUpgradeAndDowngrade(boolean upgrade) throws Exception {
     // Create hoodie table and insert into data.
     EnvironmentSettings settings = EnvironmentSettings.newInstance().inBatchMode().build();
     TableEnvironment tableEnv = TableEnvironmentImpl.create(settings);
@@ -209,7 +217,7 @@ public class ITTestHoodieFlinkCompactor {
     FlinkCompactionConfig cfg = new FlinkCompactionConfig();
     cfg.path = tempFile.getAbsolutePath();
     Configuration conf = FlinkCompactionConfig.toFlinkConfig(cfg);
-    conf.setString(FlinkOptions.TABLE_TYPE.key(), "MERGE_ON_READ");
+    conf.set(FlinkOptions.TABLE_TYPE, "MERGE_ON_READ");
 
     // create metaClient
     HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(conf);
@@ -231,10 +239,12 @@ public class ITTestHoodieFlinkCompactor {
       if (upgrade) {
         metaClient.getTableConfig().setTableVersion(HoodieTableVersion.SIX);
         HoodieTableConfig.update(metaClient.getStorage(), metaClient.getMetaPath(), metaClient.getTableConfig().getProps());
-        new UpgradeDowngrade(metaClient, writeClient.getConfig(), writeClient.getEngineContext(), FlinkUpgradeDowngradeHelper.getInstance()).run(HoodieTableVersion.EIGHT, "none");
+        new UpgradeDowngrade(metaClient, writeClient.getConfig(), writeClient.getEngineContext(),
+            FlinkUpgradeDowngradeHelper.getInstance()).run(HoodieTableVersion.current(), "none");
       } else {
-        metaClient.getTableConfig().setTableVersion(HoodieTableVersion.EIGHT);
-        new UpgradeDowngrade(metaClient, writeClient.getConfig(), writeClient.getEngineContext(), FlinkUpgradeDowngradeHelper.getInstance()).run(HoodieTableVersion.SIX, "none");
+        metaClient.getTableConfig().setTableVersion(HoodieTableVersion.current());
+        new UpgradeDowngrade(metaClient, writeClient.getConfig(), writeClient.getEngineContext(),
+            FlinkUpgradeDowngradeHelper.getInstance()).run(HoodieTableVersion.SIX, "none");
         // set table version
         conf.setString("hoodie.write.table.version", "6");
       }
@@ -277,7 +287,7 @@ public class ITTestHoodieFlinkCompactor {
     EnvironmentSettings settings = EnvironmentSettings.newInstance().inBatchMode().build();
     TableEnvironment tableEnv = TableEnvironmentImpl.create(settings);
     tableEnv.getConfig().getConfiguration()
-        .setInteger(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
+        .set(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
     Map<String, String> options = new HashMap<>();
     options.put(FlinkOptions.COMPACTION_ASYNC_ENABLED.key(), "false");
     options.put(FlinkOptions.PATH.key(), tempFile.getAbsolutePath());
@@ -297,8 +307,8 @@ public class ITTestHoodieFlinkCompactor {
     cfg.minCompactionIntervalSeconds = 3;
     cfg.schedule = true;
     Configuration conf = FlinkCompactionConfig.toFlinkConfig(cfg);
-    conf.setString(FlinkOptions.TABLE_TYPE.key(), "MERGE_ON_READ");
-    conf.setInteger(FlinkOptions.COMPACTION_TASKS.key(), FlinkMiniCluster.DEFAULT_PARALLELISM);
+    conf.set(FlinkOptions.TABLE_TYPE, "MERGE_ON_READ");
+    conf.set(FlinkOptions.COMPACTION_TASKS, FlinkMiniCluster.DEFAULT_PARALLELISM);
 
     HoodieFlinkCompactor.AsyncCompactionService asyncCompactionService = new HoodieFlinkCompactor.AsyncCompactionService(cfg, conf);
     asyncCompactionService.start(null);
@@ -318,7 +328,7 @@ public class ITTestHoodieFlinkCompactor {
     EnvironmentSettings settings = EnvironmentSettings.newInstance().inBatchMode().build();
     TableEnvironment tableEnv = TableEnvironmentImpl.create(settings);
     tableEnv.getConfig().getConfiguration()
-        .setInteger(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
+        .set(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
     Map<String, String> options = new HashMap<>();
     options.put(FlinkOptions.COMPACTION_ASYNC_ENABLED.key(), "false");
     options.put(FlinkOptions.PATH.key(), tempFile.getAbsolutePath());
@@ -332,10 +342,10 @@ public class ITTestHoodieFlinkCompactor {
     FlinkCompactionConfig cfg = new FlinkCompactionConfig();
     cfg.path = tempFile.getAbsolutePath();
     Configuration conf = FlinkCompactionConfig.toFlinkConfig(cfg);
-    conf.setString(FlinkOptions.TABLE_TYPE.key(), "MERGE_ON_READ");
+    conf.set(FlinkOptions.TABLE_TYPE, "MERGE_ON_READ");
 
     HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(conf);
-    conf.setString(FlinkOptions.TABLE_NAME, metaClient.getTableConfig().getTableName());
+    conf.set(FlinkOptions.TABLE_NAME, metaClient.getTableConfig().getTableName());
     CompactionUtil.setAvroSchema(conf, metaClient);
     CompactionUtil.inferChangelogMode(conf, metaClient);
 
@@ -396,7 +406,7 @@ public class ITTestHoodieFlinkCompactor {
     EnvironmentSettings settings = EnvironmentSettings.newInstance().inBatchMode().build();
     TableEnvironment tableEnv = TableEnvironmentImpl.create(settings);
     tableEnv.getConfig().getConfiguration()
-        .setInteger(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
+        .set(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
     Map<String, String> options = new HashMap<>();
     options.put(FlinkOptions.COMPACTION_DELTA_COMMITS.key(), "2");
     options.put(FlinkOptions.PATH.key(), tempFile.getAbsolutePath());
@@ -426,7 +436,7 @@ public class ITTestHoodieFlinkCompactor {
     Set<Pair<String, String>> fileIdCommitTimeSet = new HashSet<>();
     HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(conf);
     HoodieStorage storage = metaClient.getStorage();
-    FSUtils.getAllPartitionPaths(HoodieFlinkEngineContext.DEFAULT, metaClient.getStorage(), metaClient.getBasePath(), false).forEach(
+    FSUtils.getAllPartitionPaths(HoodieFlinkEngineContext.DEFAULT, metaClient, false).forEach(
         partition -> {
           try {
             storage.listDirectEntries(FSUtils.constructAbsolutePath(metaClient.getBasePath(), partition))
@@ -448,7 +458,7 @@ public class ITTestHoodieFlinkCompactor {
   private TableEnvironment prepareEnvAndTable() {
     EnvironmentSettings settings = EnvironmentSettings.newInstance().inBatchMode().build();
     TableEnvironment tableEnv = TableEnvironmentImpl.create(settings);
-    tableEnv.getConfig().getConfiguration().setInteger(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
+    tableEnv.getConfig().getConfiguration().set(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 4);
     tableEnv.getConfig().getConfiguration().set(TableConfigOptions.TABLE_DML_SYNC, true);
     Map<String, String> options = new HashMap<>();
     options.put(FlinkOptions.METADATA_ENABLED.key(), "false"); // to archive compaction instant
@@ -465,13 +475,13 @@ public class ITTestHoodieFlinkCompactor {
    * schedule compact, insert another batch, run compact.
    */
   private void runOfflineCompact(TableEnvironment tableEnv, Configuration conf) throws Exception {
-    conf.setString(FlinkOptions.TABLE_TYPE.key(), "MERGE_ON_READ");
+    conf.set(FlinkOptions.TABLE_TYPE, "MERGE_ON_READ");
 
     // create metaClient
     HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(conf);
 
     // set the table name
-    conf.setString(FlinkOptions.TABLE_NAME, metaClient.getTableConfig().getTableName());
+    conf.set(FlinkOptions.TABLE_NAME, metaClient.getTableConfig().getTableName());
 
     // set table schema
     CompactionUtil.setAvroSchema(conf, metaClient);
@@ -496,8 +506,11 @@ public class ITTestHoodieFlinkCompactor {
       tableEnv.executeSql(TestSQL.INSERT_T1);
 
       // Make configuration and setAvroSchema.
-      StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
-      env.setRestartStrategy(RestartStrategies.fixedDelayRestart(1, Time.milliseconds(1)));
+      Configuration envConf = new Configuration();
+      envConf.set(RestartStrategyOptions.RESTART_STRATEGY, "fixed-delay");
+      envConf.set(RestartStrategyOptions.RESTART_STRATEGY_FIXED_DELAY_ATTEMPTS, 1);
+      envConf.set(RestartStrategyOptions.RESTART_STRATEGY_FIXED_DELAY_DELAY, Duration.ofMillis(1));
+      StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment(envConf);
 
       env.addSource(new CompactionPlanSourceFunction(Collections.singletonList(Pair.of(compactionInstantTime, compactionPlan)), conf))
           .name("compaction_source")
@@ -521,4 +534,17 @@ public class ITTestHoodieFlinkCompactor {
     assertTrue(compactionInstant.isPresent(), "The compaction plan should be scheduled");
     return compactionInstant.get();
   }
+
+  /**
+   * Return test params => (enableChangelog, log block format).
+   */
+  private static Stream<Arguments> changedlogAndLogBlockParams() {
+    Object[][] data =
+        new Object[][] {
+            {true, "parquet"},
+            {true, "avro"},
+            {false, "parquet"},
+            {false, "avro"}};
+    return Stream.of(data).map(Arguments::of);
+  }
 }
