@@ -45,10 +45,15 @@ public interface TestFunctionWrapper<I> {
   void invoke(I record) throws Exception;
 
   /**
-   * Returns the event buffer sent by the write tasks.
+   * Returns the latest event buffer sent by the write tasks.
    */
   WriteMetadataEvent[] getEventBuffer();
 
+  /**
+   * Returns the event buffer sent by the write tasks with given checkpoint ID.
+   */
+  WriteMetadataEvent[] getEventBuffer(long checkpointId);
+
   /**
    * Returns the next event sent to Coordinator.
    */
@@ -158,13 +163,6 @@ public interface TestFunctionWrapper<I> {
     throw new UnsupportedOperationException();
   }
 
-  /**
-   * Returns whether the write task is confirming.
-   */
-  default boolean isConforming() {
-    throw new UnsupportedOperationException();
-  }
-
   /**
    * Close this function wrapper.
    */
