@@ -24,10 +24,10 @@ import org.apache.hudi.common.model.HoodieKey;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.model.HoodieTableType;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
+import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.table.timeline.TimelineUtils;
-import org.apache.hudi.common.testutils.HoodieTestUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.configuration.FlinkOptions;
@@ -35,10 +35,6 @@ import org.apache.hudi.configuration.OptionsResolver;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.sink.event.CommitAckEvent;
 import org.apache.hudi.sink.event.WriteMetadataEvent;
-import org.apache.hudi.sink.meta.CkpMetadata;
-import org.apache.hudi.sink.meta.CkpMetadataFactory;
-import org.apache.hudi.storage.HoodieStorage;
-import org.apache.hudi.storage.HoodieStorageUtils;
 import org.apache.hudi.storage.StoragePath;
 import org.apache.hudi.util.StreamerUtil;
 import org.apache.hudi.utils.TestConfigurations;
@@ -71,7 +67,6 @@ import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
-import static org.junit.jupiter.api.Assertions.assertNotEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
@@ -138,7 +133,7 @@ public class TestWriteBase {
   @BeforeEach
   public void before() {
     conf = TestConfigurations.getDefaultConf(tempFile.getAbsolutePath());
-    conf.setString(FlinkOptions.TABLE_TYPE, getTableType().name());
+    conf.set(FlinkOptions.TABLE_TYPE, getTableType().name());
     setUp(conf);
   }
 
@@ -167,8 +162,6 @@ public class TestWriteBase {
     private Configuration conf;
     private TestFunctionWrapper<RowData> pipeline;
 
-    private CkpMetadata ckpMetadata;
-
     private String lastPending;
     private String lastComplete;
 
@@ -180,7 +173,6 @@ public class TestWriteBase {
       // open the function and ingest data
       this.pipeline.openFunction();
       HoodieWriteConfig writeConfig = this.pipeline.getCoordinator().getWriteClient().getConfig();
-      this.ckpMetadata = CkpMetadataFactory.getCkpMetadata(writeConfig, conf);
       return this;
     }
 
@@ -200,10 +192,15 @@ public class TestWriteBase {
      * Assert the event buffer is empty.
      */
     public TestHarness emptyEventBuffer() {
-      assertTrue(
-          this.pipeline.getEventBuffer().length == 1
-              && this.pipeline.getEventBuffer()[0] == null,
-          "The coordinator events buffer expect to be empty");
+      assertNull(this.pipeline.getEventBuffer(), "The coordinator events buffer expect to be empty");
+      return this;
+    }
+
+    /**
+     * Assert the event buffer is in initial state.
+     */
+    public TestHarness initialEventBuffer() {
+      assertNull(this.pipeline.getEventBuffer()[0], "The coordinator events buffer expect to be initialized");
       return this;
     }
 
@@ -225,11 +222,19 @@ public class TestWriteBase {
     public TestHarness assertNextEvent() {
       final OperatorEvent nextEvent = this.pipeline.getNextEvent();
       MatcherAssert.assertThat("The operator expect to send an event", nextEvent, instanceOf(WriteMetadataEvent.class));
+      WriteMetadataEvent metadataEvent = (WriteMetadataEvent) nextEvent;
       this.pipeline.getCoordinator().handleEventFromOperator(0, nextEvent);
       if (!((WriteMetadataEvent) nextEvent).isBootstrap()) {
         assertNotNull(this.pipeline.getEventBuffer()[0], "The coordinator missed the event");
       } else {
-        assertNull(this.pipeline.getEventBuffer()[0], "The coordinator should reset event buffer because of the instant initialization");
+        WriteMetadataEvent[] eventBuffer = this.pipeline.getEventBuffer(metadataEvent.getCheckpointId());
+        if (metadataEvent.getInstantTime().equals(WriteMetadataEvent.BOOTSTRAP_INSTANT)) {
+          assertTrue(eventBuffer == null || eventBuffer[metadataEvent.getTaskID()] == null,
+              "The coordinator should reset the redundant events");
+        } else {
+          assertNull(eventBuffer,
+              "The coordinator should reset event buffer because of the instant recommit");
+        }
       }
       return this;
     }
@@ -263,14 +268,22 @@ public class TestWriteBase {
     public TestHarness assertEmptyEvent() {
       final OperatorEvent nextEvent = this.pipeline.getNextEvent();
       MatcherAssert.assertThat("The operator expect to send an event", nextEvent, instanceOf(WriteMetadataEvent.class));
-      List<WriteStatus> writeStatuses = ((WriteMetadataEvent) nextEvent).getWriteStatuses();
+      WriteMetadataEvent metadataEvent = (WriteMetadataEvent) nextEvent;
+      List<WriteStatus> writeStatuses = metadataEvent.getWriteStatuses();
       assertNotNull(writeStatuses);
       MatcherAssert.assertThat(writeStatuses.size(), is(0));
       this.pipeline.getCoordinator().handleEventFromOperator(0, nextEvent);
-      if (!((WriteMetadataEvent) nextEvent).isBootstrap()) {
+      if (!metadataEvent.isBootstrap()) {
         assertNotNull(this.pipeline.getEventBuffer()[0], "The coordinator missed the event");
       } else {
-        assertNull(this.pipeline.getEventBuffer()[0], "The coordinator should reset event buffer because of the instant initialization");
+        WriteMetadataEvent[] eventBuffer = this.pipeline.getEventBuffer(metadataEvent.getCheckpointId());
+        if (metadataEvent.getInstantTime().equals(WriteMetadataEvent.BOOTSTRAP_INSTANT)) {
+          assertTrue(eventBuffer == null || eventBuffer[metadataEvent.getTaskID()] == null,
+              "The coordinator should reset the redundant events");
+        } else {
+          assertNull(eventBuffer,
+              "The coordinator should reset event buffer because of the instant recommit");
+        }
       }
       return this;
     }
@@ -342,20 +355,16 @@ public class TestWriteBase {
     public TestHarness checkpointComplete(long checkpointId) {
       this.lastPending = lastPendingInstant();
       this.pipeline.checkpointComplete(checkpointId);
-      // started a new instant already
-      String newInflight = checkInflightInstant();
       checkInstantState(HoodieInstant.State.COMPLETED, lastPending);
       this.lastComplete = lastPending;
-      this.lastPending = newInflight; // refresh last pending instant
+      this.lastPending = ""; // refresh last pending instant
       return this;
     }
 
     /**
      * Flush data and commit using endInput. Asserts the commit would fail.
      */
-    public void endInputThrows(Class<?> cause, String msg) {
-      // this triggers the data write and event send
-      this.pipeline.endInput();
+    public void endInputCompleteThrows(Class<?> cause, String msg) {
       final OperatorEvent nextEvent = this.pipeline.getNextEvent();
       this.pipeline.getCoordinator().handleEventFromOperator(0, nextEvent);
       assertTrue(this.pipeline.getCoordinatorContext().isJobFailed(), "Job should have been failed");
@@ -370,6 +379,13 @@ public class TestWriteBase {
     public TestHarness endInput() {
       // this triggers the data write and event send
       this.pipeline.endInput();
+      return this;
+    }
+
+    /**
+     * Mark the pipeline end input as complete.
+     */
+    public TestHarness endInputComplete() {
       final OperatorEvent nextEvent = this.pipeline.getNextEvent();
       this.pipeline.getCoordinator().handleEventFromOperator(0, nextEvent);
       return this;
@@ -395,15 +411,10 @@ public class TestWriteBase {
       String lastPending = lastPendingInstant();
       this.pipeline.checkpointComplete(checkpointId);
       if (!OptionsResolver.allowCommitOnEmptyBatch(this.conf)) {
-        // last pending instant was reused
-        assertEquals(this.lastPending, lastPending);
         checkInstantState(HoodieInstant.State.COMPLETED, lastComplete);
       } else {
-        // started a new instant already
-        String newInflight = checkInflightInstant();
         checkInstantState(HoodieInstant.State.COMPLETED, lastPending);
         this.lastComplete = lastPending;
-        this.lastPending = newInflight; // refresh last pending instant
       }
       return this;
     }
@@ -416,7 +427,7 @@ public class TestWriteBase {
       assertFalse(this.pipeline.getCoordinatorContext().isJobFailed(),
           "The last checkpoint was aborted, ignore the events");
       // no complete instant
-      checkInstantState(HoodieInstant.State.COMPLETED, null);
+      assertNull(lastCompleteInstant(), "No complete instant");
       return this;
     }
 
@@ -426,6 +437,16 @@ public class TestWriteBase {
       return this;
     }
 
+    /**
+     * Overrides the instant time request to avoid the write task hangs up.
+     */
+    public TestHarness resetInstantTimeRequest(Configuration conf) {
+      if (OptionsResolver.isBlockingInstantGeneration(conf)) {
+        this.pipeline.getWriteFunction().setCorrespondent(new MockCorrespondentWithTimeout(this.pipeline.getCoordinator(), conf));
+      }
+      return this;
+    }
+
     /**
      * Mark the task with id {@code taskId} as failed.
      */
@@ -437,7 +458,7 @@ public class TestWriteBase {
       assertEmptyEvent();
 
       String instant2 = lastPendingInstant();
-      assertNotEquals(instant2, instant1, "The previous instant should be rolled back when starting new instant");
+      assertEquals(instant2, instant1, "The previous instant should not be rolled back when subTask restart");
       return this;
     }
 
@@ -455,9 +476,17 @@ public class TestWriteBase {
       return this;
     }
 
-    public TestHarness noCompleteInstant() {
+    public TestHarness assertInstantRecommit() {
       // no complete instant
-      checkInstantState(HoodieInstant.State.COMPLETED, null);
+      assertNotNull(lastCompleteInstant(), "The instant should recommit");
+      return this;
+    }
+
+    /**
+     * Asserts the data files are non-empty.
+     */
+    public TestHarness assertDataFilesExists() {
+      assertTrue(fileExists(false), "Data files should have been created");
       return this;
     }
 
@@ -465,19 +494,33 @@ public class TestWriteBase {
      * Asserts the data files are empty.
      */
     public TestHarness assertEmptyDataFiles() {
-      assertFalse(fileExists(), "No data files should have been created");
+      assertFalse(fileExists(false), "No data files should have been created");
       return this;
     }
 
-    private boolean fileExists() {
+    /**
+     * Asserts the data files are empty.
+     */
+    public TestHarness assertEmptyBaseFiles() {
+      assertFalse(fileExists(true), "No data files should have been created");
+      return this;
+    }
+
+    private boolean fileExists(boolean ignoreLogs) {
       List<File> dirsToCheck = new ArrayList<>();
       dirsToCheck.add(baseFile);
       while (!dirsToCheck.isEmpty()) {
         File dir = dirsToCheck.remove(0);
         for (File file : Objects.requireNonNull(dir.listFiles())) {
-          if (!file.getName().startsWith(".")) {
-            if (file.isDirectory()) {
+          if (file.isDirectory()) {
+            if (!file.getName().startsWith(".")) {
               dirsToCheck.add(file);
+            }
+          } else {
+            if (ignoreLogs) {
+              if (!file.getName().startsWith(".")) {
+                return true;
+              }
             } else {
               return true;
             }
@@ -498,16 +541,11 @@ public class TestWriteBase {
       if (OptionsResolver.isCowTable(conf)) {
         TestData.checkWrittenData(this.baseFile, expected, partitions);
       } else {
-        checkWrittenDataMor(baseFile, expected, partitions);
+        TestData.checkWrittenDataMOR(baseFile, expected, partitions);
       }
       return this;
     }
 
-    private void checkWrittenDataMor(File baseFile, Map<String, String> expected, int partitions) throws Exception {
-      HoodieStorage storage = HoodieStorageUtils.getStorage(basePath, HoodieTestUtils.getDefaultStorageConf());
-      TestData.checkWrittenDataMOR(storage, baseFile, expected, partitions);
-    }
-
     public TestHarness checkWrittenDataCOW(Map<String, List<String>> expected) throws IOException {
       TestData.checkWrittenDataCOW(this.baseFile, expected);
       return this;
@@ -531,22 +569,13 @@ public class TestWriteBase {
       return this;
     }
 
-    public TestHarness assertConfirming() {
-      assertTrue(this.pipeline.isConforming(),
-          "The write function should be waiting for the instant to commit");
-      return this;
-    }
-
-    public TestHarness assertNotConfirming() {
-      assertFalse(this.pipeline.isConforming(),
-          "The write function should finish waiting for the instant to commit");
-      return this;
-    }
-
     public TestHarness rollbackLastCompleteInstantToInflight() throws Exception {
       HoodieTableMetaClient metaClient = StreamerUtil.createMetaClient(conf);
-      Option<HoodieInstant> lastCompletedInstant =
-          metaClient.getActiveTimeline().filterCompletedInstants().lastInstant();
+      HoodieTimeline timeline = OptionsResolver.isMorTable(conf)
+          ? metaClient.getActiveTimeline().getDeltaCommitTimeline()
+          : metaClient.getActiveTimeline().getCommitTimeline();
+      Option<HoodieInstant> lastCompletedInstant = timeline
+          .filterCompletedInstants().lastInstant();
       TimelineUtils.deleteInstantFile(
           metaClient.getStorage(), metaClient.getTimelinePath(), lastCompletedInstant.get(),
           metaClient.getInstantFileNameGenerator());
@@ -561,6 +590,7 @@ public class TestWriteBase {
     }
 
     public TestHarness checkLastPendingInstantCompleted() {
+      this.pipeline.checkpointComplete(3);
       checkInstantState(HoodieInstant.State.COMPLETED, this.lastPending);
       this.lastComplete = lastPending;
       this.lastPending = lastPendingInstant();
@@ -582,41 +612,28 @@ public class TestWriteBase {
     }
 
     private String lastPendingInstant() {
-      return this.ckpMetadata.lastPendingInstant();
-    }
-
-    private String checkInflightInstant() {
-      final String instant = this.ckpMetadata.lastPendingInstant();
-      assertNotNull(instant);
-      return instant;
+      return this.pipeline.getCoordinator().getInstant();
     }
 
     private void checkInstantState(HoodieInstant.State state, String instantStr) {
-      final String instant;
+      HoodieActiveTimeline timeline = StreamerUtil.createMetaClient(conf).getActiveTimeline();
       switch (state) {
         case REQUESTED:
-          instant = lastPendingInstant();
+          assertTrue(timeline.filterInflightsAndRequested().containsInstant(instantStr));
           break;
         case COMPLETED:
-          instant = lastCompleteInstant();
+          assertTrue(timeline.filterCompletedInstants().containsInstant(instantStr));
           break;
         default:
           throw new AssertionError("Unexpected state");
       }
-      assertThat(instant, is(instantStr));
     }
 
     protected String lastCompleteInstant() {
-      // If using optimistic concurrency control, fetch last complete instant of current writer from ckp metadata
-      // because there are multiple write clients commit to the timeline.
-      if (OptionsResolver.isMultiWriter(conf)) {
-        return this.ckpMetadata.lastCompleteInstant();
-      } else {
-        // fetch the instant from timeline.
-        return OptionsResolver.isMorTable(conf)
-            ? TestUtils.getLastDeltaCompleteInstant(basePath)
-            : TestUtils.getLastCompleteInstant(basePath, HoodieTimeline.COMMIT_ACTION);
-      }
+      // fetch the instant from timeline.
+      return OptionsResolver.isMorTable(conf)
+          ? TestUtils.getLastDeltaCompleteInstant(basePath)
+          : TestUtils.getLastCompleteInstant(basePath, HoodieTimeline.COMMIT_ACTION);
     }
 
     public TestHarness checkCompletedInstantCount(int count) {
