@@ -69,6 +69,7 @@ import static org.apache.parquet.filter2.predicate.FilterApi.or;
 import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 /**
  * Test cases for {@link ExpressionPredicates}.
@@ -85,7 +86,7 @@ public class TestExpressionPredicates {
     IntColumn intColumn = intColumn("f_int");
 
     // equals
-    CallExpression equalsExpression = new CallExpression(
+    CallExpression equalsExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.EQUALS, expressions, DataTypes.BOOLEAN());
     Predicate predicate1 = Equals.getInstance().bindValueLiteral(valueLiteral).bindFieldReference(fieldReference);
     Eq<Integer> eq = eq(intColumn, 10);
@@ -94,7 +95,7 @@ public class TestExpressionPredicates {
     assertEquals(eq, predicate2.filter());
 
     // not equals
-    CallExpression notEqualsExpression = new CallExpression(
+    CallExpression notEqualsExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.NOT_EQUALS, expressions, DataTypes.BOOLEAN());
     Predicate predicate3 = NotEquals.getInstance().bindValueLiteral(valueLiteral).bindFieldReference(fieldReference);
     Predicate predicate4 = fromExpression(notEqualsExpression);
@@ -102,7 +103,7 @@ public class TestExpressionPredicates {
     assertEquals(notEq(intColumn, 10), predicate4.filter());
 
     // less than
-    CallExpression lessThanExpression = new CallExpression(
+    CallExpression lessThanExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.LESS_THAN, expressions, DataTypes.BOOLEAN());
     Predicate predicate5 = LessThan.getInstance().bindValueLiteral(valueLiteral).bindFieldReference(fieldReference);
     Lt<Integer> lt = lt(intColumn, 10);
@@ -111,7 +112,7 @@ public class TestExpressionPredicates {
     assertEquals(lt, predicate6.filter());
 
     // greater than
-    CallExpression greaterThanExpression = new CallExpression(
+    CallExpression greaterThanExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.GREATER_THAN, expressions, DataTypes.BOOLEAN());
     Predicate predicate7 = GreaterThan.getInstance().bindValueLiteral(valueLiteral).bindFieldReference(fieldReference);
     Gt<Integer> gt = gt(intColumn, 10);
@@ -120,7 +121,7 @@ public class TestExpressionPredicates {
     assertEquals(gt, predicate8.filter());
 
     // less than or equal
-    CallExpression lessThanOrEqualExpression = new CallExpression(
+    CallExpression lessThanOrEqualExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.LESS_THAN_OR_EQUAL, expressions, DataTypes.BOOLEAN());
     Predicate predicate9 = LessThanOrEqual.getInstance().bindValueLiteral(valueLiteral).bindFieldReference(fieldReference);
     Predicate predicate10 = fromExpression(lessThanOrEqualExpression);
@@ -128,7 +129,7 @@ public class TestExpressionPredicates {
     assertEquals(ltEq(intColumn, 10), predicate10.filter());
 
     // greater than or equal
-    CallExpression greaterThanOrEqualExpression = new CallExpression(
+    CallExpression greaterThanOrEqualExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.GREATER_THAN_OR_EQUAL, expressions, DataTypes.BOOLEAN());
     Predicate predicate11 = GreaterThanOrEqual.getInstance().bindValueLiteral(valueLiteral).bindFieldReference(fieldReference);
     Predicate predicate12 = fromExpression(greaterThanOrEqualExpression);
@@ -138,7 +139,7 @@ public class TestExpressionPredicates {
     // in
     ValueLiteralExpression valueLiteral1 = new ValueLiteralExpression(11);
     ValueLiteralExpression valueLiteral2 = new ValueLiteralExpression(12);
-    CallExpression inExpression = new CallExpression(
+    CallExpression inExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.IN,
         Arrays.asList(fieldReference, valueLiteral1, valueLiteral2),
         DataTypes.BOOLEAN());
@@ -148,7 +149,7 @@ public class TestExpressionPredicates {
     assertEquals(or(eq(intColumn, 11), eq(intColumn, 12)), predicate14.filter());
 
     // not
-    CallExpression notExpression = new CallExpression(
+    CallExpression notExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.NOT,
         Collections.singletonList(equalsExpression),
         DataTypes.BOOLEAN());
@@ -158,7 +159,7 @@ public class TestExpressionPredicates {
     assertEquals(not(eq), predicate16.filter());
 
     // and
-    CallExpression andExpression = new CallExpression(
+    CallExpression andExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.AND,
         Arrays.asList(lessThanExpression, greaterThanExpression),
         DataTypes.BOOLEAN());
@@ -168,7 +169,7 @@ public class TestExpressionPredicates {
     assertEquals(and(lt, gt), predicate18.filter());
 
     // or
-    CallExpression orExpression = new CallExpression(
+    CallExpression orExpression = CallExpression.permanent(
         BuiltInFunctionDefinitions.OR,
         Arrays.asList(lessThanExpression, greaterThanExpression),
         DataTypes.BOOLEAN());
@@ -184,9 +185,9 @@ public class TestExpressionPredicates {
     ValueLiteralExpression valueLiteral = new ValueLiteralExpression(BigDecimal.valueOf(100.00));
     List<ResolvedExpression> expressions = Arrays.asList(fieldReference, valueLiteral);
 
-    CallExpression greaterThanExpression = new CallExpression(BuiltInFunctionDefinitions.GREATER_THAN, expressions, DataTypes.DECIMAL(7, 2));
+    CallExpression greaterThanExpression = CallExpression.permanent(BuiltInFunctionDefinitions.GREATER_THAN, expressions, DataTypes.DECIMAL(7, 2));
     Predicate greaterThanPredicate = fromExpression(greaterThanExpression);
-    CallExpression lessThanExpression = new CallExpression(BuiltInFunctionDefinitions.LESS_THAN, expressions, DataTypes.DECIMAL(7, 2));
+    CallExpression lessThanExpression = CallExpression.permanent(BuiltInFunctionDefinitions.LESS_THAN, expressions, DataTypes.DECIMAL(7, 2));
     Predicate lessThanPredicate = fromExpression(lessThanExpression);
 
     assertNull(And.getInstance().bindPredicates(greaterThanPredicate, lessThanPredicate).filter(), "Decimal type push down is unsupported, so we expect null");
@@ -243,4 +244,168 @@ public class TestExpressionPredicates {
     ExpressionPredicates.ColumnPredicate predicate = Equals.getInstance().bindFieldReference(fieldReference).bindValueLiteral(valueLiteral);
     assertDoesNotThrow(predicate::filter, () -> String.format("Convert from %s to %s failed", literalValue.getClass().getName(), dataType));
   }
+
+  @Test
+  public void testUnaryPredicateReferences() {
+    // Test Equals predicate references method
+    FieldReferenceExpression fieldReference = new FieldReferenceExpression("field1", DataTypes.STRING(), 0, 0);
+    ValueLiteralExpression valueLiteral = new ValueLiteralExpression("value1");
+    
+    ExpressionPredicates.ColumnPredicate equalsPredicate = Equals.getInstance()
+        .bindFieldReference(fieldReference)
+        .bindValueLiteral(valueLiteral);
+    
+    assertEquals(Collections.singletonList("field1"), equalsPredicate.references());
+
+    // Test GreaterThan predicate references method
+    valueLiteral = new ValueLiteralExpression(10);
+    ExpressionPredicates.ColumnPredicate gtPredicate = GreaterThan.getInstance()
+        .bindFieldReference(fieldReference)
+        .bindValueLiteral(valueLiteral);
+    assertEquals(Collections.singletonList("field1"), gtPredicate.references());
+
+    ValueLiteralExpression valueLiteral1 = new ValueLiteralExpression("value1");
+    ValueLiteralExpression valueLiteral2 = new ValueLiteralExpression("value2");
+
+    ExpressionPredicates.ColumnPredicate inPredicate = In.getInstance()
+        .bindValueLiterals(Arrays.asList(valueLiteral1, valueLiteral2))
+        .bindFieldReference(fieldReference);
+
+    assertEquals(Collections.singletonList("field1"), inPredicate.references());
+  }
+
+  @Test
+  public void testNotPredicateReferences() {
+    // Test Not predicate references method - should delegate to underlying predicate
+    FieldReferenceExpression fieldReference = new FieldReferenceExpression("field4", DataTypes.STRING(), 0, 0);
+    ValueLiteralExpression valueLiteral = new ValueLiteralExpression("value1");
+    
+    ExpressionPredicates.ColumnPredicate innerPredicate = Equals.getInstance()
+        .bindFieldReference(fieldReference)
+        .bindValueLiteral(valueLiteral);
+    
+    Predicate notPredicate = Not.getInstance().bindPredicate(innerPredicate);
+    
+    List<String> references = notPredicate.references();
+    assertEquals(1, references.size());
+    assertEquals("field4", references.get(0));
+  }
+
+  @Test
+  public void testAndPredicateReferencesWithSameColumn() {
+    // Test And predicate references method with same column
+    FieldReferenceExpression fieldReference1 = new FieldReferenceExpression("field1", DataTypes.INT(), 0, 0);
+    FieldReferenceExpression fieldReference2 = new FieldReferenceExpression("field1", DataTypes.INT(), 0, 0);
+    ValueLiteralExpression valueLiteral1 = new ValueLiteralExpression(10);
+    ValueLiteralExpression valueLiteral2 = new ValueLiteralExpression(20);
+    
+    ExpressionPredicates.ColumnPredicate predicate1 = GreaterThan.getInstance()
+        .bindFieldReference(fieldReference1)
+        .bindValueLiteral(valueLiteral1);
+    
+    ExpressionPredicates.ColumnPredicate predicate2 = Equals.getInstance()
+        .bindFieldReference(fieldReference2)
+        .bindValueLiteral(valueLiteral2);
+    
+    Predicate andPredicate = And.getInstance().bindPredicates(predicate1, predicate2);
+    
+    List<String> references = andPredicate.references();
+    assertEquals(1, references.size());
+    assertEquals("field1", references.get(0));
+  }
+
+  @Test
+  public void testAndPredicateReferencesWithDifferentColumns() {
+    // Test And predicate references method with different columns
+    FieldReferenceExpression fieldReference1 = new FieldReferenceExpression("field1", DataTypes.INT(), 0, 0);
+    FieldReferenceExpression fieldReference2 = new FieldReferenceExpression("field2", DataTypes.STRING(), 0, 0);
+    ValueLiteralExpression valueLiteral1 = new ValueLiteralExpression(10);
+    ValueLiteralExpression valueLiteral2 = new ValueLiteralExpression("test");
+    
+    ExpressionPredicates.ColumnPredicate predicate1 = GreaterThan.getInstance()
+        .bindFieldReference(fieldReference1)
+        .bindValueLiteral(valueLiteral1);
+    
+    ExpressionPredicates.ColumnPredicate predicate2 = Equals.getInstance()
+        .bindFieldReference(fieldReference2)
+        .bindValueLiteral(valueLiteral2);
+    
+    Predicate andPredicate = And.getInstance().bindPredicates(predicate1, predicate2);
+    
+    assertEquals(Arrays.asList("field1", "field2"), andPredicate.references());
+  }
+
+  @Test
+  public void testOrPredicateReferencesWithSameColumn() {
+    // Test Or predicate references method with same column
+    FieldReferenceExpression fieldReference1 = new FieldReferenceExpression("field1", DataTypes.INT(), 0, 0);
+    FieldReferenceExpression fieldReference2 = new FieldReferenceExpression("field1", DataTypes.INT(), 0, 0);
+    ValueLiteralExpression valueLiteral1 = new ValueLiteralExpression(10);
+    ValueLiteralExpression valueLiteral2 = new ValueLiteralExpression(20);
+    
+    ExpressionPredicates.ColumnPredicate predicate1 = GreaterThan.getInstance()
+        .bindFieldReference(fieldReference1)
+        .bindValueLiteral(valueLiteral1);
+    
+    ExpressionPredicates.ColumnPredicate predicate2 = Equals.getInstance()
+        .bindFieldReference(fieldReference2)
+        .bindValueLiteral(valueLiteral2);
+    
+    Predicate orPredicate = Or.getInstance().bindPredicates(predicate1, predicate2);
+    
+    List<String> references = orPredicate.references();
+    assertEquals(1, references.size());
+    assertEquals("field1", references.get(0));
+  }
+
+  @Test
+  public void testOrPredicateReferencesWithDifferentColumns() {
+    // Test Or predicate references method with different columns
+    FieldReferenceExpression fieldReference1 = new FieldReferenceExpression("field1", DataTypes.INT(), 0, 0);
+    FieldReferenceExpression fieldReference2 = new FieldReferenceExpression("field2", DataTypes.STRING(), 0, 0);
+    ValueLiteralExpression valueLiteral1 = new ValueLiteralExpression(10);
+    ValueLiteralExpression valueLiteral2 = new ValueLiteralExpression("test");
+    
+    ExpressionPredicates.ColumnPredicate predicate1 = GreaterThan.getInstance()
+        .bindFieldReference(fieldReference1)
+        .bindValueLiteral(valueLiteral1);
+    
+    ExpressionPredicates.ColumnPredicate predicate2 = Equals.getInstance()
+        .bindFieldReference(fieldReference2)
+        .bindValueLiteral(valueLiteral2);
+    
+    Predicate orPredicate = Or.getInstance().bindPredicates(predicate1, predicate2);
+    assertEquals(Arrays.asList("field1", "field2"), orPredicate.references());
+  }
+
+  @Test
+  public void testAlwaysNullPredicateReferences() {
+    // Test AlwaysNull predicate references method - should return empty list
+    Predicate alwaysNullPredicate = ExpressionPredicates.AlwaysNull.getInstance();
+    
+    List<String> references = alwaysNullPredicate.references();
+    assertEquals(0, references.size());
+    assertTrue(references.isEmpty());
+  }
+
+  @Test
+  public void testNestedPredicateReferences() {
+    // Test nested predicates (AND inside NOT)
+    FieldReferenceExpression fieldReference1 = new FieldReferenceExpression("field1", DataTypes.INT(), 0, 0);
+    FieldReferenceExpression fieldReference2 = new FieldReferenceExpression("field2", DataTypes.STRING(), 0, 0);
+    ValueLiteralExpression valueLiteral1 = new ValueLiteralExpression(10);
+    ValueLiteralExpression valueLiteral2 = new ValueLiteralExpression("test");
+    
+    ExpressionPredicates.ColumnPredicate predicate1 = GreaterThan.getInstance()
+        .bindFieldReference(fieldReference1)
+        .bindValueLiteral(valueLiteral1);
+    
+    ExpressionPredicates.ColumnPredicate predicate2 = Equals.getInstance()
+        .bindFieldReference(fieldReference2)
+        .bindValueLiteral(valueLiteral2);
+    
+    Predicate andPredicate = And.getInstance().bindPredicates(predicate1, predicate2);
+    Predicate notPredicate = Not.getInstance().bindPredicate(andPredicate);
+    assertEquals(Arrays.asList("field1", "field2"), notPredicate.references());
+  }
 }
