@@ -20,17 +20,27 @@ package org.apache.hudi.table.format;
 
 import org.apache.hudi.client.HoodieFlinkWriteClient;
 import org.apache.hudi.client.common.HoodieFlinkEngineContext;
+import org.apache.hudi.client.model.PartialUpdateFlinkRecordMerger;
+import org.apache.hudi.common.config.RecordMergeMode;
 import org.apache.hudi.common.model.EventTimeAvroPayload;
 import org.apache.hudi.common.model.HoodieCommitMetadata;
 import org.apache.hudi.common.model.HoodieTableType;
+import org.apache.hudi.common.model.OverwriteWithLatestAvroPayload;
+import org.apache.hudi.common.model.PartialUpdateAvroPayload;
+import org.apache.hudi.common.model.WriteConcurrencyMode;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
+import org.apache.hudi.common.table.HoodieTableVersion;
 import org.apache.hudi.common.table.cdc.HoodieCDCSupplementalLoggingMode;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.testutils.HoodieTestUtils;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.configuration.HadoopConfigurations;
+import org.apache.hudi.index.HoodieIndex;
+import org.apache.hudi.io.FileGroupReaderBasedMergeHandle;
+import org.apache.hudi.io.HoodieWriteMergeHandle;
 import org.apache.hudi.source.IncrementalInputSplits;
 import org.apache.hudi.source.prune.PartitionPruners;
 import org.apache.hudi.storage.StoragePath;
@@ -52,11 +62,15 @@ import org.apache.flink.api.common.io.InputFormat;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.core.io.InputSplit;
 import org.apache.flink.table.api.DataTypes;
+import org.apache.flink.table.catalog.ResolvedSchema;
 import org.apache.flink.table.data.RowData;
+import org.apache.flink.table.data.StringData;
+import org.apache.flink.table.data.TimestampData;
 import org.apache.flink.table.expressions.CallExpression;
 import org.apache.flink.table.expressions.FieldReferenceExpression;
 import org.apache.flink.table.expressions.ValueLiteralExpression;
 import org.apache.flink.table.functions.BuiltInFunctionDefinitions;
+import org.apache.flink.table.runtime.typeutils.RowDataSerializer;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
@@ -78,10 +92,12 @@ import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
 import static org.apache.hudi.common.testutils.HoodieTestUtils.INSTANT_FILE_NAME_GENERATOR;
+import static org.apache.hudi.utils.TestData.insertRow;
 import static org.hamcrest.CoreMatchers.instanceOf;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertSame;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 /**
@@ -101,9 +117,9 @@ public class TestInputFormat {
 
   void beforeEach(HoodieTableType tableType, Map<String, String> options) throws IOException {
     conf = TestConfigurations.getDefaultConf(tempFile.getAbsolutePath());
-    conf.setString(FlinkOptions.TABLE_TYPE, tableType.name());
+    conf.set(FlinkOptions.TABLE_TYPE, tableType.name());
     if (!conf.contains(FlinkOptions.COMPACTION_ASYNC_ENABLED)) {
-      conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, false); // by default close the async compaction
+      conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, false); // by default close the async compaction
     }
     options.forEach((key, value) -> conf.setString(key, value));
 
@@ -156,8 +172,8 @@ public class TestInputFormat {
     beforeEach(HoodieTableType.MERGE_ON_READ);
 
     // write base first with compaction
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
-    conf.setInteger(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
+    conf.set(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
     TestData.writeData(TestData.DATA_SET_INSERT, conf);
 
     InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
@@ -169,7 +185,7 @@ public class TestInputFormat {
     assertThat(actual, is(expected));
 
     // write another commit using logs and read again
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, false);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, false);
     TestData.writeData(TestData.DATA_SET_UPDATE_INSERT, conf);
 
     // write another commit using logs with separate partition
@@ -209,12 +225,12 @@ public class TestInputFormat {
     beforeEach(HoodieTableType.MERGE_ON_READ, options);
 
     // write base first with compaction.
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
-    conf.setInteger(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
+    conf.set(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
     TestData.writeData(TestData.DATA_SET_INSERT, conf);
 
     // write another commit using logs and read again.
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, false);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, false);
     TestData.writeData(TestData.DATA_SET_UPDATE_DELETE, conf);
 
     InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
@@ -262,12 +278,12 @@ public class TestInputFormat {
     beforeEach(HoodieTableType.MERGE_ON_READ, options);
 
     // write base first with compaction.
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
-    conf.setInteger(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
+    conf.set(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
     TestData.writeData(TestData.DATA_SET_SINGLE_INSERT, conf);
 
     // write another commit using logs and read again.
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, compact);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, compact);
     TestData.writeData(TestData.DATA_SET_DISORDER_UPDATE_DELETE, conf);
 
     InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
@@ -293,6 +309,31 @@ public class TestInputFormat {
     assertThat(actual2, is(expected));
   }
 
+  @Test
+  void testReadWithDeletes() throws Exception {
+    Map<String, String> options = new HashMap<>();
+    options.put(FlinkOptions.RECORD_KEY_FIELD.key(), "uuid");
+    beforeEach(HoodieTableType.MERGE_ON_READ, options);
+
+    // write another commit to read again
+    TestData.writeData(TestData.DATA_SET_UPDATE_DELETE, conf);
+
+    InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
+    assertThat(inputFormat, instanceOf(MergeOnReadInputFormat.class));
+    ((MergeOnReadInputFormat) inputFormat).isEmitDelete(true);
+
+    List<RowData> result = readData(inputFormat);
+
+    final String actual = TestData.rowDataToString(result);
+    final String expected = "["
+        + "+I[id1, Danny, 24, 1970-01-01T00:00:00.001, par1], "
+        + "+I[id2, Stephen, 34, 1970-01-01T00:00:00.002, par1], "
+        + "-D[id3, null, null, null, null], "
+        + "-D[id5, null, null, null, null], "
+        + "-D[id9, null, null, null, null]]";
+    assertThat(actual, is(expected));
+  }
+
   @Test
   void testReadWithDeletesMOR() throws Exception {
     Map<String, String> options = new HashMap<>();
@@ -426,8 +467,8 @@ public class TestInputFormat {
     org.apache.hadoop.conf.Configuration hadoopConf = HadoopConfigurations.getHadoopConf(conf);
 
     // write base first with compaction
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
-    conf.setInteger(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
+    conf.set(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
     TestData.writeData(TestData.DATA_SET_INSERT, conf);
 
     InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat(true);
@@ -453,12 +494,12 @@ public class TestInputFormat {
     assertThat(actual1, is(expected1));
 
     // write another commit using logs and read again
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, false);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, false);
     TestData.writeData(TestData.DATA_SET_UPDATE_INSERT, conf);
 
     // read from the compaction commit
     String secondCommit = TestUtils.getNthCompleteInstant(metaClient.getBasePath(), 0, HoodieTimeline.COMMIT_ACTION);
-    conf.setString(FlinkOptions.READ_START_COMMIT, secondCommit);
+    conf.set(FlinkOptions.READ_START_COMMIT, secondCommit);
 
     IncrementalInputSplits.Result splits2 = incrementalInputSplits.inputSplits(metaClient, null, false);
     assertFalse(splits2.isEmpty());
@@ -489,10 +530,10 @@ public class TestInputFormat {
     beforeEach(HoodieTableType.COPY_ON_WRITE);
 
     // write base first with clustering
-    conf.setString(FlinkOptions.OPERATION, "insert");
-    conf.setBoolean(FlinkOptions.CLUSTERING_SCHEDULE_ENABLED, true);
-    conf.setBoolean(FlinkOptions.CLUSTERING_ASYNC_ENABLED, true);
-    conf.setInteger(FlinkOptions.CLUSTERING_DELTA_COMMITS, 1);
+    conf.set(FlinkOptions.OPERATION, "insert");
+    conf.set(FlinkOptions.CLUSTERING_SCHEDULE_ENABLED, true);
+    conf.set(FlinkOptions.CLUSTERING_ASYNC_ENABLED, true);
+    conf.set(FlinkOptions.CLUSTERING_DELTA_COMMITS, 1);
     TestData.writeData(TestData.DATA_SET_INSERT, conf);
 
     InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat(true);
@@ -518,12 +559,12 @@ public class TestInputFormat {
     assertThat(actual1, is(expected1));
 
     // write another commit and read again
-    conf.setBoolean(FlinkOptions.CLUSTERING_ASYNC_ENABLED, false);
+    conf.set(FlinkOptions.CLUSTERING_ASYNC_ENABLED, false);
     TestData.writeData(TestData.DATA_SET_UPDATE_INSERT, conf);
 
     // read from the clustering commit
     String secondCommit = TestUtils.getNthCompleteInstant(metaClient.getBasePath(), 0, HoodieTimeline.REPLACE_COMMIT_ACTION);
-    conf.setString(FlinkOptions.READ_START_COMMIT, secondCommit);
+    conf.set(FlinkOptions.READ_START_COMMIT, secondCommit);
 
     IncrementalInputSplits.Result splits2 = incrementalInputSplits.inputSplits(metaClient, null, false);
     assertFalse(splits2.isEmpty());
@@ -534,7 +575,7 @@ public class TestInputFormat {
 
     // write another commit with separate partition
     // so the file group has only base files
-    conf.setBoolean(FlinkOptions.CLUSTERING_ASYNC_ENABLED, true);
+    conf.set(FlinkOptions.CLUSTERING_ASYNC_ENABLED, true);
     TestData.writeData(TestData.DATA_SET_INSERT_SEPARATE_PARTITION, conf);
 
     // refresh the input format
@@ -602,7 +643,7 @@ public class TestInputFormat {
 
     // timeline: c1, c2.inflight, c3.inflight, c4
     // -> c1
-    conf.setString(FlinkOptions.READ_START_COMMIT, FlinkOptions.START_COMMIT_EARLIEST);
+    conf.set(FlinkOptions.READ_START_COMMIT, FlinkOptions.START_COMMIT_EARLIEST);
     IncrementalInputSplits.Result splits2 = incrementalInputSplits.inputSplits(metaClient, null, false);
     assertFalse(splits2.isEmpty());
     List<RowData> result2 = readData(inputFormat, splits2.getInputSplits().toArray(new MergeOnReadInputSplit[0]));
@@ -677,7 +718,7 @@ public class TestInputFormat {
 
     // read from the latest commit
     String secondCommit = TestUtils.getNthCompleteInstant(metaClient.getBasePath(), 1, HoodieTimeline.COMMIT_ACTION);
-    conf.setString(FlinkOptions.READ_START_COMMIT, secondCommit);
+    conf.set(FlinkOptions.READ_START_COMMIT, secondCommit);
 
     IncrementalInputSplits.Result splits2 = incrementalInputSplits.inputSplits(metaClient, null, false);
     assertFalse(splits2.isEmpty());
@@ -697,7 +738,7 @@ public class TestInputFormat {
     // prune to only be with partition 'par1'
     FieldReferenceExpression partRef = new FieldReferenceExpression("partition", DataTypes.STRING(), 4, 4);
     ValueLiteralExpression partLiteral = new ValueLiteralExpression("par1", DataTypes.STRING().notNull());
-    CallExpression partFilter = new CallExpression(
+    CallExpression partFilter = CallExpression.permanent(
         BuiltInFunctionDefinitions.EQUALS,
         Arrays.asList(partRef, partLiteral),
         DataTypes.BOOLEAN());
@@ -794,7 +835,7 @@ public class TestInputFormat {
     assertThat(commits.size(), is(3));
 
     // only the start commit
-    conf.setString(FlinkOptions.READ_START_COMMIT, commits.get(1));
+    conf.set(FlinkOptions.READ_START_COMMIT, commits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat1 = this.tableSource.getInputFormat();
     assertThat(inputFormat1, instanceOf(MergeOnReadInputFormat.class));
@@ -804,7 +845,7 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual1, expected1);
 
     // only the start commit: earliest
-    conf.setString(FlinkOptions.READ_START_COMMIT, FlinkOptions.START_COMMIT_EARLIEST);
+    conf.set(FlinkOptions.READ_START_COMMIT, FlinkOptions.START_COMMIT_EARLIEST);
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat2 = this.tableSource.getInputFormat();
     assertThat(inputFormat2, instanceOf(MergeOnReadInputFormat.class));
@@ -814,8 +855,8 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual2, expected2);
 
     // start and end commit: [start commit, end commit]
-    conf.setString(FlinkOptions.READ_START_COMMIT, commits.get(0));
-    conf.setString(FlinkOptions.READ_END_COMMIT, commits.get(1));
+    conf.set(FlinkOptions.READ_START_COMMIT, commits.get(0));
+    conf.set(FlinkOptions.READ_END_COMMIT, commits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat3 = this.tableSource.getInputFormat();
     assertThat(inputFormat3, instanceOf(MergeOnReadInputFormat.class));
@@ -826,7 +867,7 @@ public class TestInputFormat {
 
     // only the end commit: point in time query
     conf.removeConfig(FlinkOptions.READ_START_COMMIT);
-    conf.setString(FlinkOptions.READ_END_COMMIT, commits.get(1));
+    conf.set(FlinkOptions.READ_END_COMMIT, commits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat4 = this.tableSource.getInputFormat();
     assertThat(inputFormat4, instanceOf(MergeOnReadInputFormat.class));
@@ -836,8 +877,8 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual4, expected4);
 
     // start and end commit: start commit out of range
-    conf.setString(FlinkOptions.READ_START_COMMIT, "000");
-    conf.setString(FlinkOptions.READ_END_COMMIT, commits.get(1));
+    conf.set(FlinkOptions.READ_START_COMMIT, "000");
+    conf.set(FlinkOptions.READ_END_COMMIT, commits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat5 = this.tableSource.getInputFormat();
     assertThat(inputFormat4, instanceOf(MergeOnReadInputFormat.class));
@@ -847,8 +888,8 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual5, expected5);
 
     // start and end commit: both are out of range
-    conf.setString(FlinkOptions.READ_START_COMMIT, "001");
-    conf.setString(FlinkOptions.READ_END_COMMIT, "002");
+    conf.set(FlinkOptions.READ_START_COMMIT, "001");
+    conf.set(FlinkOptions.READ_END_COMMIT, "002");
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat6 = this.tableSource.getInputFormat();
 
@@ -937,7 +978,7 @@ public class TestInputFormat {
 
   private void testReadChangelogInternal(List<String> commits) throws IOException {
     // only the start commit
-    conf.setString(FlinkOptions.READ_START_COMMIT, commits.get(1));
+    conf.set(FlinkOptions.READ_START_COMMIT, commits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat1 = this.tableSource.getInputFormat();
     assertThat(inputFormat1, instanceOf(CdcInputFormat.class));
@@ -946,7 +987,7 @@ public class TestInputFormat {
     final List<RowData> expected1 = TestData.dataSetUpsert(2, 1, 2, 1);
     TestData.assertRowDataEquals(actual1, expected1);
     // only the start commit: earliest
-    conf.setString(FlinkOptions.READ_START_COMMIT, FlinkOptions.START_COMMIT_EARLIEST);
+    conf.set(FlinkOptions.READ_START_COMMIT, FlinkOptions.START_COMMIT_EARLIEST);
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat2 = this.tableSource.getInputFormat();
     assertThat(inputFormat2, instanceOf(CdcInputFormat.class));
@@ -956,8 +997,8 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual2, expected2);
 
     // start and end commit: [start commit, end commit]
-    conf.setString(FlinkOptions.READ_START_COMMIT, commits.get(0));
-    conf.setString(FlinkOptions.READ_END_COMMIT, commits.get(1));
+    conf.set(FlinkOptions.READ_START_COMMIT, commits.get(0));
+    conf.set(FlinkOptions.READ_END_COMMIT, commits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat3 = this.tableSource.getInputFormat();
     assertThat(inputFormat3, instanceOf(CdcInputFormat.class));
@@ -971,7 +1012,7 @@ public class TestInputFormat {
 
     // only the end commit: point in time query
     conf.removeConfig(FlinkOptions.READ_START_COMMIT);
-    conf.setString(FlinkOptions.READ_END_COMMIT, commits.get(1));
+    conf.set(FlinkOptions.READ_END_COMMIT, commits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat4 = this.tableSource.getInputFormat();
     assertThat(inputFormat4, instanceOf(CdcInputFormat.class));
@@ -981,8 +1022,8 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual4, expected4);
 
     // start and end commit: start commit out of range
-    conf.setString(FlinkOptions.READ_START_COMMIT, "000");
-    conf.setString(FlinkOptions.READ_END_COMMIT, commits.get(1));
+    conf.set(FlinkOptions.READ_START_COMMIT, "000");
+    conf.set(FlinkOptions.READ_END_COMMIT, commits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat5 = this.tableSource.getInputFormat();
     assertThat(inputFormat5, instanceOf(CdcInputFormat.class));
@@ -993,8 +1034,8 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual5, expected3);
 
     // start and end commit: both are out of range
-    conf.setString(FlinkOptions.READ_START_COMMIT, "001");
-    conf.setString(FlinkOptions.READ_END_COMMIT, "002");
+    conf.set(FlinkOptions.READ_START_COMMIT, "001");
+    conf.set(FlinkOptions.READ_END_COMMIT, "002");
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat6 = this.tableSource.getInputFormat();
 
@@ -1009,8 +1050,8 @@ public class TestInputFormat {
     beforeEach(HoodieTableType.MERGE_ON_READ, options);
 
     // write base file first with compaction.
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
-    conf.setInteger(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, true);
+    conf.set(FlinkOptions.COMPACTION_DELTA_COMMITS, 1);
     TestData.writeData(TestData.DATA_SET_DISORDER_INSERT, conf);
     InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
     final String baseResult = TestData.rowDataToString(readData(inputFormat));
@@ -1018,7 +1059,7 @@ public class TestInputFormat {
     assertThat(baseResult, is(expected));
 
     // write another commit using logs and read again.
-    conf.setBoolean(FlinkOptions.COMPACTION_ASYNC_ENABLED, false);
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, false);
     TestData.writeData(TestData.DATA_SET_SINGLE_INSERT, conf);
     this.tableSource.reset();
     inputFormat = this.tableSource.getInputFormat();
@@ -1057,6 +1098,29 @@ public class TestInputFormat {
     assertThat(baseResult, is(expected));
   }
 
+  @Test
+  void testMergeRecordWithUpdateBefore() throws Exception {
+    Map<String, String> options = new HashMap<>();
+    options.put(FlinkOptions.CHANGELOG_ENABLED.key(), "true");
+    beforeEach(HoodieTableType.COPY_ON_WRITE, options);
+
+    // write first batch with all insert data
+    TestData.writeData(TestData.DATA_SET_INSERT, conf);
+    // write second batch with UPDATE_BEFORE record, send UPDATE_BEFORE for `id1`
+    TestData.writeData(TestData.DATA_SET_UPDATE_BEFORE, conf);
+    InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
+    final String baseResult = TestData.rowDataToString(readData(inputFormat));
+    String expected = "["
+        + "+I[id2, Stephen, 33, 1970-01-01T00:00:00.002, par1], "
+        + "+I[id3, Julian, 53, 1970-01-01T00:00:00.003, par2], "
+        + "+I[id4, Fabian, 31, 1970-01-01T00:00:00.004, par2], "
+        + "+I[id5, Sophia, 18, 1970-01-01T00:00:00.005, par3], "
+        + "+I[id6, Emma, 20, 1970-01-01T00:00:00.006, par3], "
+        + "+I[id7, Bob, 44, 1970-01-01T00:00:00.007, par4], "
+        + "+I[id8, Han, 56, 1970-01-01T00:00:00.008, par4]]";
+    assertThat(baseResult, is(expected));
+  }
+
   @Test
   void testReadArchivedCommitsIncrementally() throws Exception {
     Map<String, String> options = new HashMap<>();
@@ -1092,8 +1156,8 @@ public class TestInputFormat {
     assertThat(archivedCommits.size(), is(6));
 
     // start and end commit: both are archived and cleaned
-    conf.setString(FlinkOptions.READ_START_COMMIT, archivedCommits.get(0));
-    conf.setString(FlinkOptions.READ_END_COMMIT, archivedCommits.get(1));
+    conf.set(FlinkOptions.READ_START_COMMIT, archivedCommits.get(0));
+    conf.set(FlinkOptions.READ_END_COMMIT, archivedCommits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat1 = this.tableSource.getInputFormat();
     assertThat(inputFormat1, instanceOf(MergeOnReadInputFormat.class));
@@ -1103,7 +1167,7 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual1, expected1);
 
     // only the start commit: is archived and cleaned
-    conf.setString(FlinkOptions.READ_START_COMMIT, archivedCommits.get(1));
+    conf.set(FlinkOptions.READ_START_COMMIT, archivedCommits.get(1));
     conf.removeConfig(FlinkOptions.READ_END_COMMIT);
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat2 = this.tableSource.getInputFormat();
@@ -1116,7 +1180,7 @@ public class TestInputFormat {
 
     // only the end commit: is archived and cleaned
     conf.removeConfig(FlinkOptions.READ_START_COMMIT);
-    conf.setString(FlinkOptions.READ_END_COMMIT, archivedCommits.get(1));
+    conf.set(FlinkOptions.READ_END_COMMIT, archivedCommits.get(1));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat3 = this.tableSource.getInputFormat();
     assertThat(inputFormat3, instanceOf(MergeOnReadInputFormat.class));
@@ -1126,8 +1190,8 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(actual3, expected3);
 
     // start and end commit: start is archived and cleaned, end is active and cleaned
-    conf.setString(FlinkOptions.READ_START_COMMIT, archivedCommits.get(1));
-    conf.setString(FlinkOptions.READ_END_COMMIT, commits.get(0));
+    conf.set(FlinkOptions.READ_START_COMMIT, archivedCommits.get(1));
+    conf.set(FlinkOptions.READ_END_COMMIT, commits.get(0));
     this.tableSource = getTableSource(conf);
     InputFormat<RowData, ?> inputFormat4 = this.tableSource.getInputFormat();
     // assertThat(inputFormat4, instanceOf(MergeOnReadInputFormat.class));
@@ -1152,6 +1216,30 @@ public class TestInputFormat {
     TestData.assertRowDataEquals(result, TestData.DATA_SET_INSERT);
   }
 
+  @Test
+  void testOrderingValueWithDecimalType() throws Exception {
+    conf = new Configuration();
+    conf.set(FlinkOptions.PATH, tempFile.getAbsolutePath());
+    conf.set(FlinkOptions.TABLE_NAME, "TestHoodieTable");
+    conf.set(FlinkOptions.TABLE_TYPE, HoodieTableType.MERGE_ON_READ.name());
+    conf.set(FlinkOptions.PARTITION_PATH_FIELD, "partition");
+    conf.setString(FlinkOptions.SOURCE_AVRO_SCHEMA.key(), AvroSchemaConverter.convertToSchema(TestConfigurations.ROW_TYPE_DECIMAL_ORDERING).toString());
+    conf.set(FlinkOptions.COMPACTION_ASYNC_ENABLED, false); // by default close the async compaction
+    StreamerUtil.initTableIfNotExists(conf);
+
+    tableSource = getTableSource(conf, TestConfigurations.TABLE_SCHEMA_DECIMAL_ORDERING);
+
+    TestData.writeData(TestData.DATA_SET_INSERT_DECIMAL_ORDERING, conf);
+    InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
+    List<RowData> result = readData(inputFormat, TestConfigurations.SERIALIZER_DECIMAL_ORDERING);
+
+    // global index is enabled by default
+    // row1: <id1, Danny, 23, 10, par1>
+    // row2: <id1, Bob, 44, 1, par2>
+    // the ordering value of row2: 1 is smaller than that of row1: 10, so row1 in `par1` will not be deleted.
+    TestData.assertRowDataEquals(result, TestData.DATA_SET_INSERT_DECIMAL_ORDERING, TestConfigurations.ROW_DATA_TYPE_DECIMAL_ORDERING);
+  }
+
   /**
    * Test reading file groups with compaction plan scheduled and delta logs.
    * File-slice after pending compaction-requested instant-time should also be considered valid.
@@ -1216,17 +1304,185 @@ public class TestInputFormat {
         .path(FilePathUtils.toFlinkPath(metaClient.getBasePath()))
         .skipCompaction(skipCompaction)
         .build();
-    conf.setString(FlinkOptions.READ_END_COMMIT, newCompletionTime);
+    conf.set(FlinkOptions.READ_END_COMMIT, newCompletionTime);
     IncrementalInputSplits.Result splits2 = incrementalInputSplits.inputSplits(metaClient, null, false);
     assertFalse(splits2.isEmpty());
     List<RowData> result2 = readData(inputFormat, splits2.getInputSplits().toArray(new MergeOnReadInputSplit[0]));
     TestData.assertRowDataEquals(result2, TestData.dataSetInsert(1, 2));
   }
 
+  @Test
+  void testIncReadWithNBCCAndSingleBucketNum() throws Exception {
+    Map<String, String> options = new HashMap<>();
+    // file group id for the same bucket id among different partitions are same with NBCC mode
+    options.put(HoodieWriteConfig.WRITE_CONCURRENCY_MODE.key(), WriteConcurrencyMode.NON_BLOCKING_CONCURRENCY_CONTROL.name());
+    options.put(FlinkOptions.INDEX_TYPE.key(), HoodieIndex.IndexType.BUCKET.name());
+    options.put(FlinkOptions.BUCKET_INDEX_NUM_BUCKETS.key(), "1");
+    beforeEach(HoodieTableType.MERGE_ON_READ, options);
+
+    TestData.writeData(TestData.DATA_SET_INSERT, conf);
+
+    conf.set(FlinkOptions.READ_START_COMMIT, "000");
+    conf.set(FlinkOptions.QUERY_TYPE, FlinkOptions.QUERY_TYPE_INCREMENTAL);
+    this.tableSource = getTableSource(conf);
+    InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
+    assertThat(inputFormat, instanceOf(MergeOnReadInputFormat.class));
+
+    List<RowData> result = readData(inputFormat);
+    TestData.assertRowDataEquals(result, TestData.DATA_SET_INSERT);
+  }
+
+  @ParameterizedTest
+  @ValueSource(booleans = {true, false})
+  public void testCompactWithEventTimeOrdering(boolean useLegacyConfig) throws Exception {
+    Map<String, String> options = new HashMap<>();
+    if (useLegacyConfig) {
+      options.put(FlinkOptions.PAYLOAD_CLASS_NAME.key(), EventTimeAvroPayload.class.getName());
+    } else {
+      options.put(FlinkOptions.RECORD_MERGE_MODE.key(), RecordMergeMode.EVENT_TIME_ORDERING.name());
+    }
+    options.put(FlinkOptions.COMPACTION_DELTA_COMMITS.key(), "1");
+    options.put(FlinkOptions.COMPACTION_ASYNC_ENABLED.key(), "true");
+    beforeEach(HoodieTableType.MERGE_ON_READ, options);
+
+    TestData.writeData(TestData.DATA_SET_DISORDER_INSERT, conf);
+    Map<String, String> expected = new HashMap<>();
+    // expected is row with the largest ordering value
+    expected.put("par1", "[id1,par1,id1,Danny,22,4,par1]");
+    // check result from base file
+    TestData.checkWrittenData(tempFile, expected, expected.size());
+  }
+
+  @ParameterizedTest
+  @ValueSource(booleans = {true, false})
+  public void testCompactWithCommitTimeOrdering(boolean useLegacyConfig) throws Exception {
+    Map<String, String> options = new HashMap<>();
+    if (useLegacyConfig) {
+      options.put(FlinkOptions.PAYLOAD_CLASS_NAME.key(), OverwriteWithLatestAvroPayload.class.getName());
+    } else {
+      options.put(FlinkOptions.RECORD_MERGE_MODE.key(), RecordMergeMode.COMMIT_TIME_ORDERING.name());
+    }
+    options.put(FlinkOptions.COMPACTION_DELTA_COMMITS.key(), "1");
+    options.put(FlinkOptions.COMPACTION_ASYNC_ENABLED.key(), "true");
+    beforeEach(HoodieTableType.MERGE_ON_READ, options);
+
+    TestData.writeData(TestData.DATA_SET_DISORDER_INSERT, conf);
+    Map<String, String> expected = new HashMap<>();
+    // expected is the last inserted row
+    expected.put("par1", "[id1,par1,id1,Danny,23,1,par1]");
+    // check result from base file
+    TestData.checkWrittenData(tempFile, expected, expected.size());
+  }
+
+  @ParameterizedTest
+  @ValueSource(ints = {1, 2, 3})
+  public void testCompactWithPartialUpdate(int configOrdinal) throws Exception {
+    Map<String, String> options = new HashMap<>();
+    if (configOrdinal == 1) {
+      // legacy config for partial update
+      options.put(FlinkOptions.PAYLOAD_CLASS_NAME.key(), PartialUpdateAvroPayload.class.getName());
+    } else if (configOrdinal == 2) {
+      // new config with only merge classes
+      options.put(FlinkOptions.RECORD_MERGER_IMPLS.key(), PartialUpdateFlinkRecordMerger.class.getName());
+    } else {
+      // new config with merge classes and merge mode
+      options.put(FlinkOptions.RECORD_MERGE_MODE.key(), RecordMergeMode.CUSTOM.name());
+      options.put(FlinkOptions.RECORD_MERGER_IMPLS.key(), PartialUpdateFlinkRecordMerger.class.getName());
+    }
+    options.put(FlinkOptions.COMPACTION_DELTA_COMMITS.key(), "1");
+    options.put(FlinkOptions.COMPACTION_ASYNC_ENABLED.key(), "true");
+    beforeEach(HoodieTableType.MERGE_ON_READ, options);
+
+    List<RowData> sourceData = Arrays.asList(
+        insertRow(StringData.fromString("id1"), StringData.fromString("Danny"), null,
+            TimestampData.fromEpochMillis(2), StringData.fromString("par1")),
+        insertRow(StringData.fromString("id1"), null, 23,
+            TimestampData.fromEpochMillis(1), StringData.fromString("par1"))
+    );
+    TestData.writeData(sourceData, conf);
+
+    Map<String, String> expected = new HashMap<>();
+    expected.put("par1", "[id1,par1,id1,Danny,23,2,par1]");
+    // check result from base file
+    TestData.checkWrittenData(tempFile, expected, expected.size());
+  }
+
+  @ParameterizedTest
+  @EnumSource(value = HoodieTableType.class)
+  void testStreamWriteAndReadWithUpgrade(HoodieTableType tableType) throws Exception {
+    Map<String, String> options = new HashMap<>();
+    options.put(FlinkOptions.WRITE_TABLE_VERSION.key(), HoodieTableVersion.SIX.versionCode() + "");
+    // init and write data with table version SIX
+    beforeEach(tableType, options);
+    TestData.writeData(TestData.DATA_SET_INSERT, conf);
+    assertSame(HoodieTableVersion.SIX, tableSource.getMetaClient().getTableConfig().getTableVersion());
+
+    // write another batch of data with table version EIGHT
+    conf.set(FlinkOptions.WRITE_TABLE_VERSION, HoodieTableVersion.EIGHT.versionCode());
+    TestData.writeData(TestData.DATA_SET_INSERT, conf);
+    this.tableSource = getTableSource(conf);
+    assertSame(HoodieTableVersion.EIGHT, tableSource.getMetaClient().getTableConfig().getTableVersion());
+
+    InputFormat<RowData, ?> inputFormat = this.tableSource.getInputFormat();
+    List<RowData> result = readData(inputFormat);
+    TestData.assertRowDataEquals(result, TestData.DATA_SET_INSERT);
+  }
+
+  @ParameterizedTest
+  @MethodSource("partialUpdateParams")
+  public void testWriteCowWithPartialUpdate(HoodieTableVersion tableVersion, String mergeHandleClass, boolean usePayloadConf) throws Exception {
+    Map<String, String> options = new HashMap<>();
+    options.put(HoodieWriteConfig.MERGE_HANDLE_CLASS_NAME.key(), mergeHandleClass);
+    options.put(FlinkOptions.WRITE_TABLE_VERSION.key(), tableVersion.versionCode() + "");
+    // new config with merge classes and merge mode
+    if (usePayloadConf) {
+      options.put(FlinkOptions.PAYLOAD_CLASS_NAME.key(), PartialUpdateAvroPayload.class.getName());
+    } else {
+      options.put(FlinkOptions.RECORD_MERGER_IMPLS.key(), PartialUpdateFlinkRecordMerger.class.getName());
+    }
+    beforeEach(HoodieTableType.COPY_ON_WRITE, options);
+
+    // first insert
+    List<RowData> sourceData = Arrays.asList(
+        insertRow(StringData.fromString("id1"), StringData.fromString("Danny"), null,
+            TimestampData.fromEpochMillis(2), StringData.fromString("par1"))
+    );
+    TestData.writeData(sourceData, conf);
+
+    // second insert to trigger merging during write
+    sourceData = Arrays.asList(
+        insertRow(StringData.fromString("id1"), null, 23,
+            TimestampData.fromEpochMillis(1), StringData.fromString("par1"))
+    );
+    TestData.writeData(sourceData, conf);
+
+    Map<String, String> expected = new HashMap<>();
+    expected.put("par1", "[id1,par1,id1,Danny,23,2,par1]");
+    // check result from base file
+    TestData.checkWrittenData(tempFile, expected, expected.size());
+  }
+
   // -------------------------------------------------------------------------
   //  Utilities
   // -------------------------------------------------------------------------
 
+  /**
+   * Return test params => (tableVersion, writeMergeHandleClass, usePayloadConf).
+   */
+  private static Stream<Arguments> partialUpdateParams() {
+    Object[][] data =
+        new Object[][] {
+            {HoodieTableVersion.NINE, FileGroupReaderBasedMergeHandle.class.getName(), true},
+            {HoodieTableVersion.NINE, FileGroupReaderBasedMergeHandle.class.getName(), false},
+            {HoodieTableVersion.NINE, HoodieWriteMergeHandle.class.getName(), true},
+            {HoodieTableVersion.NINE, HoodieWriteMergeHandle.class.getName(), false},
+            {HoodieTableVersion.EIGHT, FileGroupReaderBasedMergeHandle.class.getName(), true},
+            {HoodieTableVersion.EIGHT, FileGroupReaderBasedMergeHandle.class.getName(), false},
+            {HoodieTableVersion.EIGHT, HoodieWriteMergeHandle.class.getName(), true},
+            {HoodieTableVersion.EIGHT, HoodieWriteMergeHandle.class.getName(), false}};
+    return Stream.of(data).map(Arguments::of);
+  }
+
   /**
    * Return test params => (preCombining, changelog mode).
    */
@@ -1241,28 +1497,40 @@ public class TestInputFormat {
   }
 
   private HoodieTableSource getTableSource(Configuration conf) {
+    return getTableSource(conf, TestConfigurations.TABLE_SCHEMA);
+  }
+
+  private HoodieTableSource getTableSource(Configuration conf, ResolvedSchema tableSchema) {
     return new HoodieTableSource(
-        SerializableSchema.create(TestConfigurations.TABLE_SCHEMA),
+        SerializableSchema.create(tableSchema),
         new StoragePath(tempFile.getAbsolutePath()),
         Collections.singletonList("partition"),
         "default",
         conf);
   }
 
-  @SuppressWarnings("rawtypes")
   private static List<RowData> readData(InputFormat inputFormat) throws IOException {
+    return readData(inputFormat, TestConfigurations.SERIALIZER);
+  }
+
+  @SuppressWarnings("rawtypes")
+  private static List<RowData> readData(InputFormat inputFormat, RowDataSerializer serializer) throws IOException {
     InputSplit[] inputSplits = inputFormat.createInputSplits(1);
-    return readData(inputFormat, inputSplits);
+    return readData(inputFormat, inputSplits, serializer);
+  }
+
+  private static List<RowData> readData(InputFormat inputFormat, InputSplit[] inputSplits) throws IOException {
+    return readData(inputFormat, inputSplits, TestConfigurations.SERIALIZER);
   }
 
   @SuppressWarnings("unchecked, rawtypes")
-  private static List<RowData> readData(InputFormat inputFormat, InputSplit[] inputSplits) throws IOException {
+  private static List<RowData> readData(InputFormat inputFormat, InputSplit[] inputSplits, RowDataSerializer serializer) throws IOException {
     List<RowData> result = new ArrayList<>();
 
     for (InputSplit inputSplit : inputSplits) {
       inputFormat.open(inputSplit);
       while (!inputFormat.reachedEnd()) {
-        result.add(TestConfigurations.SERIALIZER.copy((RowData) inputFormat.nextRecord(null))); // no reuse
+        result.add(serializer.copy((RowData) inputFormat.nextRecord(null))); // no reuse
       }
       inputFormat.close();
     }
