@@ -34,6 +34,7 @@ import static org.hamcrest.MatcherAssert.assertThat;
  * Test cases for {@link org.apache.hudi.util.AvroSchemaConverter}.
  */
 public class TestAvroSchemaConverter {
+
   @Test
   void testUnionSchemaWithMultipleRecordTypes() {
     Schema schema = HoodieMetadataRecord.SCHEMA$;
@@ -49,7 +50,8 @@ public class TestAvroSchemaConverter {
         + "`totalSize` BIGINT, "
         + "`totalUncompressedSize` BIGINT, "
         + "`isDeleted` BOOLEAN NOT NULL, "
-        + "`isTightBound` BOOLEAN NOT NULL>";
+        + "`isTightBound` BOOLEAN NOT NULL, "
+        + "`valueType` ROW<`typeOrdinal` INT NOT NULL, `additionalInfo` STRING>>";
     assertThat(dataType.getChildren().get(pos).toString(), is(expected));
   }
 
