@@ -21,13 +21,14 @@ package org.apache.hudi.utils;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.streamer.FlinkStreamerConfig;
 import org.apache.hudi.util.AvroSchemaConverter;
+import org.apache.hudi.util.DataTypeUtils;
 import org.apache.hudi.utils.factory.CollectSinkTableFactory;
 import org.apache.hudi.utils.factory.ContinuousFileSourceFactory;
 
 import org.apache.flink.configuration.ConfigOption;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.table.api.DataTypes;
-import org.apache.flink.table.api.TableSchema;
+import org.apache.flink.table.api.Schema;
 import org.apache.flink.table.catalog.ResolvedSchema;
 import org.apache.flink.table.runtime.typeutils.RowDataSerializer;
 import org.apache.flink.table.types.DataType;
@@ -57,10 +58,25 @@ public class TestConfigurations {
 
   public static final RowType ROW_TYPE = (RowType) ROW_DATA_TYPE.getLogicalType();
 
+  public static final DataType ROW_DATA_TYPE_DECIMAL_ORDERING = DataTypes.ROW(
+          DataTypes.FIELD("uuid", DataTypes.VARCHAR(20)),// record key
+          DataTypes.FIELD("name", DataTypes.VARCHAR(10)),
+          DataTypes.FIELD("age", DataTypes.INT()),
+          DataTypes.FIELD("ts", DataTypes.DECIMAL(10, 0)), // precombine field
+          DataTypes.FIELD("partition", DataTypes.VARCHAR(10)))
+      .notNull();
+
+  public static final RowType ROW_TYPE_DECIMAL_ORDERING =
+      (RowType) ROW_DATA_TYPE_DECIMAL_ORDERING.getLogicalType();
+
   public static final ResolvedSchema TABLE_SCHEMA = SchemaBuilder.instance()
       .fields(ROW_TYPE.getFieldNames(), ROW_DATA_TYPE.getChildren())
       .build();
 
+  public static final ResolvedSchema TABLE_SCHEMA_DECIMAL_ORDERING = SchemaBuilder.instance()
+      .fields(ROW_TYPE_DECIMAL_ORDERING.getFieldNames(), ROW_DATA_TYPE_DECIMAL_ORDERING.getChildren())
+      .build();
+
   private static final List<String> FIELDS = ROW_TYPE.getFields().stream()
       .map(RowType.RowField::asSummaryString).collect(Collectors.toList());
 
@@ -244,6 +260,11 @@ public class TestConfigurations {
   }
 
   public static String getCollectSinkDDL(String tableName) {
+    // set expectedRowNum as -1 to disable forced exception to terminate a successful sink
+    return getCollectSinkDDLWithExpectedNum(tableName, -1);
+  }
+
+  public static String getCollectSinkDDLWithExpectedNum(String tableName, int expectedRowNum) {
     return "create table " + tableName + "(\n"
         + "  uuid varchar(20),\n"
         + "  name varchar(10),\n"
@@ -251,27 +272,33 @@ public class TestConfigurations {
         + "  ts timestamp(3),\n"
         + "  `partition` varchar(20)\n"
         + ") with (\n"
-        + "  'connector' = '" + CollectSinkTableFactory.FACTORY_ID + "'"
+        + "  'connector' = '" + CollectSinkTableFactory.FACTORY_ID + "',\n"
+        + "  'sink-expected-row-num' = '" + expectedRowNum + "'"
         + ")";
   }
 
-  public static String getCollectSinkDDL(String tableName, TableSchema tableSchema) {
+  public static String getCollectSinkDDL(String tableName, Schema schema) {
+    return getCollectSinkDDLWithExpectedNum(tableName, schema, -1);
+  }
+
+  public static String getCollectSinkDDLWithExpectedNum(String tableName, Schema schema, int expectRowNum) {
     final StringBuilder builder = new StringBuilder("create table " + tableName + "(\n");
-    String[] fieldNames = tableSchema.getFieldNames();
-    DataType[] fieldTypes = tableSchema.getFieldDataTypes();
-    for (int i = 0; i < fieldNames.length; i++) {
+    RowType rowType = DataTypeUtils.toRowType(schema);
+    List<RowType.RowField> fields = rowType.getFields();
+    for (int i = 0; i < rowType.getFieldCount(); i++) {
       builder.append("  `")
-          .append(fieldNames[i])
+          .append(fields.get(i).getName())
           .append("` ")
-          .append(fieldTypes[i].toString());
-      if (i != fieldNames.length - 1) {
+          .append(fields.get(i).getType().toString());
+      if (i != fields.size() - 1) {
         builder.append(",");
       }
       builder.append("\n");
     }
     final String withProps = ""
         + ") with (\n"
-        + "  'connector' = '" + CollectSinkTableFactory.FACTORY_ID + "'\n"
+        + "  'connector' = '" + CollectSinkTableFactory.FACTORY_ID + "',\n"
+        + "  'sink-expected-row-num' = '" + expectRowNum + "'"
         + ")";
     builder.append(withProps);
     return builder.toString();
@@ -295,23 +322,25 @@ public class TestConfigurations {
 
   public static final RowDataSerializer SERIALIZER = new RowDataSerializer(ROW_TYPE);
 
+  public static final RowDataSerializer SERIALIZER_DECIMAL_ORDERING = new RowDataSerializer(ROW_TYPE_DECIMAL_ORDERING);
+
   public static Configuration getDefaultConf(String tablePath) {
     Configuration conf = new Configuration();
-    conf.setString(FlinkOptions.PATH, tablePath);
-    conf.setString(FlinkOptions.SOURCE_AVRO_SCHEMA_PATH,
+    conf.set(FlinkOptions.PATH, tablePath);
+    conf.set(FlinkOptions.SOURCE_AVRO_SCHEMA_PATH,
         Objects.requireNonNull(Thread.currentThread()
             .getContextClassLoader().getResource("test_read_schema.avsc")).toString());
-    conf.setString(FlinkOptions.TABLE_NAME, "TestHoodieTable");
-    conf.setString(FlinkOptions.PARTITION_PATH_FIELD, "partition");
+    conf.set(FlinkOptions.TABLE_NAME, "TestHoodieTable");
+    conf.set(FlinkOptions.PARTITION_PATH_FIELD, "partition");
     return conf;
   }
 
   public static Configuration getDefaultConf(String tablePath, DataType dataType) {
     Configuration conf = new Configuration();
-    conf.setString(FlinkOptions.PATH, tablePath);
-    conf.setString(FlinkOptions.SOURCE_AVRO_SCHEMA, AvroSchemaConverter.convertToSchema(dataType.getLogicalType()).toString());
-    conf.setString(FlinkOptions.TABLE_NAME, "TestHoodieTable");
-    conf.setString(FlinkOptions.PARTITION_PATH_FIELD, "partition");
+    conf.set(FlinkOptions.PATH, tablePath);
+    conf.set(FlinkOptions.SOURCE_AVRO_SCHEMA, AvroSchemaConverter.convertToSchema(dataType.getLogicalType()).toString());
+    conf.set(FlinkOptions.TABLE_NAME, "TestHoodieTable");
+    conf.set(FlinkOptions.PARTITION_PATH_FIELD, "partition");
     return conf;
   }
 
@@ -400,6 +429,10 @@ public class TestConfigurations {
       return TestConfigurations.getCreateHoodieTableDDL(this.tableName, this.fields, options,
           this.withPartition, this.pkField, this.partitionField);
     }
+
+    public Map<String, String> getOptions() {
+      return this.options;
+    }
   }
 
   /**
