@@ -35,9 +35,14 @@ import org.apache.hudi.common.model.WriteConcurrencyMode;
 import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.HoodieTableVersion;
+import org.apache.hudi.common.table.marker.MarkerType;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.io.FileGroupReaderBasedMergeHandle;
+import org.apache.hudi.table.HoodieTable;
+import org.apache.hudi.table.marker.DirectWriteMarkers;
+import org.apache.hudi.table.marker.TimelineServerBasedWriteMarkers;
+import org.apache.hudi.table.marker.WriteMarkersFactory;
 import org.apache.hudi.util.FlinkWriteClients;
 import org.apache.hudi.util.StreamerUtil;
 
@@ -52,6 +57,7 @@ import java.io.File;
 
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
@@ -97,6 +103,31 @@ public class TestFlinkWriteClients {
     }
   }
 
+  @ParameterizedTest
+  @ValueSource(strings = {"", "DIRECT", "TIMELINE_SERVER_BASED"})
+  void testMarkerType(String markerType) throws Exception {
+    // create table
+    StreamerUtil.initTableIfNotExists(conf);
+    // This is expected to be used by the driver, the client can then send requests for files view.
+    FlinkWriteClients.createWriteClient(conf);
+
+    // do not set mark type to test the default behavior
+    if (!markerType.isEmpty()) {
+      conf.setString(HoodieWriteConfig.MARKERS_TYPE.key(), MarkerType.valueOf(markerType).name());
+    }
+    // This is expected to be used by writer client
+    HoodieWriteConfig writeConfig = FlinkWriteClients.getHoodieClientConfig(conf, false, true);
+    try (HoodieFlinkWriteClient writeClient = new HoodieFlinkWriteClient(HoodieFlinkEngineContext.DEFAULT, writeConfig)) {
+      HoodieTable table = writeClient.getHoodieTable();
+      String markerClass = WriteMarkersFactory.get(writeConfig.getMarkersType(), table, "001").getClass().getSimpleName();
+      if (markerType.isEmpty() || markerType.equals("DIRECT")) {
+        assertEquals(DirectWriteMarkers.class.getSimpleName(), markerClass);
+      } else {
+        assertEquals(TimelineServerBasedWriteMarkers.class.getSimpleName(), markerClass);
+      }
+    }
+  }
+
   @ParameterizedTest
   @ValueSource(booleans = {true, false})
   void testRecordMergeConfigForEventTimeOrdering(boolean useLegacyConfig) throws Exception {
