@@ -18,10 +18,20 @@
 
 package org.apache.hudi.adapter;
 
-import org.apache.flink.streaming.api.connector.sink2.SupportsPreWriteTopology;
+import org.apache.flink.streaming.api.datastream.DataStream;
 
 /**
- * Adapter clazz for {@link SupportsPreWriteTopology}.
+ * {@code SupportsPreWriteTopology} is introduced for Sink V2 since Flink 1.19,
+ * We add the adapter here to just make the compilation successful for earlier
+ * Flink versions (< 1.19).
  */
-public interface SupportsPreWriteTopologyAdapter<InputT> extends SupportsPreWriteTopology<InputT> {
+public interface SupportsPreWriteTopologyAdapter<InputT> {
+  /**
+   * Adds an arbitrary topology before the writer. The topology may be used to repartition the
+   * data.
+   *
+   * @param inputDataStream the stream of input records.
+   * @return the custom topology before {@code SinkWriter}.
+   */
+  DataStream<InputT> addPreWriteTopology(DataStream<InputT> inputDataStream);
 }
