@@ -21,11 +21,11 @@ package org.apache.hudi.table.format.cow.vector.reader;
 import org.apache.hudi.table.format.cow.vector.ParquetDecimalVector;
 
 import org.apache.flink.formats.parquet.vector.reader.ColumnReader;
-import org.apache.flink.table.data.ColumnarRowData;
 import org.apache.flink.table.data.RowData;
-import org.apache.flink.table.data.vector.ColumnVector;
-import org.apache.flink.table.data.vector.VectorizedColumnBatch;
-import org.apache.flink.table.data.vector.writable.WritableColumnVector;
+import org.apache.flink.table.data.columnar.ColumnarRowData;
+import org.apache.flink.table.data.columnar.vector.ColumnVector;
+import org.apache.flink.table.data.columnar.vector.VectorizedColumnBatch;
+import org.apache.flink.table.data.columnar.vector.writable.WritableColumnVector;
 import org.apache.flink.table.types.logical.LogicalType;
 import org.apache.flink.table.types.logical.LogicalTypeRoot;
 import org.apache.flink.util.FlinkRuntimeException;
@@ -292,10 +292,13 @@ public class ParquetColumnarRowSplitReader implements Closeable {
    */
   private boolean ensureBatch() throws IOException {
     if (nextRow >= rowsInBatch) {
-      // No more rows available in the Rows array.
-      nextRow = 0;
       // Try to read the next batch if rows from the file.
-      return nextBatch();
+      if (nextBatch()) {
+        // No more rows available in the Rows array.
+        nextRow = 0;
+        return true;
+      }
+      return false;
     }
     // there is at least one Row left in the Rows array.
     return true;
