@@ -19,7 +19,7 @@
 package org.apache.hudi.adapter;
 
 import org.apache.flink.api.common.typeutils.TypeSerializer;
-import org.apache.flink.streaming.api.legacy.io.CollectionInputFormat;
+import org.apache.flink.api.java.io.CollectionInputFormat;
 
 import java.util.Collection;
 
