@@ -20,7 +20,7 @@ package org.apache.hudi.adapter;
 
 import org.apache.flink.api.common.io.InputFormat;
 import org.apache.flink.api.common.typeinfo.TypeInformation;
-import org.apache.flink.streaming.api.functions.source.legacy.InputFormatSourceFunction;
+import org.apache.flink.streaming.api.functions.source.InputFormatSourceFunction;
 
 /**
  * Adapter clazz for {@link InputFormatSourceFunction}.
