@@ -19,7 +19,7 @@
 
 package org.apache.hudi.adapter;
 
-import org.apache.flink.table.connector.sink.legacy.SinkFunctionProvider;
+import org.apache.flink.table.connector.sink.SinkFunctionProvider;
 
 /**
  * Adapter clazz for {@link SinkFunctionProvider}.
