@@ -18,6 +18,8 @@
 
 package org.apache.hudi.adapter;
 
+import org.apache.flink.table.catalog.hive.util.Constants;
+
 /**
  * Constants for Hive Catalog.
  */
@@ -26,17 +28,17 @@ public class HiveCatalogConstants {
   // -----------------------------------------------------------------------------------
   //  Constants for ALTER DATABASE
   // -----------------------------------------------------------------------------------
-  public static final String ALTER_DATABASE_OP = "hive.alter.database.op";
+  public static final String ALTER_DATABASE_OP = Constants.ALTER_DATABASE_OP;
 
-  public static final String DATABASE_LOCATION_URI = "hive.database.location-uri";
+  public static final String DATABASE_LOCATION_URI = Constants.DATABASE_LOCATION_URI;
 
-  public static final String DATABASE_OWNER_NAME = "hive.database.owner.name";
+  public static final String DATABASE_OWNER_NAME = Constants.DATABASE_OWNER_NAME;
 
-  public static final String DATABASE_OWNER_TYPE = "hive.database.owner.type";
+  public static final String DATABASE_OWNER_TYPE = Constants.DATABASE_OWNER_TYPE;
 
-  public static final String ROLE_OWNER = "role";
+  public static final String ROLE_OWNER = Constants.ROLE_OWNER;
 
-  public static final String USER_OWNER = "user";
+  public static final String USER_OWNER = Constants.USER_OWNER;
 
   /** Type of ALTER DATABASE operation. */
   public enum AlterHiveDatabaseOp {
