@@ -18,7 +18,7 @@
 
 package org.apache.hudi.adapter;
 
-import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
+import org.apache.flink.streaming.api.functions.sink.legacy.RichSinkFunction;
 
 /**
  * Adapter clazz for {@link RichSinkFunction}.
