@@ -18,8 +18,8 @@
 
 package org.apache.hudi.adapter;
 
-import org.apache.flink.table.connector.source.TableFunctionProvider;
 import org.apache.flink.table.functions.TableFunction;
+import org.apache.flink.table.legacy.connector.source.TableFunctionProvider;
 
 /**
  * Adapter clazz for {@link TableFunctionProvider}.
