@@ -18,7 +18,7 @@
 
 package org.apache.hudi.adapter;
 
-import org.apache.flink.streaming.api.operators.YieldingOperatorFactory;
+import org.apache.flink.streaming.api.operators.legacy.YieldingOperatorFactory;
 
 /**
  * Adapter clazz for {@link YieldingOperatorFactory}.
