@@ -31,6 +31,6 @@ import java.util.Map;
  */
 public class CatalogUtils {
   public static CatalogTable createCatalogTable(Schema schema, List<String> partitionKeys, Map<String, String> options, @Nullable String comment) {
-    return CatalogTable.of(schema, comment, partitionKeys, options);
+    return CatalogTable.newBuilder().schema(schema).comment(comment).partitionKeys(partitionKeys).options(options).build();
   }
 }
