@@ -19,9 +19,9 @@
 package org.apache.hudi.utils;
 
 import org.apache.flink.api.common.typeinfo.TypeInformation;
+import org.apache.flink.table.legacy.types.logical.TypeInformationRawType;
 import org.apache.flink.table.types.AtomicDataType;
 import org.apache.flink.table.types.DataType;
-import org.apache.flink.table.types.logical.TypeInformationRawType;
 
 /**
  * Adapter utils to provide {@link DataType} utilities.
