@@ -19,13 +19,14 @@
 package org.apache.hudi.utils;
 
 import org.apache.flink.api.common.state.StateTtlConfig;
-import org.apache.flink.api.common.time.Time;
+
+import java.time.Duration;
 
 /**
  * Adapter utils to create {@link StateTtlConfig}.
  */
 public class StateTtlConfigUtils {
   public static StateTtlConfig createTtlConfig(long ttl) {
-    return StateTtlConfig.newBuilder(Time.milliseconds(ttl)).build();
+    return StateTtlConfig.newBuilder(Duration.ofMillis(ttl)).build();
   }
 }
