@@ -26,6 +26,6 @@ import org.apache.flink.runtime.executiongraph.ExecutionAttemptID;
 public class ExecutionAttemptUtil {
 
   public static ExecutionAttemptID randomId() {
-    return new ExecutionAttemptID();
+    return ExecutionAttemptID.randomId();
   }
 }
