@@ -27,6 +27,7 @@ import org.apache.avro.Schema;
 import org.apache.avro.Schema.Field;
 import org.apache.avro.generic.GenericData;
 import org.apache.avro.generic.GenericData.StringType;
+import org.apache.avro.generic.GenericRecord;
 import org.apache.avro.util.Utf8;
 import org.apache.hadoop.hive.common.type.HiveDecimal;
 import org.apache.hadoop.hive.ql.exec.vector.BytesColumnVector;
@@ -147,13 +148,13 @@ public class AvroOrcUtils {
           final Utf8 utf8 = (Utf8) value;
           bytes = utf8.getBytes();
         } else if (value instanceof GenericData.EnumSymbol) {
-          bytes = getUTF8Bytes(((GenericData.EnumSymbol) value).toString());
+          bytes = getUTF8Bytes(value.toString());
         } else {
           throw new IllegalStateException(String.format(
               "Unrecognized type for Avro %s field value, which has type %s, value %s",
               type.getCategory().getName(),
               value.getClass().getName(),
-              value.toString()
+              value
           ));
         }
 
@@ -177,7 +178,7 @@ public class AvroOrcUtils {
           throw new IllegalStateException(String.format(
               "Unrecognized type for Avro DATE field value, which has type %s, value %s",
               value.getClass().getName(),
-              value.toString()
+              value
           ));
         }
         dateColVec.vector[vectorPos] = daysSinceEpoch;
@@ -209,7 +210,7 @@ public class AvroOrcUtils {
           throw new IllegalStateException(String.format(
               "Unrecognized type for Avro TIMESTAMP field value, which has type %s, value %s",
               value.getClass().getName(),
-              value.toString()
+              value
           ));
         }
 
@@ -231,7 +232,7 @@ public class AvroOrcUtils {
           throw new IllegalStateException(String.format(
               "Unrecognized type for Avro BINARY field value, which has type %s, value %s",
               value.getClass().getName(),
-              value.toString()
+              value
           ));
         }
         binaryColVec.setRef(vectorPos, binaryBytes, 0, binaryBytes.length);
@@ -318,7 +319,7 @@ public class AvroOrcUtils {
       case STRUCT:
         StructColumnVector structColVec = (StructColumnVector) colVector;
 
-        GenericData.Record record = (GenericData.Record) value;
+        GenericRecord record = (GenericRecord) value;
 
         for (int i = 0; i < type.getFieldNames().size(); i++) {
           String fieldName = type.getFieldNames().get(i);
@@ -338,13 +339,13 @@ public class AvroOrcUtils {
           throw new IllegalStateException(String.format(
               "Failed to add value %s to union with type %s",
               value == null ? "null" : value.toString(),
-              type.toString()
+              type
           ));
         }
 
         break;
       default:
-        throw new IllegalArgumentException("Invalid TypeDescription " + type.toString() + ".");
+        throw new IllegalArgumentException("Invalid TypeDescription " + type + ".");
     }
   }
 
@@ -514,15 +515,23 @@ public class AvroOrcUtils {
         }
       case STRING:
         String stringType = avroSchema.getProp(GenericData.STRING_PROP);
+        Object parsedValue;
         if (stringType == null || !stringType.equals(StringType.String)) {
           int stringLength = ((BytesColumnVector) colVector).length[vectorPos];
           int stringOffset = ((BytesColumnVector) colVector).start[vectorPos];
           byte[] stringBytes = new byte[stringLength];
           System.arraycopy(((BytesColumnVector) colVector).vector[vectorPos], stringOffset, stringBytes, 0, stringLength);
-          return new Utf8(stringBytes);
+          parsedValue = new Utf8(stringBytes);
         } else {
-          return ((BytesColumnVector) colVector).toString(vectorPos);
+          parsedValue = ((BytesColumnVector) colVector).toString(vectorPos);
         }
+        if (avroSchema.getType() == Schema.Type.ENUM) {
+          String enumValue = parsedValue.toString();
+          if (!enumValue.isEmpty()) {
+            return new GenericData.EnumSymbol(avroSchema, enumValue);
+          }
+        }
+        return parsedValue;
       case DATE:
         // convert to daysSinceEpoch for LogicalType.Date
         return (int) ((LongColumnVector) colVector).vector[vectorPos];
@@ -554,7 +563,7 @@ public class AvroOrcUtils {
         if (baseType.equals(Schema.Type.FIXED)) {
           return new Conversions.DecimalConversion().toFixed(bigDecimal, avroSchema, logicalType);
         } else if (baseType.equals(Schema.Type.BYTES)) {
-          return bigDecimal.unscaledValue().toByteArray();
+          return ByteBuffer.wrap(bigDecimal.unscaledValue().toByteArray());
         } else {
           throw new HoodieIOException(baseType.getName() + "is not a valid type for LogicalTypes.DECIMAL.");
         }
@@ -598,7 +607,7 @@ public class AvroOrcUtils {
         ColumnVector fieldVector = unionVector.fields[tag];
         return readFromVector(type.getChildren().get(tag), fieldVector, avroSchema.getTypes().get(tag), vectorPos);
       default:
-        throw new HoodieIOException("Unrecognized TypeDescription " + type.toString());
+        throw new HoodieIOException("Unrecognized TypeDescription " + type);
     }
   }
 
@@ -811,7 +820,7 @@ public class AvroOrcUtils {
       if (nullable) {
         fields.add(new Schema.Field(field.name(), nullableSchema, null, NULL_VALUE));
       } else {
-        fields.add(new Schema.Field(field.name(), fieldSchema, null, (Object) null));
+        fields.add(new Schema.Field(field.name(), fieldSchema, null, null));
       }
     }
     Schema schema = Schema.createRecord(recordName, null, null, false);
