@@ -22,7 +22,6 @@ import org.apache.hudi.common.model.HoodieCommitMetadata;
 import org.apache.hudi.common.model.HoodieIndexDefinition;
 import org.apache.hudi.common.model.HoodieIndexMetadata;
 import org.apache.hudi.common.model.HoodieTableType;
-import org.apache.hudi.common.model.HoodieTimelineTimeZone;
 import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
@@ -30,6 +29,9 @@ import org.apache.hudi.common.testutils.HoodieCommonTestHarness;
 import org.apache.hudi.common.testutils.HoodieTestUtils;
 import org.apache.hudi.common.util.FileIOUtils;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.exception.HoodieIOException;
+import org.apache.hudi.metadata.HoodieIndexVersion;
+import org.apache.hudi.storage.HoodieInstantWriter;
 import org.apache.hudi.metadata.MetadataPartitionType;
 import org.apache.hudi.storage.StoragePath;
 
@@ -39,15 +41,13 @@ import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 
 import java.io.IOException;
+import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.Date;
 import java.util.HashMap;
-import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 
-import static org.apache.hudi.common.table.timeline.HoodieInstantTimeGenerator.MILLIS_INSTANT_TIME_FORMATTER;
 import static org.apache.hudi.common.testutils.HoodieTestUtils.INSTANT_GENERATOR;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
@@ -59,7 +59,7 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
 /**
  * Tests hoodie table meta client {@link HoodieTableMetaClient}.
  */
-public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
+class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
 
   @BeforeEach
   public void init() throws IOException {
@@ -72,7 +72,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void checkMetadata() {
+  void checkMetadata() {
     assertEquals(HoodieTestUtils.RAW_TRIPS_TEST_NAME, metaClient.getTableConfig().getTableName(),
         "Table name should be raw_trips");
     assertEquals(basePath, metaClient.getBasePath().toString(), "Basepath should be the one assigned");
@@ -84,7 +84,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testSerDe() throws IOException {
+  void testSerDe() throws IOException {
     // check if this object is serialized and de-serialized, we are able to read from the file system
     HoodieTableMetaClient deserializedMetaClient =
         HoodieTestUtils.serializeDeserialize(metaClient, HoodieTableMetaClient.class);
@@ -103,7 +103,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testCommitTimeline() throws IOException {
+  void testCommitTimeline() throws IOException {
     HoodieActiveTimeline activeTimeline = metaClient.getActiveTimeline();
     HoodieTimeline activeCommitTimeline = activeTimeline.getCommitAndReplaceTimeline();
     assertTrue(activeCommitTimeline.empty(), "Should be empty commit timeline");
@@ -129,39 +129,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testCreateNewInstantTimes() throws IOException {
-    List<String> instantTimesSoFar = new ArrayList<>();
-    // explicitly set timezone to UTC and generate timestamps
-    Properties properties = new Properties();
-    properties.setProperty(HoodieTableConfig.TIMELINE_TIMEZONE.key(), "UTC");
-    metaClient = HoodieTestUtils.init(metaClient.getStorageConf(), basePath, HoodieTableType.MERGE_ON_READ, properties);
-
-    // run for few iterations
-    for (int j = 0; j < 5; j++) {
-      instantTimesSoFar.clear();
-      // Generate an instant time in UTC and validate that all instants generated using metaClient are within few seconds apart.
-      String newCommitTimeInUTC = getNewInstantTimeInUTC();
-
-      // new instant that we generate below should be within few seconds apart compared to above time we generated. If not, the time zone is not honored
-      for (int i = 0; i < 10; i++) {
-        String newInstantTime = metaClient.createNewInstantTime(false);
-        assertTrue(!instantTimesSoFar.contains(newInstantTime));
-        instantTimesSoFar.add(newInstantTime);
-        assertTrue((Long.parseLong(newInstantTime) - Long.parseLong(newCommitTimeInUTC)) < 60000L,
-            String.format("Validation failed on new instant time created: %s, newCommitTimeInUTC=%s",
-                newInstantTime, newCommitTimeInUTC));
-      }
-    }
-  }
-
-  private String getNewInstantTimeInUTC() {
-    Date d = new Date(System.currentTimeMillis());
-    return d.toInstant().atZone(HoodieTimelineTimeZone.UTC.getZoneId())
-        .toLocalDateTime().format(MILLIS_INSTANT_TIME_FORMATTER);
-  }
-
-  @Test
-  public void testEquals() throws IOException {
+  void testEquals() throws IOException {
     HoodieTableMetaClient metaClient1 = HoodieTestUtils.init(tempDir.toAbsolutePath().toString(), getTableType());
     HoodieTableMetaClient metaClient2 = HoodieTestUtils.init(tempDir.toAbsolutePath().toString(), getTableType());
     assertEquals(metaClient1, metaClient2);
@@ -170,7 +138,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testToString() throws IOException {
+  void testToString() throws IOException {
     HoodieTableMetaClient metaClient1 = HoodieTestUtils.init(tempDir.toAbsolutePath().toString(), getTableType());
     HoodieTableMetaClient metaClient2 = HoodieTestUtils.init(tempDir.toAbsolutePath().toString(), getTableType());
     assertEquals(metaClient1.toString(), metaClient2.toString());
@@ -178,7 +146,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testTableVersion() throws IOException {
+  void testTableVersion() throws IOException {
     final String basePath = tempDir.toAbsolutePath() + Path.SEPARATOR + "t1";
     HoodieTableMetaClient metaClient1 = HoodieTableMetaClient.newTableBuilder()
         .setTableType(HoodieTableType.MERGE_ON_READ.name())
@@ -195,7 +163,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testGenerateFromAnotherMetaClient() throws IOException {
+  void testGenerateFromAnotherMetaClient() throws IOException {
     final String basePath1 = tempDir.toAbsolutePath().toString() + Path.SEPARATOR + "t2A";
     final String basePath2 = tempDir.toAbsolutePath().toString() + Path.SEPARATOR + "t2B";
 
@@ -215,7 +183,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testTableBuilderRequiresTableNameAndType() {
+  void testTableBuilderRequiresTableNameAndType() {
     assertThrows(IllegalArgumentException.class, () -> {
       HoodieTableMetaClient.builder()
           .setConf(this.metaClient.getStorageConf())
@@ -234,12 +202,12 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testCreateMetaClientFromProperties() throws IOException {
+  void testCreateMetaClientFromProperties() throws IOException {
     final String basePath = tempDir.toAbsolutePath().toString() + Path.SEPARATOR + "t5";
     Properties props = new Properties();
     props.setProperty(HoodieTableConfig.NAME.key(), "test-table");
     props.setProperty(HoodieTableConfig.TYPE.key(), HoodieTableType.COPY_ON_WRITE.name());
-    props.setProperty(HoodieTableConfig.PRECOMBINE_FIELD.key(), "timestamp");
+    props.setProperty(HoodieTableConfig.ORDERING_FIELDS.key(), "timestamp");
 
     HoodieTableMetaClient metaClient1 = HoodieTableMetaClient.newTableBuilder()
         .fromProperties(props)
@@ -253,13 +221,13 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
     // test table name and type and precombine field also match
     assertEquals(metaClient1.getTableConfig().getTableName(), metaClient2.getTableConfig().getTableName());
     assertEquals(metaClient1.getTableConfig().getTableType(), metaClient2.getTableConfig().getTableType());
-    assertEquals(metaClient1.getTableConfig().getPreCombineField(), metaClient2.getTableConfig().getPreCombineField());
+    assertEquals(metaClient1.getTableConfig().getOrderingFields(), metaClient2.getTableConfig().getOrderingFields());
     // default table version should be current version
     assertEquals(HoodieTableVersion.current(), metaClient2.getTableConfig().getTableVersion());
   }
 
   @Test
-  public void testCreateLayoutInStorage() throws IOException {
+  void testCreateLayoutInStorage() throws IOException {
     final String basePath = tempDir.toAbsolutePath().toString() + Path.SEPARATOR + "t6";
     HoodieTableMetaClient metaClient1 = HoodieTableMetaClient.newTableBuilder()
         .setTableType(HoodieTableType.COPY_ON_WRITE.name())
@@ -276,7 +244,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testGetIndexDefinitionPath() throws IOException {
+  void testGetIndexDefinitionPath() throws IOException {
     final String basePath = tempDir.toAbsolutePath() + Path.SEPARATOR + "t7";
     HoodieTableMetaClient metaClient = HoodieTableMetaClient.newTableBuilder()
         .setTableType(HoodieTableType.COPY_ON_WRITE.name())
@@ -290,7 +258,7 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
   }
 
   @Test
-  public void testDeleteDefinition() throws IOException {
+  void testDeleteDefinition() throws IOException {
     final String basePath = tempDir.toAbsolutePath() + Path.SEPARATOR + "t7";
     HoodieTableMetaClient metaClient = HoodieTableMetaClient.newTableBuilder()
         .setTableType(HoodieTableType.COPY_ON_WRITE.name())
@@ -303,18 +271,111 @@ public class TestHoodieTableMetaClient extends HoodieCommonTestHarness {
         .withIndexName(indexName)
         .withIndexType("column_stats")
         .withIndexFunction("identity")
+        .withVersion(HoodieIndexVersion.getCurrentVersion(HoodieTableVersion.current(), indexName))
         .withSourceFields(new ArrayList<>(columnsMap.keySet()))
         .withIndexOptions(Collections.emptyMap())
         .build();
     metaClient.buildIndexDefinition(indexDefinition);
-    assertTrue(metaClient.getIndexMetadata().get().getIndexDefinitions().containsKey(indexName));
+    assertTrue(metaClient.getIndexMetadata().isPresent());
+    assertTrue(metaClient.getIndexForMetadataPartition(indexName).isPresent());
     assertTrue(metaClient.getStorage().exists(new StoragePath(metaClient.getIndexDefinitionPath())));
     metaClient.deleteIndexDefinition(indexName);
-    assertTrue(metaClient.getIndexMetadata().isEmpty());
+    assertFalse(metaClient.getIndexMetadata().isPresent());
     assertTrue(metaClient.getStorage().exists(new StoragePath(metaClient.getIndexDefinitionPath())));
     // Read from storage
     HoodieIndexMetadata indexMetadata = HoodieIndexMetadata.fromJson(
         new String(FileIOUtils.readDataFromPath(metaClient.getStorage(), new StoragePath(metaClient.getIndexDefinitionPath())).get()));
     assertTrue(indexMetadata.getIndexDefinitions().isEmpty());
   }
+
+  @Test
+  void testReadIndexDefFromStorage() throws Exception {
+    final String basePath = tempDir.toAbsolutePath() + Path.SEPARATOR + "t8";
+
+    // No index definition path configured - should return empty
+    HoodieTableMetaClient metaClient = HoodieTableMetaClient.newTableBuilder()
+        .setTableType(HoodieTableType.COPY_ON_WRITE.name())
+        .setTableName("table")
+        .initTable(this.metaClient.getStorageConf(), basePath);
+
+    Method readIndexDefMethod = HoodieTableMetaClient.class
+        .getDeclaredMethod("readIndexDefFromStorage",
+            org.apache.hudi.storage.HoodieStorage.class,
+            StoragePath.class,
+            HoodieTableConfig.class);
+    readIndexDefMethod.setAccessible(true);
+
+    @SuppressWarnings("unchecked")
+    Option<HoodieIndexMetadata> result = (Option<HoodieIndexMetadata>) readIndexDefMethod.invoke(
+        null, metaClient.getStorage(), metaClient.getBasePath(), metaClient.getTableConfig());
+    assertTrue(result.isEmpty(), "Should return empty when no index definition path is configured");
+
+    // Empty index definition path - should return empty
+    metaClient.getTableConfig().setValue(HoodieTableConfig.RELATIVE_INDEX_DEFINITION_PATH.key(), "");
+    @SuppressWarnings("unchecked")
+    Option<HoodieIndexMetadata> result2 = (Option<HoodieIndexMetadata>) readIndexDefMethod.invoke(
+        null, metaClient.getStorage(), metaClient.getBasePath(), metaClient.getTableConfig());
+    assertTrue(result2.isEmpty(), "Should return empty when index definition path is empty string");
+
+    // Valid path but file doesn't exist - should return empty HoodieIndexMetadata
+    String relativePath = ".hoodie/.index_defs/index.json";
+    metaClient.getTableConfig().setValue(HoodieTableConfig.RELATIVE_INDEX_DEFINITION_PATH.key(), relativePath);
+    @SuppressWarnings("unchecked")
+    Option<HoodieIndexMetadata> result3 = (Option<HoodieIndexMetadata>) readIndexDefMethod.invoke(
+        null, metaClient.getStorage(), metaClient.getBasePath(), metaClient.getTableConfig());
+    assertTrue(result3.isPresent(), "Should return present Option when path is configured but file doesn't exist");
+    assertTrue(result3.get().getIndexDefinitions().isEmpty(), "Should return empty HoodieIndexMetadata when file doesn't exist");
+
+    // Valid path with existing empty file - should return empty HoodieIndexMetadata
+    StoragePath indexPath = new StoragePath(metaClient.getBasePath(), relativePath);
+    FileIOUtils.createFileInPath(metaClient.getStorage(), indexPath,
+        Option.of(HoodieInstantWriter.convertByteArrayToWriter("{}".getBytes())));
+    @SuppressWarnings("unchecked")
+    Option<HoodieIndexMetadata> result4 = (Option<HoodieIndexMetadata>) readIndexDefMethod.invoke(
+        null, metaClient.getStorage(), metaClient.getBasePath(), metaClient.getTableConfig());
+    assertTrue(result4.isPresent(), "Should return present Option when file exists");
+    assertTrue(result4.get().getIndexDefinitions().isEmpty(), "Should return empty HoodieIndexMetadata for empty file");
+
+    // Valid path with valid index metadata - should return populated HoodieIndexMetadata
+    Map<String, Map<String, String>> columnsMap = new HashMap<>();
+    columnsMap.put("c1", Collections.emptyMap());
+    String indexName = MetadataPartitionType.EXPRESSION_INDEX.getPartitionPath() + "test_idx";
+    HoodieIndexDefinition indexDefinition = HoodieIndexDefinition.newBuilder()
+        .withIndexName(indexName)
+        .withIndexType("column_stats")
+        .withIndexFunction("identity")
+        .withVersion(HoodieIndexVersion.getCurrentVersion(HoodieTableVersion.current(), indexName))
+        .withSourceFields(new ArrayList<>(columnsMap.keySet()))
+        .withIndexOptions(Collections.emptyMap())
+        .build();
+
+    Map<String, HoodieIndexDefinition> indexDefMap = new HashMap<>();
+    indexDefMap.put(indexName, indexDefinition);
+    HoodieIndexMetadata validIndexMetadata = new HoodieIndexMetadata(indexDefMap);
+
+    FileIOUtils.createFileInPath(metaClient.getStorage(), indexPath,
+        Option.of(HoodieInstantWriter.convertByteArrayToWriter(validIndexMetadata.toJson().getBytes())));
+    @SuppressWarnings("unchecked")
+    Option<HoodieIndexMetadata> result5 = (Option<HoodieIndexMetadata>) readIndexDefMethod.invoke(
+        null, metaClient.getStorage(), metaClient.getBasePath(), metaClient.getTableConfig());
+    assertTrue(result5.isPresent(), "Should return present Option when valid file exists");
+    assertFalse(result5.get().getIndexDefinitions().isEmpty(), "Should return populated HoodieIndexMetadata");
+    assertEquals(1, result5.get().getIndexDefinitions().size(), "Should have one index definition");
+    assertTrue(result5.get().getIndexDefinitions().containsKey(indexName), "Should contain the test index");
+    assertEquals("column_stats", result5.get().getIndexDefinitions().get(indexName).getIndexType(), "Index type should match");
+
+    // Invalid JSON file - should throw HoodieIOException
+    FileIOUtils.createFileInPath(metaClient.getStorage(), indexPath,
+        Option.of(HoodieInstantWriter.convertByteArrayToWriter("invalid json".getBytes())));
+    assertThrows(HoodieIOException.class, () -> {
+      try {
+        readIndexDefMethod.invoke(null, metaClient.getStorage(), metaClient.getBasePath(), metaClient.getTableConfig());
+      } catch (java.lang.reflect.InvocationTargetException e) {
+        if (e.getCause() instanceof HoodieIOException) {
+          throw (HoodieIOException) e.getCause();
+        }
+        throw new RuntimeException(e);
+      }
+    }, "Should throw HoodieIOException for invalid JSON");
+  }
 }
