@@ -19,15 +19,12 @@
 
 package org.apache.hudi.common.table.read;
 
+import org.apache.hudi.avro.HoodieAvroReaderContext;
 import org.apache.hudi.common.config.RecordMergeMode;
-import org.apache.hudi.common.model.HoodieAvroRecordMerger;
 import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.testutils.HoodieTestTable;
-import org.apache.hudi.common.testutils.reader.HoodieAvroRecordTestMerger;
 import org.apache.hudi.common.testutils.reader.HoodieFileGroupReaderTestHarness;
 import org.apache.hudi.common.testutils.reader.HoodieFileSliceTestUtils;
-import org.apache.hudi.common.testutils.reader.HoodieRecordTestPayload;
-import org.apache.hudi.common.testutils.reader.HoodieTestReaderContext;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.ClosableIterator;
 
@@ -47,6 +44,7 @@ import java.util.List;
 import java.util.Properties;
 import java.util.stream.Stream;
 
+import static org.apache.hudi.common.table.HoodieTableConfig.ORDERING_FIELDS;
 import static org.apache.hudi.common.testutils.HoodieTestDataGenerator.AVRO_SCHEMA;
 import static org.apache.hudi.common.testutils.reader.DataGenerationPlan.OperationType.DELETE;
 import static org.apache.hudi.common.testutils.reader.DataGenerationPlan.OperationType.INSERT;
@@ -60,17 +58,12 @@ public class TestEventTimeMerging extends HoodieFileGroupReaderTestHarness {
   protected Properties getMetaProps() {
     Properties metaProps =  super.getMetaProps();
     metaProps.setProperty(HoodieTableConfig.RECORD_MERGE_MODE.key(), RecordMergeMode.EVENT_TIME_ORDERING.name());
+    metaProps.setProperty(ORDERING_FIELDS.key(), "timestamp");
     return metaProps;
   }
 
   @BeforeAll
   public static void setUp() throws IOException {
-    // Create dedicated merger to avoid current delete logic holes.
-    // TODO: Unify delete logic (HUDI-7240).
-    HoodieAvroRecordMerger merger = new HoodieAvroRecordTestMerger();
-    readerContext = new HoodieTestReaderContext(
-        Option.of(merger),
-        Option.of(HoodieRecordTestPayload.class.getName()));
     properties.setProperty(
         "hoodie.write.record.merge.mode", RecordMergeMode.EVENT_TIME_ORDERING.name());
 
@@ -113,7 +106,6 @@ public class TestEventTimeMerging extends HoodieFileGroupReaderTestHarness {
     // Specify the instant time for each file.
     instantTimes = Arrays.asList(
         "001", "002", "003", "004", "005");
-    shouldWritePositions = Arrays.asList(false, false, false, false, false);
   }
 
   @BeforeEach
@@ -123,6 +115,10 @@ public class TestEventTimeMerging extends HoodieFileGroupReaderTestHarness {
     initMetaClient();
     initTestDataGenerator(new String[]{PARTITION_PATH});
     testTable = HoodieTestTable.of(metaClient);
+    // Create dedicated merger to avoid current delete logic holes.
+    // TODO: Unify delete logic (HUDI-7240).
+    readerContext = new HoodieAvroReaderContext(
+        storageConf, metaClient.getTableConfig(), Option.empty(), Option.empty());
     setUpMockCommits();
   }
 
@@ -131,18 +127,19 @@ public class TestEventTimeMerging extends HoodieFileGroupReaderTestHarness {
   public void testWithOneLogFile(boolean useRecordPositions) throws IOException, InterruptedException {
     shouldWritePositions = Arrays.asList(useRecordPositions, useRecordPositions);
     // The FileSlice contains a base file and a log file.
-    ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(2, useRecordPositions);
-    List<String> leftKeysExpected = Arrays.asList("6", "7", "8", "9", "10");
-    List<Long> leftTimestampsExpected = Arrays.asList(2L, 2L, 2L, 2L, 2L);
-    List<String> leftKeysActual = new ArrayList<>();
-    List<Long> leftTimestampsActual = new ArrayList<>();
-    while (iterator.hasNext()) {
-      IndexedRecord record = iterator.next();
-      leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
-      leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+    try (ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(2, useRecordPositions)) {
+      List<String> leftKeysExpected = Arrays.asList("6", "7", "8", "9", "10");
+      List<Long> leftTimestampsExpected = Arrays.asList(2L, 2L, 2L, 2L, 2L);
+      List<String> leftKeysActual = new ArrayList<>();
+      List<Long> leftTimestampsActual = new ArrayList<>();
+      while (iterator.hasNext()) {
+        IndexedRecord record = iterator.next();
+        leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
+        leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+      }
+      assertEquals(leftKeysExpected, leftKeysActual);
+      assertEquals(leftTimestampsExpected, leftTimestampsActual);
     }
-    assertEquals(leftKeysExpected, leftKeysActual);
-    assertEquals(leftTimestampsExpected, leftTimestampsActual);
   }
 
   @ParameterizedTest
@@ -150,18 +147,19 @@ public class TestEventTimeMerging extends HoodieFileGroupReaderTestHarness {
   public void testWithTwoLogFiles(boolean useRecordPositions) throws IOException, InterruptedException {
     shouldWritePositions = Arrays.asList(useRecordPositions, useRecordPositions, useRecordPositions);
     // The FileSlice contains a base file and two log files.
-    ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(3, useRecordPositions);
-    List<String> leftKeysExpected = Arrays.asList("6", "7", "8", "9", "10");
-    List<Long> leftTimestampsExpected = Arrays.asList(2L, 2L, 2L, 2L, 2L);
-    List<String> leftKeysActual = new ArrayList<>();
-    List<Long> leftTimestampsActual = new ArrayList<>();
-    while (iterator.hasNext()) {
-      IndexedRecord record = iterator.next();
-      leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
-      leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+    try (ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(3, useRecordPositions)) {
+      List<String> leftKeysExpected = Arrays.asList("6", "7", "8", "9", "10");
+      List<Long> leftTimestampsExpected = Arrays.asList(2L, 2L, 2L, 2L, 2L);
+      List<String> leftKeysActual = new ArrayList<>();
+      List<Long> leftTimestampsActual = new ArrayList<>();
+      while (iterator.hasNext()) {
+        IndexedRecord record = iterator.next();
+        leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
+        leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+      }
+      assertEquals(leftKeysExpected, leftKeysActual);
+      assertEquals(leftTimestampsExpected, leftTimestampsActual);
     }
-    assertEquals(leftKeysExpected, leftKeysActual);
-    assertEquals(leftTimestampsExpected, leftTimestampsActual);
   }
 
   @ParameterizedTest
@@ -169,35 +167,38 @@ public class TestEventTimeMerging extends HoodieFileGroupReaderTestHarness {
   public void testWithThreeLogFiles(boolean useRecordPositions) throws IOException, InterruptedException {
     shouldWritePositions = Arrays.asList(useRecordPositions, useRecordPositions, useRecordPositions, useRecordPositions);
     // The FileSlice contains a base file and three log files.
-    ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(4, useRecordPositions);
-    List<String> leftKeysExpected = Arrays.asList("6", "7", "8", "9", "10");
-    List<Long> leftTimestampsExpected = Arrays.asList(2L, 2L, 2L, 2L, 2L);
-    List<String> leftKeysActual = new ArrayList<>();
-    List<Long> leftTimestampsActual = new ArrayList<>();
-    while (iterator.hasNext()) {
-      IndexedRecord record = iterator.next();
-      leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
-      leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+    try (ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(4, useRecordPositions)) {
+      List<String> leftKeysExpected = Arrays.asList("6", "7", "8", "9", "10");
+      List<Long> leftTimestampsExpected = Arrays.asList(2L, 2L, 2L, 2L, 2L);
+      List<String> leftKeysActual = new ArrayList<>();
+      List<Long> leftTimestampsActual = new ArrayList<>();
+      while (iterator.hasNext()) {
+        IndexedRecord record = iterator.next();
+        leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
+        leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+      }
+      assertEquals(leftKeysExpected, leftKeysActual);
+      assertEquals(leftTimestampsExpected, leftTimestampsActual);
     }
-    assertEquals(leftKeysExpected, leftKeysActual);
-    assertEquals(leftTimestampsExpected, leftTimestampsActual);
   }
 
   @Test
   public void testWithFourLogFiles() throws IOException, InterruptedException {
+    shouldWritePositions = Arrays.asList(false, false, false, false, false);
     // The FileSlice contains a base file and three log files.
-    ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(5);
-    List<String> leftKeysExpected = Arrays.asList("1", "2", "6", "7", "8", "9", "10");
-    List<Long> leftTimestampsExpected = Arrays.asList(4L, 4L, 2L, 2L, 2L, 2L, 2L);
-    List<String> leftKeysActual = new ArrayList<>();
-    List<Long> leftTimestampsActual = new ArrayList<>();
-    while (iterator.hasNext()) {
-      IndexedRecord record = iterator.next();
-      leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
-      leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+    try (ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(5)) {
+      List<String> leftKeysExpected = Arrays.asList("1", "2", "6", "7", "8", "9", "10");
+      List<Long> leftTimestampsExpected = Arrays.asList(4L, 4L, 2L, 2L, 2L, 2L, 2L);
+      List<String> leftKeysActual = new ArrayList<>();
+      List<Long> leftTimestampsActual = new ArrayList<>();
+      while (iterator.hasNext()) {
+        IndexedRecord record = iterator.next();
+        leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
+        leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+      }
+      assertEquals(leftKeysExpected, leftKeysActual);
+      assertEquals(leftTimestampsExpected, leftTimestampsActual);
     }
-    assertEquals(leftKeysExpected, leftKeysActual);
-    assertEquals(leftTimestampsExpected, leftTimestampsActual);
   }
 
   @ParameterizedTest
@@ -206,18 +207,19 @@ public class TestEventTimeMerging extends HoodieFileGroupReaderTestHarness {
                                         boolean log3haspositions, boolean log4haspositions) throws IOException, InterruptedException {
     shouldWritePositions = Arrays.asList(true, log1haspositions, log2haspositions, log3haspositions, log4haspositions);
     // The FileSlice contains a base file and three log files.
-    ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(5, true);
-    List<String> leftKeysExpected = Arrays.asList("1", "2", "6", "7", "8", "9", "10");
-    List<Long> leftTimestampsExpected = Arrays.asList(4L, 4L, 2L, 2L, 2L, 2L, 2L);
-    List<String> leftKeysActual = new ArrayList<>();
-    List<Long> leftTimestampsActual = new ArrayList<>();
-    while (iterator.hasNext()) {
-      IndexedRecord record = iterator.next();
-      leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
-      leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+    try (ClosableIterator<IndexedRecord> iterator = getFileGroupIterator(5, true)) {
+      List<String> leftKeysExpected = Arrays.asList("1", "2", "6", "7", "8", "9", "10");
+      List<Long> leftTimestampsExpected = Arrays.asList(4L, 4L, 2L, 2L, 2L, 2L, 2L);
+      List<String> leftKeysActual = new ArrayList<>();
+      List<Long> leftTimestampsActual = new ArrayList<>();
+      while (iterator.hasNext()) {
+        IndexedRecord record = iterator.next();
+        leftKeysActual.add(record.get(AVRO_SCHEMA.getField(ROW_KEY).pos()).toString());
+        leftTimestampsActual.add((Long) record.get(AVRO_SCHEMA.getField("timestamp").pos()));
+      }
+      assertEquals(leftKeysExpected, leftKeysActual);
+      assertEquals(leftTimestampsExpected, leftTimestampsActual);
     }
-    assertEquals(leftKeysExpected, leftKeysActual);
-    assertEquals(leftTimestampsExpected, leftTimestampsActual);
   }
 
   //generate all possible combos of 4 booleans
