@@ -19,16 +19,20 @@
 
 package org.apache.hudi.common.testutils.reader;
 
+import org.apache.hudi.common.config.HoodieCommonConfig;
+import org.apache.hudi.common.config.HoodieMemoryConfig;
 import org.apache.hudi.common.config.TypedProperties;
 import org.apache.hudi.common.engine.HoodieReaderContext;
 import org.apache.hudi.common.model.FileSlice;
 import org.apache.hudi.common.model.HoodieTableType;
+import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.read.HoodieFileGroupReader;
 import org.apache.hudi.common.testutils.HoodieCommonTestHarness;
 import org.apache.hudi.common.testutils.HoodieTestTable;
 import org.apache.hudi.common.testutils.HoodieTestUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.ClosableIterator;
+import org.apache.hudi.common.util.collection.ExternalSpillableMap;
 import org.apache.hudi.storage.HoodieStorage;
 import org.apache.hudi.storage.StorageConfiguration;
 import org.apache.hudi.storage.hadoop.HoodieHadoopStorage;
@@ -42,8 +46,10 @@ import java.util.List;
 import java.util.Properties;
 
 import static org.apache.hudi.common.table.HoodieTableConfig.POPULATE_META_FIELDS;
+import static org.apache.hudi.common.table.HoodieTableConfig.RECORDKEY_FIELDS;
 import static org.apache.hudi.common.testutils.HoodieTestDataGenerator.AVRO_SCHEMA;
 import static org.apache.hudi.common.testutils.HoodieTestUtils.getDefaultStorageConf;
+import static org.apache.hudi.common.testutils.reader.HoodieFileSliceTestUtils.ROW_KEY;
 
 public class HoodieFileGroupReaderTestHarness extends HoodieCommonTestHarness {
   protected static final String PARTITION_PATH = "any-partition-path";
@@ -56,23 +62,13 @@ public class HoodieFileGroupReaderTestHarness extends HoodieCommonTestHarness {
   protected static List<DataGenerationPlan.OperationType> operationTypes;
   // Set the instantTime for each record set.
   protected static List<String> instantTimes;
-  protected static List<Boolean> shouldWritePositions;
+  protected List<Boolean> shouldWritePositions;
 
   // Environmental variables.
-  protected static StorageConfiguration<?> storageConf;
+  protected static StorageConfiguration<?> storageConf = getDefaultStorageConf();
   protected static HoodieTestTable testTable;
-  protected static HoodieReaderContext<IndexedRecord> readerContext;
-  protected static TypedProperties properties;
-
-  static {
-    // Note: Make `timestamp` as ordering field.
-    properties = new TypedProperties();
-    properties.setProperty(
-        "hoodie.datasource.write.precombine.field", "timestamp");
-    storageConf = getDefaultStorageConf();
-    readerContext = new HoodieTestReaderContext(
-        Option.empty(), Option.empty());
-  }
+  protected static TypedProperties properties = new TypedProperties();
+  protected HoodieReaderContext<IndexedRecord> readerContext;
 
   @AfterAll
   public static void tearDown() throws IOException {
@@ -95,6 +91,7 @@ public class HoodieFileGroupReaderTestHarness extends HoodieCommonTestHarness {
   protected void initMetaClient() throws IOException {
     Properties metaProps = getMetaProps();
     metaProps.setProperty(POPULATE_META_FIELDS.key(), "false");
+    metaProps.setProperty(RECORDKEY_FIELDS.key(), ROW_KEY);
     if (basePath == null) {
       initPath();
     }
@@ -136,22 +133,22 @@ public class HoodieFileGroupReaderTestHarness extends HoodieCommonTestHarness {
             FILE_ID
         );
 
-    HoodieFileGroupReader<IndexedRecord> fileGroupReader =
-        HoodieFileGroupReaderTestUtils.createFileGroupReader(
-            fileSliceOpt,
-            basePath,
-            "1000", // Not used internally.
-            AVRO_SCHEMA,
-            shouldReadPositions,
-            0L,
-            Long.MAX_VALUE,
-            properties,
-            hoodieStorage,
-            readerContext,
-            metaClient,
-            allowInflightCommits);
+    properties.setProperty(HoodieMemoryConfig.MAX_MEMORY_FOR_MERGE.key(),String.valueOf(1024 * 1024 * 1000));
+    properties.setProperty(HoodieMemoryConfig.SPILLABLE_MAP_BASE_PATH.key(),  basePath + "/" + HoodieTableMetaClient.TEMPFOLDER_NAME);
+    properties.setProperty(HoodieCommonConfig.SPILLABLE_DISK_MAP_TYPE.key(), ExternalSpillableMap.DiskMapType.ROCKS_DB.name());
+    properties.setProperty(HoodieCommonConfig.DISK_MAP_BITCASK_COMPRESSION_ENABLED.key(), "false");
+    HoodieFileGroupReader<IndexedRecord> fileGroupReader = HoodieFileGroupReader.<IndexedRecord>newBuilder()
+        .withReaderContext(readerContext)
+        .withHoodieTableMetaClient(metaClient)
+        .withLatestCommitTime("1000") // Not used internally.
+        .withFileSlice(fileSliceOpt.orElseThrow(() -> new IllegalArgumentException("FileSlice is not present")))
+        .withDataSchema(AVRO_SCHEMA)
+        .withRequestedSchema(AVRO_SCHEMA)
+        .withProps(properties)
+        .withShouldUseRecordPosition(shouldReadPositions)
+        .withAllowInflightInstants(allowInflightCommits)
+        .build();
 
-    fileGroupReader.initRecordIterators();
     return fileGroupReader.getClosableIterator();
   }
 }
