@@ -18,11 +18,12 @@
 
 package org.apache.hudi.metadata;
 
-import org.apache.hudi.common.model.HoodieColumnRangeMetadata;
 import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.common.testutils.HoodieCommonTestHarness;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
+import org.apache.hudi.stats.HoodieColumnRangeMetadata;
+import org.apache.hudi.stats.ValueMetadata;
 
 import org.apache.avro.generic.IndexedRecord;
 import org.junit.jupiter.api.Test;
@@ -35,6 +36,7 @@ import java.util.List;
 import java.util.Map;
 
 import static org.apache.hudi.common.util.CollectionUtils.createImmutableMap;
+import static org.apache.hudi.metadata.HoodieIndexVersion.V1;
 import static org.apache.hudi.metadata.HoodieMetadataPayload.SECONDARY_INDEX_RECORD_KEY_SEPARATOR;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
@@ -184,7 +186,7 @@ public class TestHoodieMetadataPayload extends HoodieCommonTestHarness {
     String targetColName = "c1";
 
     HoodieColumnRangeMetadata<Comparable> c1Metadata =
-        HoodieColumnRangeMetadata.<Comparable>create(fileName, targetColName, 100, 1000, 5, 1000, 123456, 123456);
+        HoodieColumnRangeMetadata.<Comparable>create(fileName, targetColName, 100, 1000, 5, 1000, 123456, 123456, ValueMetadata.V1EmptyMetadata.get());
 
     HoodieRecord<HoodieMetadataPayload> columnStatsRecord =
         HoodieMetadataPayload.createColumnStatsRecords(PARTITION_NAME, Collections.singletonList(c1Metadata), false)
@@ -197,7 +199,7 @@ public class TestHoodieMetadataPayload extends HoodieCommonTestHarness {
     // NOTE: Column Stats record will only be merged in case existing file will be modified,
     //       which could only happen on storages schemes supporting appends
     HoodieColumnRangeMetadata<Comparable> c1AppendedBlockMetadata =
-        HoodieColumnRangeMetadata.<Comparable>create(fileName, targetColName, 0, 500, 0, 100, 12345, 12345);
+        HoodieColumnRangeMetadata.<Comparable>create(fileName, targetColName, 0, 500, 0, 100, 12345, 12345, ValueMetadata.V1EmptyMetadata.get());
 
     HoodieRecord<HoodieMetadataPayload> updatedColumnStatsRecord =
         HoodieMetadataPayload.createColumnStatsRecords(PARTITION_NAME, Collections.singletonList(c1AppendedBlockMetadata), false)
@@ -207,7 +209,7 @@ public class TestHoodieMetadataPayload extends HoodieCommonTestHarness {
         columnStatsRecord.getData().preCombine(updatedColumnStatsRecord.getData());
 
     HoodieColumnRangeMetadata<Comparable> expectedColumnRangeMetadata =
-        HoodieColumnRangeMetadata.<Comparable>create(fileName, targetColName, 0, 1000, 5, 1100, 135801, 135801);
+        HoodieColumnRangeMetadata.<Comparable>create(fileName, targetColName, 0, 1000, 5, 1100, 135801, 135801, ValueMetadata.V1EmptyMetadata.get());
 
     HoodieRecord<HoodieMetadataPayload> expectedColumnStatsRecord =
         HoodieMetadataPayload.createColumnStatsRecords(PARTITION_NAME, Collections.singletonList(expectedColumnRangeMetadata), false)
@@ -227,7 +229,7 @@ public class TestHoodieMetadataPayload extends HoodieCommonTestHarness {
     ////////////////////////////////////////////////////////////////////////
 
     HoodieColumnRangeMetadata<Comparable> c1StubbedMetadata =
-        HoodieColumnRangeMetadata.<Comparable>stub(fileName, targetColName);
+        HoodieColumnRangeMetadata.<Comparable>stub(fileName, targetColName, V1);
 
     HoodieRecord<HoodieMetadataPayload> deletedColumnStatsRecord =
         HoodieMetadataPayload.createColumnStatsRecords(PARTITION_NAME, Collections.singletonList(c1StubbedMetadata), true)
@@ -253,17 +255,17 @@ public class TestHoodieMetadataPayload extends HoodieCommonTestHarness {
   @Test
   public void testPartitionStatsPayloadMerging() {
     HoodieColumnRangeMetadata<Comparable> fileColumnRange1 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file", "columnName", 1, 5, 0, 10, 100, 200);
+        "path/to/file", "columnName", 1, 5, 0, 10, 100, 200, ValueMetadata.V1EmptyMetadata.get());
     HoodieRecord<HoodieMetadataPayload> firstPartitionStatsRecord =
         HoodieMetadataPayload.createPartitionStatsRecords(PARTITION_NAME, Collections.singletonList(fileColumnRange1), false, false, Option.empty()).findFirst().get();
     HoodieColumnRangeMetadata<Comparable> fileColumnRange2 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file", "columnName", 3, 8, 1, 15, 120, 250);
+        "path/to/file", "columnName", 3, 8, 1, 15, 120, 250, ValueMetadata.V1EmptyMetadata.get());
     HoodieRecord<HoodieMetadataPayload> updatedPartitionStatsRecord =
         HoodieMetadataPayload.createPartitionStatsRecords(PARTITION_NAME, Collections.singletonList(fileColumnRange2), false, false, Option.empty()).findFirst().get();
     HoodieMetadataPayload combinedPartitionStatsRecordPayload =
         updatedPartitionStatsRecord.getData().preCombine(firstPartitionStatsRecord.getData());
     HoodieColumnRangeMetadata<Comparable> expectedColumnRange = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file", "columnName", 1, 8, 1, 25, 220, 450);
+        "path/to/file", "columnName", 1, 8, 1, 25, 220, 450, ValueMetadata.V1EmptyMetadata.get());
     HoodieMetadataPayload expectedColumnRangeMetadata = (HoodieMetadataPayload) HoodieMetadataPayload.createPartitionStatsRecords(
         PARTITION_NAME, Collections.singletonList(expectedColumnRange), false, false, Option.empty()).findFirst().get().getData();
     assertEquals(expectedColumnRangeMetadata, combinedPartitionStatsRecordPayload);
@@ -272,11 +274,11 @@ public class TestHoodieMetadataPayload extends HoodieCommonTestHarness {
   @Test
   public void testPartitionStatsPayloadMergingWithDelete() {
     HoodieColumnRangeMetadata<Comparable> fileColumnRange1 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file", "columnName", 1, 5, 0, 10, 100, 200);
+        "path/to/file", "columnName", 1, 5, 0, 10, 100, 200, ValueMetadata.V1EmptyMetadata.get());
     HoodieRecord<HoodieMetadataPayload> firstPartitionStatsRecord =
         HoodieMetadataPayload.createPartitionStatsRecords(PARTITION_NAME, Collections.singletonList(fileColumnRange1), false, false, Option.empty()).findFirst().get();
     HoodieColumnRangeMetadata<Comparable> fileColumnRange2 = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file", "columnName", 3, 8, 1, 15, 120, 250);
+        "path/to/file", "columnName", 3, 8, 1, 15, 120, 250, ValueMetadata.V1EmptyMetadata.get());
     // create delete payload
     HoodieRecord<HoodieMetadataPayload> deletedPartitionStatsRecord =
         HoodieMetadataPayload.createPartitionStatsRecords(PARTITION_NAME, Collections.singletonList(fileColumnRange2), true, false, Option.empty()).findFirst().get();
@@ -284,7 +286,7 @@ public class TestHoodieMetadataPayload extends HoodieCommonTestHarness {
     HoodieMetadataPayload combinedPartitionStatsRecordPayload =
         deletedPartitionStatsRecord.getData().preCombine(firstPartitionStatsRecord.getData());
     HoodieColumnRangeMetadata<Comparable> expectedColumnRange = HoodieColumnRangeMetadata.<Comparable>create(
-        "path/to/file", "columnName", 3, 8, 1, 15, 120, 250);
+        "path/to/file", "columnName", 3, 8, 1, 15, 120, 250, ValueMetadata.V1EmptyMetadata.get());
     HoodieMetadataPayload expectedColumnRangeMetadata = (HoodieMetadataPayload) HoodieMetadataPayload.createPartitionStatsRecords(
         PARTITION_NAME, Collections.singletonList(expectedColumnRange), true, false, Option.empty()).findFirst().get().getData();
     assertEquals(expectedColumnRangeMetadata, combinedPartitionStatsRecordPayload);
