@@ -123,15 +123,15 @@ public class HoodieParquetInputFormat extends HoodieParquetInputFormatBase {
           return super.getRecordReader(split, job, reporter);
         }
         if (supportAvroRead && HoodieColumnProjectionUtils.supportTimestamp(job)) {
-          return new HoodieFileGroupReaderBasedRecordReader((s, j) -> {
+          return new HoodieFileGroupReaderBasedRecordReader((s, j, d) -> {
             try {
-              return new ParquetRecordReaderWrapper(new HoodieTimestampAwareParquetInputFormat(Option.empty()), s, j, reporter);
+              return new ParquetRecordReaderWrapper(new HoodieTimestampAwareParquetInputFormat(Option.empty(), Option.of(d)), s, j, reporter);
             } catch (InterruptedException e) {
               throw new RuntimeException(e);
             }
           }, split, job);
         } else {
-          return new HoodieFileGroupReaderBasedRecordReader((s, j) -> super.getRecordReader(s, j, reporter), split, job);
+          return new HoodieFileGroupReaderBasedRecordReader((s, j, d) -> super.getRecordReader(s, j, reporter), split, job);
         }
       } catch (final IOException e) {
         throw new RuntimeException("Cannot create a RecordReaderWrapper", e);
@@ -157,9 +157,7 @@ public class HoodieParquetInputFormat extends HoodieParquetInputFormatBase {
     SchemaEvolutionContext schemaEvolutionContext = new SchemaEvolutionContext(split, job);
     schemaEvolutionContext.doEvolutionForParquetFormat();
 
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("EMPLOYING DEFAULT RECORD READER - " + split);
-    }
+    LOG.debug("EMPLOYING DEFAULT RECORD READER - {}", split);
 
     return getRecordReaderInternal(split, job, reporter, schemaEvolutionContext.internalSchemaOption);
   }
@@ -176,7 +174,7 @@ public class HoodieParquetInputFormat extends HoodieParquetInputFormatBase {
                                                                             Option<InternalSchema> internalSchemaOption) throws IOException {
     try {
       if (supportAvroRead && HoodieColumnProjectionUtils.supportTimestamp(job)) {
-        return new ParquetRecordReaderWrapper(new HoodieTimestampAwareParquetInputFormat(internalSchemaOption), split, job, reporter);
+        return new ParquetRecordReaderWrapper(new HoodieTimestampAwareParquetInputFormat(internalSchemaOption, Option.empty()), split, job, reporter);
       } else {
         return super.getRecordReader(split, job, reporter);
       }
