@@ -357,10 +357,8 @@ public class HoodieCombineHiveInputFormat<K extends WritableComparable, V extend
 
     // Store the previous value for the path specification
     String oldPaths = job.get(org.apache.hadoop.mapreduce.lib.input.FileInputFormat.INPUT_DIR);
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("The received input paths are: [" + oldPaths + "] against the property "
-          + org.apache.hadoop.mapreduce.lib.input.FileInputFormat.INPUT_DIR);
-    }
+    LOG.debug("The received input paths are: [{}] against the property {}", oldPaths,
+        org.apache.hadoop.mapreduce.lib.input.FileInputFormat.INPUT_DIR);
 
     // Process the normal splits
     if (nonCombinablePaths.size() > 0) {
@@ -411,7 +409,7 @@ public class HoodieCombineHiveInputFormat<K extends WritableComparable, V extend
           String avroSchema = schemaUtil.getTableAvroSchema().toString();
           Option<InternalSchema> internalSchema = schemaUtil.getTableInternalSchemaFromCommitMetadata();
           if (internalSchema.isPresent()) {
-            LOG.info("Set internal and avro schema cache with path: " + path);
+            LOG.info("Set internal and avro schema cache with path: {}", path);
             job.set(SCHEMA_CACHE_KEY_PREFIX + "." + path, avroSchema);
             job.set(INTERNAL_SCHEMA_CACHE_KEY_PREFIX + "." + path, SerDeHelper.toJson(internalSchema.get()));
           } else {
@@ -421,11 +419,11 @@ public class HoodieCombineHiveInputFormat<K extends WritableComparable, V extend
           }
         }
       } catch (Exception e) {
-        LOG.warn("Fail to set schema cache", e);
+        LOG.warn("Failed to set schema cache", e);
       }
     }
 
-    LOG.info("Number of all splits " + result.size());
+    LOG.info("Number of all splits {}", result.size());
     perfLogger.PerfLogEnd(CLASS_NAME, PerfLogger.GET_SPLITS);
     return result.toArray(new InputSplit[result.size()]);
   }
@@ -1062,9 +1060,7 @@ public class HoodieCombineHiveInputFormat<K extends WritableComparable, V extend
         InputFormat<WritableComparable, Writable> inputFormat = getInputFormatFromCache(inputFormatClass, conf);
         if (inputFormat instanceof AvoidSplitCombination
             && ((AvoidSplitCombination) inputFormat).shouldSkipCombine(paths[i + start], conf)) {
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("The path [" + paths[i + start] + "] is being parked for HiveInputFormat.getSplits");
-          }
+          LOG.debug("The path [{}] is being parked for HiveInputFormat.getSplits", paths[i + start]);
           nonCombinablePathIndices.add(i + start);
         }
       }
