@@ -55,6 +55,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.stream.Collectors;
 
+import static org.apache.hudi.avro.HoodieAvroUtils.createNewSchemaField;
 import static org.apache.hudi.common.util.StringUtils.EMPTY_STRING;
 
 /**
@@ -90,8 +91,8 @@ public abstract class AbstractRealtimeRecordReader {
       metaClient = HoodieTableMetaClient.builder()
           .setConf(HadoopFSUtils.getStorageConfWithCopy(jobConf)).setBasePath(split.getBasePath()).build();
       payloadProps.putAll(metaClient.getTableConfig().getProps(true));
-      if (metaClient.getTableConfig().getPreCombineField() != null) {
-        this.payloadProps.setProperty(HoodiePayloadProps.PAYLOAD_ORDERING_FIELD_PROP_KEY, metaClient.getTableConfig().getPreCombineField());
+      if (metaClient.getTableConfig().getOrderingFieldsStr().isPresent()) {
+        this.payloadProps.setProperty(HoodiePayloadProps.PAYLOAD_ORDERING_FIELD_PROP_KEY, metaClient.getTableConfig().getOrderingFieldsStr().orElse(null));
       }
       this.usesCustomPayload = usesCustomPayload(metaClient);
       LOG.info("usesCustomPayload ==> " + this.usesCustomPayload);
@@ -135,7 +136,7 @@ public abstract class AbstractRealtimeRecordReader {
         if (writerSchemaColNames.contains(lastColName)) {
           break;
         }
-        LOG.debug(String.format("remove virtual column: %s", lastColName));
+        LOG.debug("remove virtual column: {}", lastColName);
         columnNameList.remove(columnNameList.size() - 1);
         columnTypeList.remove(columnTypeList.size() - 1);
       }
@@ -143,7 +144,7 @@ public abstract class AbstractRealtimeRecordReader {
       this.serializer = new HiveAvroSerializer(new ArrayWritableObjectInspector(rowTypeInfo), columnNameList, columnTypeList);
     } catch (Exception e) {
       // fallback to origin logical
-      LOG.warn("fall to init HiveAvroSerializer to support payload merge", e);
+      LOG.warn("Failed to init HiveAvroSerializer to support payload merge. Fallback to origin logical", e);
       this.supportPayload = false;
     }
   }
@@ -185,18 +186,18 @@ public abstract class AbstractRealtimeRecordReader {
       Field field = schemaFieldsMap.get(columnName.toLowerCase());
 
       if (field != null) {
-        hiveSchemaFields.add(new Schema.Field(field.name(), field.schema(), field.doc(), field.defaultVal()));
+        hiveSchemaFields.add(createNewSchemaField(field));
       } else {
         // Hive has some extra virtual columns like BLOCK__OFFSET__INSIDE__FILE which do not exist in table schema.
         // They will get skipped as they won't be found in the original schema.
-        LOG.debug("Skipping Hive Column => " + columnName);
+        LOG.debug("Skipping Hive Column => {}", columnName);
       }
     }
 
     Schema hiveSchema = Schema.createRecord(writerSchema.getName(), writerSchema.getDoc(), writerSchema.getNamespace(),
         writerSchema.isError());
     hiveSchema.setFields(hiveSchemaFields);
-    LOG.debug("HIVE Schema is :" + hiveSchema.toString(true));
+    LOG.debug("HIVE Schema is :{}", hiveSchema);
     return hiveSchema;
   }
 
