@@ -36,7 +36,6 @@ import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.exception.HoodieIOException;
 import org.apache.hudi.hadoop.BootstrapBaseFileSplit;
 import org.apache.hudi.hadoop.FileStatusWithBootstrapBaseFile;
-import org.apache.hudi.hadoop.HiveHoodieTableFileIndex;
 import org.apache.hudi.hadoop.HoodieCopyOnWriteTableInputFormat;
 import org.apache.hudi.hadoop.LocatedFileStatusWithBootstrapBaseFile;
 import org.apache.hudi.hadoop.RealtimeFileStatus;
@@ -95,14 +94,12 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
   }
 
   @Override
-  protected FileStatus createFileStatusUnchecked(FileSlice fileSlice, HiveHoodieTableFileIndex fileIndex, HoodieTableMetaClient metaClient) {
+  protected FileStatus createFileStatusUnchecked(FileSlice fileSlice, Option<HoodieInstant> latestCompletedInstantOpt,
+                                                 String tableBasePath, HoodieTableMetaClient metaClient) {
     Option<HoodieBaseFile> baseFileOpt = fileSlice.getBaseFile();
     Option<HoodieLogFile> latestLogFileOpt = fileSlice.getLatestLogFile();
     Stream<HoodieLogFile> logFiles = fileSlice.getLogFiles();
 
-    Option<HoodieInstant> latestCompletedInstantOpt = fileIndex.getLatestCompletedInstant();
-    String tableBasePath = fileIndex.getBasePath().toString();
-
     // Check if we're reading a MOR table
     if (baseFileOpt.isPresent()) {
       return createRealtimeFileStatusUnchecked(baseFileOpt.get(), logFiles, tableBasePath, latestCompletedInstantOpt, getHoodieVirtualKeyInfo(metaClient));
@@ -193,13 +190,13 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
 
     // build fileGroup from fsView
     List<StoragePathInfo> affectedPathInfoList = HoodieInputFormatUtils
-        .listAffectedFilesForCommits(job, tableMetaClient.getBasePathV2(),
+        .listAffectedFilesForCommits(job, tableMetaClient.getBasePath(),
             metadataList);
     // step3
     HoodieTableFileSystemView fsView = new HoodieTableFileSystemView(
         tableMetaClient, commitsTimelineToReturn, affectedPathInfoList);
     // build fileGroup from fsView
-    Path basePath = new Path(tableMetaClient.getBasePath());
+    Path basePath = new Path(tableMetaClient.getBasePath().toString());
     // filter affectedPartition by inputPaths
     List<String> affectedPartition =
         HoodieTableMetadataUtil.getWritePartitionPaths(metadataList).stream()
@@ -226,7 +223,7 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
     }
 
     Option<HoodieVirtualKeyInfo> virtualKeyInfoOpt = getHoodieVirtualKeyInfo(tableMetaClient);
-    String maxCommitTime = fsView.getLastInstant().get().getTimestamp();
+    String maxCommitTime = fsView.getLastInstant().get().requestedTime();
     // step6
     result.addAll(collectAllIncrementalFiles(fileGroups, maxCommitTime, basePath.toString(), candidateFileStatus, virtualKeyInfoOpt));
     return result;
@@ -375,7 +372,7 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
         HoodieInstant latestCompletedInstant = latestCompletedInstantOpt.get();
         checkState(latestCompletedInstant.isCompleted());
 
-        rtFileStatus.setMaxCommitTime(latestCompletedInstant.getTimestamp());
+        rtFileStatus.setMaxCommitTime(latestCompletedInstant.requestedTime());
       }
 
       if (baseFileStatus instanceof LocatedFileStatusWithBootstrapBaseFile || baseFileStatus instanceof FileStatusWithBootstrapBaseFile) {
@@ -406,7 +403,7 @@ public class HoodieMergeOnReadTableInputFormat extends HoodieCopyOnWriteTableInp
         HoodieInstant latestCompletedInstant = latestCompletedInstantOpt.get();
         checkState(latestCompletedInstant.isCompleted());
 
-        rtFileStatus.setMaxCommitTime(latestCompletedInstant.getTimestamp());
+        rtFileStatus.setMaxCommitTime(latestCompletedInstant.requestedTime());
       }
 
       return rtFileStatus;
