@@ -20,8 +20,8 @@ package org.apache.hudi.hadoop.realtime;
 
 import org.apache.hudi.common.model.HoodieLogFile;
 import org.apache.hudi.common.util.Option;
-import org.apache.hudi.hadoop.CachingPath;
 import org.apache.hudi.hadoop.InputSplitUtils;
+import org.apache.hudi.storage.StoragePath;
 
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.mapred.InputSplitWithLocationInfo;
@@ -128,7 +128,7 @@ public interface RealtimeSplit extends InputSplitWithLocationInfo {
     for (int i = 0; i < totalLogFiles; i++) {
       String logFilePath = InputSplitUtils.readString(in);
       long logFileSize = in.readLong();
-      deltaLogPaths.add(new HoodieLogFile(new CachingPath(logFilePath), logFileSize));
+      deltaLogPaths.add(new HoodieLogFile(new StoragePath(logFilePath), logFileSize));
     }
     setDeltaLogFiles(deltaLogPaths);
 
