@@ -34,7 +34,6 @@ import org.apache.avro.JsonProperties;
 import org.apache.avro.LogicalType;
 import org.apache.avro.LogicalTypes;
 import org.apache.avro.Schema;
-import org.apache.avro.generic.GenericArray;
 import org.apache.avro.generic.GenericFixed;
 import org.apache.avro.generic.GenericRecord;
 import org.apache.hadoop.fs.Path;
@@ -58,6 +57,7 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
@@ -68,6 +68,7 @@ import java.util.stream.Collectors;
 
 import static org.apache.hudi.avro.AvroSchemaUtils.appendFieldsToSchema;
 import static org.apache.hudi.avro.AvroSchemaUtils.createNullableSchema;
+import static org.apache.hudi.avro.HoodieAvroUtils.createNewSchemaField;
 import static org.apache.hudi.common.util.ConfigUtils.getReaderConfigs;
 import static org.apache.hudi.hadoop.fs.HadoopFSUtils.convertToStoragePath;
 
@@ -139,7 +140,7 @@ public class HoodieRealtimeRecordReaderUtils {
         throw new HoodieException("Field " + fn + " not found in log schema. Query cannot proceed! "
             + "Derived Schema Fields: " + new ArrayList<>(schemaFieldsMap.keySet()));
       } else {
-        projectedFields.add(new Schema.Field(field.name(), field.schema(), field.doc(), field.defaultVal()));
+        projectedFields.add(createNewSchemaField(field));
       }
     }
 
@@ -208,9 +209,7 @@ public class HoodieRealtimeRecordReaderUtils {
           if (record.getSchema().getField(field.name()) != null) {
             fieldValue = record.get(field.name());
           } else {
-            if (LOG.isDebugEnabled()) {
-              LOG.debug("Field:" + field.name() + "not found in Schema:" + schema);
-            }
+            LOG.debug("Field: {} not found in Schema: {}", field.name(), schema);
           }
           recordValues[recordValueIndex++] = avroToArrayWritable(fieldValue, field.schema(), supportTimestamp);
         }
@@ -218,7 +217,7 @@ public class HoodieRealtimeRecordReaderUtils {
       case ENUM:
         return new BytesWritable(value.toString().getBytes());
       case ARRAY:
-        GenericArray arrayValue = (GenericArray) value;
+        Collection arrayValue = (Collection) value;
         Writable[] arrayValues = new Writable[arrayValue.size()];
         int arrayValueIndex = 0;
         for (Object obj : arrayValue) {
