@@ -22,6 +22,9 @@ package org.apache.hudi.hadoop.realtime;
 import org.apache.hudi.common.model.HoodieLogFile;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.hadoop.PathWithBootstrapFileStatus;
+import org.apache.hudi.storage.HoodieStorage;
+import org.apache.hudi.storage.StoragePath;
+import org.apache.hudi.storage.hadoop.HoodieHadoopStorage;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
@@ -43,11 +46,13 @@ public class TestHoodieMergeOnReadTableInputFormat {
 
   @TempDir
   java.nio.file.Path tempDir;
+  private HoodieStorage storage;
   private FileSystem fs;
 
   @BeforeEach
   void setUp() throws IOException {
     fs = FileSystem.get(tempDir.toUri(), new Configuration());
+    storage = new HoodieHadoopStorage(fs);
   }
 
   @AfterEach
@@ -74,7 +79,7 @@ public class TestHoodieMergeOnReadTableInputFormat {
     assertTrue(new HoodieMergeOnReadTableInputFormat().isSplitable(fs, rtPath), "Path only contains the base file should be splittable");
 
     URI logPath = Files.createTempFile(tempDir, ".test", ".log.4_1-149-180").toUri();
-    HoodieLogFile logFile = new HoodieLogFile(fs.getFileStatus(new Path(logPath)));
+    HoodieLogFile logFile = new HoodieLogFile(storage.getPathInfo(new StoragePath(logPath)));
     rtPath = new HoodieRealtimePath(new Path("foo"), "bar", basePath.toString(), Collections.singletonList(logFile), "000", false, Option.empty());
     assertFalse(new HoodieMergeOnReadTableInputFormat().isSplitable(fs, rtPath), "Path contains log files should not be splittable.");
   }
