@@ -26,6 +26,7 @@ import org.apache.avro.generic.GenericRecord;
 import org.apache.spark.api.java.JavaRDD;
 import org.apache.spark.sql.Dataset;
 import org.apache.spark.sql.Row;
+import org.apache.spark.sql.SQLContext;
 import org.apache.spark.sql.SparkSession;
 
 import java.util.List;
@@ -57,10 +58,10 @@ public class SparkBasedReader {
         .toJavaRDD();
   }
 
-  public static JavaRDD<GenericRecord> readParquet(SparkSession sparkSession, List<String>
+  public static JavaRDD<GenericRecord> readParquet(SQLContext sqlContext, List<String>
       listOfPaths, Option<String> structName, Option<String> nameSpace) {
 
-    Dataset<Row> dataSet = sparkSession.read()
+    Dataset<Row> dataSet = sqlContext.read()
         .parquet((JavaConverters.asScalaIteratorConverter(listOfPaths.iterator()).asScala().toSeq()));
 
     return HoodieSparkUtils
@@ -69,10 +70,10 @@ public class SparkBasedReader {
         .toJavaRDD();
   }
 
-  public static JavaRDD<GenericRecord> readOrc(SparkSession sparkSession, List<String>
+  public static JavaRDD<GenericRecord> readOrc(SQLContext sqlContext, List<String>
       listOfPaths, Option<String> structName, Option<String> nameSpace) {
 
-    Dataset<Row> dataSet = sparkSession.read()
+    Dataset<Row> dataSet = sqlContext.read()
         .orc((JavaConverters.asScalaIteratorConverter(listOfPaths.iterator()).asScala().toSeq()));
 
     return HoodieSparkUtils.createRdd(dataSet.toDF(),
