@@ -21,6 +21,16 @@ package org.apache.hudi.io.hfile;
 
 import org.apache.hudi.common.util.Option;
 
+import java.io.ByteArrayOutputStream;
+import java.io.DataOutputStream;
+import java.io.IOException;
+import java.nio.ByteBuffer;
+import java.util.ArrayList;
+import java.util.List;
+
+import static org.apache.hudi.io.hfile.DataSize.SIZEOF_BYTE;
+import static org.apache.hudi.io.hfile.DataSize.SIZEOF_INT16;
+import static org.apache.hudi.io.hfile.DataSize.SIZEOF_INT64;
 import static org.apache.hudi.io.hfile.HFileReader.SEEK_TO_BEFORE_BLOCK_FIRST_KEY;
 import static org.apache.hudi.io.hfile.HFileReader.SEEK_TO_FOUND;
 import static org.apache.hudi.io.hfile.HFileReader.SEEK_TO_IN_RANGE;
@@ -30,15 +40,24 @@ import static org.apache.hudi.io.hfile.KeyValue.KEY_OFFSET;
  * Represents a {@link HFileBlockType#DATA} block.
  */
 public class HFileDataBlock extends HFileBlock {
+  private static final int KEY_LENGTH_LENGTH = SIZEOF_INT16;
+  private static final int COLUMN_FAMILY_LENGTH = SIZEOF_BYTE;
+  private static final int VERSION_TIMESTAMP_LENGTH = SIZEOF_INT64;
+  private static final int KEY_TYPE_LENGTH = SIZEOF_BYTE;
   // Hudi does not use HFile MVCC timestamp version so the version
   // is always 0, thus the byte length of the version is always 1.
   // This assumption is also validated when parsing {@link HFileInfo},
   // i.e., the maximum MVCC timestamp in a HFile must be 0.
   private static final long ZERO_TS_VERSION_BYTE_LENGTH = 1;
+  // Hudi does not set version timestamp for key value pairs,
+  // so the latest timestamp is used.
+  private static final long LATEST_TIMESTAMP = Long.MAX_VALUE;
 
   // End offset of content in the block, relative to the start of the start of the block
   protected final int uncompressedContentEndRelativeOffset;
+  private final List<KeyValueEntry> entriesToWrite = new ArrayList<>();
 
+  // For read purpose.
   protected HFileDataBlock(HFileContext context,
                            byte[] byteBuff,
                            int startOffsetInBuff) {
@@ -48,6 +67,18 @@ public class HFileDataBlock extends HFileBlock {
         this.uncompressedEndOffset - this.sizeCheckSum - this.startOffsetInBuff;
   }
 
+  // For write purpose.
+  private HFileDataBlock(HFileContext context, long previousBlockOffset) {
+    super(context, HFileBlockType.DATA, previousBlockOffset);
+    // This is not used for write.
+    uncompressedContentEndRelativeOffset = -1;
+  }
+
+  static HFileDataBlock createDataBlockToWrite(HFileContext context,
+                                               long previousBlockOffset) {
+    return new HFileDataBlock(context, previousBlockOffset);
+  }
+
   /**
    * Seeks to the key to look up. The key may not have an exact match.
    *
@@ -70,7 +101,7 @@ public class HFileDataBlock extends HFileBlock {
    * of the data block; the cursor points to the actual first key of the data block which is
    * lexicographically greater than the lookup key.
    */
-  public int seekTo(HFileCursor cursor, Key key, int blockStartOffsetInFile) {
+  int seekTo(HFileCursor cursor, Key key, int blockStartOffsetInFile) {
     int relativeOffset = cursor.getOffset() - blockStartOffsetInFile;
     int lastRelativeOffset = relativeOffset;
     Option<KeyValue> lastKeyValue = cursor.getKeyValue();
@@ -123,7 +154,7 @@ public class HFileDataBlock extends HFileBlock {
    * @param offset offset to read relative to the start of {@code byteBuff}.
    * @return the {@link KeyValue} instance.
    */
-  public KeyValue readKeyValue(int offset) {
+  KeyValue readKeyValue(int offset) {
     return new KeyValue(byteBuff, offset);
   }
 
@@ -135,7 +166,7 @@ public class HFileDataBlock extends HFileBlock {
    *                               HFile.
    * @return {@code true} if there is next {@link KeyValue}; {code false} otherwise.
    */
-  public boolean next(HFileCursor cursor, int blockStartOffsetInFile) {
+  boolean next(HFileCursor cursor, int blockStartOffsetInFile) {
     int offset = cursor.getOffset() - blockStartOffsetInFile;
     Option<KeyValue> keyValue = cursor.getKeyValue();
     if (!keyValue.isPresent()) {
@@ -149,4 +180,66 @@ public class HFileDataBlock extends HFileBlock {
   private boolean isAtFirstKey(int relativeOffset) {
     return relativeOffset == HFILEBLOCK_HEADER_SIZE;
   }
+
+  // ================ Below are for Write ================
+
+  boolean isEmpty() {
+    return entriesToWrite.isEmpty();
+  }
+
+  void add(byte[] key, byte[] value) {
+    KeyValueEntry kv = new KeyValueEntry(key, value);
+    // Assume all entries are sorted before write.
+    entriesToWrite.add(kv);
+  }
+
+  int getNumOfEntries() {
+    return entriesToWrite.size();
+  }
+
+  byte[] getFirstKey() {
+    return entriesToWrite.get(0).key;
+  }
+
+  byte[] getLastKeyContent() {
+    if (entriesToWrite.isEmpty()) {
+      return new byte[0];
+    }
+    return entriesToWrite.get(entriesToWrite.size() - 1).key;
+  }
+
+  @Override
+  protected ByteBuffer getUncompressedBlockDataToWrite() throws IOException {
+    ByteArrayOutputStream baos = new ByteArrayOutputStream(context.getBlockSize());
+    try (DataOutputStream dataOutputStream = new DataOutputStream(baos)) {
+      for (KeyValueEntry kv : entriesToWrite) {
+        // Length of key + length of a short variable indicating length of key.
+        // Note that 10 extra bytes are required by hbase reader.
+        // That is: 1 byte for column family length, 8 bytes for timestamp, 1 bytes for key type.
+        dataOutputStream.writeInt(
+            kv.key.length + KEY_LENGTH_LENGTH + COLUMN_FAMILY_LENGTH + VERSION_TIMESTAMP_LENGTH + KEY_TYPE_LENGTH);
+        // Length of value.
+        dataOutputStream.writeInt(kv.value.length);
+        // Key content length.
+        dataOutputStream.writeShort((short)kv.key.length);
+        // Key.
+        dataOutputStream.write(kv.key);
+        // Column family length: constant 0.
+        dataOutputStream.write(0);
+        // Column qualifier: assume 0 bits.
+        // Timestamp: using the latest.
+        dataOutputStream.writeLong(LATEST_TIMESTAMP);
+        // Key type: constant Put (4) in Hudi.
+        // Minimum((byte) 0), Put((byte) 4), Delete((byte) 8),
+        // DeleteFamilyVersion((byte) 10), DeleteColumn((byte) 12),
+        // DeleteFamily((byte) 14), Maximum((byte) 255).
+        dataOutputStream.write(4);
+        // Value.
+        dataOutputStream.write(kv.value);
+        // MVCC.
+        dataOutputStream.write(0);
+      }
+    }
+    return ByteBuffer.wrap(baos.toByteArray());
+  }
 }
