@@ -374,8 +374,7 @@ public abstract class HoodieStorage implements Closeable {
         if (!renameSuccess && null != tmpPath) {
           try {
             deleteFile(tmpPath);
-            LOG.warn("Fail to rename " + tmpPath + " to " + path
-                + ", target file exists: " + exists(path));
+            LOG.debug("Failed to rename {} to {}, target file exists: {}", tmpPath, path, exists(path));
           } catch (IOException e) {
             throw new HoodieIOException("Failed to delete tmp file " + tmpPath, e);
           }
@@ -389,7 +388,10 @@ public abstract class HoodieStorage implements Closeable {
    */
   @PublicAPIMethod(maturity = ApiMaturityLevel.EVOLVING)
   public final boolean needCreateTempFile() {
-    return StorageSchemes.HDFS.getScheme().equals(getScheme());
+    return StorageSchemes.HDFS.getScheme().equals(getScheme())
+        // Local file will be visible immediately after LocalFileSystem#create(..), even before the output
+        // stream is closed, so temporary file is also needed for atomic file creating with content written.
+        || StorageSchemes.FILE.getScheme().equals(getScheme());
   }
 
   /**
