@@ -18,9 +18,7 @@
 
 package org.apache.hudi.metaserver.store;
 
-import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
 import org.apache.hudi.common.table.timeline.HoodieInstantTimeGenerator;
-import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.metaserver.store.bean.InstantBean;
 import org.apache.hudi.metaserver.store.bean.TableBean;
@@ -32,11 +30,14 @@ import org.apache.hudi.metaserver.thrift.Table;
 
 import java.io.Serializable;
 import java.util.Collections;
+import java.util.Date;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.stream.Collectors;
 
+import static org.apache.hudi.common.table.timeline.InstantComparison.LESSER_THAN_OR_EQUALS;
+import static org.apache.hudi.common.table.timeline.InstantComparison.compareTimestamps;
 import static org.apache.hudi.common.util.CollectionUtils.isNullOrEmpty;
 import static org.apache.hudi.common.util.ValidationUtils.checkState;
 
@@ -126,9 +127,9 @@ public class RelationalDBBasedStorage implements MetaserverStorage, Serializable
     do {
       oldTimestamp = getLatestTimestamp(tableId);
       do {
-        newTimestamp = HoodieInstantTimeGenerator.createNewInstantTime(0L);
-      } while (oldTimestamp != null && HoodieTimeline.compareTimestamps(newTimestamp,
-          HoodieActiveTimeline.LESSER_THAN_OR_EQUALS, oldTimestamp));
+        newTimestamp = HoodieInstantTimeGenerator.formatDate(new Date(System.currentTimeMillis()));
+      } while (oldTimestamp != null && compareTimestamps(newTimestamp,
+          LESSER_THAN_OR_EQUALS, oldTimestamp));
       Map<String, Object> params = new HashMap<>();
       params.put("tableId", tableId);
       params.put("oldTimestamp", oldTimestamp);
