@@ -22,13 +22,14 @@ import org.apache.hudi.HoodieBaseRelation.BaseFileReader
 import org.apache.hudi.HoodieConversionUtils.{toJavaOption, toScalaOption}
 import org.apache.hudi.HoodieDataSourceHelper.AvroDeserializerSupport
 import org.apache.hudi.LogFileIterator.{getPartitionPath, scanLog}
-import org.apache.hudi.avro.AvroSchemaCache
+import org.apache.hudi.avro.{AvroRecordContext, AvroSchemaCache}
 import org.apache.hudi.common.config.{HoodieCommonConfig, HoodieMemoryConfig, HoodieMetadataConfig, TypedProperties}
-import org.apache.hudi.common.engine.{EngineType, HoodieLocalEngineContext}
+import org.apache.hudi.common.engine.{EngineType, HoodieLocalEngineContext, RecordContext}
 import org.apache.hudi.common.fs.FSUtils.getRelativePartitionPath
 import org.apache.hudi.common.model.{HoodieAvroIndexedRecord, HoodieEmptyRecord, HoodieLogFile, HoodieOperation, HoodieRecord, HoodieSparkRecord}
 import org.apache.hudi.common.model.HoodieRecord.HoodieRecordType
 import org.apache.hudi.common.table.log.HoodieMergedLogRecordScanner
+import org.apache.hudi.common.table.read.{BufferedRecord, BufferedRecords, DeleteContext}
 import org.apache.hudi.common.util.{FileIOUtils, HoodieRecordUtils}
 import org.apache.hudi.config.HoodiePayloadConfig
 import org.apache.hudi.hadoop.fs.HadoopFSUtils
@@ -40,7 +41,7 @@ import org.apache.hudi.storage.{HoodieStorageUtils, StoragePath}
 import org.apache.hudi.util.CachingIterator
 
 import org.apache.avro.Schema
-import org.apache.avro.generic.GenericRecord
+import org.apache.avro.generic.{GenericRecord, IndexedRecord}
 import org.apache.hadoop.conf.Configuration
 import org.apache.hadoop.mapred.JobConf
 import org.apache.spark.sql.HoodieCatalystExpressionUtils.generateUnsafeProjection
@@ -86,20 +87,21 @@ class LogFileIterator(logFiles: List[HoodieLogFile],
   }
   private val maxCompactionMemoryInBytes: Long = getMaxCompactionMemoryInBytes(new JobConf(config))
 
-  protected val payloadProps: TypedProperties = tableState.preCombineFieldOpt
-    .map { preCombineField =>
-      HoodiePayloadConfig.newBuilder
-        .withPayloadOrderingField(preCombineField)
-        .build
-        .getProps
-    }
-    .getOrElse(new TypedProperties())
+  protected val payloadProps: TypedProperties = if (tableState.orderingFields.nonEmpty) {
+    HoodiePayloadConfig.newBuilder
+      .withPayloadOrderingFields(String.join(",", tableState.orderingFields: _*))
+      .build
+      .getProps
+  } else {
+    new TypedProperties()
+  }
 
   protected override val avroSchema: Schema = requiredAvroSchema
   protected override val structTypeSchema: StructType = requiredStructTypeSchema
 
   protected val logFileReaderAvroSchema: Schema = AvroSchemaCache.intern(new Schema.Parser().parse(tableSchema.avroSchemaStr))
   protected val logFileReaderStructType: StructType = tableSchema.structTypeSchema
+  protected val deleteContext: DeleteContext = new DeleteContext(payloadProps, logFileReaderAvroSchema).withReaderSchema(logFileReaderAvroSchema);
 
   private val requiredSchemaAvroProjection: AvroProjection = AvroProjection.create(avroSchema)
   private val requiredSchemaRowProjection: Projection = generateUnsafeProjection(logFileReaderStructType, structTypeSchema)
@@ -268,6 +270,10 @@ class RecordMergingFileIterator(logFiles: List[HoodieLogFile],
   private val recordMerger = HoodieRecordUtils.createRecordMerger(tableState.tablePath, EngineType.SPARK,
     tableState.recordMergeImplClasses.asJava, tableState.recordMergeStrategyId)
 
+  private val rowRecordContext: RecordContext[InternalRow] = SparkFileFormatInternalRecordContext.getFieldAccessorInstance
+  private val avroRecordContext: RecordContext[IndexedRecord] = AvroRecordContext.getFieldAccessorInstance
+  private val orderingFields: Array[String] = tableState.orderingFields.toArray
+
   override def doHasNext: Boolean = hasNextInternal
 
   // NOTE: It's crucial for this method to be annotated w/ [[@tailrec]] to make sure
@@ -305,31 +311,26 @@ class RecordMergingFileIterator(logFiles: List[HoodieLogFile],
     //       on the record from the Delta Log
     recordMerger.getRecordType match {
       case HoodieRecordType.SPARK =>
-        val curRecord = new HoodieSparkRecord(curRow, readerSchema)
-        val result = recordMerger.merge(curRecord, baseFileReaderAvroSchema, newRecord, logFileReaderAvroSchema, payloadProps)
-        toScalaOption(result)
-          .flatMap { r =>
-            val data = r.getLeft.getData.asInstanceOf[InternalRow]
-            if (isDeleteOperation(data)) {
-              None
-            } else {
-              val schema = HoodieInternalRowUtils.getCachedSchema(r.getRight)
-              val projection = HoodieInternalRowUtils.getCachedUnsafeProjection(schema, structTypeSchema)
-              Some(projection.apply(data))
-            }
-          }
+        val curRecord = BufferedRecords.fromEngineRecord(curRow, baseFileReaderAvroSchema, rowRecordContext, orderingFields, newRecord.getRecordKey, false)
+        val newBufferedRecord = BufferedRecords.fromHoodieRecord(newRecord, logFileReaderAvroSchema, rowRecordContext, payloadProps, orderingFields, deleteContext)
+        val result = recordMerger.merge(curRecord, newBufferedRecord, rowRecordContext, payloadProps).asInstanceOf[BufferedRecord[InternalRow]]
+        if (result.isDelete) {
+          None
+        } else {
+          val schema = HoodieInternalRowUtils.getCachedSchema(rowRecordContext.getSchemaFromBufferRecord(result))
+          val projection = HoodieInternalRowUtils.getCachedUnsafeProjection(schema, structTypeSchema)
+          Some(projection.apply(result.getRecord))
+        }
       case _ =>
-        val curRecord = new HoodieAvroIndexedRecord(serialize(curRow))
-        val result = recordMerger.merge(curRecord, baseFileReaderAvroSchema, newRecord, logFileReaderAvroSchema, payloadProps)
-        toScalaOption(result)
-          .flatMap { r =>
-            val avroRecord = r.getLeft.toIndexedRecord(r.getRight, payloadProps).get.getData.asInstanceOf[GenericRecord]
-            if (isDeleteOperation(avroRecord)) {
-              None
-            } else {
-              Some(deserialize(requiredSchemaAvroProjection(avroRecord)))
-            }
-          }
+        val curRecord = BufferedRecords.fromEngineRecord(serialize(curRow), baseFileReaderAvroSchema, avroRecordContext, orderingFields, newRecord.getRecordKey, false)
+        val newBufferedRecord = BufferedRecords.fromHoodieRecord(newRecord, logFileReaderAvroSchema, avroRecordContext, payloadProps, orderingFields, deleteContext)
+        val result = recordMerger.merge(curRecord, newBufferedRecord, avroRecordContext, payloadProps).asInstanceOf[BufferedRecord[IndexedRecord]]
+        if (result.isDelete) {
+          None
+        } else {
+          val avroRecord = result.getRecord.asInstanceOf[GenericRecord]
+          Some(deserialize(requiredSchemaAvroProjection(avroRecord)))
+        }
     }
   }
 }
@@ -346,75 +347,43 @@ object LogFileIterator extends SparkAdapterSupport {
     val tablePath = tableState.tablePath
     val storage = HoodieStorageUtils.getStorage(tablePath, HadoopFSUtils.getStorageConf(hadoopConf))
 
-    if (HoodieTableMetadata.isMetadataTable(tablePath)) {
-      val metadataConfig = HoodieMetadataConfig.newBuilder()
-        .fromProperties(tableState.metadataConfig.getProps)
-        .withSpillableMapDir(hadoopConf.get(HoodieMemoryConfig.SPILLABLE_MAP_BASE_PATH.key(),
+    val logRecordScannerBuilder = HoodieMergedLogRecordScanner.newBuilder()
+      .withStorage(storage)
+      .withBasePath(tablePath)
+      .withLogFilePaths(logFiles.map(logFile => logFile.getPath.toString).asJava)
+      .withReaderSchema(logSchema)
+      // NOTE: This part shall only be reached when at least one log is present in the file-group
+      //       entailing that table has to have at least one commit
+      .withLatestInstantTime(tableState.latestCommitTimestamp.get)
+      .withReverseReader(false)
+      .withInternalSchema(internalSchema)
+      .withBufferSize(
+        hadoopConf.getInt(HoodieMemoryConfig.MAX_DFS_STREAM_BUFFER_SIZE.key(),
+          HoodieMemoryConfig.DEFAULT_MR_MAX_DFS_STREAM_BUFFER_SIZE))
+      .withMaxMemorySizeInBytes(maxCompactionMemoryInBytes)
+      .withSpillableMapBasePath(
+        hadoopConf.get(HoodieMemoryConfig.SPILLABLE_MAP_BASE_PATH.key,
           FileIOUtils.getDefaultSpillableMapBasePath))
-        .enable(true).build()
-      val dataTableBasePath = getDataTableBasePathFromMetadataTable(tablePath)
-      val metadataTable = new HoodieBackedTableMetadata(
-        new HoodieLocalEngineContext(HadoopFSUtils.getStorageConf(hadoopConf)), storage, metadataConfig,
-        dataTableBasePath)
+      .withDiskMapType(
+        hadoopConf.getEnum(HoodieCommonConfig.SPILLABLE_DISK_MAP_TYPE.key,
+          HoodieCommonConfig.SPILLABLE_DISK_MAP_TYPE.defaultValue))
+      .withBitCaskDiskMapCompressionEnabled(
+        hadoopConf.getBoolean(HoodieCommonConfig.DISK_MAP_BITCASK_COMPRESSION_ENABLED.key(),
+          HoodieCommonConfig.DISK_MAP_BITCASK_COMPRESSION_ENABLED.defaultValue()))
 
-      // We have to force full-scan for the MT log record reader, to make sure
-      // we can iterate over all of the partitions, since by default some of the partitions (Column Stats,
-      // Bloom Filter) are in "point-lookup" mode
-      val forceFullScan = true
+    if (logFiles.nonEmpty) {
+      logRecordScannerBuilder.withPartition(getRelativePartitionPath(
+        new StoragePath(tableState.tablePath), logFiles.head.getPath.getParent))
+    }
 
-      // NOTE: In case of Metadata Table partition path equates to partition name (since there's just one level
-      //       of indirection among MT partitions)
-      val relativePartitionPath = getRelativePartitionPath(
-        new StoragePath(tablePath), partitionPath)
+    logRecordScannerBuilder.withRecordMerger(
+      HoodieRecordUtils.createRecordMerger(tableState.tablePath, EngineType.SPARK, tableState.recordMergeImplClasses.asJava, tableState.recordMergeStrategyId))
 
-      val logRecordReader =
-        metadataTable.getLogRecordScanner(logFiles.asJava, relativePartitionPath, toJavaOption(Some(forceFullScan)), toJavaOption(Some(tableState.latestCommitTimestamp.get)))
-          .getLeft
+    val scanner = logRecordScannerBuilder.build()
 
-      val recordList = closing(logRecordReader) {
-        logRecordReader.getRecords
-      }
-
-      mutable.HashMap(recordList.asScala.map(r => (r.getRecordKey, r)).toSeq: _*)
-    } else {
-      val logRecordScannerBuilder = HoodieMergedLogRecordScanner.newBuilder()
-        .withStorage(storage)
-        .withBasePath(tablePath)
-        .withLogFilePaths(logFiles.map(logFile => logFile.getPath.toString).asJava)
-        .withReaderSchema(logSchema)
-        // NOTE: This part shall only be reached when at least one log is present in the file-group
-        //       entailing that table has to have at least one commit
-        .withLatestInstantTime(tableState.latestCommitTimestamp.get)
-        .withReverseReader(false)
-        .withInternalSchema(internalSchema)
-        .withBufferSize(
-          hadoopConf.getInt(HoodieMemoryConfig.MAX_DFS_STREAM_BUFFER_SIZE.key(),
-            HoodieMemoryConfig.DEFAULT_MR_MAX_DFS_STREAM_BUFFER_SIZE))
-        .withMaxMemorySizeInBytes(maxCompactionMemoryInBytes)
-        .withSpillableMapBasePath(
-          hadoopConf.get(HoodieMemoryConfig.SPILLABLE_MAP_BASE_PATH.key,
-            FileIOUtils.getDefaultSpillableMapBasePath))
-        .withDiskMapType(
-          hadoopConf.getEnum(HoodieCommonConfig.SPILLABLE_DISK_MAP_TYPE.key,
-            HoodieCommonConfig.SPILLABLE_DISK_MAP_TYPE.defaultValue))
-        .withBitCaskDiskMapCompressionEnabled(
-          hadoopConf.getBoolean(HoodieCommonConfig.DISK_MAP_BITCASK_COMPRESSION_ENABLED.key(),
-            HoodieCommonConfig.DISK_MAP_BITCASK_COMPRESSION_ENABLED.defaultValue()))
-
-      if (logFiles.nonEmpty) {
-        logRecordScannerBuilder.withPartition(getRelativePartitionPath(
-          new StoragePath(tableState.tablePath), logFiles.head.getPath.getParent))
-      }
-
-      logRecordScannerBuilder.withRecordMerger(
-        HoodieRecordUtils.createRecordMerger(tableState.tablePath, EngineType.SPARK, tableState.recordMergeImplClasses.asJava, tableState.recordMergeStrategyId))
-
-      val scanner = logRecordScannerBuilder.build()
-
-      closing(scanner) {
-        // NOTE: We have to copy record-map (by default immutable copy is exposed)
-        mutable.HashMap(scanner.getRecords.asScala.toSeq: _*)
-      }
+    closing(scanner) {
+      // NOTE: We have to copy record-map (by default immutable copy is exposed)
+      mutable.HashMap(scanner.getRecords.asScala.toSeq: _*)
     }
   }
 
