@@ -24,6 +24,8 @@ import org.apache.hudi.common.model.{FileSlice, HoodieLogFile, OverwriteWithLate
 import org.apache.hudi.common.table.HoodieTableMetaClient
 import org.apache.hudi.storage.StoragePath
 
+import org.apache.hadoop.conf.Configuration
+import org.apache.hadoop.mapred.JobConf
 import org.apache.spark.rdd.RDD
 import org.apache.spark.sql.SQLContext
 import org.apache.spark.sql.catalyst.InternalRow
@@ -90,7 +92,7 @@ abstract class BaseMergeOnReadSnapshotRelation(sqlContext: SQLContext,
    *       by the query), therefore saving on throughput
    */
   protected lazy val mandatoryFieldsForMerging: Seq[String] =
-    Seq(recordKeyField) ++ preCombineFieldOpt.map(Seq(_)).getOrElse(Seq())
+    Seq(recordKeyField) ++ orderingFields
 
   override lazy val mandatoryFields: Seq[String] = mandatoryFieldsForMerging
 
@@ -111,16 +113,21 @@ abstract class BaseMergeOnReadSnapshotRelation(sqlContext: SQLContext,
     val requiredFilters = Seq.empty
     val optionalFilters = filters
     val readers = createBaseFileReaders(tableSchema, requiredSchema, requestedColumns, requiredFilters, optionalFilters)
+    val confWithSchema = embedInternalSchema(new Configuration(conf), internalSchemaOpt)
 
     new HoodieMergeOnReadRDDV2(
       sqlContext.sparkContext,
-      config = jobConf,
+      config = new JobConf(confWithSchema),
+      sqlConf = sqlContext.sparkSession.sessionState.conf,
       fileReaders = readers,
       tableSchema = tableSchema,
       requiredSchema = requiredSchema,
       tableState = tableState,
       mergeType = mergeType,
-      fileSplits = fileSplits)
+      fileSplits = fileSplits,
+      optionalFilters = optionalFilters,
+      metaClient = metaClient,
+      options = optParams)
   }
 
   protected override def collectFileSplits(partitionFilters: Seq[Expression], dataFilters: Seq[Expression]): List[HoodieMergeOnReadFileSplit] = {
