@@ -19,6 +19,7 @@ package org.apache.spark.sql.hudi
 
 import org.apache.hudi.ColumnStatsIndexSupport.{getMaxColumnNameFor, getMinColumnNameFor, getNullCountColumnNameFor, getValueCountColumnNameFor}
 import org.apache.hudi.SparkAdapterSupport
+import org.apache.hudi.SparkAdapterSupport.sparkAdapter
 import org.apache.hudi.common.util.ValidationUtils.checkState
 
 import org.apache.spark.internal.Logging
@@ -28,6 +29,7 @@ import org.apache.spark.sql.catalyst.expressions.{Alias, And, Attribute, Attribu
 import org.apache.spark.sql.catalyst.expressions.Literal.TrueLiteral
 import org.apache.spark.sql.functions.col
 import org.apache.spark.sql.hudi.ColumnStatsExpressionUtils._
+import org.apache.spark.sql.hudi.command.exception.HoodieAnalysisException
 import org.apache.spark.unsafe.types.UTF8String
 
 import java.util.concurrent.atomic.AtomicBoolean
@@ -405,9 +407,9 @@ object DataSkippingUtils extends Logging {
       case Alias(c, _) => getTargetColNameParts(c)
       case GetStructField(c, _, Some(name)) => getTargetColNameParts(c) :+ name
       case ex: ExtractValue =>
-        throw new AnalysisException(s"convert reference to name failed, Updating nested fields is only supported for StructType: ${ex}.")
+        throw new HoodieAnalysisException(s"convert reference to name failed, Updating nested fields is only supported for StructType: ${ex}.")
       case other =>
-        throw new AnalysisException(s"convert reference to name failed,  Found unsupported expression ${other}")
+        throw new HoodieAnalysisException(s"convert reference to name failed,  Found unsupported expression ${other}")
     }
   }
 
@@ -472,10 +474,10 @@ object DataSkippingUtils extends Logging {
 
 object ColumnStatsExpressionUtils {
 
-  @inline def genColMinValueExpr(colName: String): Expression = col(getMinColumnNameFor(colName)).expr
-  @inline def genColMaxValueExpr(colName: String): Expression = col(getMaxColumnNameFor(colName)).expr
-  @inline def genColNumNullsExpr(colName: String): Expression = col(getNullCountColumnNameFor(colName)).expr
-  @inline def genColValueCountExpr: Expression = col(getValueCountColumnNameFor).expr
+  @inline def genColMinValueExpr(colName: String): Expression = sparkAdapter.getExpressionFromColumn(col(getMinColumnNameFor(colName)))
+  @inline def genColMaxValueExpr(colName: String): Expression = sparkAdapter.getExpressionFromColumn(col(getMaxColumnNameFor(colName)))
+  @inline def genColNumNullsExpr(colName: String): Expression = sparkAdapter.getExpressionFromColumn(col(getNullCountColumnNameFor(colName)))
+  @inline def genColValueCountExpr: Expression = sparkAdapter.getExpressionFromColumn(col(getValueCountColumnNameFor))
 
   @inline def genColumnValuesEqualToExpression(colName: String,
                                                value: Expression,
