@@ -18,15 +18,21 @@
 
 package org.apache.hudi.cli;
 
-import org.apache.hudi.client.HoodieTimelineArchiver;
+import org.apache.hudi.client.WriteStatus;
 import org.apache.hudi.client.common.HoodieSparkEngineContext;
+import org.apache.hudi.client.timeline.HoodieTimelineArchiver;
+import org.apache.hudi.client.timeline.TimelineArchivers;
 import org.apache.hudi.common.config.HoodieMetadataConfig;
+import org.apache.hudi.common.data.HoodieData;
 import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.model.HoodieAvroPayload;
+import org.apache.hudi.common.model.HoodieKey;
+import org.apache.hudi.common.model.HoodieRecord;
 import org.apache.hudi.config.HoodieArchivalConfig;
 import org.apache.hudi.config.HoodieCleanConfig;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.table.HoodieSparkTable;
+import org.apache.hudi.util.CommonClientUtils;
 
 import org.apache.spark.api.java.JavaSparkContext;
 import org.slf4j.Logger;
@@ -44,11 +50,11 @@ public final class ArchiveExecutorUtils {
   }
 
   public static int archive(JavaSparkContext jsc,
-       int minCommits,
-       int maxCommits,
-       int commitsRetained,
-       boolean enableMetadata,
-       String basePath) throws IOException {
+                            int minCommits,
+                            int maxCommits,
+                            int commitsRetained,
+                            boolean enableMetadata,
+                            String basePath) throws IOException {
     HoodieWriteConfig config = HoodieWriteConfig.newBuilder().withPath(basePath)
         .withArchivalConfig(HoodieArchivalConfig.newBuilder().archiveCommitsWith(minCommits, maxCommits).build())
         .withCleanConfig(HoodieCleanConfig.newBuilder().retainCommits(commitsRetained).build())
@@ -57,11 +63,13 @@ public final class ArchiveExecutorUtils {
         .build();
     HoodieEngineContext context = new HoodieSparkEngineContext(jsc);
     HoodieSparkTable<HoodieAvroPayload> table = HoodieSparkTable.create(config, context);
+    CommonClientUtils.validateTableVersion(table.getMetaClient().getTableConfig(), config);
     try {
-      HoodieTimelineArchiver archiver = new HoodieTimelineArchiver(config, table);
+      HoodieTimelineArchiver<HoodieAvroPayload, HoodieData<HoodieRecord<HoodieAvroPayload>>, HoodieData<HoodieKey>, HoodieData<WriteStatus>> archiver =
+          TimelineArchivers.getInstance(table.getMetaClient().getTimelineLayoutVersion(), config, table);
       archiver.archiveIfRequired(context, true);
     } catch (IOException ioe) {
-      LOG.error("Failed to archive with IOException: " + ioe);
+      LOG.error("Failed to archive with IOException: {}", ioe.getMessage());
       throw ioe;
     }
     return 0;
