@@ -32,15 +32,14 @@ class RunCleanProcedure extends BaseProcedure with ProcedureBuilder with Logging
 
   private val PARAMETERS = Array[ProcedureParameter](
     ProcedureParameter.required(0, "table", DataTypes.StringType),
-    ProcedureParameter.optional(1, "skip_locking", DataTypes.BooleanType, false),
-    ProcedureParameter.optional(2, "schedule_in_line", DataTypes.BooleanType, true),
-    ProcedureParameter.optional(3, "clean_policy", DataTypes.StringType),
-    ProcedureParameter.optional(4, "retain_commits", DataTypes.IntegerType),
-    ProcedureParameter.optional(5, "hours_retained", DataTypes.IntegerType),
-    ProcedureParameter.optional(6, "file_versions_retained", DataTypes.IntegerType),
-    ProcedureParameter.optional(7, "trigger_strategy", DataTypes.StringType),
-    ProcedureParameter.optional(8, "trigger_max_commits", DataTypes.IntegerType),
-    ProcedureParameter.optional(9, "options", DataTypes.StringType)
+    ProcedureParameter.optional(1, "schedule_in_line", DataTypes.BooleanType, true),
+    ProcedureParameter.optional(2, "clean_policy", DataTypes.StringType),
+    ProcedureParameter.optional(3, "retain_commits", DataTypes.IntegerType),
+    ProcedureParameter.optional(4, "hours_retained", DataTypes.IntegerType),
+    ProcedureParameter.optional(5, "file_versions_retained", DataTypes.IntegerType),
+    ProcedureParameter.optional(6, "trigger_strategy", DataTypes.StringType),
+    ProcedureParameter.optional(7, "trigger_max_commits", DataTypes.IntegerType),
+    ProcedureParameter.optional(8, "options", DataTypes.StringType)
   )
 
   private val OUTPUT_TYPE = new StructType(Array[StructField](
@@ -68,29 +67,28 @@ class RunCleanProcedure extends BaseProcedure with ProcedureBuilder with Logging
     super.checkArgs(PARAMETERS, args)
 
     val tableName = getArgValueOrDefault(args, PARAMETERS(0))
-    val skipLocking = getArgValueOrDefault(args, PARAMETERS(1)).get.asInstanceOf[Boolean]
-    val scheduleInLine = getArgValueOrDefault(args, PARAMETERS(2)).get.asInstanceOf[Boolean]
+    val scheduleInLine = getArgValueOrDefault(args, PARAMETERS(1)).get.asInstanceOf[Boolean]
     var confs: Map[String, String] = Map.empty
+    if (getArgValueOrDefault(args, PARAMETERS(2)).isDefined) {
+      confs += HoodieCleanConfig.CLEANER_POLICY.key() -> getArgValueOrDefault(args, PARAMETERS(2)).get.toString
+    }
     if (getArgValueOrDefault(args, PARAMETERS(3)).isDefined) {
-      confs += HoodieCleanConfig.CLEANER_POLICY.key() -> getArgValueOrDefault(args, PARAMETERS(3)).get.toString
+      confs += HoodieCleanConfig.CLEANER_COMMITS_RETAINED.key() -> getArgValueOrDefault(args, PARAMETERS(3)).get.toString
     }
     if (getArgValueOrDefault(args, PARAMETERS(4)).isDefined) {
-      confs += HoodieCleanConfig.CLEANER_COMMITS_RETAINED.key() -> getArgValueOrDefault(args, PARAMETERS(4)).get.toString
+      confs += HoodieCleanConfig.CLEANER_HOURS_RETAINED.key() -> getArgValueOrDefault(args, PARAMETERS(4)).get.toString
     }
     if (getArgValueOrDefault(args, PARAMETERS(5)).isDefined) {
-      confs += HoodieCleanConfig.CLEANER_HOURS_RETAINED.key() -> getArgValueOrDefault(args, PARAMETERS(5)).get.toString
+      confs += HoodieCleanConfig.CLEANER_FILE_VERSIONS_RETAINED.key() -> getArgValueOrDefault(args, PARAMETERS(5)).get.toString
     }
     if (getArgValueOrDefault(args, PARAMETERS(6)).isDefined) {
-      confs += HoodieCleanConfig.CLEANER_FILE_VERSIONS_RETAINED.key() -> getArgValueOrDefault(args, PARAMETERS(6)).get.toString
+      confs += HoodieCleanConfig.CLEAN_TRIGGER_STRATEGY.key() -> getArgValueOrDefault(args, PARAMETERS(6)).get.toString
     }
     if (getArgValueOrDefault(args, PARAMETERS(7)).isDefined) {
-      confs += HoodieCleanConfig.CLEAN_TRIGGER_STRATEGY.key() -> getArgValueOrDefault(args, PARAMETERS(7)).get.toString
+      confs += HoodieCleanConfig.CLEAN_MAX_COMMITS.key() -> getArgValueOrDefault(args, PARAMETERS(7)).get.toString
     }
     if (getArgValueOrDefault(args, PARAMETERS(8)).isDefined) {
-      confs += HoodieCleanConfig.CLEAN_MAX_COMMITS.key() -> getArgValueOrDefault(args, PARAMETERS(8)).get.toString
-    }
-    if (getArgValueOrDefault(args, PARAMETERS(9)).isDefined) {
-      confs ++= HoodieCLIUtils.extractOptions(getArgValueOrDefault(args, PARAMETERS(9)).get.asInstanceOf[String])
+      confs ++= HoodieCLIUtils.extractOptions(getArgValueOrDefault(args, PARAMETERS(8)).get.asInstanceOf[String])
     }
 
     val basePath = getBasePath(tableName, Option.empty)
@@ -99,8 +97,7 @@ class RunCleanProcedure extends BaseProcedure with ProcedureBuilder with Logging
     try {
       client = HoodieCLIUtils.createHoodieWriteClient(sparkSession, basePath, confs,
         tableName.asInstanceOf[Option[String]])
-      val cleanInstantTime = client.createNewInstantTime()
-      val hoodieCleanMeta = client.clean(cleanInstantTime, scheduleInLine, skipLocking)
+      val hoodieCleanMeta = client.clean(scheduleInLine)
 
       if (hoodieCleanMeta == null) Seq.empty
       else Seq(Row(hoodieCleanMeta.getStartCleanTime,
