@@ -28,7 +28,9 @@ import java.util.function.Supplier
 
 class ShowBootstrapPartitionsProcedure extends BaseProcedure with ProcedureBuilder {
   private val PARAMETERS = Array[ProcedureParameter](
-    ProcedureParameter.required(0, "table", DataTypes.StringType)
+    ProcedureParameter.optional(0, "table", DataTypes.StringType),
+    ProcedureParameter.optional(1, "path", DataTypes.StringType),
+    ProcedureParameter.optional(2, "filter", DataTypes.StringType, "")
   )
 
   private val OUTPUT_TYPE = new StructType(Array[StructField](
@@ -43,14 +45,18 @@ class ShowBootstrapPartitionsProcedure extends BaseProcedure with ProcedureBuild
     super.checkArgs(PARAMETERS, args)
 
     val tableName = getArgValueOrDefault(args, PARAMETERS(0))
+    val tablePath = getArgValueOrDefault(args, PARAMETERS(1))
+    val filter = getArgValueOrDefault(args, PARAMETERS(2)).get.asInstanceOf[String]
 
-    val basePath: String = getBasePath(tableName)
+    validateFilter(filter, outputType)
+    val basePath: String = getBasePath(tableName, tablePath)
     val metaClient = createMetaClient(jsc, basePath)
 
     val indexReader = createBootstrapIndexReader(metaClient)
     val indexedPartitions = indexReader.getIndexedPartitionPaths
 
-    indexedPartitions.stream().toArray.map(r => Row(r)).toList
+    val results = indexedPartitions.stream().toArray.map(r => Row(r)).toList
+    applyFilter(results, filter, outputType)
   }
 
   private def createBootstrapIndexReader(metaClient: HoodieTableMetaClient) = {
