@@ -36,7 +36,9 @@ import scala.collection.JavaConverters.asScalaIteratorConverter
 
 class ShowMetadataTablePartitionsProcedure() extends BaseProcedure with ProcedureBuilder with Logging {
   private val PARAMETERS = Array[ProcedureParameter](
-    ProcedureParameter.required(0, "table", DataTypes.StringType)
+    ProcedureParameter.optional(0, "table", DataTypes.StringType),
+    ProcedureParameter.optional(1, "path", DataTypes.StringType),
+    ProcedureParameter.optional(2, "filter", DataTypes.StringType, "")
   )
 
   private val OUTPUT_TYPE = new StructType(Array[StructField](
@@ -51,8 +53,11 @@ class ShowMetadataTablePartitionsProcedure() extends BaseProcedure with Procedur
     super.checkArgs(PARAMETERS, args)
 
     val table = getArgValueOrDefault(args, PARAMETERS(0))
+    val path = getArgValueOrDefault(args, PARAMETERS(1))
+    val filter = getArgValueOrDefault(args, PARAMETERS(2)).get.asInstanceOf[String]
 
-    val basePath = getBasePath(table)
+    validateFilter(filter, outputType)
+    val basePath = getBasePath(table, path)
     val storage = new HoodieHadoopStorage(basePath, spark.sessionState.newHadoopConf())
     val config = HoodieMetadataConfig.newBuilder.enable(true).build
     val metadata = new HoodieBackedTableMetadata(new HoodieSparkEngineContext(jsc), storage, config, basePath)
@@ -69,7 +74,8 @@ class ShowMetadataTablePartitionsProcedure() extends BaseProcedure with Procedur
     partitions.stream.iterator().asScala.foreach((p: String) => {
         rows.add(Row(p))
     })
-    rows.stream().toArray().map(r => r.asInstanceOf[Row]).toList
+    val results = rows.stream().toArray().map(r => r.asInstanceOf[Row]).toList
+    applyFilter(results, filter, outputType)
   }
 
   override def build: Procedure = new ShowMetadataTablePartitionsProcedure()
