@@ -76,6 +76,7 @@ import org.apache.spark.SparkException;
 import org.apache.spark.api.java.JavaRDD;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Tag;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.Arguments;
@@ -131,6 +132,7 @@ import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 
+@Tag("functional")
 public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
 
   private Properties lockProperties = null;
@@ -285,7 +287,6 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
             .withSecondaryStorageType(FileSystemViewStorageType.MEMORY).build())
         .withWriteConcurrencyMode(WriteConcurrencyMode.OPTIMISTIC_CONCURRENCY_CONTROL)
         .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(InProcessLockProvider.class).build())
-        .withAutoCommit(false)
         .withProperties(properties);
     HoodieWriteConfig writeConfig = writeConfigBuilder.build();
     final SparkRDDWriteClient client1 = getHoodieWriteClient(writeConfig);
@@ -295,7 +296,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     final String nextCommitTime12 = "0012";
     if (createInitialCommit) {
       // Create the first commit ingesting some data.
-      createCommitWithInserts(writeConfig, client1, "000", nextCommitTime11, 100, true);
+      createCommitWithInserts(writeConfig, client1, "000", nextCommitTime11, 100);
       createCommitWithUpserts(writeConfig, client1, nextCommitTime11, Option.empty(), nextCommitTime12, 100);
       totalCommits += 2;
     }
@@ -306,12 +307,13 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
       // to save the checkpoint.
       HoodieWriteConfig writeConfig22 = HoodieWriteConfig.newBuilder().withProperties(writeConfig.getProps()).build();
       writeConfig22.setSchema("\"null\"");
-      final SparkRDDWriteClient client22 = getHoodieWriteClient(writeConfig22);
-      JavaRDD<HoodieRecord> emptyRDD = jsc.emptyRDD();
-      // Perform upsert with empty RDD
-      client22.startCommitWithTime("0013");
-      JavaRDD<WriteStatus> writeStatusRDD = client22.upsert(emptyRDD, "0013");
-      client22.commit("0013", writeStatusRDD);
+      try (final SparkRDDWriteClient client22 = getHoodieWriteClient(writeConfig22)) {
+        JavaRDD<HoodieRecord> emptyRDD = jsc.emptyRDD();
+        // Perform upsert with empty RDD
+        WriteClientTestUtils.startCommitWithTime(client22, "0013");
+        JavaRDD<WriteStatus> writeStatusRDD = client22.upsert(emptyRDD, "0013");
+        client22.commit("0013", writeStatusRDD);
+      }
       totalCommits += 1;
 
       // Validate table schema in the end.
@@ -324,15 +326,13 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     HoodieWriteConfig writeConfig2 = HoodieWriteConfig.newBuilder().withProperties(writeConfig.getProps()).build();
     writeConfig2.setSchema(writerSchema1);
     final SparkRDDWriteClient client2 = getHoodieWriteClient(writeConfig2);
-    final String nextCommitTime21 = "0021";
-    startSchemaEvolutionTransaction(metaClient, client2, nextCommitTime21, tableType);
+    final String nextCommitTime21 = startSchemaEvolutionTransaction(metaClient, client2, tableType);
 
     // Start concurrent txn 003 alter table schema
     HoodieWriteConfig writeConfig3 = HoodieWriteConfig.newBuilder().withProperties(writeConfig.getProps()).build();
     writeConfig3.setSchema(writerSchema2);
     final SparkRDDWriteClient client3 = getHoodieWriteClient(writeConfig3);
-    final String nextCommitTime31 = "0031";
-    startSchemaEvolutionTransaction(metaClient, client3, nextCommitTime31, tableType);
+    final String nextCommitTime31 = startSchemaEvolutionTransaction(metaClient, client3, tableType);
 
     Properties props = new TypedProperties();
     HoodieWriteConfig tableServiceWriteCfg = tableType.equals(MERGE_ON_READ)
@@ -349,7 +349,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     // schedule clustering (COW) or compaction (MOR)
     Option<String> tableServiceInstant = Option.empty();
     if (createInitialCommit) {
-      tableServiceInstant = tableServiceClient.scheduleTableService(tableServiceCommit32,
+      tableServiceInstant = tableServiceClient.scheduleTableService(Option.of(tableServiceCommit32),
         Option.empty(), tableType.equals(MERGE_ON_READ) ? TableServiceType.COMPACT : TableServiceType.CLUSTER);
       if (!writerSchema1.equals(writerSchema2)) {
         assertTrue(tableServiceInstant.isPresent());
@@ -485,7 +485,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
 
     // Create the first commit
     final String nextCommitTime1 = "001";
-    createCommitWithInserts(writeConfig, client1, "000", nextCommitTime1, 200, true);
+    createCommitWithInserts(writeConfig, client1, "000", nextCommitTime1, 200);
 
     final SparkRDDWriteClient client2 = getHoodieWriteClient(writeConfig);
     final SparkRDDWriteClient client3 = getHoodieWriteClient(writeConfig);
@@ -606,10 +606,10 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         .withMarkersType(MarkerType.DIRECT.name())
         .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(providerClass)
             .withConflictResolutionStrategy(resolutionStrategy)
-            .build()).withAutoCommit(false).withProperties(lockProperties).build();
+            .build()).withProperties(lockProperties).build();
 
     // Create the first commit
-    createCommitWithInserts(writeConfig, getHoodieWriteClient(writeConfig), "000", "001", 200, true);
+    createCommitWithInserts(writeConfig, getHoodieWriteClient(writeConfig), "000", "001", 200);
 
     final int threadCount = 2;
     final ExecutorService executors = Executors.newFixedThreadPool(2);
@@ -622,7 +622,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
 
     Future future1 = executors.submit(() -> {
       try {
-        final String nextCommitTime = client1.createNewInstantTime();
+        final String nextCommitTime = WriteClientTestUtils.createNewInstantTime();
         final JavaRDD<WriteStatus> writeStatusList = startCommitForUpdate(writeConfig, client1, nextCommitTime, 100);
 
         // Wait for the 2nd writer to start the commit
@@ -643,7 +643,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
 
     Future future2 = executors.submit(() -> {
       try {
-        final String nextCommitTime = client2.createNewInstantTime();
+        final String nextCommitTime = WriteClientTestUtils.createNewInstantTime();
 
         // Wait for the 1st writer to make progress with the commit
         cyclicBarrier.await(60, TimeUnit.SECONDS);
@@ -692,7 +692,6 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         .withLockConfig(HoodieLockConfig.newBuilder()
             .withLockProvider(InProcessLockProvider.class)
             .build())
-        .withAutoCommit(false)
         .withEmbeddedTimelineServerEnabled(false)
         // Timeline-server-based markers are not used for multi-writer tests
         .withMarkersType(MarkerType.DIRECT.name())
@@ -779,7 +778,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         .withWriteConcurrencyMode(WriteConcurrencyMode.OPTIMISTIC_CONCURRENCY_CONTROL)
         .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(providerClass)
             .withConflictResolutionStrategy(resolutionStrategy)
-            .build()).withAutoCommit(false).withProperties(lockProperties)
+            .build()).withProperties(lockProperties)
         .withProperties(properties);
 
     Set<String> validInstants = new HashSet<>();
@@ -787,14 +786,14 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     // Create the first commit with inserts
     HoodieWriteConfig cfg = writeConfigBuilder.build();
     SparkRDDWriteClient client = getHoodieWriteClient(cfg);
-    String firstCommitTime = client.createNewInstantTime();
-    createCommitWithInserts(cfg, client, "000", firstCommitTime, 200, true);
+    String firstCommitTime = WriteClientTestUtils.createNewInstantTime();
+    createCommitWithInserts(cfg, client, "000", firstCommitTime, 200);
     validInstants.add(firstCommitTime);
 
     // Create 2 commits with upserts
-    String secondCommitTime = client.createNewInstantTime();
+    String secondCommitTime = WriteClientTestUtils.createNewInstantTime();
     createCommitWithUpserts(cfg, client, firstCommitTime, Option.of("000"), secondCommitTime, 100);
-    String thirdCommitTime = client.createNewInstantTime();
+    String thirdCommitTime = WriteClientTestUtils.createNewInstantTime();
     createCommitWithUpserts(cfg, client, secondCommitTime, Option.of("000"), thirdCommitTime, 100);
     validInstants.add(secondCommitTime);
     validInstants.add(thirdCommitTime);
@@ -811,10 +810,10 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
             .withInlineClusteringNumCommits(1)
             .build())
         .build();
-    final SparkRDDWriteClient client1 = getHoodieWriteClient(cfg2);
-    final SparkRDDWriteClient client2 = getHoodieWriteClient(cfg);
-    final SparkRDDWriteClient client3 = getHoodieWriteClient(cfg);
-    final String upsertCommitTime = client1.createNewInstantTime(); // upsert commit time has to be lesser than compaction instant time.
+    final SparkRDDWriteClient<?> client1 = getHoodieWriteClient(cfg2);
+    final SparkRDDWriteClient<?> client2 = getHoodieWriteClient(cfg);
+    final SparkRDDWriteClient<?> client3 = getHoodieWriteClient(cfg);
+    final String upsertCommitTime = WriteClientTestUtils.createNewInstantTime(); // upsert commit time has to be lesser than compaction instant time.
     // and w/ MOR table, during conflict resolution, we will definitely hit conflict resolution exception.
     // if the delta commit's instant time is not guaranteed to be < compaction instant time, then delta commit will succeed w/o issues.
 
@@ -831,32 +830,16 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
       // We want the upsert to go through only after the compaction
       // and cleaning schedule completion. So, waiting on latch here.
       latchCountDownAndWait(scheduleCountDownLatch, waitAndRunSecond);
-      if (tableType == HoodieTableType.MERGE_ON_READ && !(resolutionStrategy instanceof PreferWriterConflictResolutionStrategy)) {
-        // HUDI-6897: Improve SimpleConcurrentFileWritesConflictResolutionStrategy for NB-CC
-        // There is no need to throw concurrent modification exception for the simple strategy under NB-CC, because the compactor would finally resolve the conflicts instead.
-
-        // Since the concurrent modifications went in, this upsert has
-        // to fail
-        assertThrows(HoodieWriteConflictException.class, () -> {
-          createCommitWithUpserts(cfg, client1, thirdCommitTime, Option.of(commitTimeBetweenPrevAndNew), upsertCommitTime, numRecords);
-        });
-      } else {
-        // We don't have the compaction for COW and so this upsert
-        // has to pass
-        final String newCommitTime = client1.createNewInstantTime();
-        assertDoesNotThrow(() -> {
-          createCommitWithUpserts(cfg, client1, thirdCommitTime, Option.of(commitTimeBetweenPrevAndNew), newCommitTime, numRecords);
-        });
-        validInstants.add(newCommitTime);
-      }
+      // Writes should pass since scheduled compaction does not conflict with upsert for v8 and above
+      assertDoesNotThrow(() -> createCommitWithUpserts(cfg, client1, thirdCommitTime, Option.of(commitTimeBetweenPrevAndNew), upsertCommitTime, numRecords));
+      validInstants.add(upsertCommitTime);
     });
 
     Future future2 = executors.submit(() -> {
       if (tableType == MERGE_ON_READ) {
         assertDoesNotThrow(() -> {
-          String compactionTimeStamp = client2.createNewInstantTime();
+          String compactionTimeStamp = client2.scheduleTableService(Option.empty(), TableServiceType.COMPACT).get();
           ValidationUtils.checkArgument(InstantComparison.compareTimestamps(compactionTimeStamp, InstantComparison.GREATER_THAN, upsertCommitTime));
-          client2.scheduleTableService(compactionTimeStamp, Option.empty(), TableServiceType.COMPACT);
         });
       }
       latchCountDownAndWait(scheduleCountDownLatch, waitAndRunFirst);
@@ -865,8 +848,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     Future future3 = executors.submit(() -> {
       assertDoesNotThrow(() -> {
         latchCountDownAndWait(scheduleCountDownLatch, waitAndRunFirst);
-        String cleanCommitTime = client3.createNewInstantTime();
-        client3.scheduleTableService(cleanCommitTime, Option.empty(), TableServiceType.CLEAN);
+        client3.scheduleTableService(Option.empty(), TableServiceType.CLEAN);
       });
     });
     future1.get();
@@ -881,16 +863,16 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         .firstInstant();
     String pendingCleanTime = pendingCleanInstantOp.isPresent()
         ? pendingCleanInstantOp.get().requestedTime()
-        : client.createNewInstantTime();
+        : WriteClientTestUtils.createNewInstantTime();
 
     CountDownLatch runCountDownLatch = new CountDownLatch(threadCount);
     // Create inserts, run cleaning, run compaction in parallel
     future1 = executors.submit(() -> {
-      final String newCommitTime = client1.createNewInstantTime();
+      final String newCommitTime = WriteClientTestUtils.createNewInstantTime();
       final int numRecords = 100;
       latchCountDownAndWait(runCountDownLatch, waitAndRunSecond);
       assertDoesNotThrow(() -> {
-        createCommitWithInserts(cfg, client1, thirdCommitTime, newCommitTime, numRecords, true);
+        createCommitWithInserts(cfg, client1, thirdCommitTime, newCommitTime, numRecords);
         validInstants.add(newCommitTime);
       });
     });
@@ -900,7 +882,8 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
       if (tableType == HoodieTableType.MERGE_ON_READ) {
         assertDoesNotThrow(() -> {
           HoodieWriteMetadata<JavaRDD<WriteStatus>> compactionMetadata = client2.compact(pendingCompactionTime);
-          client2.commitCompaction(pendingCompactionTime, compactionMetadata.getCommitMetadata().get(), Option.empty());
+          client2.commitCompaction(pendingCompactionTime, compactionMetadata, Option.empty());
+          assertTrue(metaClient.reloadActiveTimeline().filterCompletedInstants().containsInstant(pendingCompactionTime));
           validInstants.add(pendingCompactionTime);
         });
       }
@@ -909,7 +892,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     future3 = executors.submit(() -> {
       latchCountDownAndWait(runCountDownLatch, waitAndRunFirst);
       assertDoesNotThrow(() -> {
-        client3.clean(pendingCleanTime, false);
+        client3.clean(false);
         validInstants.add(pendingCleanTime);
       });
     });
@@ -955,12 +938,12 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         .withWriteConcurrencyMode(WriteConcurrencyMode.OPTIMISTIC_CONCURRENCY_CONTROL)
         // Set the config so that heartbeat will expire in 1 second without update
         .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(InProcessLockProvider.class)
-            .build()).withAutoCommit(false).withProperties(lockProperties);
+            .build()).withProperties(lockProperties);
     Set<String> validInstants = new HashSet<>();
     // Create the first commit with inserts
     HoodieWriteConfig cfg = writeConfigBuilder.build();
     SparkRDDWriteClient client = getHoodieWriteClient(cfg);
-    createCommitWithInserts(cfg, client, "000", "001", 200, true);
+    createCommitWithInserts(cfg, client, "000", "001", 200);
     validInstants.add("001");
 
     // Three clients running actions in parallel
@@ -1038,7 +1021,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         .withMarkersType(MarkerType.DIRECT.name())
         .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(InProcessLockProvider.class)
             .withConflictResolutionStrategy(new SimpleConcurrentFileWritesConflictResolutionStrategy())
-            .build()).withAutoCommit(false).withProperties(properties);
+            .build()).withProperties(properties);
     HoodieWriteConfig cfg = writeConfigBuilder.build();
     HoodieWriteConfig cfg2 = writeConfigBuilder.build();
     HoodieWriteConfig cfg3 = writeConfigBuilder
@@ -1047,7 +1030,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
 
     // Create the first commit
     try (SparkRDDWriteClient client = getHoodieWriteClient(cfg)) {
-      createCommitWithInserts(cfg, client, "000", "001", 200, true);
+      createCommitWithInserts(cfg, client, "000", "001", 200);
     }
     // Start another inflight commit
     String newCommitTime = "003";
@@ -1056,14 +1039,13 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     String commitTimeBetweenPrevAndNew = "002";
     JavaRDD<WriteStatus> result1 = updateBatch(cfg, client1, newCommitTime, "001",
         Option.of(Arrays.asList(commitTimeBetweenPrevAndNew)), "000", numRecords, SparkRDDWriteClient::upsert, false, false,
-        numRecords, 200, 2, INSTANT_GENERATOR);
+        numRecords, 200, 2, true, INSTANT_GENERATOR, true);
     // Start and finish another commit while the previous writer for commit 003 is running
     newCommitTime = "004";
     SparkRDDWriteClient client2 = getHoodieWriteClient(cfg);
     JavaRDD<WriteStatus> result2 = updateBatch(cfg2, client2, newCommitTime, "001",
         Option.of(Arrays.asList(commitTimeBetweenPrevAndNew)), "000", numRecords, SparkRDDWriteClient::upsert, false, false,
         numRecords, 200, 2, INSTANT_GENERATOR);
-    client2.commit(newCommitTime, result2);
     // Schedule and run clustering while previous writer for commit 003 is running
     SparkRDDWriteClient client3 = getHoodieWriteClient(cfg3);
     // schedule clustering
@@ -1092,13 +1074,13 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         // Timeline-server-based markers are not used for multi-writer tests
         .withMarkersType(MarkerType.DIRECT.name())
         .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(InProcessLockProvider.class)
-            .build()).withAutoCommit(true).withProperties(lockProperties);
+            .build()).withProperties(lockProperties);
     HoodieWriteConfig cfg = writeConfigBuilder.build();
     HoodieWriteConfig cfg2 = writeConfigBuilder.build();
 
     // Create the first commit
     try (SparkRDDWriteClient client = getHoodieWriteClient(cfg)) {
-      createCommitWithInserts(cfg, client, "000", "001", 5000, false);
+      createCommitWithInserts(cfg, client, "000", "001", 5000);
     }
     // Start another inflight commit
     String newCommitTime1 = "003";
@@ -1180,13 +1162,13 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         // Timeline-server-based markers are not used for multi-writer tests
         .withMarkersType(MarkerType.DIRECT.name())
         .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(InProcessLockProvider.class)
-            .build()).withAutoCommit(true).withProperties(properties);
+            .build()).withProperties(properties);
     HoodieWriteConfig cfg = writeConfigBuilder.build();
     HoodieWriteConfig cfg2 = writeConfigBuilder.build();
 
     // Create the first commit
     try (SparkRDDWriteClient client = getHoodieWriteClient(cfg)) {
-      createCommitWithInserts(cfg, client, "000", "001", 200, false);
+      createCommitWithInserts(cfg, client, "000", "001", 200);
     }
     // Start another inflight commit
     String newCommitTime1 = "003";
@@ -1216,14 +1198,14 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     HoodieWriteConfig.Builder writeConfigBuilder = getConfigBuilder()
         .withMetadataConfig(HoodieMetadataConfig.newBuilder()
             .withMetadataIndexColumnStats(true)
-            .withEnableRecordIndex(true).build())
+            .withEnableGlobalRecordLevelIndex(true).build())
         .withArchivalConfig(HoodieArchivalConfig.newBuilder()
             .withAutoArchive(false).build())
         .withWriteConcurrencyMode(WriteConcurrencyMode.OPTIMISTIC_CONCURRENCY_CONTROL)
         .withMarkersType(MarkerType.DIRECT.name())
         .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(InProcessLockProvider.class)
             .withConflictResolutionStrategy(new SimpleConcurrentFileWritesConflictResolutionStrategy())
-            .build()).withAutoCommit(false);
+            .build());
     HoodieWriteConfig writeConfig1 = writeConfigBuilder.build();
 
     // clean every commit for writer2
@@ -1238,7 +1220,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     // Simulate the first commit with Writer 1
     final SparkRDDWriteClient client1 = getHoodieWriteClient(writeConfig1);
     final SparkRDDWriteClient client2 = getHoodieWriteClient(writeConfig2);
-    createCommitWithInserts(writeConfig1, getHoodieWriteClient(writeConfig1), client1.createNewInstantTime(), client1.createNewInstantTime(), 200, true);
+    createCommitWithInserts(writeConfig1, getHoodieWriteClient(writeConfig1), WriteClientTestUtils.createNewInstantTime(), WriteClientTestUtils.createNewInstantTime(), 200);
 
     // multi-writer setup
     final int threadCount = 2;
@@ -1250,7 +1232,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     // Writer 1 - Simulating the index update process
     Future future1 = executors.submit(() -> {
       try {
-        final String nextCommitTime = client1.createNewInstantTime();
+        final String nextCommitTime = WriteClientTestUtils.createNewInstantTime();
         final JavaRDD<WriteStatus> writeStatusList = startCommitForUpdate(writeConfig1, client1, nextCommitTime, 100);
 
         // Wait for Writer 2 to start cleaning
@@ -1278,7 +1260,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
         // Simulate aggressive cleaning
         metaClient.reloadActiveTimeline();
         HoodieTable table = HoodieSparkTable.create(writeConfig2, context, metaClient);
-        table.clean(context, client2.createNewInstantTime()); // clean old file slices
+        table.clean(context, WriteClientTestUtils.createNewInstantTime()); // clean old file slices
 
         // Signal Writer 1 to complete its update
         cyclicBarrier.await(60, TimeUnit.SECONDS);
@@ -1303,11 +1285,11 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
   private void ingestBatch(Function3<JavaRDD<WriteStatus>, SparkRDDWriteClient, JavaRDD<HoodieRecord>, String> writeFn,
                            SparkRDDWriteClient writeClient, String commitTime, JavaRDD<HoodieRecord> records,
                            CountDownLatch countDownLatch) throws IOException, InterruptedException {
-    writeClient.startCommitWithTime(commitTime);
+    WriteClientTestUtils.startCommitWithTime(writeClient, commitTime);
     countDownLatch.countDown();
     countDownLatch.await();
-    JavaRDD<WriteStatus> statusJavaRDD = writeFn.apply(writeClient, records, commitTime);
-    statusJavaRDD.collect();
+    List<WriteStatus> statuses = writeFn.apply(writeClient, records, commitTime).collect();
+    writeClient.commit(commitTime, jsc.parallelize(statuses));
   }
 
   private void createCommitWithInsertsForPartition(HoodieWriteConfig cfg, SparkRDDWriteClient client,
@@ -1315,29 +1297,34 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
                                                    String partition) throws Exception {
     JavaRDD<WriteStatus> result = insertBatch(cfg, client, newCommitTime, prevCommitTime, numRecords, SparkRDDWriteClient::insert,
         false, false, numRecords, numRecords, 1, Option.of(partition), INSTANT_GENERATOR);
-    assertTrue(client.commit(newCommitTime, result), "Commit should succeed");
+  }
+
+  private JavaRDD<WriteStatus> createCommitWithInserts(HoodieWriteConfig cfg, SparkRDDWriteClient client,
+                                                       String prevCommitTime, String newCommitTime, int numRecords) throws Exception {
+    return createCommitWithInserts(cfg, client, prevCommitTime, newCommitTime, numRecords, true);
   }
 
   private JavaRDD<WriteStatus> createCommitWithInserts(HoodieWriteConfig cfg, SparkRDDWriteClient client,
                                                        String prevCommitTime, String newCommitTime, int numRecords,
                                                        boolean doCommit) throws Exception {
     // Finish first base commit
-    JavaRDD<WriteStatus> result = insertFirstBatch(cfg, client, newCommitTime, prevCommitTime, numRecords, SparkRDDWriteClient::bulkInsert,
-        false, false, numRecords, INSTANT_GENERATOR);
+    List<WriteStatus> result = insertFirstBatch(cfg, client, newCommitTime, prevCommitTime, numRecords, SparkRDDWriteClient::bulkInsert,
+        false, false, numRecords, true, INSTANT_GENERATOR, true).collect();
     if (doCommit) {
-      assertTrue(client.commit(newCommitTime, result), "Commit should succeed");
+      assertTrue(client.commit(newCommitTime, jsc.parallelize(result)), "Commit should succeed");
     }
-    return result;
+    return jsc.parallelize(result);
   }
 
-  private static void startSchemaEvolutionTransaction(HoodieTableMetaClient metaClient, SparkRDDWriteClient client, String nextCommitTime2, HoodieTableType tableType) throws IOException {
+  private static String startSchemaEvolutionTransaction(HoodieTableMetaClient metaClient, SparkRDDWriteClient client, HoodieTableType tableType) throws IOException {
     String commitActionType = CommitUtils.getCommitActionType(WriteOperationType.UPSERT, tableType);
-    client.startCommitWithTime(nextCommitTime2, commitActionType);
-    client.preWrite(nextCommitTime2, WriteOperationType.UPSERT, client.createMetaClient(true));
-    HoodieInstant requested = metaClient.createNewInstant(HoodieInstant.State.REQUESTED, commitActionType, nextCommitTime2);
+    String instant = client.startCommit(commitActionType);
+    client.preWrite(instant, WriteOperationType.UPSERT, client.createMetaClient(true));
+    HoodieInstant requested = metaClient.createNewInstant(HoodieInstant.State.REQUESTED, commitActionType, instant);
     HoodieCommitMetadata metadata = new HoodieCommitMetadata();
     metadata.setOperationType(WriteOperationType.UPSERT);
     client.createMetaClient(true).getActiveTimeline().transitionRequestedToInflight(requested, Option.of(metadata));
+    return instant;
   }
 
   private void createCommitWithUpserts(HoodieWriteConfig cfg, SparkRDDWriteClient client, String prevCommit,
@@ -1347,7 +1334,6 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
     JavaRDD<WriteStatus> result = updateBatch(cfg, client, newCommitTime, prevCommit,
         Option.of(commitsBetweenPrevAndNew), "000", numRecords, SparkRDDWriteClient::upsert, false, false,
         numRecords, 200, 2, INSTANT_GENERATOR);
-    client.commit(newCommitTime, result);
   }
 
   /**
@@ -1363,7 +1349,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
   private JavaRDD<WriteStatus> startCommitForUpdate(HoodieWriteConfig writeConfig, SparkRDDWriteClient writeClient,
                                                     String newCommitTime, int numRecords) throws Exception {
     // Start the new commit
-    writeClient.startCommitWithTime(newCommitTime);
+    WriteClientTestUtils.startCommitWithTime(writeClient, newCommitTime);
 
     // Prepare update records
     final Function2<List<HoodieRecord>, String, Integer> recordGenFunction =
@@ -1418,7 +1404,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
           .withAsyncConflictDetectorInitialDelayMs(0)
           .withAsyncConflictDetectorPeriodMs(100)
           .withLockConfig(HoodieLockConfig.newBuilder().withLockProvider(lockProvider).build())
-          .withAutoCommit(false).withProperties(properties).build();
+          .withProperties(properties).build();
     } else {
       return getConfigBuilder()
           .withStorageConfig(HoodieStorageConfig.newBuilder().parquetMaxFileSize(20 * 1024).build())
@@ -1443,7 +1429,7 @@ public class TestHoodieClientMultiWriter extends HoodieClientTestBase {
           .withEarlyConflictDetectionStrategy(earlyConflictDetectionStrategy)
           .withAsyncConflictDetectorInitialDelayMs(0)
           .withAsyncConflictDetectorPeriodMs(100)
-          .withAutoCommit(false).withProperties(properties).build();
+          .withProperties(properties).build();
     }
   }
 }
