@@ -21,6 +21,7 @@ package org.apache.hudi.functional;
 
 import org.apache.hudi.DataSourceWriteOptions;
 import org.apache.hudi.client.SparkRDDWriteClient;
+import org.apache.hudi.client.WriteClientTestUtils;
 import org.apache.hudi.common.config.HoodieStorageConfig;
 import org.apache.hudi.common.model.HoodieBaseFile;
 import org.apache.hudi.common.model.HoodieRecord;
@@ -75,14 +76,71 @@ class TestHoodieSparkMergeOnReadTableClustering extends SparkClientFunctionalTes
   @ParameterizedTest
   @MethodSource
   void testClustering(boolean clusteringAsRow, boolean doUpdates, boolean populateMetaFields) throws Exception {
-    // set low compaction small File Size to generate more file groups.
+    HoodieWriteConfig cfg = getHoodieWriteConfig(populateMetaFields, false);
+    HoodieTableMetaClient metaClient = getHoodieMetaClient(HoodieTableType.MERGE_ON_READ, cfg.getProps());
+    HoodieTestDataGenerator dataGen = new HoodieTestDataGenerator();
+    int totalFiles = 0;
+    try (SparkRDDWriteClient client = getHoodieWriteClient(cfg)) {
+
+      /*
+       * Write 1 (only inserts)
+       */
+      String newCommitTime = "001";
+      WriteClientTestUtils.startCommitWithTime(client, newCommitTime);
+
+      List<HoodieRecord> records = dataGen.generateInserts(newCommitTime, 400);
+      Stream<HoodieBaseFile> dataFiles = insertRecordsToMORTable(metaClient, records.subList(0, 200), client, cfg, newCommitTime);
+      assertTrue(dataFiles.findAny().isPresent(), "should list the base files we wrote in the delta commit");
+
+      /*
+       * Write 2 (more inserts to create new files)
+       */
+      // we already set small file size to small number to force inserts to go into new file.
+      newCommitTime = "002";
+      WriteClientTestUtils.startCommitWithTime(client, newCommitTime);
+      dataFiles = insertRecordsToMORTable(metaClient, records.subList(200, 400), client, cfg, newCommitTime);
+      assertTrue(dataFiles.findAny().isPresent(), "should list the base files we wrote in the delta commit");
+
+      if (doUpdates) {
+        /*
+         * Write 3 (updates)
+         */
+        newCommitTime = "003";
+        WriteClientTestUtils.startCommitWithTime(client, newCommitTime);
+        records = dataGen.generateUpdates(newCommitTime, 100);
+        updateRecordsInMORTable(metaClient, records, client, cfg, newCommitTime, false);
+      }
+
+      HoodieTable hoodieTable = HoodieSparkTable.create(cfg, context(), metaClient);
+      hoodieTable.getHoodieView().sync();
+      List<StoragePathInfo> allFiles = listAllBaseFilesInPath(hoodieTable);
+      totalFiles = allFiles.size();
+      // expect 2 base files for each partition
+      assertEquals(dataGen.getPartitionPaths().length * 2, allFiles.size());
+    }
+
+    HoodieWriteConfig cfgClusteringEnabled = getHoodieWriteConfig(populateMetaFields, true);
+    try (SparkRDDWriteClient client = getHoodieWriteClient(cfgClusteringEnabled)) {
+
+      String clusteringCommitTime = client.scheduleClustering(Option.empty()).get().toString();
+      metaClient = HoodieTableMetaClient.reload(metaClient);
+      HoodieTable hoodieTable = HoodieSparkTable.create(cfg, context(), metaClient);
+      // verify all files are included in clustering plan.
+      assertEquals(totalFiles,
+          hoodieTable.getFileSystemView().getFileGroupsInPendingClustering().map(Pair::getLeft).count());
+
+      // Do the clustering and validate
+      doClusteringAndValidate(client, clusteringCommitTime, metaClient, cfg, dataGen, clusteringAsRow);
+    }
+  }
+
+  private HoodieWriteConfig getHoodieWriteConfig(boolean populateMetaFields, boolean enableInlineClustering) {
     HoodieWriteConfig.Builder cfgBuilder = HoodieWriteConfig.newBuilder()
         .forTable("test-trip-table")
         .withPath(basePath())
         .withSchema(TRIP_EXAMPLE_SCHEMA)
         .withParallelism(2, 2)
         .withDeleteParallelism(2)
-        .withAutoCommit(true)
         .withCompactionConfig(HoodieCompactionConfig.newBuilder()
             .compactionSmallFileSize(10L)
             .withInlineCompaction(false).withMaxNumDeltaCommitsBeforeCompaction(1).build())
@@ -96,62 +154,12 @@ class TestHoodieSparkMergeOnReadTableClustering extends SparkClientFunctionalTes
         .withClusteringConfig(HoodieClusteringConfig.newBuilder()
             .withClusteringMaxNumGroups(10)
             .withClusteringTargetPartitions(0)
-            .withInlineClustering(true)
+            .withInlineClustering(enableInlineClustering)
             .withInlineClusteringNumCommits(1)
             .build())
         .withRollbackUsingMarkers(false);
     addConfigsForPopulateMetaFields(cfgBuilder, populateMetaFields);
-    HoodieWriteConfig cfg = cfgBuilder.build();
-    HoodieTableMetaClient metaClient = getHoodieMetaClient(HoodieTableType.MERGE_ON_READ, cfg.getProps());
-    HoodieTestDataGenerator dataGen = new HoodieTestDataGenerator();
-
-    try (SparkRDDWriteClient client = getHoodieWriteClient(cfg)) {
-
-      /*
-       * Write 1 (only inserts)
-       */
-      String newCommitTime = "001";
-      client.startCommitWithTime(newCommitTime);
-
-      List<HoodieRecord> records = dataGen.generateInserts(newCommitTime, 400);
-      Stream<HoodieBaseFile> dataFiles = insertRecordsToMORTable(metaClient, records.subList(0, 200), client, cfg, newCommitTime);
-      assertTrue(dataFiles.findAny().isPresent(), "should list the base files we wrote in the delta commit");
-
-      /*
-       * Write 2 (more inserts to create new files)
-       */
-      // we already set small file size to small number to force inserts to go into new file.
-      newCommitTime = "002";
-      client.startCommitWithTime(newCommitTime);
-      dataFiles = insertRecordsToMORTable(metaClient, records.subList(200, 400), client, cfg, newCommitTime);
-      assertTrue(dataFiles.findAny().isPresent(), "should list the base files we wrote in the delta commit");
-
-      if (doUpdates) {
-        /*
-         * Write 3 (updates)
-         */
-        newCommitTime = "003";
-        client.startCommitWithTime(newCommitTime);
-        records = dataGen.generateUpdates(newCommitTime, 100);
-        updateRecordsInMORTable(metaClient, records, client, cfg, newCommitTime, false);
-      }
-
-      HoodieTable hoodieTable = HoodieSparkTable.create(cfg, context(), metaClient);
-      hoodieTable.getHoodieView().sync();
-      List<StoragePathInfo> allFiles = listAllBaseFilesInPath(hoodieTable);
-      // expect 2 base files for each partition
-      assertEquals(dataGen.getPartitionPaths().length * 2, allFiles.size());
-
-      String clusteringCommitTime = client.scheduleClustering(Option.empty()).get().toString();
-      metaClient = HoodieTableMetaClient.reload(metaClient);
-      hoodieTable = HoodieSparkTable.create(cfg, context(), metaClient);
-      // verify all files are included in clustering plan.
-      assertEquals(allFiles.size(),
-          hoodieTable.getFileSystemView().getFileGroupsInPendingClustering().map(Pair::getLeft).count());
-
-      // Do the clustering and validate
-      doClusteringAndValidate(client, clusteringCommitTime, metaClient, cfg, dataGen, clusteringAsRow);
-    }
+    return cfgBuilder.build();
   }
 
   private static Stream<Arguments> testClusteringWithNoBaseFiles() {
@@ -168,14 +176,56 @@ class TestHoodieSparkMergeOnReadTableClustering extends SparkClientFunctionalTes
   @ParameterizedTest
   @MethodSource
   void testClusteringWithNoBaseFiles(boolean clusteringAsRow, boolean doUpdates, boolean shouldWriteRecordPositions) throws Exception {
-    // set low compaction small File Size to generate more file groups.
+    HoodieWriteConfig cfg = getWriteConfigToTestClusteringWithNoBaseFiles(shouldWriteRecordPositions, false);
+    HoodieTableMetaClient metaClient = getHoodieMetaClient(HoodieTableType.MERGE_ON_READ, cfg.getProps());
+    HoodieTestDataGenerator dataGen = new HoodieTestDataGenerator();
+
+    try (SparkRDDWriteClient client = getHoodieWriteClient(cfg)) {
+      // test 2 inserts
+      String newCommitTime = "001";
+      WriteClientTestUtils.startCommitWithTime(client, newCommitTime);
+      List<HoodieRecord> records = dataGen.generateInserts(newCommitTime, 400);
+      Stream<HoodieBaseFile> dataFiles = insertRecordsToMORTable(metaClient, records.subList(0, 200), client, cfg, newCommitTime);
+      assertTrue(!dataFiles.findAny().isPresent(), "should not have any base files");
+      newCommitTime = "002";
+      WriteClientTestUtils.startCommitWithTime(client, newCommitTime);
+      dataFiles = insertRecordsToMORTable(metaClient, records.subList(200, 400), client, cfg, newCommitTime);
+      assertTrue(!dataFiles.findAny().isPresent(), "should not have any base files");
+      // run updates
+      if (doUpdates) {
+        newCommitTime = "003";
+        WriteClientTestUtils.startCommitWithTime(client, newCommitTime);
+        records = dataGen.generateUpdates(newCommitTime, 100);
+        updateRecordsInMORTable(metaClient, records, client, cfg, newCommitTime, false);
+      }
+
+      HoodieTable hoodieTable = HoodieSparkTable.create(cfg, context(), metaClient);
+      hoodieTable.getHoodieView().sync();
+      List<StoragePathInfo> allBaseFiles = listAllBaseFilesInPath(hoodieTable);
+      // expect 0 base files for each partition
+      assertEquals(0, allBaseFiles.size());
+    }
+
+    HoodieWriteConfig cfgClusteringEnabled = getWriteConfigToTestClusteringWithNoBaseFiles(shouldWriteRecordPositions, true);
+    try (SparkRDDWriteClient client = getHoodieWriteClient(cfgClusteringEnabled)) {
+      String clusteringCommitTime = client.scheduleClustering(Option.empty()).get().toString();
+      metaClient = HoodieTableMetaClient.reload(metaClient);
+      HoodieTable hoodieTable = HoodieSparkTable.create(cfg, context(), metaClient);
+      // verify log files are included in clustering plan for each partition.
+      assertEquals(dataGen.getPartitionPaths().length, hoodieTable.getFileSystemView().getFileGroupsInPendingClustering().map(Pair::getLeft).count());
+
+      // do the clustering and validate
+      doClusteringAndValidate(client, clusteringCommitTime, metaClient, cfg, dataGen, clusteringAsRow);
+    }
+  }
+
+  private HoodieWriteConfig getWriteConfigToTestClusteringWithNoBaseFiles(boolean shouldWriteRecordPositions, boolean enableInlineClustering) {
     HoodieWriteConfig.Builder cfgBuilder = HoodieWriteConfig.newBuilder()
         .forTable("test-trip-table")
         .withPath(basePath())
         .withSchema(TRIP_EXAMPLE_SCHEMA)
         .withParallelism(2, 2)
         .withDeleteParallelism(2)
-        .withAutoCommit(true)
         .withCompactionConfig(HoodieCompactionConfig.newBuilder()
             .compactionSmallFileSize(10L)
             .withInlineCompaction(false).withMaxNumDeltaCommitsBeforeCompaction(1).build())
@@ -192,47 +242,10 @@ class TestHoodieSparkMergeOnReadTableClustering extends SparkClientFunctionalTes
         .withClusteringConfig(HoodieClusteringConfig.newBuilder()
             .withClusteringMaxNumGroups(10)
             .withClusteringTargetPartitions(0)
-            .withInlineClustering(true)
+            .withInlineClustering(enableInlineClustering)
             .withInlineClusteringNumCommits(1).build())
         .withRollbackUsingMarkers(false);
-    HoodieWriteConfig cfg = cfgBuilder.build();
-    HoodieTableMetaClient metaClient = getHoodieMetaClient(HoodieTableType.MERGE_ON_READ, cfg.getProps());
-    HoodieTestDataGenerator dataGen = new HoodieTestDataGenerator();
-
-    try (SparkRDDWriteClient client = getHoodieWriteClient(cfg)) {
-      // test 2 inserts
-      String newCommitTime = "001";
-      client.startCommitWithTime(newCommitTime);
-      List<HoodieRecord> records = dataGen.generateInserts(newCommitTime, 400);
-      Stream<HoodieBaseFile> dataFiles = insertRecordsToMORTable(metaClient, records.subList(0, 200), client, cfg, newCommitTime);
-      assertTrue(!dataFiles.findAny().isPresent(), "should not have any base files");
-      newCommitTime = "002";
-      client.startCommitWithTime(newCommitTime);
-      dataFiles = insertRecordsToMORTable(metaClient, records.subList(200, 400), client, cfg, newCommitTime);
-      assertTrue(!dataFiles.findAny().isPresent(), "should not have any base files");
-      // run updates
-      if (doUpdates) {
-        newCommitTime = "003";
-        client.startCommitWithTime(newCommitTime);
-        records = dataGen.generateUpdates(newCommitTime, 100);
-        updateRecordsInMORTable(metaClient, records, client, cfg, newCommitTime, false);
-      }
-
-      HoodieTable hoodieTable = HoodieSparkTable.create(cfg, context(), metaClient);
-      hoodieTable.getHoodieView().sync();
-      List<StoragePathInfo> allBaseFiles = listAllBaseFilesInPath(hoodieTable);
-      // expect 0 base files for each partition
-      assertEquals(0, allBaseFiles.size());
-
-      String clusteringCommitTime = client.scheduleClustering(Option.empty()).get().toString();
-      metaClient = HoodieTableMetaClient.reload(metaClient);
-      hoodieTable = HoodieSparkTable.create(cfg, context(), metaClient);
-      // verify log files are included in clustering plan for each partition.
-      assertEquals(dataGen.getPartitionPaths().length, hoodieTable.getFileSystemView().getFileGroupsInPendingClustering().map(Pair::getLeft).count());
-
-      // do the clustering and validate
-      doClusteringAndValidate(client, clusteringCommitTime, metaClient, cfg, dataGen, clusteringAsRow);
-    }
+    return cfgBuilder.build();
   }
 
   private void doClusteringAndValidate(SparkRDDWriteClient client,
