@@ -24,8 +24,8 @@ import org.apache.hudi.testutils.HoodieClientTestUtils
 
 import org.apache.commons.io.FileUtils
 import org.apache.spark.SparkContext
+import org.apache.spark.sql.{Dataset, Row, SparkSession, SQLContext}
 import org.apache.spark.sql.hudi.HoodieSparkSessionExtension
-import org.apache.spark.sql.{Dataset, Row, SQLContext, SparkSession}
 import org.junit.jupiter.api.{AfterEach, BeforeEach}
 
 import scala.collection.JavaConverters
