@@ -20,6 +20,7 @@ package org.apache.hudi
 
 import org.apache.hudi.common.config.{HoodieMetadataConfig, HoodieReaderConfig}
 import org.apache.hudi.common.model.HoodieTableType
+import org.apache.hudi.common.table.HoodieTableConfig
 import org.apache.hudi.config.HoodieWriteConfig
 import org.apache.hudi.exception.SchemaCompatibilityException
 import org.apache.hudi.testutils.HoodieClientTestBase
@@ -45,7 +46,7 @@ class TestAvroSchemaResolutionSupport extends HoodieClientTestBase with ScalaAss
     "hoodie.insert.shuffle.parallelism" -> "1",
     "hoodie.upsert.shuffle.parallelism" -> "1",
     DataSourceWriteOptions.RECORDKEY_FIELD.key -> "id",
-    DataSourceWriteOptions.PRECOMBINE_FIELD.key -> "id",
+    HoodieTableConfig.ORDERING_FIELDS.key() -> "id",
     DataSourceWriteOptions.PARTITIONPATH_FIELD.key -> "name",
     DataSourceWriteOptions.KEYGENERATOR_CLASS_NAME.key -> "org.apache.hudi.keygen.SimpleKeyGenerator",
     HoodieMetadataConfig.ENABLE.key -> "false"
@@ -127,8 +128,6 @@ class TestAvroSchemaResolutionSupport extends HoodieClientTestBase with ScalaAss
     def doTest(colInitType: String, start: Int, end: Int): Unit = {
       for (a <- Range(start, end)) {
         try {
-          Console.println(s"Performing test: $a with initialColType of: $colInitType")
-
           // convert int to string first before conversion to binary
           val initDF = prepDataFrame(df1, colInitType)
           initDF.printSchema()
@@ -874,9 +873,8 @@ class TestAvroSchemaResolutionSupport extends HoodieClientTestBase with ScalaAss
     upsertData(df2, tempRecordPath, isCow)
 
     // after implicit type change, read the table with vectorized read enabled
-    //fg reader with mor does not support vectorized currently and will auto read by row
-    if (isCow || !useFileGroupReader) {
-      assertThrows(classOf[SparkException]){
+    if (HoodieSparkUtils.gteqSpark3_4 || !useFileGroupReader) {
+      assertThrows(classOf[SparkException]) {
         withSQLConf("spark.sql.parquet.enableNestedColumnVectorizedReader" -> "true") {
           readTable(tempRecordPath, useFileGroupReader)
         }
