@@ -18,9 +18,10 @@
 package org.apache.hudi
 
 import org.apache.hudi.ColumnStatsIndexSupport.composeIndexSchema
+import org.apache.hudi.SparkAdapterSupport.sparkAdapter
 import org.apache.hudi.testutils.HoodieSparkClientTestBase
 
-import org.apache.spark.sql.{Column, Row, SparkSession}
+import org.apache.spark.sql.{Row, SparkSession}
 import org.apache.spark.sql.HoodieCatalystExpressionUtils.resolveExpr
 import org.apache.spark.sql.catalyst.analysis.UnresolvedAttribute
 import org.apache.spark.sql.catalyst.encoders.DummyExpressionHolder
@@ -132,7 +133,7 @@ class TestDataSkippingUtils extends HoodieSparkClientTestBase with SparkAdapterS
 
     val indexDf = spark.createDataset(input)
 
-    val rows = indexDf.where(new Column(lookupFilter))
+    val rows = indexDf.where(sparkAdapter.createColumnFromExpression(lookupFilter))
       .select("fileName")
       .collect()
       .map(_.getString(0))
@@ -159,7 +160,7 @@ class TestDataSkippingUtils extends HoodieSparkClientTestBase with SparkAdapterS
 
     val indexDf = spark.createDataFrame(input.map(_.toRow).asJava, indexSchema)
 
-    indexDf.where(new Column(lookupFilter))
+    indexDf.where(sparkAdapter.createColumnFromExpression(lookupFilter))
       .select("fileName")
       .collect()
       .map(_.getString(0))
@@ -171,7 +172,7 @@ object TestDataSkippingUtils {
   def testStringsLookupFilterExpressionsSource(): java.util.stream.Stream[Arguments] = {
     java.util.stream.Stream.of(
       arguments(
-        col("B").startsWith("abc").expr,
+        sparkAdapter.getExpressionFromColumn(col("B").startsWith("abc")),
         Seq(
           IndexRow("file_1", valueCount = 1, B_minValue = "aba", B_maxValue = "adf", B_nullCount = 1), // may contain strings starting w/ "abc"
           IndexRow("file_2", valueCount = 1, B_minValue = "adf", B_maxValue = "azy", B_nullCount = 0),
@@ -179,7 +180,7 @@ object TestDataSkippingUtils {
         ),
         Seq("file_1")),
       arguments(
-        Not(col("B").startsWith("abc").expr),
+        Not(sparkAdapter.getExpressionFromColumn(col("B").startsWith("abc"))),
         Seq(
           IndexRow("file_1", valueCount = 1, B_minValue = "aba", B_maxValue = "adf", B_nullCount = 1), // may contain strings starting w/ "abc"
           IndexRow("file_2", valueCount = 1, B_minValue = "adf", B_maxValue = "azy", B_nullCount = 0),
@@ -189,7 +190,7 @@ object TestDataSkippingUtils {
         Seq("file_1", "file_2", "file_3")),
       arguments(
         // Composite expression
-        Not(lower(col("B")).startsWith("abc").expr),
+        Not(sparkAdapter.getExpressionFromColumn(lower(col("B")).startsWith("abc"))),
         Seq(
           IndexRow("file_1", valueCount = 1, B_minValue = "ABA", B_maxValue = "ADF", B_nullCount = 1), // may contain strings starting w/ "ABC" (after upper)
           IndexRow("file_2", valueCount = 1, B_minValue = "ADF", B_maxValue = "AZY", B_nullCount = 0),
