@@ -20,10 +20,12 @@ package org.apache.hudi.functional
 
 import org.apache.hudi.DataSourceWriteOptions._
 import org.apache.hudi.common.config.{HoodieMetadataConfig, TypedProperties}
+import org.apache.hudi.common.data.HoodieListData
 import org.apache.hudi.common.model.HoodieTableType
 import org.apache.hudi.common.table.{HoodieTableConfig, HoodieTableMetaClient}
 import org.apache.hudi.common.table.timeline.HoodieInstant
-import org.apache.hudi.common.testutils.RawTripTestPayload.recordsToStrings
+import org.apache.hudi.common.testutils.HoodieTestDataGenerator.recordsToStrings
+import org.apache.hudi.common.util.HoodieDataUtils
 import org.apache.hudi.common.util.Option
 import org.apache.hudi.config.{HoodieClusteringConfig, HoodieWriteConfig}
 import org.apache.hudi.metadata.{HoodieBackedTableMetadata, HoodieTableMetadataUtil, MetadataPartitionType}
@@ -31,7 +33,7 @@ import org.apache.hudi.testutils.HoodieSparkClientTestBase
 
 import org.apache.spark.sql._
 import org.junit.jupiter.api._
-import org.junit.jupiter.api.Assertions.{assertEquals, assertFalse, assertTrue}
+import org.junit.jupiter.api.Assertions.{assertEquals, assertTrue}
 import org.junit.jupiter.params.ParameterizedTest
 import org.junit.jupiter.params.provider.EnumSource
 
@@ -46,7 +48,7 @@ class TestMetadataRecordIndex extends HoodieSparkClientTestBase {
   var instantTime: AtomicInteger = _
   val metadataOpts = Map(
     HoodieMetadataConfig.ENABLE.key -> "true",
-    HoodieMetadataConfig.RECORD_INDEX_ENABLE_PROP.key -> "true"
+    HoodieMetadataConfig.GLOBAL_RECORD_LEVEL_INDEX_ENABLE_PROP.key -> "true"
   )
   val commonOpts = Map(
     "hoodie.insert.shuffle.parallelism" -> "4",
@@ -54,7 +56,7 @@ class TestMetadataRecordIndex extends HoodieSparkClientTestBase {
     HoodieWriteConfig.TBL_NAME.key -> "hoodie_test",
     RECORDKEY_FIELD.key -> "_row_key",
     PARTITIONPATH_FIELD.key -> "partition",
-    PRECOMBINE_FIELD.key -> "timestamp",
+    HoodieTableConfig.ORDERING_FIELDS.key -> "timestamp",
     HoodieTableConfig.POPULATE_META_FIELDS.key -> "true"
   ) ++ metadataOpts
   var mergedDfList: List[DataFrame] = List.empty
@@ -80,34 +82,42 @@ class TestMetadataRecordIndex extends HoodieSparkClientTestBase {
     cleanupSparkContexts()
   }
 
+  protected def withRDDPersistenceValidation(f: => Unit): Unit = {
+    org.apache.hudi.testutils.SparkRDDValidationUtils.withRDDPersistenceValidation(spark, new org.apache.hudi.testutils.SparkRDDValidationUtils.ThrowingRunnable {
+      override def run(): Unit = f
+    })
+  }
+
   @ParameterizedTest
   @EnumSource(classOf[HoodieTableType])
   def testClusteringWithRecordIndex(tableType: HoodieTableType): Unit = {
-    val hudiOpts = commonOpts ++ Map(
-      TABLE_TYPE.key -> tableType.name(),
-      HoodieClusteringConfig.INLINE_CLUSTERING.key() -> "true",
-      HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key() -> "2"
-    )
+    withRDDPersistenceValidation {
+      val hudiOpts = commonOpts ++ Map(
+        TABLE_TYPE.key -> tableType.name(),
+        HoodieClusteringConfig.INLINE_CLUSTERING.key() -> "true",
+        HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key() -> "2"
+      )
 
-    doWriteAndValidateDataAndRecordIndex(hudiOpts,
-      operation = INSERT_OPERATION_OPT_VAL,
-      saveMode = SaveMode.Overwrite)
-    doWriteAndValidateDataAndRecordIndex(hudiOpts,
-      operation = UPSERT_OPERATION_OPT_VAL,
-      saveMode = SaveMode.Append)
+      doWriteAndValidateDataAndRecordIndex(hudiOpts,
+        operation = INSERT_OPERATION_OPT_VAL,
+        saveMode = SaveMode.Overwrite)
+      doWriteAndValidateDataAndRecordIndex(hudiOpts,
+        operation = UPSERT_OPERATION_OPT_VAL,
+        saveMode = SaveMode.Append)
 
-    val lastClusteringInstant = getLatestClusteringInstant()
-    assertTrue(lastClusteringInstant.isPresent)
+      val lastClusteringInstant = getLatestClusteringInstant()
+      assertTrue(lastClusteringInstant.isPresent)
 
-    doWriteAndValidateDataAndRecordIndex(hudiOpts,
-      operation = UPSERT_OPERATION_OPT_VAL,
-      saveMode = SaveMode.Append)
-    doWriteAndValidateDataAndRecordIndex(hudiOpts,
-      operation = UPSERT_OPERATION_OPT_VAL,
-      saveMode = SaveMode.Append)
+      doWriteAndValidateDataAndRecordIndex(hudiOpts,
+        operation = UPSERT_OPERATION_OPT_VAL,
+        saveMode = SaveMode.Append)
+      doWriteAndValidateDataAndRecordIndex(hudiOpts,
+        operation = UPSERT_OPERATION_OPT_VAL,
+        saveMode = SaveMode.Append)
 
-    assertTrue(getLatestClusteringInstant().get().requestedTime.compareTo(lastClusteringInstant.get().requestedTime) > 0)
-    validateDataAndRecordIndices(hudiOpts)
+      assertTrue(getLatestClusteringInstant().get().requestedTime.compareTo(lastClusteringInstant.get().requestedTime) > 0)
+      validateDataAndRecordIndices(hudiOpts)
+    }
   }
 
   private def getLatestClusteringInstant(): Option[HoodieInstant] = {
@@ -148,9 +158,7 @@ class TestMetadataRecordIndex extends HoodieSparkClientTestBase {
       val prevDf = prevDfOpt.get
       val prevDfOld = prevDf.join(inputDF1, prevDf("_row_key") === inputDF1("_row_key")
         && prevDf("partition") === inputDF1("partition"), "leftanti")
-      prevDfOld.show(500, false)
       val unionDf = prevDfOld.union(inputDF1)
-      unionDf.show(500, false)
       mergedDfList = mergedDfList :+ unionDf
     }
   }
@@ -178,8 +186,10 @@ class TestMetadataRecordIndex extends HoodieSparkClientTestBase {
     val metadata = metadataWriter(writeConfig).getTableMetadata
     val readDf = spark.read.format("hudi").load(basePath)
     val rowArr = readDf.collect()
-    val recordIndexMap = metadata.readRecordIndex(
-      rowArr.map(row => row.getAs("_hoodie_record_key").toString).toList.asJava)
+    val res = metadata.readRecordIndexLocationsWithKeys(
+      HoodieListData.eager(rowArr.map(row => row.getAs("_hoodie_record_key").toString).toList.asJava));
+    val recordIndexMap = HoodieDataUtils.dedupeAndCollectAsMap(res);
+    res.unpersistWithDependencies()
 
     assertTrue(rowArr.length > 0)
     for (row <- rowArr) {
@@ -196,18 +206,15 @@ class TestMetadataRecordIndex extends HoodieSparkClientTestBase {
 
     assertEquals(rowArr.length, recordIndexMap.keySet.size)
     val estimatedFileGroupCount = HoodieTableMetadataUtil.estimateFileGroupCount(
-      MetadataPartitionType.RECORD_INDEX, rowArr.length, 48,
-      writeConfig.getRecordIndexMinFileGroupCount, writeConfig.getRecordIndexMaxFileGroupCount,
+      MetadataPartitionType.RECORD_INDEX, () => rowArr.length, 48,
+      writeConfig.getGlobalRecordLevelIndexMinFileGroupCount, writeConfig.getGlobalRecordLevelIndexMaxFileGroupCount,
       writeConfig.getRecordIndexGrowthFactor, writeConfig.getRecordIndexMaxFileGroupSizeBytes)
     assertEquals(estimatedFileGroupCount, getFileGroupCountForRecordIndex(writeConfig))
     val prevDf = mergedDfList.last.drop("tip_history")
     val nonMatchingRecords = readDf.drop("_hoodie_commit_time", "_hoodie_commit_seqno", "_hoodie_record_key",
       "_hoodie_partition_path", "_hoodie_file_name", "tip_history")
       .join(prevDf, prevDf.columns, "leftanti")
-    nonMatchingRecords.show(500, false)
     assertEquals(0, nonMatchingRecords.count())
-    readDf.show(500, false)
-    prevDf.show(500, false)
     assertEquals(readDf.count(), prevDf.count())
   }
 }
