@@ -19,9 +19,10 @@ package org.apache.hudi.functional
 
 import org.apache.hudi.DataSourceReadOptions
 import org.apache.hudi.DataSourceReadOptions.{START_OFFSET, STREAMING_READ_TABLE_VERSION}
-import org.apache.hudi.DataSourceWriteOptions.{PRECOMBINE_FIELD, RECORDKEY_FIELD}
+import org.apache.hudi.DataSourceWriteOptions.{ORDERING_FIELDS, RECORDKEY_FIELD}
+import org.apache.hudi.common.model.HoodieTableType
 import org.apache.hudi.common.model.HoodieTableType.{COPY_ON_WRITE, MERGE_ON_READ}
-import org.apache.hudi.common.table.{HoodieTableMetaClient, HoodieTableVersion}
+import org.apache.hudi.common.table.{HoodieTableConfig, HoodieTableMetaClient, HoodieTableVersion}
 import org.apache.hudi.common.table.timeline.HoodieTimeline
 import org.apache.hudi.config.{HoodieClusteringConfig, HoodieCompactionConfig}
 import org.apache.hudi.config.HoodieWriteConfig.{DELETE_PARALLELISM_VALUE, INSERT_PARALLELISM_VALUE, TBL_NAME, UPSERT_PARALLELISM_VALUE, WRITE_TABLE_VERSION}
@@ -37,7 +38,7 @@ class TestStreamingSource extends StreamTest {
   import testImplicits._
   protected val commonOptions: Map[String, String] = Map(
     RECORDKEY_FIELD.key -> "id",
-    PRECOMBINE_FIELD.key -> "ts",
+    HoodieTableConfig.ORDERING_FIELDS.key -> "ts",
     INSERT_PARALLELISM_VALUE.key -> "4",
     UPSERT_PARALLELISM_VALUE.key -> "4",
     DELETE_PARALLELISM_VALUE.key -> "4"
@@ -60,7 +61,7 @@ class TestStreamingSource extends StreamTest {
         .setTableType(COPY_ON_WRITE)
         .setTableName(getTableName(tablePath))
         .setRecordKeyFields("id")
-        .setPreCombineField("ts")
+        .setOrderingFields("ts")
         .initTable(HadoopFSUtils.getStorageConf(spark.sessionState.newHadoopConf()), tablePath)
 
       addData(tablePath, Seq(("1", "a1", "10", "000")))
@@ -111,7 +112,7 @@ class TestStreamingSource extends StreamTest {
         .setTableType(MERGE_ON_READ)
         .setTableName(getTableName(tablePath))
         .setRecordKeyFields("id")
-        .setPreCombineField("ts")
+        .setOrderingFields("ts")
         .initTable(HadoopFSUtils.getStorageConf(spark.sessionState.newHadoopConf()), tablePath)
 
       addData(tablePath, Seq(("1", "a1", "10", "000")))
@@ -156,7 +157,7 @@ class TestStreamingSource extends StreamTest {
         .setTableType(COPY_ON_WRITE)
         .setTableName(getTableName(tablePath))
         .setRecordKeyFields("id")
-        .setPreCombineField("ts")
+        .setOrderingFields("ts")
         .initTable(HadoopFSUtils.getStorageConf(spark.sessionState.newHadoopConf()), tablePath)
 
       addData(tablePath, Seq(("1", "a1", "10", "000")))
@@ -187,7 +188,7 @@ class TestStreamingSource extends StreamTest {
         .setTableType(COPY_ON_WRITE)
         .setTableName(getTableName(tablePath))
         .setRecordKeyFields("id")
-        .setPreCombineField("ts")
+        .setOrderingFields("ts")
         .initTable(HadoopFSUtils.getStorageConf(spark.sessionState.newHadoopConf()), tablePath)
 
       addData(tablePath, Seq(("1", "a1", "10", "000")))
@@ -213,59 +214,55 @@ class TestStreamingSource extends StreamTest {
   }
 
   test("Test mor streaming source with clustering") {
-    Array("true", "false").foreach(skipCluster => {
-      withTempDir { inputDir =>
-        val tablePath = s"${inputDir.getCanonicalPath}/test_mor_stream_cluster"
-        val metaClient = HoodieTableMetaClient.newTableBuilder()
-          .setTableType(MERGE_ON_READ)
-          .setTableName(getTableName(tablePath))
-          .setRecordKeyFields("id")
-          .setPreCombineField("ts")
-          .initTable(HadoopFSUtils.getStorageConf(spark.sessionState.newHadoopConf()), tablePath)
+    withTempDir { inputDir =>
+      val tablePath = s"${inputDir.getCanonicalPath}/test_mor_stream_cluster"
+      val metaClient = HoodieTableMetaClient.newTableBuilder()
+        .setTableType(MERGE_ON_READ)
+        .setTableName(getTableName(tablePath))
+        .setRecordKeyFields("id")
+        .setOrderingFields("ts")
+        .initTable(HadoopFSUtils.getStorageConf(spark.sessionState.newHadoopConf()), tablePath)
 
-        addData(tablePath, Seq(("1", "a1", "10", "000")))
-        addData(tablePath, Seq(("2", "a1", "11", "001")))
-        addData(tablePath, Seq(("3", "a1", "12", "002")))
-        addData(tablePath, Seq(("4", "a1", "13", "003")), enableInlineCluster = true)
-        addData(tablePath, Seq(("5", "a1", "14", "004")))
+      addData(tablePath, Seq(("1", "a1", "10", "000")))
+      addData(tablePath, Seq(("2", "a1", "11", "001")))
+      addData(tablePath, Seq(("3", "a1", "12", "002")))
+      addData(tablePath, Seq(("4", "a1", "13", "003")), enableInlineCluster = true)
+      addData(tablePath, Seq(("5", "a1", "14", "004")))
 
-        val timestamp =
-          metaClient.getActiveTimeline.getCommitsTimeline.filterCompletedInstants()
-            .firstInstant().get().getCompletionTime
+      val timestamp =
+        metaClient.getActiveTimeline.getCommitsTimeline.filterCompletedInstants()
+          .firstInstant().get().getCompletionTime
 
-        val df = spark.readStream
-          .format("org.apache.hudi")
-          .option(START_OFFSET.key(), timestamp)
-          .option(DataSourceReadOptions.INCREMENTAL_READ_SKIP_CLUSTER.key(), skipCluster)
-          .load(tablePath)
-          .select("id", "name", "price", "ts")
+      val df = spark.readStream
+        .format("org.apache.hudi")
+        .option(START_OFFSET.key(), timestamp)
+        .load(tablePath)
+        .select("id", "name", "price", "ts")
 
-        testStream(df)(
-          AssertOnQuery { q => q.processAllAvailable(); true },
-          // Start after the first commit
-          CheckAnswerRows(Seq(
-            Row("2", "a1", "11", "001"),
-            Row("3", "a1", "12", "002"),
-            Row("4", "a1", "13", "003"),
-            Row("5", "a1", "14", "004")), lastOnly = true, isSorted = false)
-        )
-        assertTrue(metaClient.reloadActiveTimeline
-          .filter(JavaConversions.getPredicate(
-            e => e.isCompleted && HoodieTimeline.REPLACE_COMMIT_ACTION.equals(e.getAction)))
-          .countInstants() > 0)
-      }
-    })
+      testStream(df)(
+        AssertOnQuery { q => q.processAllAvailable(); true },
+        // Start after the first commit
+        CheckAnswerRows(Seq(
+          Row("2", "a1", "11", "001"),
+          Row("3", "a1", "12", "002"),
+          Row("4", "a1", "13", "003"),
+          Row("5", "a1", "14", "004")), lastOnly = true, isSorted = false))
+      assertTrue(metaClient.reloadActiveTimeline
+        .filter(JavaConversions.getPredicate(
+          e => e.isCompleted && HoodieTimeline.REPLACE_COMMIT_ACTION.equals(e.getAction)))
+        .countInstants() > 0)
+    }
   }
 
   test("test mor stream source with compaction") {
     Array("true", "false").foreach(skipCompact => {
       withTempDir { inputDir =>
-        val tablePath = s"${inputDir.getCanonicalPath}/test_mor_stream"
+        val tablePath = s"${inputDir.getCanonicalPath}/test_mor_stream_$skipCompact"
         val metaClient = HoodieTableMetaClient.newTableBuilder()
           .setTableType(MERGE_ON_READ)
           .setTableName(getTableName(tablePath))
           .setRecordKeyFields("id")
-          .setPreCombineField("ts")
+          .setOrderingFields("ts")
           .initTable(HadoopFSUtils.getStorageConf(spark.sessionState.newHadoopConf()), tablePath)
 
         addData(tablePath, Seq(("1", "a1", "10", "000")))
@@ -297,61 +294,120 @@ class TestStreamingSource extends StreamTest {
     })
   }
 
-  test("Test checkpoint translation") {
+  private def testCheckpointTranslation(tableName: String,
+                                        tableType: HoodieTableType,
+                                        writeTableVersion: HoodieTableVersion,
+                                        streamingReadVersions: List[Int]): Unit = {
     withTempDir { inputDir =>
-      val tablePath = s"${inputDir.getCanonicalPath}/test_cow_stream_ckpt"
+      val tablePath = s"${inputDir.getCanonicalPath}/$tableName"
       val metaClient = HoodieTableMetaClient.newTableBuilder()
-        .setTableType(COPY_ON_WRITE)
+        .setTableType(tableType)
         .setTableName(getTableName(tablePath))
+        .setTableVersion(writeTableVersion)
         .setRecordKeyFields("id")
-        .setPreCombineField("ts")
+        .setOrderingFields("ts")
         .initTable(HadoopFSUtils.getStorageConf(spark.sessionState.newHadoopConf()), tablePath)
 
-      addData(tablePath, Seq(("1", "a1", "10", "000")))
-      addData(tablePath, Seq(("2", "a1", "11", "001")))
-      addData(tablePath, Seq(("3", "a1", "12", "002")))
+      // Add initial data
+      addData(tablePath, Seq(("1", "a1", "10", "000")), tableVersion = writeTableVersion)
+      addData(tablePath, Seq(("2", "a1", "11", "001")), tableVersion = writeTableVersion)
+      addData(tablePath, Seq(("3", "a1", "12", "002")), tableVersion = writeTableVersion)
+
+      // Add update for MOR tests
+      if (tableType == MERGE_ON_READ) {
+        addData(tablePath, Seq(("2", "a2_updated", "16", "003")), tableVersion = writeTableVersion)
+      }
 
       val instants = metaClient.getActiveTimeline.getCommitsTimeline.filterCompletedInstants.getInstants
-      assertEquals(3, instants.size())
+      val expectedInstantCount = if (tableType == MERGE_ON_READ) 4 else 3
+      assertEquals(expectedInstantCount, instants.size())
 
-      // If the request time is used, i.e., V1, then the second record is included in the output.
-      // Otherwise, only third record in the output.
-      val startTimestamp = instants.get(1).requestedTime
-      for (streamingReadTableVersion <- List(HoodieTableVersion.SIX.versionCode(), HoodieTableVersion.EIGHT.versionCode())) {
+      val startTimestampIndex = if (tableType == MERGE_ON_READ) 2 else 1
+      val startTimestamp = instants.get(startTimestampIndex).requestedTime
+
+      for (streamingReadTableVersion <- streamingReadVersions) {
         val df = spark.readStream
           .format("org.apache.hudi")
           .option(START_OFFSET.key, startTimestamp)
-          .option(WRITE_TABLE_VERSION.key, HoodieTableVersion.current().versionCode().toString)
+          .option(WRITE_TABLE_VERSION.key, writeTableVersion.versionCode().toString)
           .option(STREAMING_READ_TABLE_VERSION.key, streamingReadTableVersion.toString)
           .load(tablePath)
           .select("id", "name", "price", "ts")
-        val expectedRows = if (streamingReadTableVersion == HoodieTableVersion.EIGHT.versionCode()) {
-          Seq(Row("2", "a1", "11", "001"), Row("3", "a1", "12", "002"))
+
+        val expectedRows = if (tableType == MERGE_ON_READ) {
+          if (streamingReadTableVersion == HoodieTableVersion.current().versionCode()) {
+            Seq(Row("3", "a1", "12", "002"), Row("2", "a2_updated", "16", "003"))
+          } else {
+            Seq(Row("2", "a2_updated", "16", "003"))
+          }
         } else {
-          Seq(Row("3", "a1", "12", "002"))
+          if (streamingReadTableVersion == HoodieTableVersion.current().versionCode()) {
+            Seq(Row("2", "a1", "11", "001"), Row("3", "a1", "12", "002"))
+          } else {
+            Seq(Row("3", "a1", "12", "002"))
+          }
         }
+
         testStream(df)(
           AssertOnQuery { q => q.processAllAvailable(); true },
-          // Start after the first commit
           CheckAnswerRows(expectedRows, lastOnly = true, isSorted = false)
         )
       }
     }
   }
 
+  test("Test checkpoint translation on COW table") {
+    testCheckpointTranslation(
+      "test_cow_stream_ckpt",
+      COPY_ON_WRITE,
+      HoodieTableVersion.current(),
+      List(HoodieTableVersion.SIX.versionCode(), HoodieTableVersion.current().versionCode())
+    )
+  }
+
+  test("Test checkpoint translation on MOR table") {
+    testCheckpointTranslation(
+      "test_mor_stream_ckpt",
+      MERGE_ON_READ,
+      HoodieTableVersion.current(),
+      List(HoodieTableVersion.SIX.versionCode(), HoodieTableVersion.current().versionCode())
+    )
+  }
+
+  test("Test checkpoint translation on COW table with table version 6") {
+    testCheckpointTranslation(
+      "test_cow_stream_ckpt_v6",
+      COPY_ON_WRITE,
+      HoodieTableVersion.SIX,
+      List(HoodieTableVersion.SIX.versionCode())
+    )
+  }
+
+  test("Test checkpoint translation on MOR table with table version 6") {
+    testCheckpointTranslation(
+      "test_mor_stream_ckpt_v6",
+      MERGE_ON_READ,
+      HoodieTableVersion.SIX,
+      List(HoodieTableVersion.SIX.versionCode())
+    )
+  }
+
   private def addData(inputPath: String,
                       rows: Seq[(String, String, String, String)],
                       enableInlineCompaction: Boolean = false,
-                      enableInlineCluster: Boolean = false) : Unit = {
+                      enableInlineCluster: Boolean = false,
+                      tableVersion: HoodieTableVersion = HoodieTableVersion.current) : Unit = {
     rows.toDF(columns: _*)
       .write
       .format("org.apache.hudi")
       .options(commonOptions)
       .option(TBL_NAME.key, getTableName(inputPath))
+      .option(WRITE_TABLE_VERSION.key, tableVersion.versionCode().toString)
       .option(HoodieCompactionConfig.INLINE_COMPACT.key(), enableInlineCompaction.toString)
       .option(HoodieCompactionConfig.INLINE_COMPACT_NUM_DELTA_COMMITS.key(), "2")
       .option(HoodieClusteringConfig.INLINE_CLUSTERING.key(), enableInlineCluster.toString)
       .option(HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key(), "2")
+      .option(HoodieCompactionConfig.PARQUET_SMALL_FILE_LIMIT.key, "0")
       .mode(SaveMode.Append)
       .save(inputPath)
   }
