@@ -18,7 +18,7 @@
 
 package org.apache.spark.sql.hudi.common
 
-import org.apache.hudi.AvroConversionUtils
+import org.apache.hudi.{AvroConversionUtils, SparkRowSerDe}
 import org.apache.hudi.SparkAdapterSupport.sparkAdapter
 import org.apache.hudi.avro.HoodieAvroUtils
 import org.apache.hudi.internal.schema.Types
@@ -79,29 +79,153 @@ class TestHoodieInternalRowUtils extends FunSuite with Matchers with BeforeAndAf
     val data = sparkSession.sparkContext.parallelize(rows)
     val oldRow = sparkSession.createDataFrame(data, mergedSchema).queryExecution.toRdd.first()
 
-    val rowWriter1 = HoodieInternalRowUtils.genUnsafeRowWriter(mergedSchema, schema1, JCollections.emptyMap())
+    val rowWriter1 = HoodieInternalRowUtils.genUnsafeRowWriter(mergedSchema, schema1, JCollections.emptyMap(), JCollections.emptyMap())
     val newRow1 = rowWriter1(oldRow)
 
-    val serDe1 = sparkAdapter.createSparkRowSerDe(schema1)
+    val serDe1 = createSparkRowSerDe(schema1)
     assertEquals(serDe1.deserializeRow(newRow1), Row("Andrew", 18, Row("Mission st", "SF")));
 
-    val rowWriter2 = HoodieInternalRowUtils.genUnsafeRowWriter(mergedSchema, schema2, JCollections.emptyMap())
+    val rowWriter2 = HoodieInternalRowUtils.genUnsafeRowWriter(mergedSchema, schema2, JCollections.emptyMap(), JCollections.emptyMap())
     val newRow2 = rowWriter2(oldRow)
 
-    val serDe2 = sparkAdapter.createSparkRowSerDe(schema2)
+    val serDe2 = createSparkRowSerDe(schema2)
     assertEquals(serDe2.deserializeRow(newRow2), Row("John", 19));
   }
 
   test("Test simple rewriting (with nullable value)") {
     val data = sparkSession.sparkContext.parallelize(Seq(Row("Rob", 18, null.asInstanceOf[StructType])))
     val oldRow = sparkSession.createDataFrame(data, schema1).queryExecution.toRdd.first()
-    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(schema1, mergedSchema, JCollections.emptyMap())
+    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(schema1, mergedSchema, JCollections.emptyMap(), JCollections.emptyMap())
     val newRow = rowWriter(oldRow)
 
-    val serDe = sparkAdapter.createSparkRowSerDe(mergedSchema)
+    val serDe = createSparkRowSerDe(mergedSchema)
     assertEquals(serDe.deserializeRow(newRow), Row("Rob", 18, null.asInstanceOf[StructType], null.asInstanceOf[StringType], null.asInstanceOf[IntegerType]))
   }
 
+  test("Test rewriting with field value injections") {
+    val rowWithNull = Seq(
+      Row("Andrew", null, Row("Mission st", "SF"), "John", 19)
+    )
+    val oldRow = sparkSession.createDataFrame(sparkSession.sparkContext.parallelize(rowWithNull), mergedSchema).queryExecution.toRdd.first()
+
+    val updatedValuesMap: java.util.Map[Integer, Object] = JCollections.singletonMap(1, 18).asInstanceOf[java.util.Map[Integer, Object]]
+    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(mergedSchema, schema1, JCollections.emptyMap(), updatedValuesMap)
+    val newRow1 = rowWriter(oldRow)
+
+    val serDe = createSparkRowSerDe(schema1)
+    assertEquals(serDe.deserializeRow(newRow1), Row("Andrew", 18, Row("Mission st", "SF")));
+
+    // non-nul value should not be rewritten
+    val rowWithoutNull = Seq(
+      Row("Andrew", 25, Row("Mission st", "SF"), "John", 19)
+    )
+    val oldRow2 = sparkSession.createDataFrame(sparkSession.sparkContext.parallelize(rowWithoutNull), mergedSchema).queryExecution.toRdd.first()
+    val newRow2 = rowWriter(oldRow2)
+    assertEquals(serDe.deserializeRow(newRow2), Row("Andrew", 25, Row("Mission st", "SF")));
+  }
+
+  test("Test rewrite row with renamed columns") {
+    // Original schema
+    val oldSchema = StructType(Seq(
+      StructField("first_name", StringType),
+      StructField("years_old", IntegerType)
+    ))
+
+    // Renamed schema
+    val newSchema = StructType(Seq(
+      StructField("name", StringType),
+      StructField("age", IntegerType)
+    ))
+
+    // Rename mapping: new -> old
+    val renameMap: java.util.Map[String, String] = new java.util.HashMap()
+    renameMap.put("name", "first_name")
+    renameMap.put("age", "years_old")
+
+    // Sample row
+    val oldRowData = sparkSession.sparkContext.parallelize(Seq(Row("Alice", 30)))
+    val oldRow = sparkSession.createDataFrame(oldRowData, oldSchema).queryExecution.toRdd.first()
+
+    // Generate writer with rename map
+    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(oldSchema, newSchema, renameMap, JCollections.emptyMap())
+    val newRow = rowWriter(oldRow)
+
+    val serDe = createSparkRowSerDe(newSchema)
+    assertEquals(Row("Alice", 30), serDe.deserializeRow(newRow))
+  }
+
+  test("Test rewrite row with columns swap") {
+    // Original schema
+    val oldSchema = StructType(Seq(
+      StructField("first_name", StringType),
+      StructField("years_old", IntegerType)
+    ))
+
+    // Renamed schema
+    val newSchema = StructType(Seq(
+      StructField("years_old", StringType),
+      StructField("first_name", IntegerType)
+    ))
+
+    // Rename mapping: new -> old
+    val renameMap: java.util.Map[String, String] = new java.util.HashMap()
+    renameMap.put("years_old", "first_name")
+    renameMap.put("first_name", "years_old")
+
+    // Sample row
+    val oldRowData = sparkSession.sparkContext.parallelize(Seq(Row("Alice", 30)))
+    val oldRow = sparkSession.createDataFrame(oldRowData, oldSchema).queryExecution.toRdd.first()
+
+    // Generate writer with rename map
+    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(oldSchema, newSchema, renameMap, JCollections.emptyMap())
+    val newRow = rowWriter(oldRow)
+
+    val serDe = createSparkRowSerDe(newSchema)
+    assertEquals(Row("Alice", 30), serDe.deserializeRow(newRow))
+  }
+
+  test("Test rewrite row with columns swap nested") {
+    // Original schema
+    val oldSchema = StructType(Seq(
+      StructField("first_name", StringType),
+      StructField("years_old", IntegerType),
+      StructField("address",
+        StructType(Seq(
+          StructField("city", StringType),
+          StructField("street", StringType)
+        )
+    ))))
+
+    // Renamed schema
+    val newSchema = StructType(Seq(
+      StructField("years_old", StringType),
+      StructField("first_name", IntegerType),
+      StructField("address",
+        StructType(Seq(
+          StructField("street", StringType),
+          StructField("city", StringType)
+        )
+        ))))
+
+    // Rename mapping: new -> old
+    val renameMap: java.util.Map[String, String] = new java.util.HashMap()
+    renameMap.put("years_old", "first_name")
+    renameMap.put("first_name", "years_old")
+    renameMap.put("address.city", "street")
+    renameMap.put("address.street", "city")
+
+    // Sample row
+    val oldRowData = sparkSession.sparkContext.parallelize(Seq(Row("Alice", 30, Row("SF", "Mission st"))))
+    val oldRow = sparkSession.createDataFrame(oldRowData, oldSchema).queryExecution.toRdd.first()
+
+    // Generate writer with rename map
+    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(oldSchema, newSchema, renameMap, JCollections.emptyMap())
+    val newRow = rowWriter(oldRow)
+
+    val serDe = createSparkRowSerDe(newSchema)
+    assertEquals(Row("Alice", 30, Row("SF", "Mission st")), serDe.deserializeRow(newRow))
+  }
+
   /**
    * test record data type changes.
    * int => long/float/double/string
@@ -192,7 +316,7 @@ class TestHoodieInternalRowUtils extends FunSuite with Matchers with BeforeAndAf
     val newRowExpected = AvroConversionUtils.createAvroToInternalRowConverter(newAvroSchema, newStructTypeSchema)
       .apply(newRecord).get
 
-    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(structTypeSchema, newStructTypeSchema, new HashMap[String, String])
+    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(structTypeSchema, newStructTypeSchema, JCollections.emptyMap(), JCollections.emptyMap())
     val newRow = rowWriter(row)
 
     internalRowCompare(newRowExpected, newRow, newStructTypeSchema)
@@ -247,7 +371,7 @@ class TestHoodieInternalRowUtils extends FunSuite with Matchers with BeforeAndAf
     val row = AvroConversionUtils.createAvroToInternalRowConverter(schema, structTypeSchema).apply(avroRecord).get
     val newRowExpected = AvroConversionUtils.createAvroToInternalRowConverter(newAvroSchema, newStructTypeSchema).apply(newAvroRecord).get
 
-    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(structTypeSchema, newStructTypeSchema, new HashMap[String, String])
+    val rowWriter = HoodieInternalRowUtils.genUnsafeRowWriter(structTypeSchema, newStructTypeSchema, JCollections.emptyMap(), JCollections.emptyMap())
     val newRow = rowWriter(row)
 
     internalRowCompare(newRowExpected, newRow, newStructTypeSchema)
@@ -290,4 +414,8 @@ class TestHoodieInternalRowUtils extends FunSuite with Matchers with BeforeAndAf
   private def checkNull(left: Any, right: Any): Boolean = {
     (left == null && right != null) || (left == null && right != null)
   }
+
+  private def createSparkRowSerDe(schema: StructType): SparkRowSerDe = {
+    new SparkRowSerDe(sparkAdapter.getCatalystExpressionUtils.getEncoder(schema))
+  }
 }
