@@ -103,47 +103,49 @@ class TestPartialUpdateForMergeInto extends HoodieSparkSqlTestBase {
     withTempDir { tmp =>
       val tableName = generateTableName
       val basePath = tmp.getCanonicalPath + "/" + tableName
-      spark.sql(s"set ${HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key} = 0")
-      spark.sql(s"set ${DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key} = true")
-      spark.sql(s"set ${HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key} = true")
+      withSQLConf(
+        HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key() -> "0",
+        DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key() -> "true",
+        HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key() -> "true") {
 
-      // Create a table with five data fields
-      spark.sql(
-        s"""
-           |create table $tableName (
-           | id int,
-           | name string,
-           | price double,
-           | _ts int,
-           | description string
-           |) using hudi
-           |tblproperties(
-           | type ='mor',
-           | primaryKey = 'id',
-           | preCombineField = '_ts'
-           |)
-           |location '$basePath'
+        // Create a table with five data fields
+        spark.sql(
+          s"""
+             |create table $tableName (
+             | id int,
+             | name string,
+             | price double,
+             | _ts int,
+             | description string
+             |) using hudi
+             |tblproperties(
+             | type ='mor',
+             | primaryKey = 'id',
+             | preCombineField = '_ts'
+             |)
+             |location '$basePath'
         """.stripMargin)
-      spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
-        "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
+        spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
+          "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
 
-      // Update all fields
-      spark.sql(
-        s"""
-           |merge into $tableName t0
-           |using ( select 1 as id, 'a1' as name, 12 as price, 1001 as _ts, 'a1: updated' as description
-           |union select 3 as id, 'a3' as name, 25 as price, 1260 as _ts, 'a3: updated' as description) s0
-           |on t0.id = s0.id
-           |when matched then update set *
-           |""".stripMargin)
+        // Update all fields
+        spark.sql(
+          s"""
+             |merge into $tableName t0
+             |using ( select 1 as id, 'a1' as name, 12 as price, 1001 as _ts, 'a1: updated' as description
+             |union select 3 as id, 'a3' as name, 25 as price, 1260 as _ts, 'a3: updated' as description) s0
+             |on t0.id = s0.id
+             |when matched then update set *
+             |""".stripMargin)
 
-      checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-        Seq(1, "a1", 12.0, 1001, "a1: updated"),
-        Seq(2, "a2", 20.0, 1200, "a2: desc2"),
-        Seq(3, "a3", 25.0, 1260, "a3: updated")
-      )
+        checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+          Seq(1, "a1", 12.0, 1001, "a1: updated"),
+          Seq(2, "a2", 20.0, 1200, "a2: desc2"),
+          Seq(3, "a3", 25.0, 1260, "a3: updated")
+        )
 
-      validateLogBlock(basePath, 1, Seq(Seq("id", "name", "price", "_ts", "description")), false)
+        validateLogBlock(basePath, 1, Seq(Seq("id", "name", "price", "_ts", "description")), false)
+      }
     }
   }
 
@@ -151,55 +153,57 @@ class TestPartialUpdateForMergeInto extends HoodieSparkSqlTestBase {
     withTempDir { tmp =>
       val tableName = generateTableName
       val basePath = tmp.getCanonicalPath + "/" + tableName
-      spark.sql(s"set ${HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key} = 0")
-      spark.sql(s"set ${DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key} = true")
-      spark.sql(s"set ${HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key} = true")
-      // Write inserts to log block
-      spark.sql(s"set ${HoodieIndexConfig.INDEX_TYPE.key} = INMEMORY")
-
-      // Create a table with five data fields
-      spark.sql(
-        s"""
-           |create table $tableName (
-           | id int,
-           | name string,
-           | price double,
-           | _ts int,
-           | description string
-           |) using hudi
-           |tblproperties(
-           | type ='mor',
-           | primaryKey = 'id',
-           | preCombineField = '_ts'
-           |)
-           |location '$basePath'
+      withSQLConf(
+        HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key() -> "0",
+        DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key() -> "true",
+        HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key() -> "true",
+        // Write inserts to log block
+        HoodieIndexConfig.INDEX_TYPE.key() -> "INMEMORY"
+      ) {
+        // Create a table with five data fields
+        spark.sql(
+          s"""
+             |create table $tableName (
+             | id int,
+             | name string,
+             | price double,
+             | _ts int,
+             | description string
+             |) using hudi
+             |tblproperties(
+             | type ='mor',
+             | primaryKey = 'id',
+             | preCombineField = '_ts'
+             |)
+             |location '$basePath'
         """.stripMargin)
-      spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
-        "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
+        spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
+          "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
 
-      // Inserts only
-      spark.sql(
-        s"""
-           |merge into $tableName t0
-           |using ( select 1 as id, 'a1' as name, 12 as price, 1001 as _ts, 'a1: updated' as description
-           |union select 3 as id, 'a3' as name, 25 as price, 1260 as _ts, 'a3: updated' as description
-           |union select 4 as id, 'a4' as name, 60 as price, 1270 as _ts, 'a4: desc4' as description) s0
-           |on t0.id = s0.id
-           |when not matched then insert *
-           |""".stripMargin)
+        // Inserts only
+        spark.sql(
+          s"""
+             |merge into $tableName t0
+             |using ( select 1 as id, 'a1' as name, 12 as price, 1001 as _ts, 'a1: updated' as description
+             |union select 3 as id, 'a3' as name, 25 as price, 1260 as _ts, 'a3: updated' as description
+             |union select 4 as id, 'a4' as name, 60 as price, 1270 as _ts, 'a4: desc4' as description) s0
+             |on t0.id = s0.id
+             |when not matched then insert *
+             |""".stripMargin)
 
-      checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-        Seq(1, "a1", 10.0, 1000, "a1: desc1"),
-        Seq(2, "a2", 20.0, 1200, "a2: desc2"),
-        Seq(3, "a3", 30.0, 1250, "a3: desc3"),
-        Seq(4, "a4", 60.0, 1270, "a4: desc4")
-      )
+        checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+          Seq(1, "a1", 10.0, 1000, "a1: desc1"),
+          Seq(2, "a2", 20.0, 1200, "a2: desc2"),
+          Seq(3, "a3", 30.0, 1250, "a3: desc3"),
+          Seq(4, "a4", 60.0, 1270, "a4: desc4")
+        )
 
-      validateLogBlock(
-        basePath,
-        2,
-        Seq(Seq("id", "name", "price", "_ts", "description"), Seq("id", "name", "price", "_ts", "description")),
-        false)
+        validateLogBlock(
+          basePath,
+          2,
+          Seq(Seq("id", "name", "price", "_ts", "description"), Seq("id", "name", "price", "_ts", "description")),
+          false)
+      }
     }
   }
 
@@ -215,65 +219,67 @@ class TestPartialUpdateForMergeInto extends HoodieSparkSqlTestBase {
     withTempDir { tmp =>
       val tableName = generateTableName
       val basePath = tmp.getCanonicalPath + "/" + tableName
-      spark.sql(s"set ${HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key} = 0")
-      spark.sql(s"set ${DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key} = true")
+      withSQLConf(
+        HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key() -> "0",
+        DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key() -> "true") {
 
-      // Create a table with five data fields
-      spark.sql(
-        s"""
-           |create table $tableName (
-           | id int,
-           | name string,
-           | price double,
-           | _ts int,
-           | description string
-           |) using hudi
-           |tblproperties(
-           | type ='$tableType',
-           | primaryKey = 'id',
-           | preCombineField = '_ts'
-           |)
-           |location '$basePath'
+        // Create a table with five data fields
+        spark.sql(
+          s"""
+             |create table $tableName (
+             | id int,
+             | name string,
+             | price double,
+             | _ts int,
+             | description string
+             |) using hudi
+             |tblproperties(
+             | type ='$tableType',
+             | primaryKey = 'id',
+             | preCombineField = '_ts'
+             |)
+             |location '$basePath'
         """.stripMargin)
-      val structFields = scala.collection.immutable.List(
-        StructField("id", IntegerType, nullable = true),
-        StructField("name", StringType, nullable = true),
-        StructField("price", DoubleType, nullable = true),
-        StructField("_ts", IntegerType, nullable = true),
-        StructField("description", StringType, nullable = true))
+        val structFields = scala.collection.immutable.List(
+          StructField("id", IntegerType, nullable = true),
+          StructField("name", StringType, nullable = true),
+          StructField("price", DoubleType, nullable = true),
+          StructField("_ts", IntegerType, nullable = true),
+          StructField("description", StringType, nullable = true))
 
-      spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
-        "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
-      validateTableSchema(tableName, structFields)
+        spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
+          "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
+        validateTableSchema(tableName, structFields)
 
-      // Partial updates using MERGE INTO statement with changed fields: "price", "_ts"
-      // This is OK since the "UPDATE SET" clause does not contain the new column
-      spark.sql(
-        s"""
-           |merge into $tableName t0
-           |using ( select 1 as id, 'a1' as name, 12.0 as price, 1001 as ts, 'x' as new_col
-           |union select 3 as id, 'a3' as name, 25.0 as price, 1260 as ts, 'y' as new_col) s0
-           |on t0.id = s0.id
-           |when matched then update set price = s0.price, _ts = s0.ts
-           |""".stripMargin)
+        // Partial updates using MERGE INTO statement with changed fields: "price", "_ts"
+        // This is OK since the "UPDATE SET" clause does not contain the new column
+        spark.sql(
+          s"""
+             |merge into $tableName t0
+             |using ( select 1 as id, 'a1' as name, 12.0 as price, 1001 as ts, 'x' as new_col
+             |union select 3 as id, 'a3' as name, 25.0 as price, 1260 as ts, 'y' as new_col) s0
+             |on t0.id = s0.id
+             |when matched then update set price = s0.price, _ts = s0.ts
+             |""".stripMargin)
 
-      validateTableSchema(tableName, structFields)
-      checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-        Seq(1, "a1", 12.0, 1001, "a1: desc1"),
-        Seq(2, "a2", 20.0, 1200, "a2: desc2"),
-        Seq(3, "a3", 25.0, 1260, "a3: desc3")
-      )
+        validateTableSchema(tableName, structFields)
+        checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+          Seq(1, "a1", 12.0, 1001, "a1: desc1"),
+          Seq(2, "a2", 20.0, 1200, "a2: desc2"),
+          Seq(3, "a3", 25.0, 1260, "a3: desc3")
+        )
 
-      // Partial updates using MERGE INTO statement with changed fields: "price", "_ts", "new_col"
-      // This throws an error since the "UPDATE SET" clause contains the new column
-      checkExceptionContain(
-        s"""
-           |merge into $tableName
-           |using ( select 1 as id, 'a1' as name, 12.0 as price, 1001 as ts, 'x' as new_col
-           |union select 3 as id, 'a3' as name, 25.0 as price, 1260 as ts, 'y' as new_col) s0
-           |on $tableName.id = s0.id
-           |when matched then update set price = s0.price, _ts = s0.ts, new_col = s0.new_col
-           |""".stripMargin)(getExpectedUnresolvedColumnExceptionMessage("new_col", tableName))
+        // Partial updates using MERGE INTO statement with changed fields: "price", "_ts", "new_col"
+        // This throws an error since the "UPDATE SET" clause contains the new column
+        checkExceptionContain(
+          s"""
+             |merge into $tableName
+             |using ( select 1 as id, 'a1' as name, 12.0 as price, 1001 as ts, 'x' as new_col
+             |union select 3 as id, 'a3' as name, 25.0 as price, 1260 as ts, 'y' as new_col) s0
+             |on $tableName.id = s0.id
+             |when matched then update set price = s0.price, _ts = s0.ts, new_col = s0.new_col
+             |""".stripMargin)(getExpectedUnresolvedColumnExceptionMessage("new_col", tableName))
+      }
     }
   }
 
@@ -288,163 +294,160 @@ class TestPartialUpdateForMergeInto extends HoodieSparkSqlTestBase {
     withTempDir { tmp =>
       val tableName = generateTableName
       val basePath = tmp.getCanonicalPath + "/" + tableName
-      spark.sql(s"set ${HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key} = 0")
-      spark.sql(s"set ${DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key} = true")
-      spark.sql(s"set ${HoodieStorageConfig.LOGFILE_DATA_BLOCK_FORMAT.key} = $logDataBlockFormat")
-      spark.sql(s"set ${HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key} = true")
-      val mergeMode = if (commitTimeOrdering) {
-        RecordMergeMode.COMMIT_TIME_ORDERING.name()
-      } else {
-        RecordMergeMode.EVENT_TIME_ORDERING.name()
-      }
+      withSQLConf(
+        HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key() -> "0",
+        DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key() -> "true",
+        HoodieStorageConfig.LOGFILE_DATA_BLOCK_FORMAT.key() -> logDataBlockFormat,
+        HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key() -> "true") {
+        val mergeMode = if (commitTimeOrdering) {
+          RecordMergeMode.COMMIT_TIME_ORDERING.name()
+        } else {
+          RecordMergeMode.EVENT_TIME_ORDERING.name()
+        }
 
-      val preCombineString = if (commitTimeOrdering) {
-        ""
-      } else {
-        "preCombineField = '_ts',"
-      }
-      // Create a table with five data fields
-      spark.sql(
-        s"""
-           |create table $tableName (
-           | id int,
-           | name string,
-           | price double,
-           | _ts int,
-           | description string
-           |) using hudi
-           |tblproperties(
-           | type ='$tableType',
-           | primaryKey = 'id',
-           | $preCombineString
-           | recordMergeMode = '$mergeMode'
-           |)
-           |location '$basePath'
+        val preCombineString = if (commitTimeOrdering) {
+          ""
+        } else {
+          "preCombineField = '_ts',"
+        }
+        // Create a table with five data fields
+        spark.sql(
+          s"""
+             |create table $tableName (
+             | id int,
+             | name string,
+             | price double,
+             | _ts int,
+             | description string
+             |) using hudi
+             |tblproperties(
+             | type ='$tableType',
+             | primaryKey = 'id',
+             | $preCombineString
+             | recordMergeMode = '$mergeMode'
+             |)
+             |location '$basePath'
         """.stripMargin)
-      val structFields = scala.collection.immutable.List(
-        StructField("id", IntegerType, nullable = true),
-        StructField("name", StringType, nullable = true),
-        StructField("price", DoubleType, nullable = true),
-        StructField("_ts", IntegerType, nullable = true),
-        StructField("description", StringType, nullable = true))
-      spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
-        "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
-      validateTableSchema(tableName, structFields)
+        val structFields = scala.collection.immutable.List(
+          StructField("id", IntegerType, nullable = true),
+          StructField("name", StringType, nullable = true),
+          StructField("price", DoubleType, nullable = true),
+          StructField("_ts", IntegerType, nullable = true),
+          StructField("description", StringType, nullable = true))
+        spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
+          "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
+        validateTableSchema(tableName, structFields)
 
-      // Partial updates using MERGE INTO statement with changed fields: "price" and "_ts"
-      spark.sql(
-        s"""
-           |merge into $tableName t0
-           |using ( select 1 as id, 1.0 as price, 100 as ts
-           |union select 1 as id, 12.0 as price, 999 as ts
-           |union select 3 as id, 25.0 as price, 1260 as ts) s0
-           |on t0.id = s0.id
-           |when matched then update set price = s0.price, _ts = s0.ts
-           |""".stripMargin)
-      validateTableSchema(tableName, structFields)
-      if (commitTimeOrdering) {
-        checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-          Seq(1, "a1", 12.0, 999, "a1: desc1"),
-          Seq(2, "a2", 20.0, 1200, "a2: desc2"),
-          Seq(3, "a3", 25.0, 1260, "a3: desc3")
-        )
-      } else {
-        checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-          Seq(1, "a1", 10.0, 1000, "a1: desc1"),
-          Seq(2, "a2", 20.0, 1200, "a2: desc2"),
-          Seq(3, "a3", 25.0, 1260, "a3: desc3")
-        )
-      }
-      if (tableType.equals("mor")) {
-        validateLogBlock(basePath, 1, Seq(Seq("price", "_ts")), true)
-      }
-
-      // TODO: [HUDI-9375] get rid of this update and fix the rest of the test accordingly
-      // showcase the difference between event time and commit time ordering
-      // Partial updates using MERGE INTO statement with changed fields: "price" and "_ts"
-      spark.sql(
-        s"""
-           |merge into $tableName t0
-           |using ( select 1 as id, 'a1' as name, 12.0 as price, 1001 as ts) s0
-           |on t0.id = s0.id
-           |when matched then update set price = s0.price, _ts = s0.ts
-           |""".stripMargin)
-
-      checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-        Seq(1, "a1", 12.0, 1001, "a1: desc1"),
-        Seq(2, "a2", 20.0, 1200, "a2: desc2"),
-        Seq(3, "a3", 25.0, 1260, "a3: desc3")
-      )
-      if (tableType.equals("mor")) {
-        validateLogBlock(basePath, 2, Seq(Seq("price", "_ts"), Seq("price", "_ts")), true)
-      }
-
-      // Partial updates using MERGE INTO statement with changed fields: "description" and "_ts"
-      spark.sql(
-        s"""
-           |merge into $tableName t0
-           |using ( select 1 as id, 'a1' as name, 'a1: updated desc1' as new_description, 1023 as _ts
-           |union select 2 as id, 'a2' as name, 'a2: updated desc2' as new_description, 1270 as _ts) s0
-           |on t0.id = s0.id
-           |when matched then update set description = s0.new_description, _ts = s0._ts
-           |""".stripMargin)
-
-      validateTableSchema(tableName, structFields)
-      checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-        Seq(1, "a1", 12.0, 1023, "a1: updated desc1"),
-        Seq(2, "a2", 20.0, 1270, "a2: updated desc2"),
-        Seq(3, "a3", 25.0, 1260, "a3: desc3")
-      )
-
-      if (tableType.equals("mor")) {
-        validateLogBlock(basePath, 3, Seq(Seq("price", "_ts"), Seq("price", "_ts"), Seq("_ts", "description")), true)
-
-        spark.sql(s"set ${HoodieCompactionConfig.INLINE_COMPACT_NUM_DELTA_COMMITS.key} = 4")
-        // Partial updates that trigger compaction
+        // Partial updates using MERGE INTO statement with changed fields: "price" and "_ts"
         spark.sql(
           s"""
              |merge into $tableName t0
-             |using ( select 2 as id, '_a2' as name, 18.0 as _price, 1275 as _ts
-             |union select 3 as id, '_a3' as name, 28.0 as _price, 1280 as _ts) s0
+             |using ( select 1 as id, 1.0 as price, 100 as ts
+             |union select 1 as id, 12.0 as price, 999 as ts
+             |union select 3 as id, 25.0 as price, 1260 as ts) s0
              |on t0.id = s0.id
-             |when matched then update set price = s0._price, _ts = s0._ts
+             |when matched then update set price = s0.price, _ts = s0.ts
              |""".stripMargin)
-        validateCompactionExecuted(basePath)
         validateTableSchema(tableName, structFields)
-        checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-          Seq(1, "a1", 12.0, 1023, "a1: updated desc1"),
-          Seq(2, "a2", 18.0, 1275, "a2: updated desc2"),
-          Seq(3, "a3", 28.0, 1280, "a3: desc3")
-        )
+        if (commitTimeOrdering) {
+          checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+            Seq(1, "a1", 12.0, 999, "a1: desc1"),
+            Seq(2, "a2", 20.0, 1200, "a2: desc2"),
+            Seq(3, "a3", 25.0, 1260, "a3: desc3")
+          )
+        } else {
+          checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+            Seq(1, "a1", 10.0, 1000, "a1: desc1"),
+            Seq(2, "a2", 20.0, 1200, "a2: desc2"),
+            Seq(3, "a3", 25.0, 1260, "a3: desc3")
+          )
+        }
+        if (tableType.equals("mor")) {
+          validateLogBlock(basePath, 1, Seq(Seq("price", "_ts")), true)
+        }
 
-        // trigger one more MIT and do inline clustering
-        spark.sql(s"set ${HoodieClusteringConfig.INLINE_CLUSTERING.key} = true")
-        spark.sql(s"set ${HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key} = 3")
+        // TODO: [HUDI-9375] get rid of this update and fix the rest of the test accordingly
+        // showcase the difference between event time and commit time ordering
+        // Partial updates using MERGE INTO statement with changed fields: "price" and "_ts"
         spark.sql(
           s"""
              |merge into $tableName t0
-             |using ( select 2 as id, '_a2' as name, 48.0 as _price, 1275 as _ts
-             |union select 3 as id, '_a3' as name, 58.0 as _price, 1280 as _ts) s0
+             |using ( select 1 as id, 'a1' as name, 12.0 as price, 1001 as ts) s0
              |on t0.id = s0.id
-             |when matched then update set price = s0._price, _ts = s0._ts
+             |when matched then update set price = s0.price, _ts = s0.ts
+             |""".stripMargin)
+
+        checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+          Seq(1, "a1", 12.0, 1001, "a1: desc1"),
+          Seq(2, "a2", 20.0, 1200, "a2: desc2"),
+          Seq(3, "a3", 25.0, 1260, "a3: desc3")
+        )
+        if (tableType.equals("mor")) {
+          validateLogBlock(basePath, 2, Seq(Seq("price", "_ts"), Seq("price", "_ts")), true)
+        }
+
+        // Partial updates using MERGE INTO statement with changed fields: "description" and "_ts"
+        spark.sql(
+          s"""
+             |merge into $tableName t0
+             |using ( select 1 as id, 'a1' as name, 'a1: updated desc1' as new_description, 1023 as _ts
+             |union select 2 as id, 'a2' as name, 'a2: updated desc2' as new_description, 1270 as _ts) s0
+             |on t0.id = s0.id
+             |when matched then update set description = s0.new_description, _ts = s0._ts
              |""".stripMargin)
 
-        validateClusteringExecuted(basePath)
         validateTableSchema(tableName, structFields)
         checkAnswer(s"select id, name, price, _ts, description from $tableName")(
           Seq(1, "a1", 12.0, 1023, "a1: updated desc1"),
-          Seq(2, "a2", 48.0, 1275, "a2: updated desc2"),
-          Seq(3, "a3", 58.0, 1280, "a3: desc3")
+          Seq(2, "a2", 20.0, 1270, "a2: updated desc2"),
+          Seq(3, "a3", 25.0, 1260, "a3: desc3")
         )
 
-        // revert the config overrides.
-        spark.sql(s"set ${HoodieCompactionConfig.INLINE_COMPACT_NUM_DELTA_COMMITS.key}"
-          + s" = ${HoodieCompactionConfig.INLINE_COMPACT_NUM_DELTA_COMMITS.defaultValue()}")
+        if (tableType.equals("mor")) {
+          validateLogBlock(basePath, 3, Seq(Seq("price", "_ts"), Seq("price", "_ts"), Seq("_ts", "description")), true)
 
-        spark.sql(s"set ${HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key}"
-          + s" = ${HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.defaultValue()}")
-        spark.sql(s"set ${HoodieClusteringConfig.INLINE_CLUSTERING.key}"
-          + s" = ${HoodieClusteringConfig.INLINE_CLUSTERING.defaultValue()}")
+          withSQLConf(HoodieCompactionConfig.INLINE_COMPACT_NUM_DELTA_COMMITS.key() -> "4") {
+            // Partial updates that trigger compaction
+            spark.sql(
+              s"""
+                 |merge into $tableName t0
+                 |using ( select 2 as id, '_a2' as name, 18.0 as _price, 1275 as _ts
+                 |union select 3 as id, '_a3' as name, 28.0 as _price, 1280 as _ts) s0
+                 |on t0.id = s0.id
+                 |when matched then update set price = s0._price, _ts = s0._ts
+                 |""".stripMargin)
+            validateCompactionExecuted(basePath)
+            validateTableSchema(tableName, structFields)
+            checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+              Seq(1, "a1", 12.0, 1023, "a1: updated desc1"),
+              Seq(2, "a2", 18.0, 1275, "a2: updated desc2"),
+              Seq(3, "a3", 28.0, 1280, "a3: desc3")
+            )
+          }
+
+          // trigger one more MIT and do inline clustering
+          withSQLConf(
+            HoodieClusteringConfig.INLINE_CLUSTERING.key() -> "true",
+            HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key() -> "3"
+          ) {
+            spark.sql(
+              s"""
+                 |merge into $tableName t0
+                 |using ( select 2 as id, '_a2' as name, 48.0 as _price, 1275 as _ts
+                 |union select 3 as id, '_a3' as name, 58.0 as _price, 1280 as _ts) s0
+                 |on t0.id = s0.id
+                 |when matched then update set price = s0._price, _ts = s0._ts
+                 |""".stripMargin)
+
+            validateClusteringExecuted(basePath)
+            validateTableSchema(tableName, structFields)
+            checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+              Seq(1, "a1", 12.0, 1023, "a1: updated desc1"),
+              Seq(2, "a2", 48.0, 1275, "a2: updated desc2"),
+              Seq(3, "a3", 58.0, 1280, "a3: desc3")
+            )
+          }
+        }
       }
 
       if (tableType.equals("cow")) {
@@ -466,27 +469,24 @@ class TestPartialUpdateForMergeInto extends HoodieSparkSqlTestBase {
         """.stripMargin)
         spark.sql(s"insert into $tableName2 values(1, 'a1', 10)")
 
-        spark.sql(s"set ${HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key} = true")
-        spark.sql(s"set ${HoodieClusteringConfig.INLINE_CLUSTERING.key} = true")
-        spark.sql(s"set ${HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key} = 1")
+        withSQLConf(
+          HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key() -> "true",
+          HoodieClusteringConfig.INLINE_CLUSTERING.key() -> "true",
+          HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key() -> "1"
+        ) {
+          spark.sql(
+            s"""
+               |merge into $tableName2 t0
+               |using ( select 1 as id, 'a2' as name, 12.0 as price) s0
+               |on t0.id = s0.id
+               |when matched then update set price = s0.price
+               |""".stripMargin)
 
-        spark.sql(
-          s"""
-             |merge into $tableName2 t0
-             |using ( select 1 as id, 'a2' as name, 12.0 as price) s0
-             |on t0.id = s0.id
-             |when matched then update set price = s0.price
-             |""".stripMargin)
-
-        validateClusteringExecuted(basePath2)
-        checkAnswer(s"select id, name, price from $tableName2")(
-          Seq(1, "a1", 12.0)
-        )
-
-        spark.sql(s"set ${HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.key}"
-          + s" = ${HoodieClusteringConfig.INLINE_CLUSTERING_MAX_COMMITS.defaultValue()}")
-        spark.sql(s"set ${HoodieClusteringConfig.INLINE_CLUSTERING.key}"
-          + s" = ${HoodieClusteringConfig.INLINE_CLUSTERING.defaultValue()}")
+          validateClusteringExecuted(basePath2)
+          checkAnswer(s"select id, name, price from $tableName2")(
+            Seq(1, "a1", 12.0)
+          )
+        }
       }
     }
   }
@@ -502,64 +502,66 @@ class TestPartialUpdateForMergeInto extends HoodieSparkSqlTestBase {
     withTempDir { tmp =>
       val tableName = generateTableName
       val basePath = tmp.getCanonicalPath + "/" + tableName
-      spark.sql(s"set ${HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key} = 0")
-      spark.sql(s"set ${DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key} = true")
-      spark.sql(s"set ${HoodieStorageConfig.LOGFILE_DATA_BLOCK_FORMAT.key} = $logDataBlockFormat")
-      spark.sql(s"set ${HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key} = true")
-      val mergeMode = if (commitTimeOrdering) {
-        RecordMergeMode.COMMIT_TIME_ORDERING.name()
-      } else {
-        RecordMergeMode.EVENT_TIME_ORDERING.name()
-      }
+      withSQLConf(
+        HoodieWriteConfig.MERGE_SMALL_FILE_GROUP_CANDIDATES_LIMIT.key() -> "0",
+        DataSourceWriteOptions.ENABLE_MERGE_INTO_PARTIAL_UPDATES.key() -> "true",
+        HoodieStorageConfig.LOGFILE_DATA_BLOCK_FORMAT.key() -> logDataBlockFormat,
+        HoodieReaderConfig.FILE_GROUP_READER_ENABLED.key() -> "true") {
+        val mergeMode = if (commitTimeOrdering) {
+          RecordMergeMode.COMMIT_TIME_ORDERING.name()
+        } else {
+          RecordMergeMode.EVENT_TIME_ORDERING.name()
+        }
 
-      val preCombineString = if (commitTimeOrdering) {
-        ""
-      } else {
-        "preCombineField = '_ts',"
-      }
+        val preCombineString = if (commitTimeOrdering) {
+          ""
+        } else {
+          "preCombineField = '_ts',"
+        }
 
-      // Create a table with five data fields
-      spark.sql(
-        s"""
-           |create table $tableName (
-           | id int,
-           | name string,
-           | price double,
-           | _ts int,
-           | description string
-           |) using hudi
-           |tblproperties(
-           | type ='$tableType',
-           | primaryKey = 'id',
-           | $preCombineString
-           | recordMergeMode = '$mergeMode'
-           |)
-           |location '$basePath'
+        // Create a table with five data fields
+        spark.sql(
+          s"""
+             |create table $tableName (
+             | id int,
+             | name string,
+             | price double,
+             | _ts int,
+             | description string
+             |) using hudi
+             |tblproperties(
+             | type ='$tableType',
+             | primaryKey = 'id',
+             | $preCombineString
+             | recordMergeMode = '$mergeMode'
+             |)
+             |location '$basePath'
         """.stripMargin)
-      spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
-        "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
+        spark.sql(s"insert into $tableName values (1, 'a1', 10, 1000, 'a1: desc1')," +
+          "(2, 'a2', 20, 1200, 'a2: desc2'), (3, 'a3', 30, 1250, 'a3: desc3')")
 
-      // Partial updates with changed fields: "price" and "_ts" and inserts using MERGE INTO statement
-      spark.sql(
-        s"""
-           |merge into $tableName t0
-           |using ( select 1 as id, 'a1' as name, 12 as price, 1001 as _ts, '' as description
-           |union select 3 as id, 'a3' as name, 25 as price, 1260 as _ts, '' as description
-           |union select 4 as id, 'a4' as name, 70 as price, 1270 as _ts, 'a4: desc4' as description) s0
-           |on t0.id = s0.id
-           |when matched then update set price = s0.price, _ts = s0._ts
-           |when not matched then insert *
-           |""".stripMargin)
+        // Partial updates with changed fields: "price" and "_ts" and inserts using MERGE INTO statement
+        spark.sql(
+          s"""
+             |merge into $tableName t0
+             |using ( select 1 as id, 'a1' as name, 12 as price, 1001 as _ts, '' as description
+             |union select 3 as id, 'a3' as name, 25 as price, 1260 as _ts, '' as description
+             |union select 4 as id, 'a4' as name, 70 as price, 1270 as _ts, 'a4: desc4' as description) s0
+             |on t0.id = s0.id
+             |when matched then update set price = s0.price, _ts = s0._ts
+             |when not matched then insert *
+             |""".stripMargin)
 
-      checkAnswer(s"select id, name, price, _ts, description from $tableName")(
-        Seq(1, "a1", 12.0, 1001, "a1: desc1"),
-        Seq(2, "a2", 20.0, 1200, "a2: desc2"),
-        Seq(3, "a3", 25.0, 1260, "a3: desc3"),
-        Seq(4, "a4", 70.0, 1270, "a4: desc4")
-      )
+        checkAnswer(s"select id, name, price, _ts, description from $tableName")(
+          Seq(1, "a1", 12.0, 1001, "a1: desc1"),
+          Seq(2, "a2", 20.0, 1200, "a2: desc2"),
+          Seq(3, "a3", 25.0, 1260, "a3: desc3"),
+          Seq(4, "a4", 70.0, 1270, "a4: desc4")
+        )
 
-      if (tableType.equals("mor")) {
-        validateLogBlock(basePath, 1, Seq(Seq("price", "_ts")), true)
+        if (tableType.equals("mor")) {
+          validateLogBlock(basePath, 1, Seq(Seq("price", "_ts")), true)
+        }
       }
     }
   }
