@@ -20,7 +20,7 @@
 package org.apache.spark.sql.hudi.procedure
 
 import org.apache.hudi.SparkDatasetMixin
-import org.apache.hudi.client.SparkRDDWriteClient
+import org.apache.hudi.client.{SparkRDDWriteClient, WriteClientTestUtils}
 import org.apache.hudi.client.common.HoodieSparkEngineContext
 import org.apache.hudi.common.model.{HoodieRecord, HoodieTableType}
 import org.apache.hudi.common.table.HoodieTableConfig
@@ -31,7 +31,7 @@ import org.apache.hudi.config.HoodieWriteConfig
 
 import org.apache.spark.api.java.JavaSparkContext
 
-import java.util.Properties
+import java.util.{Collections, Properties}
 
 import scala.collection.JavaConverters._
 
@@ -57,7 +57,7 @@ class TestTTLProcedure extends HoodieSparkProcedureTestBase with SparkDatasetMix
         val partitionPath1 = partitionPaths(1)
         writeRecordsForPartition(client, dataGen, partitionPath1, instant1)
 
-        val currentInstant = client.createNewInstantTime()
+        val currentInstant = WriteClientTestUtils.createNewInstantTime()
         val partitionPath2 = partitionPaths(2)
         writeRecordsForPartition(client, dataGen, partitionPath2, currentInstant)
         spark.sql(
@@ -66,7 +66,7 @@ class TestTTLProcedure extends HoodieSparkProcedureTestBase with SparkDatasetMix
              | location '$basePath'
              | tblproperties (
              |   primaryKey = '_row_key',
-             |   preCombineField = '_row_key',
+             |   orderingFields = '_row_key',
              |   type = 'cow'
              | )
              |""".stripMargin)
@@ -75,6 +75,7 @@ class TestTTLProcedure extends HoodieSparkProcedureTestBase with SparkDatasetMix
           Seq(partitionPath0),
           Seq(partitionPath1)
         )
+        client.close()
       }
       }
     }
@@ -87,8 +88,9 @@ class TestTTLProcedure extends HoodieSparkProcedureTestBase with SparkDatasetMix
       dataGen.generateInsertsForPartition(instantTime, 10, partition)
         .asInstanceOf[java.util.List[HoodieRecord[Nothing]]]
     // Use this JavaRDD to call the insert method
-    client.startCommitWithTime(instantTime, HoodieTimeline.COMMIT_ACTION)
-    client.insert(spark.sparkContext.parallelize(records.asScala.toSeq).toJavaRDD(), instantTime)
+    WriteClientTestUtils.startCommitWithTime(client, instantTime, HoodieTimeline.COMMIT_ACTION)
+    val statuses = client.insert(spark.sparkContext.parallelize(records.asScala.toSeq).toJavaRDD(), instantTime)
+    client.commit(instantTime, statuses)
   }
 
   private def getHoodieWriteClient(cfg: HoodieWriteConfig): SparkRDDWriteClient[Nothing] = {
@@ -112,8 +114,6 @@ class TestTTLProcedure extends HoodieSparkProcedureTestBase with SparkDatasetMix
       .newBuilder
       .withPath(basePath)
       .withSchema(TRIP_EXAMPLE_SCHEMA)
-      .withAutoCommit(autoCommit)
-      .withPreCombineField("_row_key")
       .forTable(tableName)
-
+      .withProps(Collections.singletonMap(HoodieTableConfig.ORDERING_FIELDS.key(), "_row_key"))
 }
