@@ -29,7 +29,7 @@ import org.apache.spark.sql.execution.datasources.{HadoopFsRelation, LogicalRela
 import org.apache.spark.sql.execution.datasources.parquet.{HoodieFormatTrait, ParquetFileFormat}
 import org.apache.spark.sql.types.StructType
 
-object HoodieSpark33CatalystPlanUtils extends HoodieSpark3CatalystPlanUtils {
+object HoodieSpark33CatalystPlanUtils extends BaseHoodieCatalystPlanUtils {
 
   def unapplyResolvedTable(plan: LogicalPlan): Option[(TableCatalog, Identifier, Table)] =
     plan match {
@@ -76,6 +76,10 @@ object HoodieSpark33CatalystPlanUtils extends HoodieSpark3CatalystPlanUtils {
     a.failAnalysis(s"cannot resolve ${a.sql} in MERGE command given columns [$cols]")
   }
 
+  override def failTableNotFound(tableName: String): Unit = {
+    throw new AnalysisException(s"Table or view not found: $tableName")
+  }
+
   override def unapplyCreateIndex(plan: LogicalPlan): Option[(LogicalPlan, String, String, Boolean, Seq[(Seq[String], Map[String, String])], Map[String, String])] = {
     plan match {
       case ci @ CreateIndex(table, indexName, indexType, ignoreIfExists, columns, properties) =>
