@@ -18,11 +18,10 @@
 package org.apache.spark.sql.avro
 
 import org.apache.avro.Schema
-import org.apache.spark.sql.internal.SQLConf
-import org.apache.spark.sql.internal.SQLConf.LegacyBehaviorPolicy
+import org.apache.spark.sql.internal.{LegacyBehaviorPolicy, SQLConf}
 import org.apache.spark.sql.types.DataType
 
-class HoodieSpark3_3AvroDeserializer(rootAvroType: Schema, rootCatalystType: DataType)
+class HoodieSpark4_0AvroDeserializer(rootAvroType: Schema, rootCatalystType: DataType)
   extends HoodieAvroDeserializer {
 
   private val avroDeserializer = new AvroDeserializer(rootAvroType, rootCatalystType,
