@@ -81,9 +81,7 @@ public class HiveSchemaUtil {
     } catch (IOException e) {
       throw new HoodieHiveSyncException("Failed to convert parquet schema to hive schema", e);
     }
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("Getting schema difference for " + tableSchema + "\r\n\r\n" + newTableSchema);
-    }
+    LOG.debug("Getting schema difference for {} \r\n\r\n{}", tableSchema, newTableSchema);
 
     SchemaDifference.Builder schemaDiffBuilder = SchemaDifference.newBuilder(storageSchema, tableSchema);
     Set<String> tableColumns = new HashSet<>();
@@ -102,7 +100,7 @@ public class HiveSchemaUtil {
             continue;
           }
           // We will log this and continue. Hive schema is a superset of all parquet schemas
-          LOG.warn("Ignoring table column " + fieldName + " as its not present in the parquet schema");
+          LOG.info("Ignoring table column {} as its not present in the parquet schema", fieldName);
           continue;
         }
         tableColumnType = tableColumnType.replaceAll("\\s+", "");
@@ -123,11 +121,9 @@ public class HiveSchemaUtil {
         schemaDiffBuilder.addTableColumn(entry.getKey(), entry.getValue());
       }
     }
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("Difference between schemas: " + schemaDiffBuilder.build().toString());
-    }
-
-    return schemaDiffBuilder.build();
+    SchemaDifference result = schemaDiffBuilder.build();
+    LOG.debug("Difference between schemas: {}", result);
+    return result;
   }
 
   private static String getExpectedType(Map<String, String> newTableSchema, String fieldName) {
