@@ -19,7 +19,6 @@
 package org.apache.hudi.timeline.service.handlers;
 
 import org.apache.hudi.common.conflict.detection.TimelineServerBasedDetectionStrategy;
-import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.metrics.Registry;
 import org.apache.hudi.common.model.IOType;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
@@ -29,7 +28,6 @@ import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.ReflectionUtils;
 import org.apache.hudi.exception.HoodieEarlyConflictDetectionException;
 import org.apache.hudi.exception.HoodieException;
-import org.apache.hudi.storage.HoodieStorage;
 import org.apache.hudi.storage.StorageConfiguration;
 import org.apache.hudi.timeline.service.TimelineService;
 import org.apache.hudi.timeline.service.handlers.marker.MarkerCreationDispatchingRunnable;
@@ -37,12 +35,10 @@ import org.apache.hudi.timeline.service.handlers.marker.MarkerCreationFuture;
 import org.apache.hudi.timeline.service.handlers.marker.MarkerDirState;
 
 import com.fasterxml.jackson.core.JsonProcessingException;
-import com.fasterxml.jackson.databind.ObjectMapper;
 import io.javalin.http.Context;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.io.IOException;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.Map;
@@ -96,20 +92,16 @@ public class MarkerHandler extends Handler {
   private final MarkerCreationDispatchingRunnable markerCreationDispatchingRunnable;
   private final Object firstCreationRequestSeenLock = new Object();
   private final Object earlyConflictDetectionLock = new Object();
-  private transient HoodieEngineContext hoodieEngineContext;
   private ScheduledFuture<?> dispatchingThreadFuture;
   private boolean firstCreationRequestSeen;
   private String currentMarkerDir = null;
   private TimelineServerBasedDetectionStrategy earlyConflictDetectionStrategy;
 
   public MarkerHandler(StorageConfiguration<?> conf, TimelineService.Config timelineServiceConfig,
-                       HoodieEngineContext hoodieEngineContext, HoodieStorage storage,
-                       FileSystemViewManager viewManager, Registry metricsRegistry) throws IOException {
-    super(conf, timelineServiceConfig, storage, viewManager);
-    LOG.debug("MarkerHandler FileSystem: " + this.storage.getScheme());
-    LOG.debug("MarkerHandler batching params: batchNumThreads=" + timelineServiceConfig.markerBatchNumThreads
-        + " batchIntervalMs=" + timelineServiceConfig.markerBatchIntervalMs + "ms");
-    this.hoodieEngineContext = hoodieEngineContext;
+                       FileSystemViewManager viewManager, Registry metricsRegistry) {
+    super(conf, timelineServiceConfig, viewManager);
+    LOG.debug("MarkerHandler batching params: batchNumThreads={} batchIntervalMs={}ms",
+        timelineServiceConfig.markerBatchNumThreads, timelineServiceConfig.markerBatchIntervalMs);
     this.metricsRegistry = metricsRegistry;
     this.parallelism = timelineServiceConfig.markerParallelism;
     this.dispatchingExecutorService = Executors.newSingleThreadScheduledExecutor();
@@ -165,6 +157,16 @@ public class MarkerHandler extends Handler {
         .collect(Collectors.toSet());
   }
 
+  /**
+   * @param markerDir marker directory path
+   * @return all marker paths of write IO type "APPEND"
+   */
+  public Set<String> getAppendMarkers(String markerDir) {
+    return getAllMarkers(markerDir).stream()
+        .filter(markerName -> markerName.endsWith(IOType.APPEND.name()))
+        .collect(Collectors.toSet());
+  }
+
   /**
    * @param markerDir  marker directory path
    * @return {@code true} if the marker directory exists; {@code false} otherwise.
@@ -193,9 +195,9 @@ public class MarkerHandler extends Handler {
           if (earlyConflictDetectionStrategy == null) {
             String strategyClassName = timelineServiceConfig.earlyConflictDetectionStrategy;
             if (!ReflectionUtils.isSubClass(strategyClassName, TimelineServerBasedDetectionStrategy.class)) {
-              LOG.warn("Cannot use " + strategyClassName + " for timeline-server-based markers.");
+              LOG.warn("Cannot use {} for timeline-server-based markers.", strategyClassName);
               strategyClassName = "org.apache.hudi.timeline.service.handlers.marker.AsyncTimelineServerBasedDetectionStrategy";
-              LOG.warn("Falling back to " + strategyClassName);
+              LOG.warn("Falling back to {}", strategyClassName);
             }
 
             earlyConflictDetectionStrategy =
@@ -221,18 +223,18 @@ public class MarkerHandler extends Handler {
                 timelineServiceConfig.asyncConflictDetectorInitialDelayMs,
                 timelineServiceConfig.asyncConflictDetectorPeriodMs,
                 markerDir, basePath, timelineServiceConfig.maxAllowableHeartbeatIntervalInMs,
-                storage, this, completedCommits);
+                getStorage(basePath), this, completedCommits);
           }
         }
 
         earlyConflictDetectionStrategy.detectAndResolveConflictIfNecessary();
 
       } catch (HoodieEarlyConflictDetectionException he) {
-        LOG.warn("Detected the write conflict due to a concurrent writer, "
+        LOG.error("Detected the write conflict due to a concurrent writer, "
             + "failing the marker creation as the early conflict detection is enabled", he);
         return finishCreateMarkerFuture(context, markerDir, markerName);
       } catch (Exception e) {
-        LOG.warn("Failed to execute early conflict detection." + e.getMessage());
+        LOG.warn("Failed to execute early conflict detection. Marker creation will continue.", e);
         // When early conflict detection fails to execute, we still allow the marker creation
         // to continue
         return addMarkerCreationRequestForAsyncProcessing(context, markerDir, markerName);
@@ -267,7 +269,7 @@ public class MarkerHandler extends Handler {
     MarkerCreationFuture future = new MarkerCreationFuture(context, markerDir, markerName);
     try {
       future.complete(jsonifyResult(
-          future.getContext(), future.isSuccessful(), metricsRegistry, new ObjectMapper(), LOG));
+          future.getContext(), future.isSuccessful(), metricsRegistry));
     } catch (JsonProcessingException e) {
       throw new HoodieException("Failed to JSON encode the value", e);
     }
@@ -280,7 +282,7 @@ public class MarkerHandler extends Handler {
    * @param markerDir marker directory path
    * @return {@code true} if successful; {@code false} otherwise.
    */
-  public Boolean deleteMarkers(String markerDir) {
+  public boolean deleteMarkers(String markerDir) {
     boolean result = getMarkerDirState(markerDir).deleteAllMarkers();
     markerDirStateMap.remove(markerDir);
     return result;
@@ -297,7 +299,7 @@ public class MarkerHandler extends Handler {
                   ? Option.of(earlyConflictDetectionStrategy) : Option.empty();
           markerDirState = new MarkerDirState(
               markerDir, timelineServiceConfig.markerBatchNumThreads,
-              strategy, storage, metricsRegistry, hoodieEngineContext, parallelism);
+              strategy, getStorage(markerDir), metricsRegistry, parallelism);
           markerDirStateMap.put(markerDir, markerDirState);
         } else {
           markerDirState = markerDirStateMap.get(markerDir);
