@@ -184,7 +184,8 @@ public class HoodieSnapshotExporter {
 
   private List<String> getPartitions(HoodieEngineContext engineContext, Config cfg,
                                      HoodieStorage storage) {
-    return FSUtils.getAllPartitionPaths(engineContext, storage, cfg.sourceBasePath, true);
+    HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder().setBasePath(cfg.sourceBasePath).setConf(storage.getConf()).build();
+    return FSUtils.getAllPartitionPaths(engineContext, metaClient, true);
   }
 
   private void createSuccessTag(FileSystem fs, Config cfg) throws IOException {
@@ -218,7 +219,7 @@ public class HoodieSnapshotExporter {
             .map(HoodieBaseFile::getPath).iterator())
         .toLocalIterator();
 
-    Dataset<Row> sourceDataset = new SQLContext(jsc).read().parquet(JavaScalaConverters.convertJavaIteratorToScalaIterator(exportingFilePaths).toSeq());
+    Dataset<Row> sourceDataset = SQLContext.getOrCreate(jsc.sc()).read().parquet(JavaScalaConverters.convertJavaIteratorToScalaIterator(exportingFilePaths).toSeq());
 
     if (!StringUtils.isNullOrEmpty(cfg.transformerClassName)) {
       Option<Transformer> transformer = UtilHelpers.createTransformer(
