@@ -20,6 +20,8 @@
 package org.apache.hudi.utilities.deltastreamer;
 
 import org.apache.hudi.config.HoodieWriteConfig;
+import org.apache.hudi.config.metrics.HoodieMetricsConfig;
+import org.apache.hudi.storage.HoodieStorage;
 import org.apache.hudi.utilities.streamer.HoodieStreamerMetrics;
 
 /**
@@ -28,7 +30,12 @@ import org.apache.hudi.utilities.streamer.HoodieStreamerMetrics;
  */
 @Deprecated
 public class HoodieDeltaStreamerMetrics extends HoodieStreamerMetrics {
-  public HoodieDeltaStreamerMetrics(HoodieWriteConfig writeConfig) {
-    super(writeConfig);
+
+  public HoodieDeltaStreamerMetrics(HoodieWriteConfig writeConfig, HoodieStorage storage) {
+    super(writeConfig.getMetricsConfig(), storage);
+  }
+
+  public HoodieDeltaStreamerMetrics(HoodieMetricsConfig metricsConfig, HoodieStorage storage) {
+    super(metricsConfig, storage);
   }
 }
