@@ -26,9 +26,9 @@ import org.apache.hudi.common.table.HoodieTableConfig;
 import org.apache.hudi.common.table.HoodieTableVersion;
 import org.apache.hudi.common.table.checkpoint.Checkpoint;
 import org.apache.hudi.common.table.checkpoint.CheckpointUtils;
-import org.apache.hudi.common.table.checkpoint.UnresolvedStreamerCheckpointBasedOnCfg;
 import org.apache.hudi.common.table.checkpoint.StreamerCheckpointV1;
 import org.apache.hudi.common.table.checkpoint.StreamerCheckpointV2;
+import org.apache.hudi.common.table.checkpoint.UnresolvedStreamerCheckpointBasedOnCfg;
 import org.apache.hudi.common.table.log.InstantRange;
 import org.apache.hudi.common.table.read.IncrementalQueryAnalyzer;
 import org.apache.hudi.common.table.read.IncrementalQueryAnalyzer.QueryContext;
@@ -269,11 +269,11 @@ public class HoodieIncrSource extends RowSource {
         Option<SnapshotLoadQuerySplitter.CheckpointWithPredicates> newCheckpointAndPredicate =
             snapshotLoadQuerySplitter.get().getNextCheckpointWithPredicates(snapshot, queryContext);
         if (newCheckpointAndPredicate.isPresent()) {
-          endCompletionTime = newCheckpointAndPredicate.get().endCompletionTime;
-          predicate = Option.of(newCheckpointAndPredicate.get().predicateFilter);
+          endCompletionTime = newCheckpointAndPredicate.get().getEndCompletionTime();
+          predicate = Option.ofNullable(newCheckpointAndPredicate.get().getPredicateFilter());
           instantTimeList = queryContext.getInstants().stream()
               .filter(instant -> compareTimestamps(
-                  instant.getCompletionTime(), LESSER_THAN_OR_EQUALS, newCheckpointAndPredicate.get().endCompletionTime))
+                  instant.getCompletionTime(), LESSER_THAN_OR_EQUALS, newCheckpointAndPredicate.get().getEndCompletionTime()))
               .map(HoodieInstant::requestedTime)
               .collect(Collectors.toList());
         } else {
@@ -287,18 +287,19 @@ public class HoodieIncrSource extends RowSource {
           .filter(String.format("%s IN ('%s')", HoodieRecord.COMMIT_TIME_METADATA_FIELD,
               String.join("','", instantTimeList)));
     } else {
-      // normal incremental query
-      TimelineLayout layout = TimelineLayout.fromVersion(queryContext.getActiveTimeline().getTimelineLayoutVersion());
-      String inclusiveStartCompletionTime = queryContext.getInstants().stream()
-          .min(layout.getInstantComparator().completionTimeOrderedComparator())
+      String exclusiveStartCompletionTime = analyzer.getStartCompletionTime().isPresent()
+          ? analyzer.getStartCompletionTime().get()
+          : String.valueOf(Long.parseLong(queryContext.getInstants().stream()
+          .min(TimelineLayout.fromVersion(queryContext.getActiveTimeline().getTimelineLayoutVersion())
+              .getInstantComparator().completionTimeOrderedComparator())
           .map(HoodieInstant::getCompletionTime)
-          .get();
-
+          .get()) - 1);
+      // normal incremental query
       source = reader
           .options(readOpts)
           .option(QUERY_TYPE().key(), QUERY_TYPE_INCREMENTAL_OPT_VAL())
           .option(INCREMENTAL_READ_TABLE_VERSION().key(), HoodieTableVersion.EIGHT.versionCode())
-          .option(START_COMMIT().key(), inclusiveStartCompletionTime)
+          .option(START_COMMIT().key(), exclusiveStartCompletionTime)
           .option(END_COMMIT().key(), endCompletionTime)
           .option(INCREMENTAL_FALLBACK_TO_FULL_TABLE_SCAN().key(),
               props.getString(INCREMENTAL_FALLBACK_TO_FULL_TABLE_SCAN().key(),
