@@ -105,6 +105,17 @@ public class PubsubMessagesFetcher {
     );
   }
 
+  /**
+   * <p>Fetches messages from the Pub/Sub subscription based on the configured limits and timeouts.</p>
+   *
+   * The method pulls messages until one of the following conditions is met:
+   * <li>Number of unacknowledged messages in the subscription is reached</li>
+   * <li>Maximum messages per sync limit is reached</li>
+   * <li>Maximum fetch time per sync is exceeded</li>
+   *
+   * @return list of received messages from the Pub/Sub subscription
+   * @throws HoodieException if an error occurs while fetching messages
+   */
   public List<ReceivedMessage> fetchMessages() {
     List<ReceivedMessage> messageList = new ArrayList<>();
     try (SubscriberStub subscriber = pubsubQueueClient.getSubscriber(subscriberStubSettings)) {
@@ -123,6 +134,16 @@ public class PubsubMessagesFetcher {
     }
   }
 
+  /**
+   * <p>Sends acknowledgment for the processed messages to the Pub/Sub subscription.</p>
+   *
+   * Messages are processed in parallel batches for improved performance. Each batch
+   * contains up to {@link #DEFAULT_BATCH_SIZE_ACK_API} messages. The method blocks
+   * until all acknowledgments are completed or the timeout is reached.
+   *
+   * @param messagesToAck list of message acknowledgment IDs to be acknowledged
+   * @throws IOException if acknowledgment fails due to timeout, interruption, or execution error
+   */
   public void sendAcks(List<String> messagesToAck) throws IOException {
     try (SubscriberStub subscriber = pubsubQueueClient.getSubscriber(subscriberStubSettings)) {
       int numberOfBatches = (int) Math.ceil((double) messagesToAck.size() / DEFAULT_BATCH_SIZE_ACK_API);
@@ -131,15 +152,26 @@ public class PubsubMessagesFetcher {
               .boxed()
               .map(batchIndex -> getTask(subscriber, messagesToAck, batchIndex)).toArray(CompletableFuture[]::new))
           .get(MAX_WAIT_TIME_TO_ACK_MESSAGES, TimeUnit.MILLISECONDS);
-      LOG.debug("Flushed out all outstanding acknowledged messages: " + messagesToAck.size());
+      LOG.debug("Flushed out all outstanding acknowledged messages: {}", messagesToAck.size());
     } catch (ExecutionException | InterruptedException | TimeoutException e) {
       throw new IOException("Failed to ack messages from PubSub", e);
     }
   }
 
+  /**
+   * Creates a batch of messages for the given batch index and sends acknowledgement asynchronously.
+   *
+   * @param subscriber the Pub/Sub subscriber stub to use for the acknowledgment request
+   * @param messagesToAck the complete list of message IDs to be acknowledged
+   * @param batchIndex the zero-based index of the batch to process
+   * @return CompletableFuture that completes when the batch acknowledgment is finished
+   */
   private CompletableFuture<Void> getTask(SubscriberStub subscriber, List<String> messagesToAck, int batchIndex) {
     String subscriptionName = ProjectSubscriptionName.format(googleProjectId, pubsubSubscriptionId);
-    List<String> messages = messagesToAck.subList(batchIndex, Math.min(batchIndex + DEFAULT_BATCH_SIZE_ACK_API, messagesToAck.size()));
+    List<String> messages = messagesToAck.subList(
+        batchIndex * DEFAULT_BATCH_SIZE_ACK_API,
+        Math.min((batchIndex + 1) * DEFAULT_BATCH_SIZE_ACK_API, messagesToAck.size()));
+    LOG.debug("Sending ack for batch {} with {} messages: {}", batchIndex, messages.size(), messages);
     return CompletableFuture.runAsync(() -> pubsubQueueClient.makeAckRequest(subscriber, subscriptionName, messages), threadPool);
   }
 }
