@@ -234,7 +234,7 @@ public class HoodieMultiTableStreamer {
       tableConfig.enableMetaSync = globalConfig.enableMetaSync;
       tableConfig.syncClientToolClassNames = globalConfig.syncClientToolClassNames;
       tableConfig.schemaProviderClassName = globalConfig.schemaProviderClassName;
-      tableConfig.sourceOrderingField = globalConfig.sourceOrderingField;
+      tableConfig.sourceOrderingFields = globalConfig.sourceOrderingField;
       tableConfig.sourceClassName = globalConfig.sourceClassName;
       tableConfig.tableType = globalConfig.tableType;
       tableConfig.targetTableName = globalConfig.targetTableName;
@@ -275,8 +275,8 @@ public class HoodieMultiTableStreamer {
     }
 
     if (config.targetTableName != null) {
-      LOG.warn(String.format("--target-table is deprecated and will be removed in a future release due to it's useless;"
-          + " please use %s to configure multiple target tables", HoodieStreamerConfig.TABLES_TO_BE_INGESTED.key()));
+      LOG.warn("--target-table is deprecated and will be removed in a future release due to it's useless;"
+          + " please use {} to configure multiple target tables", HoodieStreamerConfig.TABLES_TO_BE_INGESTED.key());
     }
 
     JavaSparkContext jssc = UtilHelpers.buildSparkContext("multi-table-streamer", Constants.LOCAL_SPARK_MASTER);
