@@ -157,11 +157,10 @@ public class HoodieStreamer implements Serializable {
                         Option<TypedProperties> propsOverride, Option<SourceProfileSupplier> sourceProfileSupplier) throws IOException {
     this.properties = combineProperties(cfg, propsOverride, jssc.hadoopConfiguration());
     Triple<RecordMergeMode, String, String> mergingConfigs =
-        HoodieTableConfig.inferCorrectMergingBehavior(
-            cfg.recordMergeMode, cfg.payloadClassName, cfg.recordMergeStrategyId, cfg.sourceOrderingField,
+        HoodieTableConfig.inferMergingConfigsForWrites(
+            cfg.recordMergeMode, cfg.payloadClassName, cfg.recordMergeStrategyId, cfg.sourceOrderingFields,
             HoodieTableVersion.fromVersionCode(ConfigUtils.getIntWithAltKeys(this.properties, HoodieWriteConfig.WRITE_TABLE_VERSION)));
     cfg.recordMergeMode = mergingConfigs.getLeft();
-    cfg.payloadClassName = mergingConfigs.getMiddle();
     cfg.recordMergeStrategyId = mergingConfigs.getRight();
     if (cfg.initialCheckpointProvider != null && cfg.checkpoint == null) {
       InitialCheckPointProvider checkPointProvider =
@@ -272,9 +271,9 @@ public class HoodieStreamer implements Serializable {
             + "JsonKafkaSource, AvroKafkaSource, HiveIncrPullSource}")
     public String sourceClassName = JsonDFSSource.class.getName();
 
-    @Parameter(names = {"--source-ordering-field"}, description = "Field within source record to decide how"
-        + " to break ties between records with same key in input data.")
-    public String sourceOrderingField = null;
+    @Parameter(names = {"--source-ordering-fields", "--source-ordering-field"}, description = "Comma separated list of fields within source record to decide how"
+        + " to break ties between records with same key in input data. --source-ordering-field is deprecated, please use --source-ordering-fields instead")
+    public String sourceOrderingFields = null;
 
     @Parameter(names = {"--payload-class"}, description = "Deprecated. "
         + "Use --merge-mode for commit time or event time merging. "
@@ -505,7 +504,7 @@ public class HoodieStreamer implements Serializable {
           && Objects.equals(propsFilePath, config.propsFilePath)
           && Objects.equals(configs, config.configs)
           && Objects.equals(sourceClassName, config.sourceClassName)
-          && Objects.equals(sourceOrderingField, config.sourceOrderingField)
+          && Objects.equals(sourceOrderingFields, config.sourceOrderingFields)
           && Objects.equals(payloadClassName, config.payloadClassName)
           && Objects.equals(schemaProviderClassName, config.schemaProviderClassName)
           && Objects.equals(transformerClassNames, config.transformerClassNames)
@@ -538,7 +537,7 @@ public class HoodieStreamer implements Serializable {
     public int hashCode() {
       return Objects.hash(targetBasePath, targetTableName, tableType,
           baseFileFormat, propsFilePath, configs, sourceClassName,
-          sourceOrderingField, payloadClassName, schemaProviderClassName,
+          sourceOrderingFields, payloadClassName, schemaProviderClassName,
           transformerClassNames, sourceLimit, operation, filterDupes,
           enableHiveSync, enableMetaSync, forceEmptyMetaSync, syncClientToolClassNames, maxPendingCompactions, maxPendingClustering,
           continuousMode, minSyncIntervalSeconds, sparkMaster, commitOnErrors,
@@ -557,7 +556,7 @@ public class HoodieStreamer implements Serializable {
           + ", propsFilePath='" + propsFilePath + '\''
           + ", configs=" + configs
           + ", sourceClassName='" + sourceClassName + '\''
-          + ", sourceOrderingField='" + sourceOrderingField + '\''
+          + ", sourceOrderingField='" + sourceOrderingFields + '\''
           + ", payloadClassName='" + payloadClassName + '\''
           + ", schemaProviderClassName='" + schemaProviderClassName + '\''
           + ", transformerClassNames=" + transformerClassNames
@@ -750,7 +749,7 @@ public class HoodieStreamer implements Serializable {
           properties.get().forEach((k, v) -> propsToValidate.put(k.toString(), v.toString()));
           HoodieWriterUtils.validateTableConfig(this.sparkSession, org.apache.hudi.HoodieConversionUtils.mapAsScalaImmutableMap(propsToValidate), meta.getTableConfig());
         } catch (HoodieIOException e) {
-          LOG.warn("Full exception msg " + e.getLocalizedMessage() + ",  msg " + e.getMessage());
+          LOG.warn("Full exception msg {},  msg {}", e.getLocalizedMessage(), e.getMessage());
           if (e.getMessage().contains("Could not load Hoodie properties") && e.getMessage().contains(HoodieTableConfig.HOODIE_PROPERTIES_FILE)) {
             initializeTableTypeAndBaseFileFormat();
           } else {
@@ -859,7 +858,7 @@ public class HoodieStreamer implements Serializable {
               Option<HoodieData<WriteStatus>> lastWriteStatuses = Option.ofNullable(
                   scheduledCompactionInstantAndRDD.isPresent() ? HoodieJavaRDD.of(scheduledCompactionInstantAndRDD.get().getRight()) : null);
               if (requestShutdownIfNeeded(lastWriteStatuses)) {
-                LOG.warn("Closing and shutting down ingestion service");
+                LOG.info("Closing and shutting down ingestion service");
                 error = true;
                 onIngestionCompletes(false);
                 shutdown(true);
@@ -901,13 +900,13 @@ public class HoodieStreamer implements Serializable {
      * Shutdown async services like compaction/clustering as DeltaSync is shutdown.
      */
     private void shutdownAsyncServices(boolean error) {
-      LOG.info("Delta Sync shutdown. Error ?" + error);
+      LOG.info("Delta Sync shutdown. Error ?{}", error);
       if (asyncCompactService.isPresent()) {
-        LOG.warn("Gracefully shutting down compactor");
+        LOG.info("Gracefully shutting down compactor");
         asyncCompactService.get().shutdown(false);
       }
       if (asyncClusteringService.isPresent()) {
-        LOG.warn("Gracefully shutting down clustering service");
+        LOG.info("Gracefully shutting down clustering service");
         asyncClusteringService.get().shutdown(false);
       }
     }
@@ -925,8 +924,11 @@ public class HoodieStreamer implements Serializable {
 
     @Override
     protected boolean requestShutdownIfNeeded(Option<HoodieData<WriteStatus>> lastWriteStatuses) {
-      Option<JavaRDD<WriteStatus>> lastWriteStatusRDD = Option.ofNullable(lastWriteStatuses.isPresent() ? HoodieJavaRDD.getJavaRDD(lastWriteStatuses.get()) : null);
-      return postWriteTerminationStrategy.isPresent() && postWriteTerminationStrategy.get().shouldShutdown(lastWriteStatusRDD);
+      if (postWriteTerminationStrategy.isPresent()) {
+        Option<JavaRDD<WriteStatus>> lastWriteStatusRDD = lastWriteStatuses.map(HoodieJavaRDD::getJavaRDD);
+        return postWriteTerminationStrategy.get().shouldShutdown(lastWriteStatusRDD);
+      }
+      return false;
     }
 
     /**
