@@ -43,12 +43,10 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
-import java.time.Instant;
-import java.time.ZoneId;
 import java.util.List;
+import java.util.UUID;
 
 import static org.apache.hudi.common.table.HoodieTableMetaClient.SAMPLE_WRITES_FOLDER_PATH;
-import static org.apache.hudi.common.table.timeline.HoodieInstantTimeGenerator.getInstantFromTemporalAccessor;
 import static org.apache.hudi.common.util.ValidationUtils.checkState;
 import static org.apache.hudi.config.HoodieCompactionConfig.COPY_ON_WRITE_RECORD_SIZE_ESTIMATE;
 import static org.apache.hudi.utilities.config.HoodieStreamerConfig.SAMPLE_WRITES_ENABLED;
@@ -75,8 +73,7 @@ public class SparkSampleWritesUtils {
       return Option.empty();
     }
     try {
-      String instantTime = getInstantFromTemporalAccessor(Instant.now().atZone(ZoneId.systemDefault()));
-      Pair<Boolean, String> result = doSampleWrites(jsc, recordsOpt, writeConfig, instantTime);
+      Pair<Boolean, String> result = doSampleWrites(jsc, recordsOpt, writeConfig);
       if (result.getLeft()) {
         long avgSize = getAvgSizeFromSampleWrites(jsc, result.getRight());
         LOG.info("Overwriting record size estimate to {}", avgSize);
@@ -90,12 +87,13 @@ public class SparkSampleWritesUtils {
     return Option.empty();
   }
 
-  private static Pair<Boolean, String> doSampleWrites(JavaSparkContext jsc, Option<JavaRDD<HoodieRecord>> recordsOpt, HoodieWriteConfig writeConfig, String instantTime)
+  private static Pair<Boolean, String> doSampleWrites(JavaSparkContext jsc, Option<JavaRDD<HoodieRecord>> recordsOpt, HoodieWriteConfig writeConfig)
       throws IOException {
-    final String sampleWritesBasePath = getSampleWritesBasePath(jsc, writeConfig, instantTime);
+    String uniqueId = UUID.randomUUID().toString();
+    final String sampleWritesBasePath = getSampleWritesBasePath(jsc, writeConfig, uniqueId);
     HoodieTableMetaClient.newTableBuilder()
         .setTableType(HoodieTableType.COPY_ON_WRITE)
-        .setTableName(String.format("%s_samples_%s", writeConfig.getTableName(), instantTime))
+        .setTableName(String.format("%s_samples_%s", writeConfig.getTableName(), uniqueId))
         .setCDCEnabled(false)
         .initTable(HadoopFSUtils.getStorageConfWithCopy(jsc.hadoopConfiguration()), sampleWritesBasePath);
     TypedProperties props = writeConfig.getProps();
@@ -106,7 +104,6 @@ public class SparkSampleWritesUtils {
         .withMetadataConfig(HoodieMetadataConfig.newBuilder().enable(false).build())
         .withSchemaEvolutionEnable(false)
         .withBulkInsertParallelism(1)
-        .withAutoCommit(true)
         .withPath(sampleWritesBasePath)
         .build();
     Pair<Boolean, String> emptyRes = Pair.of(false, null);
@@ -117,7 +114,7 @@ public class SparkSampleWritesUtils {
         if (samples.isEmpty()) {
           return emptyRes;
         }
-        sampleWriteClient.startCommitWithTime(instantTime);
+        String instantTime = sampleWriteClient.startCommit();
         JavaRDD<WriteStatus> writeStatusRDD = sampleWriteClient.bulkInsert(jsc.parallelize(samples, 1), instantTime);
         if (writeStatusRDD.filter(WriteStatus::hasErrors).count() > 0) {
           LOG.error("sample writes for table {} failed with errors.", writeConfig.getTableName());
@@ -131,14 +128,15 @@ public class SparkSampleWritesUtils {
           }
           return emptyRes;
         } else {
+          sampleWriteClient.commit(instantTime, writeStatusRDD);
           return Pair.of(true, sampleWritesBasePath);
         }
       }).orElse(emptyRes);
     }
   }
 
-  private static String getSampleWritesBasePath(JavaSparkContext jsc, HoodieWriteConfig writeConfig, String instantTime) throws IOException {
-    StoragePath basePath = new StoragePath(writeConfig.getBasePath(), SAMPLE_WRITES_FOLDER_PATH + StoragePath.SEPARATOR + instantTime);
+  private static String getSampleWritesBasePath(JavaSparkContext jsc, HoodieWriteConfig writeConfig, String uniqueId) throws IOException {
+    StoragePath basePath = new StoragePath(writeConfig.getBasePath(), SAMPLE_WRITES_FOLDER_PATH + StoragePath.SEPARATOR + uniqueId);
     HoodieStorage storage = getMetaClient(jsc, writeConfig.getBasePath()).getStorage();
     if (storage.exists(basePath)) {
       storage.deleteDirectory(basePath);
