@@ -21,6 +21,7 @@ package org.apache.hudi.utilities.streamer;
 
 import org.apache.hudi.DataSourceWriteOptions;
 import org.apache.hudi.client.common.HoodieSparkEngineContext;
+import org.apache.hudi.common.config.RecordMergeMode;
 import org.apache.hudi.common.config.TypedProperties;
 import org.apache.hudi.common.model.HoodieCommitMetadata;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
@@ -28,6 +29,7 @@ import org.apache.hudi.common.table.HoodieTableVersion;
 import org.apache.hudi.common.table.checkpoint.Checkpoint;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
+import org.apache.hudi.common.util.collection.Triple;
 import org.apache.hudi.config.HoodieErrorTableConfig;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.storage.HoodieStorage;
@@ -46,12 +48,14 @@ import org.apache.spark.api.java.JavaSparkContext;
 import org.apache.spark.sql.Dataset;
 import org.apache.spark.sql.Row;
 import org.apache.spark.sql.SparkSession;
+import org.junit.jupiter.api.Nested;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.Arguments;
 import org.junit.jupiter.params.provider.MethodSource;
 
 import java.io.IOException;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
@@ -66,6 +70,7 @@ import static org.apache.hudi.utilities.streamer.HoodieStreamer.CHECKPOINT_RESET
 import static org.apache.hudi.utilities.streamer.StreamSync.CHECKPOINT_IGNORE_KEY;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyLong;
@@ -389,4 +394,61 @@ public class TestStreamSync extends SparkClientFunctionalTestHarness {
     expected.put(CHECKPOINT_RESET_KEY, "test-checkpoint");
     assertEquals(expected, result, "Should return default metadata when checkpoint is null");
   }
+
+  @Nested
+  class TestsWithoutSetup {
+    @Test
+    void testParseOverridingMergeConfigsWithEmptyConfigs() {
+      List<String> configs = Collections.emptyList();
+      Option<Triple<RecordMergeMode, String, String>> result = StreamSync.parseOverridingMergeConfigs(configs);
+      assertTrue(result.isEmpty());
+    }
+
+    @Test
+    void testParseOverridingMergeConfigsWithNullConfigs() {
+      Option<Triple<RecordMergeMode, String, String>> result = StreamSync.parseOverridingMergeConfigs(null);
+      assertTrue(result.isEmpty(), "Should return empty option when configs list is null");
+    }
+
+    @Test
+    void testParseOverridingMergeConfigsWithNonMergeConfigs() {
+      List<String> configs = Arrays.asList(
+          "hoodie.datasource.write.table.name=testTable",
+          "hoodie.datasource.write.operation=upsert"
+      );
+      Option<Triple<RecordMergeMode, String, String>> result = StreamSync.parseOverridingMergeConfigs(configs);
+      assertTrue(result.isEmpty());
+    }
+
+    @Test
+    void testParseOverridingMergeConfigsWithSomeMergeConfigs() {
+      List<String> configs = Arrays.asList(
+          "hoodie.datasource.write.table.name=testTable",
+          "hoodie.datasource.write.operation=upsert",
+          "hoodie.write.record.merge.mode=COMMIT_TIME_ORDERING"
+      );
+      Option<Triple<RecordMergeMode, String, String>> result = StreamSync.parseOverridingMergeConfigs(configs);
+      assertTrue(result.isPresent());
+      assertEquals(RecordMergeMode.COMMIT_TIME_ORDERING, result.get().getLeft());
+      assertNull(result.get().getMiddle());
+      assertNull(result.get().getRight());
+    }
+
+    @Test
+    void testParseOverridingMergeConfigsWithMixedConfigs() {
+      List<String> configs = Arrays.asList(
+          "hoodie.datasource.write.table.name=testTable",
+          "hoodie.write.record.merge.mode=COMMIT_TIME_ORDERING",
+          "hoodie.datasource.write.operation=upsert",
+          "hoodie.write.record.merge.strategy.id=any_id",
+          "hoodie.datasource.write.payload.class=org.apache.hudi.common.model.OverwriteWithLatestPayload"
+      );
+      Option<Triple<RecordMergeMode, String, String>> result = StreamSync.parseOverridingMergeConfigs(configs);
+      assertTrue(result.isPresent());
+      Triple<RecordMergeMode, String, String> triple = result.get();
+      assertEquals(RecordMergeMode.COMMIT_TIME_ORDERING, triple.getLeft());
+      assertEquals("org.apache.hudi.common.model.OverwriteWithLatestPayload", triple.getMiddle());
+      assertEquals("any_id", triple.getRight());
+    }
+  }
 }
